<?php

namespace Ems\Logic\ProcessamentoSolicitacao;
use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoAdvertencia extends ProcessamentoSolicitacao {
        
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrorg              = Utils::getParamters()->getNrorg();
        $nrvinculom         = $this->getSolicitacao()->getNrvinculom();
        $dsobssolicitacao   = $this->getSolicitacao()->getDsobssolicitacao();
        $dtcompetencia      = DateUtil::getDataDeString(Utils::getParamters()->getDtmescomptencDiaCorrente(), DateUtil::FORMATO_BRASILEIRO,true);
        $observacao         = $this->getObservacaoComProtocolo($dsobssolicitacao);
        
        // ADVÊRTENCIA/SUSPENSÃO
        $nrsitufuncm       = $itens['NRSITUFUNCM']     ?? null; 
        $dtinisitufunc      = isset($itens['DTINISITUFUNC']) ? DateUtil::getDataDeString(substr($itens['DTINISITUFUNC'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtfimsitufunc      = isset($itens['DTFIMSITUFUNC']) ? DateUtil::getDataDeString(substr($itens['DTFIMSITUFUNC'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $nrqtdiasitufunc   = $itens['NRQTDIASITUFUNC']  ?? null;

        // VALIDAÇÕES
        $vinculom = Utils::getEntityManager()->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg, "nrvinculom" => $nrvinculom));
        if($vinculom->getDtrescisaovinc())
            throw new \Exception("O vínculo está rescindido!");


        $existSituFunc = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(array("nrorg"=> $nrorg, "nrvinculom" => $nrvinculom,"dtinisitufunc" => $dtinisitufunc));
        if($existSituFunc)
            throw new \Exception("Já existe uma alteração de situação funcional com esta data");


        $proximaSituFunc = $this->retornaProximaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc, null);
        if($proximaSituFunc)
            throw new \Exception("Operação bloqueada! <br>Já existe uma alteração funcional com data posterior a alteração funcional adicionada!");


        $dtinisitufuncAnteriorAdmissaoVinc = DateUtil::getDataDeString(substr($dtinisitufunc,0, 10),DateUtil::FORMATO_BRASILEIRO, true)->format('Y/m/d') < $vinculom->getDtadmissaovinc()->format('Y/m/d');
        if($dtinisitufuncAnteriorAdmissaoVinc)
            throw new \Exception("Data de Início da Situação Funcional anterior à admissão do vínculo");

        // END

        // Atualizar data fim da ultima situação funcional se necessario        
        $ultimaSituFunc =  ProcessamentoSolicitacaoUtil::atualizaDatafimUltimaSituFunc($nrorg, $nrvinculom, $dtinisitufunc);

        // Cria a nova situação funcional
        $nrproximasitufunc = $ultimaSituFunc->getNrsitufuncm() ? $ultimaSituFunc->getNrsitufuncm() : null;
        $alteSituFunc = ProcessamentoSolicitacaoUtil::novaAlteSituacaoFuncional($nrorg, $nrvinculom, $nrsitufuncm, $observacao, $dtinisitufunc, $dtfimsitufunc, $nrqtdiasitufunc, $nrproximasitufunc);

        // Cria a proxima situação funcional
        if($ultimaSituFunc) {
            $proxdtinisitufunc = DateUtil::adicionaIntervalo($dtfimsitufunc, 1);
            $ultimanrsitufuncm = $ultimaSituFunc->getNrsitufuncm() ? $ultimaSituFunc->getNrsitufuncm() : '1';
            $alteSituFunc = ProcessamentoSolicitacaoUtil::novaAlteSituacaoFuncional($nrorg, $nrvinculom, $ultimanrsitufuncm, $observacao, $proxdtinisitufunc);
        }

        // $this->atualizaVinculoHistorico($nrorg, $nrvinculom, $nrsitufuncm, Utils::getParamters()->getCdoperador());
    }
    
    public function rejeitar() {}
    
    public function atualizaVinculoHistorico($nrorg, $nrvinculom, $nrsitufuncm, $cdoperador) {
        $sql = <<<SQL
        UPDATE GPE_VINCULOH VH
        SET VH.NRSITUFUNCM = :P_NRSITUFUNCM,
            VH.NRORGULTATU = :P_NRORG,
            VH.CDOPERULTATU = :P_CDOPERADOR
        WHERE VH.DTMESCOMPETENC = (
            SELECT MAX(DTMESCOMPETENC)
            FROM GPE_VINCULOH VHH
            WHERE VHH.NRORG = VH.NRORG
            AND VHH.NRVINCULOM = VH.NRVINCULOM
        )
        AND VH.NRORG = :P_NRORG
        AND VH.NRVINCULOM = :P_NRVINCULOM
SQL;

        return Utils::getEntityManager()->getConnection()->prepare($sql)
            ->execute(array(
                "P_NRORG"        => $nrorg,
                "P_NRVINCULOM"   => $nrvinculom,
                "P_NRSITUFUNCM"  => $nrsitufuncm,
                "P_CDOPERADOR"   => $cdoperador
            )
        );
    }

    public function retornaProximaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc, $nrproxsitufunc=null) {
        $gpeAltesitufunc = Repositories::GPE_ALTESITUFUNC;
        $sql = <<<SQL
            SELECT a
            FROM $gpeAltesitufunc a
            WHERE a.nrorg         = :NRORG
            AND a.nrvinculom    = :NRVINCULOM
            AND a.dtinisitufunc > :DTINISITUFUNC
            AND (:NRSITUFUNCM IS NULL OR a.nrsitufuncm = :NRSITUFUNCM)
  
SQL;
        $params = array(
            'NRORG' => $nrorg,
            'NRVINCULOM' => $nrvinculom,
            'NRSITUFUNCM' => $nrproxsitufunc,
            'DTINISITUFUNC' => $dtinisitufunc
        );

        $result = Utils::getEntityManager()->createQuery($sql)->execute($params);

        return empty($result) ? $result : $result[0];
    }

}