<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Teknisa\Model\Gestaopessoas\Entities\GpeSolicitacao;

class ProcessamentoSolicitacaoFactory {
    
    const TP_SOLI_FERIAS                    = 1;
    const TP_SOLI_MOVIM_LEGAL               = 2;
    const TP_SOLI_SALARIO                   = 3;
    const TP_SOLI_TREINAMENTO               = 4;
    const TP_SOLI_DESLIGAMENTO              = 5;
    const TP_SOLI_PESSOAL                   = 6;
    const TP_SOLI_ALTERADADOS               = 7;
    const TP_SOLI_ALTERAENDERECO            = 8;
    const TP_SOLI_MOVIMENT_OCUPACAO_SALARIO = 9;
    const TP_SOLI_VTRANS                    = 10;
    const TP_SOLI_DEP                       = 11;
    const TP_SOLI_ALTESITU                  = 12;
    const TP_SOLI_PLANOSAUDE                = 13;
    const TP_SOLI_ALTERAESCALA              = 14;
    const TP_SOLI_ALTERACONTA               = 15;
    const TP_SOLI_ALTERACOMU                = 16;
    const TP_SOLI_ADMISSAO                  = 17;
    const TP_SOLI_APROVAHE                  = 18;
    const TP_SOLI_REEMBOLSO                 = 19;
    const TP_SOLI_ADVERTENCIA               = 20;
    const TP_SOLI_VCOMBUST                  = 21;
    const TP_SOLI_AUXILIOMORADIA            = 22;
    const TP_SOLI_VMANUTENCAO               = 23;
    const TP_SOLI_BENEFICIOS                = 24;
    const TP_SOLI_SUBSTITUICAO              = 25;
    const TP_SOLI_MOVIM_GERENCIAL           = 26;
    const TP_SOLI_OCUPACAO                  = 27;
    const TP_SOLI_AUMENTO_QUADRO            = 28;
    const TP_SOLI_SALARIO_ESCALA            = 29;

    /**
     * Instancia entidades de processamento das solicitacoes a partir de uma solicitacao e informacoes extras opcionais.
     * 
     * @param GpeSolicitacao  $solicitacao,
     * @param ?array          $extraData = []
     * 
     * @return ProcessamentoSolicitacao
     */
    public static function createProcessamentoSolicitacao(
        GpeSolicitacao  $solicitacao,
        ?array          $extraData = []
    ): ?ProcessamentoSolicitacao {
        switch ($solicitacao->getNrtiposolicitacao()) {
            case self::TP_SOLI_FERIAS:
                return new ProcessamentoSolicitacaoFerias($solicitacao, $extraData);
            case self::TP_SOLI_ALTERAESCALA:
                return new ProcessamentoSolicitacaoEscala($solicitacao, $extraData);
            case self::TP_SOLI_ALTERADADOS:
                return new ProcessamentoSolicitacaoDadosPessoa($solicitacao, $extraData);
            case self::TP_SOLI_MOVIMENT_OCUPACAO_SALARIO:
                return new ProcessamentoSolicitacaoMovimentacaoOcupacaoSalario($solicitacao, $extraData);
            case self::TP_SOLI_SALARIO:
                return new ProcessamentoSolicitacaoSalario($solicitacao, $extraData);
            case self::TP_SOLI_DESLIGAMENTO:
                return new ProcessamentoSolicitacaoDesligamento($solicitacao, $extraData);
            case self::TP_SOLI_DEP:
                return new ProcessamentoSolicitacaoDependente($solicitacao, $extraData);
            case self::TP_SOLI_PESSOAL:
                return new ProcessamentoSolicitacaoPessoal($solicitacao, $extraData);
            case self::TP_SOLI_PLANOSAUDE:
                return new ProcessamentoSolicitacaoPlanoSaude($solicitacao, $extraData);
            case self::TP_SOLI_MOVIM_LEGAL:
                return new ProcessamentoSolicitacaoRemanejamentoLegal($solicitacao, $extraData);
            case self::TP_SOLI_MOVIM_GERENCIAL:
                return new ProcessamentoSolicitacaoRemanejamentoGerencial($solicitacao, $extraData);
            case self::TP_SOLI_VTRANS:
                return new ProcessamentoSolicitacaoValeTransporte($solicitacao, $extraData);
            case self::TP_SOLI_ALTERACOMU:
                return new ProcessamentoSolicitacaoFormaComunicacao($solicitacao, $extraData);
            case self::TP_SOLI_ALTERAENDERECO:
                return new ProcessamentoSolicitacaoEndereco($solicitacao, $extraData);
            case self::TP_SOLI_ALTERACONTA:
                return new ProcessamentoSolicitacaoContaCorrente($solicitacao, $extraData);
            case self::TP_SOLI_APROVAHE:
                return new ProcessamentoSolicitacaoAprovaHorasExtras($solicitacao, $extraData);
            case self::TP_SOLI_TREINAMENTO:
                return new ProcessamentoSolicitacaoTreinamento($solicitacao, $extraData);
            case self::TP_SOLI_REEMBOLSO:
                return new ProcessamentoSolicitacaoAprovaReembolso($solicitacao, $extraData);
            case self::TP_SOLI_ADVERTENCIA:
                return new ProcessamentoSolicitacaoAdvertencia($solicitacao, $extraData);
            case self::TP_SOLI_VCOMBUST:
                return new ProcessamentoSolicitacaoValeCombustivel($solicitacao, $extraData);
            case self::TP_SOLI_AUXILIOMORADIA:
                return new ProcessamentoSolicitacaoAuxilioMoradia($solicitacao, $extraData);
            case self::TP_SOLI_VMANUTENCAO:
                return new ProcessamentoSolicitacaoValeManutencao($solicitacao, $extraData);
            case self::TP_SOLI_BENEFICIOS:
                return new ProcessamentoSolicitacaoBeneficios($solicitacao, $extraData);
            case self::TP_SOLI_SUBSTITUICAO:
                return new ProcessamentoSolicitacaoSubstituicao($solicitacao, $extraData);
            case self::TP_SOLI_OCUPACAO:
                return new ProcessamentoSolicitacaoOcupacao($solicitacao, $extraData);
            case self::TP_SOLI_AUMENTO_QUADRO:
                return new ProcessamentoSolicitacaoAumentoQuadro($solicitacao, $extraData);
            case self::TP_SOLI_SALARIO_ESCALA:
                return new ProcessamentoSolicitacaoSalarioEscala($solicitacao, $extraData);    
            default:
                return null;
        }
    }
}