<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Ems\Util\Utils;

class IndicadoresSaudeTrabalhoRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function getTotalAfastamentosPorCIDDashboard(
        $row
   ) {

       $sql = <<<SQL
      
       SELECT COUNT(1) AS QTD_AFASTAMENTOS
FROM GPE_ALTESITUFUNC ASF
JOIN GPE_VINCULOM VM
   ON VM.NRORG = ASF.NRORG
       AND ASF.NRVINCULOM = VM.NRVINCULOM
JOIN GPE_VINCULOH VH
   ON VM.NRORG = VH.NRORG
       AND VM.NRVINCULOM = VH.NRVINCULOM
       AND VH.DTMESCOMPETENC =
   (SELECT DTMESCOMPETENC
   FROM 
       (SELECT H1.DTMESCOMPETENC
       FROM GPE_VINCULOH H1
       WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
               AND H1.NRORG = VM.NRORG
               AND H1.NRVINCULOM = VM.NRVINCULOM
       ORDER BY  H1.DTMESCOMPETENC DESC)
       WHERE ROWNUM = 1)
   LEFT JOIN GPE_ALTEOCUPACAO AO
   ON AO.NRORG = VM.NRORG
       AND AO.NRVINCULOM = VM.NRVINCULOM
       AND AO.DTINIOCUPACAO = 
   (SELECT MAX(DTINIOCUPACAO)
   FROM GPE_ALTEOCUPACAO
   WHERE NRORG = VM.NRORG
           AND NRVINCULOM = VM.NRVINCULOM
           AND DTINIOCUPACAO <= LAST_DAY(:P_DTMESCOMPETENC))
LEFT JOIN GPE_OCUPACAOH OH
   ON AO.NROCUPACAOM = OH.NROCUPACAOM
       AND OH.NRORG = AO.NRORG
       AND OH.DTMESCOMPETENC = 
   (SELECT MAX(OHH.DTMESCOMPETENC)
   FROM GPE_OCUPACAOH OHH
   WHERE OHH.NRORG = :P_NRORG
           AND OHH.NROCUPACAOM = OH.NROCUPACAOM
           AND OHH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
LEFT JOIN TIPOHIERARQORG TH
   ON (TH.NRORG = VM.NRORG
       OR TH.NRORG = 0)
       AND TH.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
       AND TH.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTOMADOR, 7)
LEFT JOIN HIERARQUIAORG HO
   ON HO.NRORG = VM.NRORG
       AND HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG
       AND HO.NRESTRUTURAMREL = VH.NRESTRUTGEREN
       AND HO.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
       AND (HO.DTFIMVIGENCIA IS NULL
       OR HO.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
       AND HO.DTINIVIGENCIA = 
   (SELECT MAX(DTINIVIGENCIA)
   FROM HIERARQUIAORG HI
   WHERE HI.NRORG = HO.NRORG
           AND HI.NRESTRUTURAMREL = HO.NRESTRUTURAMREL
           AND NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG
           AND HI.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
           AND (HI.DTFIMVIGENCIA IS NULL
           OR HI.DTFIMVIGENCIA >= :P_DTMESCOMPETENC))
LEFT JOIN ESTRUTURAM EMSUP
   ON EMSUP.NRESTRUTURAM = HO.NRESTRUTURAMSUP
       AND EMSUP.NRORG = HO.NRORG
LEFT JOIN ESTRUTURAM EM
   ON EM.NRESTRUTURAM = VH.NRESTRUTLEGAL
       AND EM.NRORG = VH.NRORG
JOIN GPE_SITUFUNCM SF
   ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
JOIN GPE_SITUFUNCH SFH
   ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
       AND SFH.DTEFETIVA = 
   (SELECT MAX(DTEFETIVA)
   FROM GPE_SITUFUNCH
   WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
           AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
           AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
LEFT JOIN DIAGNOST DG
   ON DG.CDDIAGNOST = ASF.CDDIAGNOST
WHERE ASF.NRORG = :P_NRORG
       AND (VH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL
       OR :P_NRESTRUTLEGAL IS NULL)
       AND (:P_NRESTRUTGEREN IS NULL OR INSTR(',' || :P_NRESTRUTGEREN || ',', ',' || VH.NRESTRUTGEREN || ',') > 0)
       AND (VH.NRESTRUTSIND = :P_NRESTRUTSIND
       OR :P_NRESTRUTSIND IS NULL)
       AND (HO.NRESTRUTURAMSUP = :P_NRESTRUTTOMAD
       OR :P_NRESTRUTTOMAD IS NULL)
       AND (VM.DTRESCISAOVINC IS NULL
       OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
       AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
       AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
       OR SF.NRSITUFUNCM IN (12, 100012, 15, 16, 100084, 3, 5, 11))/*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
       AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
       AND ASF.CDDIAGNOST IS NOT NULL
SQL;

   $query = $this->connection->prepare($sql);
   $query->execute(
       array(
           'P_NRORG'       => (int) $row['NRORG'],
           'P_DTMESCOMPETENC'   => $row['DTMESCOMPETENC'],
           'P_NRTIPOESTRUTOMADOR' => $row['NRTIPOESTRUTOMADOR'],
           'P_NRESTRUTLEGAL' => $row['NRESTRUTLEGAL'],
           'P_NRESTRUTGEREN' => $row['NRESTRUTGEREN'],
           'P_NRESTRUTSIND' => null,
           'P_NRESTRUTTOMAD' => null,
           'P_NRTIPOESTRUTGEREN' => $row['NRTIPOESTRUTGEREN'],
       )
   );

   return $query->fetchAll();
   }


    public function getTotalAfastamentosPorCIDLegalOcupacao(
         $row
    ) {

        $sql = <<<SQL
        SELECT 'Estrutura: ' || EM.NMESTRUTURAM || ' || Ocupação: ' || OH.NMOCUPACAOH || ' || CID: ' || ASF.CDDIAGNOST AS INDICADOR, COUNT(1) AS QTD_AFASTAMENTOS
FROM GPE_ALTESITUFUNC ASF
JOIN GPE_VINCULOM VM
    ON VM.NRORG = ASF.NRORG
        AND ASF.NRVINCULOM = VM.NRVINCULOM
JOIN GPE_VINCULOH VH
    ON VM.NRORG = VH.NRORG
        AND VM.NRVINCULOM = VH.NRVINCULOM
        AND VH.DTMESCOMPETENC =
    (SELECT DTMESCOMPETENC
    FROM 
        (SELECT H1.DTMESCOMPETENC
        FROM GPE_VINCULOH H1
        WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                AND H1.NRORG = VM.NRORG
                AND H1.NRVINCULOM = VM.NRVINCULOM
        ORDER BY  H1.DTMESCOMPETENC DESC)
        WHERE ROWNUM = 1)
    LEFT JOIN GPE_ALTEOCUPACAO AO
    ON AO.NRORG = VM.NRORG
        AND AO.NRVINCULOM = VM.NRVINCULOM
        AND AO.DTINIOCUPACAO = 
    (SELECT MAX(DTINIOCUPACAO)
    FROM GPE_ALTEOCUPACAO
    WHERE NRORG = VM.NRORG
            AND NRVINCULOM = VM.NRVINCULOM
            AND DTINIOCUPACAO <= LAST_DAY(:P_DTMESCOMPETENC))
LEFT JOIN GPE_OCUPACAOH OH
    ON AO.NROCUPACAOM = OH.NROCUPACAOM
        AND OH.NRORG = AO.NRORG
        AND OH.DTMESCOMPETENC = 
    (SELECT MAX(OHH.DTMESCOMPETENC)
    FROM GPE_OCUPACAOH OHH
    WHERE OHH.NRORG = :P_NRORG
            AND OHH.NROCUPACAOM = OH.NROCUPACAOM
            AND OHH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
LEFT JOIN TIPOHIERARQORG TH
    ON (TH.NRORG = VM.NRORG
        OR TH.NRORG = 0)
        AND TH.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
        AND TH.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTOMADOR, 7)
LEFT JOIN HIERARQUIAORG HO
    ON HO.NRORG = VM.NRORG
        AND HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG
        AND HO.NRESTRUTURAMREL = VH.NRESTRUTGEREN
        AND HO.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
        AND (HO.DTFIMVIGENCIA IS NULL
        OR HO.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
        AND HO.DTINIVIGENCIA = 
    (SELECT MAX(DTINIVIGENCIA)
    FROM HIERARQUIAORG HI
    WHERE HI.NRORG = HO.NRORG
            AND HI.NRESTRUTURAMREL = HO.NRESTRUTURAMREL
            AND NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG
            AND HI.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
            AND (HI.DTFIMVIGENCIA IS NULL
            OR HI.DTFIMVIGENCIA >= :P_DTMESCOMPETENC))
LEFT JOIN ESTRUTURAM EMSUP
    ON EMSUP.NRESTRUTURAM = HO.NRESTRUTURAMSUP
        AND EMSUP.NRORG = HO.NRORG
LEFT JOIN ESTRUTURAM EM
    ON EM.NRESTRUTURAM = VH.NRESTRUTLEGAL
        AND EM.NRORG = VH.NRORG
JOIN GPE_SITUFUNCM SF
    ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
JOIN GPE_SITUFUNCH SFH
    ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
        AND SFH.DTEFETIVA = 
    (SELECT MAX(DTEFETIVA)
    FROM GPE_SITUFUNCH
    WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
            AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
            AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
LEFT JOIN DIAGNOST DG
    ON DG.CDDIAGNOST = ASF.CDDIAGNOST
WHERE ASF.NRORG = :P_NRORG
        AND (VH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL
        OR :P_NRESTRUTLEGAL IS NULL)
        AND (:P_NRESTRUTGEREN IS NULL OR INSTR(',' || :P_NRESTRUTGEREN || ',', ',' || VH.NRESTRUTGEREN || ',') > 0)
        AND (VH.NRESTRUTSIND = :P_NRESTRUTSIND
        OR :P_NRESTRUTSIND IS NULL)
        AND (HO.NRESTRUTURAMSUP = :P_NRESTRUTTOMAD
        OR :P_NRESTRUTTOMAD IS NULL)
        AND (VM.DTRESCISAOVINC IS NULL
        OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
        AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
        AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
        OR SF.NRSITUFUNCM IN (12, 100012, 15, 16, 100084, 3, 5, 11))/*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
        AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
GROUP BY  EM.NMESTRUTURAM , OH.NMOCUPACAOH, ASF.CDDIAGNOST
HAVING ASF.CDDIAGNOST IS NOT NULL
ORDER BY QTD_AFASTAMENTOS DESC
OFFSET 0 ROWS FETCH NEXT 10 ROWS ONLY
SQL;

    $query = $this->connection->prepare($sql);
    $query->execute(
        array(
            'P_NRORG'       => (int) $row['NRORG'],
            'P_DTMESCOMPETENC'   => $row['DTMESCOMPETENC'],
            'P_NRTIPOESTRUTOMADOR' => $row['NRTIPOESTRUTOMADOR'],
            'P_NRESTRUTLEGAL' => $row['NRESTRUTLEGAL'],
            'P_NRESTRUTGEREN' => $row['NRESTRUTGEREN'],
            'P_NRESTRUTSIND' => null,
            'P_NRESTRUTTOMAD' => null,
            'P_NRTIPOESTRUTGEREN' => $row['NRTIPOESTRUTGEREN'],
        )
    );

    return $query->fetchAll();
    }

    public function getTotalAfastamentosPorCIDOcupacao(
        $row
   ) {

       $sql = <<<SQL
       SELECT 'Ocupação: ' || OH.NMOCUPACAOH || ' || CID: ' || ASF.CDDIAGNOST AS INDICADOR,COUNT(1) AS QTD_AFASTAMENTOS
FROM GPE_ALTESITUFUNC ASF
JOIN GPE_VINCULOM VM
    ON VM.NRORG = ASF.NRORG
        AND ASF.NRVINCULOM = VM.NRVINCULOM
JOIN GPE_VINCULOH VH
    ON VM.NRORG = VH.NRORG
        AND VM.NRVINCULOM = VH.NRVINCULOM
        AND VH.DTMESCOMPETENC =
    (SELECT DTMESCOMPETENC
    FROM 
        (SELECT H1.DTMESCOMPETENC
        FROM GPE_VINCULOH H1
        WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                AND H1.NRORG = VM.NRORG
                AND H1.NRVINCULOM = VM.NRVINCULOM
        ORDER BY  H1.DTMESCOMPETENC DESC)
        WHERE ROWNUM = 1)
    LEFT JOIN GPE_ALTEOCUPACAO AO
    ON AO.NRORG = VM.NRORG
        AND AO.NRVINCULOM = VM.NRVINCULOM
        AND AO.DTINIOCUPACAO = 
    (SELECT MAX(DTINIOCUPACAO)
    FROM GPE_ALTEOCUPACAO
    WHERE NRORG = VM.NRORG
            AND NRVINCULOM = VM.NRVINCULOM
            AND DTINIOCUPACAO <= LAST_DAY(:P_DTMESCOMPETENC))
LEFT JOIN GPE_OCUPACAOH OH
    ON AO.NROCUPACAOM = OH.NROCUPACAOM
        AND OH.NRORG = AO.NRORG
        AND OH.DTMESCOMPETENC = 
    (SELECT MAX(OHH.DTMESCOMPETENC)
    FROM GPE_OCUPACAOH OHH
    WHERE OHH.NRORG = :P_NRORG
            AND OHH.NROCUPACAOM = OH.NROCUPACAOM
            AND OHH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
LEFT JOIN TIPOHIERARQORG TH
    ON (TH.NRORG = VM.NRORG
        OR TH.NRORG = 0)
        AND TH.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
        AND TH.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTOMADOR, 7)
LEFT JOIN HIERARQUIAORG HO
    ON HO.NRORG = VM.NRORG
        AND HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG
        AND HO.NRESTRUTURAMREL = VH.NRESTRUTGEREN
        AND HO.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
        AND (HO.DTFIMVIGENCIA IS NULL
        OR HO.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
        AND HO.DTINIVIGENCIA = 
    (SELECT MAX(DTINIVIGENCIA)
    FROM HIERARQUIAORG HI
    WHERE HI.NRORG = HO.NRORG
            AND HI.NRESTRUTURAMREL = HO.NRESTRUTURAMREL
            AND NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG
            AND HI.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
            AND (HI.DTFIMVIGENCIA IS NULL
            OR HI.DTFIMVIGENCIA >= :P_DTMESCOMPETENC))
LEFT JOIN ESTRUTURAM EMSUP
    ON EMSUP.NRESTRUTURAM = HO.NRESTRUTURAMSUP
        AND EMSUP.NRORG = HO.NRORG
LEFT JOIN ESTRUTURAM EM
    ON EM.NRESTRUTURAM = VH.NRESTRUTLEGAL
        AND EM.NRORG = VH.NRORG
JOIN GPE_SITUFUNCM SF
    ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
JOIN GPE_SITUFUNCH SFH
    ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
        AND SFH.DTEFETIVA = 
    (SELECT MAX(DTEFETIVA)
    FROM GPE_SITUFUNCH
    WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
            AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
            AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
LEFT JOIN DIAGNOST DG
    ON DG.CDDIAGNOST = ASF.CDDIAGNOST
WHERE ASF.NRORG = :P_NRORG
        AND (VH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL
        OR :P_NRESTRUTLEGAL IS NULL)
        AND (:P_NRESTRUTGEREN IS NULL OR INSTR(',' || :P_NRESTRUTGEREN || ',', ',' || VH.NRESTRUTGEREN || ',') > 0)
        AND (VH.NRESTRUTSIND = :P_NRESTRUTSIND
        OR :P_NRESTRUTSIND IS NULL)
        AND (HO.NRESTRUTURAMSUP = :P_NRESTRUTTOMAD
        OR :P_NRESTRUTTOMAD IS NULL)
        AND (VM.DTRESCISAOVINC IS NULL
        OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
        AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
        AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
        OR SF.NRSITUFUNCM IN (12, 100012, 15, 16, 100084, 3, 5, 11))/*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
        AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
GROUP BY  OH.NMOCUPACAOH, ASF.CDDIAGNOST
HAVING ASF.CDDIAGNOST IS NOT NULL
ORDER BY QTD_AFASTAMENTOS DESC
OFFSET 0 ROWS FETCH NEXT 10 ROWS ONLY
SQL;

   $query = $this->connection->prepare($sql);
   $query->execute(
    array(
        'P_NRORG'       => (int) $row['NRORG'],
        'P_DTMESCOMPETENC'   => $row['DTMESCOMPETENC'],
        'P_NRTIPOESTRUTOMADOR' => $row['NRTIPOESTRUTOMADOR'],
        'P_NRESTRUTLEGAL' => $row['NRESTRUTLEGAL'],
        'P_NRESTRUTGEREN' => $row['NRESTRUTGEREN'],
        'P_NRESTRUTSIND' => null,
        'P_NRESTRUTTOMAD' => null,
        'P_NRTIPOESTRUTGEREN' => $row['NRTIPOESTRUTGEREN'],
        )
    );

   return $query->fetchAll();
   }

   public function getTotalAfastamentosPorNTEPLegalOcupacao(
    $row
) {
   $sql = <<<SQL
   SELECT 'Estrutura: ' || EM.NMESTRUTURAM || ' || Ocupação: ' || OH.NMOCUPACAOH || ' || CID: ' || ASF.CDDIAGNOST AS INDICADOR, COUNT(1) AS QTD_AFASTAMENTOS
FROM GPE_ALTESITUFUNC ASF
JOIN GPE_VINCULOM VM
    ON VM.NRORG = ASF.NRORG
        AND ASF.NRVINCULOM = VM.NRVINCULOM
JOIN GPE_VINCULOH VH
    ON VM.NRORG = VH.NRORG
        AND VM.NRVINCULOM = VH.NRVINCULOM
        AND VH.DTMESCOMPETENC =
    (SELECT DTMESCOMPETENC
    FROM 
        (SELECT H1.DTMESCOMPETENC
        FROM GPE_VINCULOH H1
        WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                AND H1.NRORG = VM.NRORG
                AND H1.NRVINCULOM = VM.NRVINCULOM
        ORDER BY  H1.DTMESCOMPETENC DESC)
        WHERE ROWNUM = 1)
    LEFT JOIN GPE_ALTEOCUPACAO AO
    ON AO.NRORG = VM.NRORG
        AND AO.NRVINCULOM = VM.NRVINCULOM
        AND AO.DTINIOCUPACAO = 
    (SELECT MAX(DTINIOCUPACAO)
    FROM GPE_ALTEOCUPACAO
    WHERE NRORG = VM.NRORG
            AND NRVINCULOM = VM.NRVINCULOM
            AND DTINIOCUPACAO <= LAST_DAY(:P_DTMESCOMPETENC))
LEFT JOIN GPE_OCUPACAOH OH
    ON AO.NROCUPACAOM = OH.NROCUPACAOM
        AND OH.NRORG = AO.NRORG
        AND OH.DTMESCOMPETENC = 
    (SELECT MAX(OHH.DTMESCOMPETENC)
    FROM GPE_OCUPACAOH OHH
    WHERE OHH.NRORG = :P_NRORG
            AND OHH.NROCUPACAOM = OH.NROCUPACAOM
            AND OHH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
LEFT JOIN TIPOHIERARQORG TH
    ON (TH.NRORG = VM.NRORG
        OR TH.NRORG = 0)
        AND TH.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
        AND TH.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTOMADOR, 7)
LEFT JOIN HIERARQUIAORG HO
    ON HO.NRORG = VM.NRORG
        AND HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG
        AND HO.NRESTRUTURAMREL = VH.NRESTRUTGEREN
        AND HO.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
        AND (HO.DTFIMVIGENCIA IS NULL
        OR HO.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
        AND HO.DTINIVIGENCIA = 
    (SELECT MAX(DTINIVIGENCIA)
    FROM HIERARQUIAORG HI
    WHERE HI.NRORG = HO.NRORG
            AND HI.NRESTRUTURAMREL = HO.NRESTRUTURAMREL
            AND NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG
            AND HI.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
            AND (HI.DTFIMVIGENCIA IS NULL
            OR HI.DTFIMVIGENCIA >= :P_DTMESCOMPETENC))
LEFT JOIN ESTRUTURAM EMSUP
    ON EMSUP.NRESTRUTURAM = HO.NRESTRUTURAMSUP
        AND EMSUP.NRORG = HO.NRORG
LEFT JOIN ESTRUTURAM EM
    ON EM.NRESTRUTURAM = VH.NRESTRUTLEGAL
        AND EM.NRORG = VH.NRORG
JOIN GPE_SITUFUNCM SF
    ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
JOIN GPE_SITUFUNCH SFH
    ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
        AND SFH.DTEFETIVA = 
    (SELECT MAX(DTEFETIVA)
    FROM GPE_SITUFUNCH
    WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
            AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
            AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
LEFT JOIN DIAGNOST DG
    ON DG.CDDIAGNOST = ASF.CDDIAGNOST
WHERE ASF.NRORG = :P_NRORG
        AND (VH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL
        OR :P_NRESTRUTLEGAL IS NULL)
        AND (:P_NRESTRUTGEREN IS NULL OR INSTR(',' || :P_NRESTRUTGEREN || ',', ',' || VH.NRESTRUTGEREN || ',') > 0)
        AND (VH.NRESTRUTSIND = :P_NRESTRUTSIND
        OR :P_NRESTRUTSIND IS NULL)
        AND (HO.NRESTRUTURAMSUP = :P_NRESTRUTTOMAD
        OR :P_NRESTRUTTOMAD IS NULL)
        AND (VM.DTRESCISAOVINC IS NULL
        OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
        AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
        AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
        OR SF.NRSITUFUNCM IN (12, 100012, 15, 16, 100084, 3, 5, 11))/*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
        AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
        AND ASF.CDDIAGNOST IN 
    (SELECT CID.CDCID
    FROM SST_CNAE_CID CNAECID
    INNER JOIN SST_CID CID
        ON CNAECID.NRCID = CID.NRCID
    INNER JOIN CNAEFISCAL CNAE
        ON CNAECID.CDCNAE = CNAE.CDCNAE)
GROUP BY  EM.NMESTRUTURAM, OH.NMOCUPACAOH, ASF.CDDIAGNOST
HAVING ASF.CDDIAGNOST IS NOT NULL
ORDER BY QTD_AFASTAMENTOS DESC
OFFSET 0 ROWS FETCH NEXT 10 ROWS ONLY
SQL;

$query = $this->connection->prepare($sql);
$query->execute(
    array(
        'P_NRORG'       => (int) $row['NRORG'],
        'P_DTMESCOMPETENC'   => $row['DTMESCOMPETENC'],
        'P_NRTIPOESTRUTOMADOR' => $row['NRTIPOESTRUTOMADOR'],
        'P_NRESTRUTLEGAL' => $row['NRESTRUTLEGAL'],
        'P_NRESTRUTGEREN' => $row['NRESTRUTGEREN'],
        'P_NRESTRUTSIND' => null,
        'P_NRESTRUTTOMAD' => null,
        'P_NRTIPOESTRUTGEREN' => $row['NRTIPOESTRUTGEREN'],
        )
    );

return $query->fetchAll();
}

public function getTotalAfastamentosPorNTEPOcupacao(
    $row
) {

   $sql = <<<SQL
  SELECT 'Ocupação: ' || OH.NMOCUPACAOH || ' || CID: ' || ASF.CDDIAGNOST AS INDICADOR,COUNT(1) AS QTD_AFASTAMENTOS
FROM GPE_ALTESITUFUNC ASF
JOIN GPE_VINCULOM VM
    ON VM.NRORG = ASF.NRORG
        AND ASF.NRVINCULOM = VM.NRVINCULOM
JOIN GPE_VINCULOH VH
    ON VM.NRORG = VH.NRORG
        AND VM.NRVINCULOM = VH.NRVINCULOM
        AND VH.DTMESCOMPETENC =
    (SELECT DTMESCOMPETENC
    FROM 
        (SELECT H1.DTMESCOMPETENC
        FROM GPE_VINCULOH H1
        WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                AND H1.NRORG = VM.NRORG
                AND H1.NRVINCULOM = VM.NRVINCULOM
        ORDER BY  H1.DTMESCOMPETENC DESC)
        WHERE ROWNUM = 1)
    LEFT JOIN GPE_ALTEOCUPACAO AO
    ON AO.NRORG = VM.NRORG
        AND AO.NRVINCULOM = VM.NRVINCULOM
        AND AO.DTINIOCUPACAO = 
    (SELECT MAX(DTINIOCUPACAO)
    FROM GPE_ALTEOCUPACAO
    WHERE NRORG = VM.NRORG
            AND NRVINCULOM = VM.NRVINCULOM
            AND DTINIOCUPACAO <= LAST_DAY(:P_DTMESCOMPETENC))
LEFT JOIN GPE_OCUPACAOH OH
    ON AO.NROCUPACAOM = OH.NROCUPACAOM
        AND OH.NRORG = AO.NRORG
        AND OH.DTMESCOMPETENC = 
    (SELECT MAX(OHH.DTMESCOMPETENC)
    FROM GPE_OCUPACAOH OHH
    WHERE OHH.NRORG = :P_NRORG
            AND OHH.NROCUPACAOM = OH.NROCUPACAOM
            AND OHH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY'))
LEFT JOIN TIPOHIERARQORG TH
    ON (TH.NRORG = VM.NRORG
        OR TH.NRORG = 0)
        AND TH.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
        AND TH.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTOMADOR, 7)
LEFT JOIN HIERARQUIAORG HO
    ON HO.NRORG = VM.NRORG
        AND HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG
        AND HO.NRESTRUTURAMREL = VH.NRESTRUTGEREN
        AND HO.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
        AND (HO.DTFIMVIGENCIA IS NULL
        OR HO.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
        AND HO.DTINIVIGENCIA = 
    (SELECT MAX(DTINIVIGENCIA)
    FROM HIERARQUIAORG HI
    WHERE HI.NRORG = HO.NRORG
            AND HI.NRESTRUTURAMREL = HO.NRESTRUTURAMREL
            AND NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG
            AND HI.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
            AND (HI.DTFIMVIGENCIA IS NULL
            OR HI.DTFIMVIGENCIA >= :P_DTMESCOMPETENC))
LEFT JOIN ESTRUTURAM EMSUP
    ON EMSUP.NRESTRUTURAM = HO.NRESTRUTURAMSUP
        AND EMSUP.NRORG = HO.NRORG
LEFT JOIN ESTRUTURAM EM
    ON EM.NRESTRUTURAM = VH.NRESTRUTLEGAL
        AND EM.NRORG = VH.NRORG
JOIN GPE_SITUFUNCM SF
    ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
JOIN GPE_SITUFUNCH SFH
    ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
        AND SFH.DTEFETIVA = 
    (SELECT MAX(DTEFETIVA)
    FROM GPE_SITUFUNCH
    WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
            AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
            AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
LEFT JOIN DIAGNOST DG
    ON DG.CDDIAGNOST = ASF.CDDIAGNOST
WHERE ASF.NRORG = :P_NRORG
        AND (VH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL
        OR :P_NRESTRUTLEGAL IS NULL)
        AND (:P_NRESTRUTGEREN IS NULL OR INSTR(',' || :P_NRESTRUTGEREN || ',', ',' || VH.NRESTRUTGEREN || ',') > 0)
        AND (VH.NRESTRUTSIND = :P_NRESTRUTSIND
        OR :P_NRESTRUTSIND IS NULL)
        AND (HO.NRESTRUTURAMSUP = :P_NRESTRUTTOMAD
        OR :P_NRESTRUTTOMAD IS NULL)
        AND (VM.DTRESCISAOVINC IS NULL
        OR VM.DTRESCISAOVINC >= TRUNC(SYSDATE))
        AND VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
        AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
        OR SF.NRSITUFUNCM IN (12, 100012, 15, 16, 100084, 3, 5, 11))/*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
        AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
        AND ASF.CDDIAGNOST IN 
    (SELECT CID.CDCID
    FROM SST_CNAE_CID CNAECID
    INNER JOIN SST_CID CID
        ON CNAECID.NRCID = CID.NRCID
    INNER JOIN CNAEFISCAL CNAE
        ON CNAECID.CDCNAE = CNAE.CDCNAE)
GROUP BY  OH.NMOCUPACAOH, ASF.CDDIAGNOST
HAVING ASF.CDDIAGNOST IS NOT NULL
ORDER BY QTD_AFASTAMENTOS DESC
OFFSET 0 ROWS FETCH NEXT 10 ROWS ONLY
SQL;

$query = $this->connection->prepare($sql);
$query->execute(
    array(
        'P_NRORG'       => (int) $row['NRORG'],
        'P_DTMESCOMPETENC'   => $row['DTMESCOMPETENC'],
        'P_NRTIPOESTRUTOMADOR' => $row['NRTIPOESTRUTOMADOR'],
        'P_NRESTRUTLEGAL' => $row['NRESTRUTLEGAL'],
        'P_NRESTRUTGEREN' => $row['NRESTRUTGEREN'],
        'P_NRESTRUTSIND' => null,
        'P_NRESTRUTTOMAD' => null,
        'P_NRTIPOESTRUTGEREN' => $row['NRTIPOESTRUTGEREN'],
        )
    );

return $query->fetchAll();
}
}