<?php

namespace Ems\Services;

use Util\Environment;
use Ems\Repository\IndicadoresSaudeTrabalhoRepository;
use Ems\Repository\OcorrenciaRepository;
use Ems\Services\EstruturaService;
use Ems\Util\ArrayUtil;
use Util\DateUtil;
use Ems\Util\Utils;

class IndicadoresSaudeTrabalhoService {

    /** @var Environment */
    protected $environment;

    /** @var IndicadoresSaudeTrabalhoRepository */
    protected $indicadoresSaudeTrabalhoRepository;

    /** @var OcorrenciaRepository */
    protected $ocorrenciaRepository;

    /** @var EstruturaService */
    protected $estruturaService;
    
    public function __construct(
        Environment                            $environment,
        IndicadoresSaudeTrabalhoRepository $indicadoresSaudeTrabalhoRepository,
        OcorrenciaRepository                   $ocorrenciaRepository,
        EstruturaService                       $estruturaService
    )
    {
        $this->environment                            = $environment;
        $this->indicadoresSaudeTrabalhoRepository = $indicadoresSaudeTrabalhoRepository;
        $this->ocorrenciaRepository                   = $ocorrenciaRepository;
        $this->estruturaService                       = $estruturaService;
    }

    private function verificaPerfilAcesso($row)
    {
        $perfilSesmt = null;
        $nrEstruturasString = null;

        $perfilSesmt = $this->ocorrenciaRepository->verificaPerfilAcessoSESMT($row['NRORG'], $row['CDOPERADOR']);
        if(!empty($perfilSesmt)) {
            $row['NRESTRUTLEGAL'] = null;
            return $row;
        } else {
            if ($row['NRESTRUTURAM'] !== "-1") {
                $nrEstruturas = $this->estruturaService->getTodasEstruturasInferiores($row['NRESTRUTURAM'], true);
    
                $nrEstruturasString = ArrayUtil::toStringSQL($nrEstruturas);
                $row['NRESTRUTGEREN'] = $nrEstruturasString;
                return $row;
            }
            $row['NRESTRUTLEGAL'] = null;
            return $row;
        }

        return $row;
    }

    private function adicionaCondicao($row) 
    {

        $row['NRTIPOESTRUTGEREN'] = $this->environment->getTipoEstruturaGerencialOrg();
        $row['NRTIPOESTRUTOMADOR'] =  $this->environment->getTipoEstruturaTomadorOrg();

        return $row;
    }


    public function getTotalAfastamentosPorCIDLegalOcupacao($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);

        $row = $this->adicionaCondicao($row);

        if(!isset($row['DTMESCOMPETENC'])) {
            $row['DTMESCOMPETENC'] = Utils::getParamters()->getDtmescomptencDiaCorrente();

            $totais = $this->indicadoresSaudeTrabalhoRepository->getTotalAfastamentosPorCIDDashboard(
                $row
            );
            
            return $totais;
        }
        
        $totais = $this->indicadoresSaudeTrabalhoRepository->getTotalAfastamentosPorCIDLegalOcupacao(
            $row
        );

        return $totais;
    }

    public function getTotalAfastamentosPorCIDOcupacao($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        
        $totais = $this->indicadoresSaudeTrabalhoRepository->getTotalAfastamentosPorCIDOcupacao(
            $row
        );

        return $totais;
    }

    public function getTotalAfastamentosPorNTEPLegalOcupacao($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);


        $totais = $this->indicadoresSaudeTrabalhoRepository->getTotalAfastamentosPorNTEPLegalOcupacao(
            $row
        );

        return $totais;
    }

    public function getTotalAfastamentosPorNTEPOcupacao($row)
    {   
        $totais = array();
        $row['NRESTRUTLEGAL'] = null;
        $row['NRESTRUTGEREN'] = null;
        $row = $this->verificaPerfilAcesso($row);
        $row = $this->adicionaCondicao($row);

        $totais = $this->indicadoresSaudeTrabalhoRepository->getTotalAfastamentosPorNTEPOcupacao(
            $row
        );

        return $totais;
    }
}