<?php

namespace HCM\FPA\Entity;

use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\TkEntity;
use Zeedhi\Framework\ORM\DateTime;

class FpaCnabmagnetEntity extends TkEntity
{
    
    public function __construct($entityManager, $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        
        $this->cnabMagnet = Repositories::FPA_CNABMAGNET;
        parent::__construct(null,$entityManager);
    }
    
    public function retornaRegistros($mesCompetencia, $nrOrg, $cdBanco, $nrTipoMovimento, $cdoperinclusao){
        return $this->entityManager->getRepository(Repositories::FPA_CNABMAGNET)->retornaRegistros($mesCompetencia, $nrOrg, $cdBanco, $nrTipoMovimento, $cdoperinclusao);
    }

    public function retornaRegistrosPorTomador($mesCompetencia, $nrOrg, $cdBanco, $nrTipoMovimento, $cdoperinclusao, $tomador){
        return $this->entityManager->getRepository(Repositories::FPA_CNABMAGNET)->retornaRegistrosPorTomador($mesCompetencia, $nrOrg, $cdBanco, $nrTipoMovimento, $cdoperinclusao, $tomador);
    }

    public function inicializaListaRegistros($mesCompetencia, $tipoRegistro, $nrOrg, $cdBanco, $nrTipoMovimento, $cdoperinclusao){
        $resultados = $this->entityManager->getRepository(Repositories::FPA_CNABMAGNET)->retornaRegistrosPorTipo($mesCompetencia, $tipoRegistro, $nrOrg, $cdBanco, $nrTipoMovimento, $cdoperinclusao);
        if (is_array($resultados) === true) {
            foreach ($resultados as $resultado) {
                $this->addItem($resultado, $resultado->getNrcnabmagnet());
            }

            return true;
        } else {
            return false;
        }
    }

    public function insereRegistroLista(
        $nrOrg, $cdoperinclusao, $mesCompetencia, $tipoRegistro, $tipoInscricao, $nrInscricao, $linha, $cdBanco, $nrTipoMovimento, $nrSeqArquivo = null, $valor = null, $nrvinculom = null, $sequencia = null
    ){
        $item = new $this->cnabMagnet;
        
        $item->setNrcnabmagnet(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_CNABMAGNET', 1, 12, true));
        $item->setIdativo('S');
        $item->setNrorg($nrOrg);
        $item->setMescompetencia(DateUtil::getDataDeString($mesCompetencia, DateUtil::FORMATO_BRASILEIRO, true));
        $item->setIdtiporegistro($tipoRegistro);
        $item->setIdtpinscestrutu($tipoInscricao);
        $item->setCdinscestrut($nrInscricao);
        $item->setDslinharegistro($linha);
        $item->setCdbanco($cdBanco);
        $item->setNrtipomoviment($nrTipoMovimento);
        $item->setNrvinculom($nrvinculom);
        $item->setValor($valor);
        $item->setNrsequencialarquivo($sequencia);
        $item->setNrsequencialarquivo($nrSeqArquivo);
        $item->setDtinclusao(DateUtil::getDataAtual());
        $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $item->setDtultatu(DateUtil::getDataAtual());
        $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $item->setCdoperultatu($this->hcmParameters->getCdoperador());
        
        $this->addItem($item);

    }

    public function persisteRegistro(
        $nrOrg, $mesCompetencia, $tipoRegistro, $segmento, $tipoInscricao, $nrInscricao,
        $linha, $nrVinculoM, $cdBanco, $nrTipoMovimento, $valor, $tipoInscricaoVinculo, $nrInscricaoVinculo, $cdoperinclusao
    ){
        $item = $this->entityManager->getRepository(Repositories::FPA_CNABMAGNET)->retornaRegistro(
            $mesCompetencia, $nrOrg, $tipoRegistro, $segmento, $tipoInscricao, $nrInscricao, $nrVinculoM, $cdBanco, $nrTipoMovimento, $cdoperinclusao
        );

        if (is_object($item) === true) {
            $item->setNrcnabmagnet($item->getNrcnabmagnet());
            $item->setDslinharegistro($linha);
        } else {
            $item = new $this->cnabMagnet;
            
            $item->setNrcnabmagnet(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_CNABMAGNET', 1, 12, true));
            $item->setIdativo('S');
            $item->setNrorg($nrOrg);
            $item->setMescompetencia($mesCompetencia);
            $item->setIdtiporegistro($tipoRegistro);
            $item->setIdtpinscestrutu($tipoInscricao);
            $item->setCdinscestrut($nrInscricao);
            $item->setNrvinculom($nrVinculoM);
            $item->setDslinharegistro($linha);
            $item->setIdsegmento($segmento);
            $item->setCdbanco($cdBanco);
            $item->setNrtipomoviment($nrTipoMovimento);
            $item->setValor($valor);
            $item->setIdtpinscvinculo($tipoInscricaoVinculo);
            $item->setCdinscvinculo($nrInscricaoVinculo);
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setDtultatu(DateUtil::getDataAtual());
            $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperultatu($this->hcmParameters->getCdoperador());
        }

        $this->entityManager->persist($item);
        $this->entityManager->flush($item);
    }

    public function procuraRegistroLista($nrInscricao, $tpRegistro, $cdBanco, $nrTipoMovimento, $cdoperinclusao, $nrorg) {
        $itens = $this->getItems();
        foreach ($itens as $key => $item) {
            if (($item->getCdinscestrut() == $nrInscricao)
                && ($item->getIdtiporegistro() == $tpRegistro)
                && ($item->getCdbanco() == $cdBanco)
                && ($item->getNrtipomoviment() == $nrTipoMovimento)
                && ($item->getCdoperinclusao() == $cdoperinclusao)
                && ($item->getNrorginclusao() == $nrorg)
            ) {
                return $key;
            }
        }

        return false;
    }

    public function retroceder($nrOrg, $mesCompetencia, $cdBanco, $nrTipoMovimento, $cdoperinclusao){

        $registros = $this->retornaRegistros($mesCompetencia, $nrOrg, $cdBanco, $nrTipoMovimento, $cdoperinclusao);
        if (count($registros) > 0) {
            foreach ($registros as $registro) {
                $this->addItem($registro);
                $this->removeItem($registro);
            }
            $this->persist();
        }
    }

    public function totalRegistrosPorTipos($mesCompetencia, $nrOrg, $tiposRegistro, $cdBanco, $nrTipoMovimento, $cdoperinclusao){
        $totalRegistros = $this->entityManager->getRepository(Repositories::FPA_CNABMAGNET)->totalRegistrosPorTipos($mesCompetencia, $nrOrg, $tiposRegistro, $cdBanco, $nrTipoMovimento, $cdoperinclusao)[0];
        
        if (end($totalRegistros)){
            $totalRegistros = end($totalRegistros);
            return $totalRegistros;
        }
        return false;
    }
    
    public function retornaUltimoSequencialRegistro0($mesCompetencia, $nrOrg, $cdBanco, $cdoperinclusao){
        $ultimoSequencial = $this->entityManager->getRepository(Repositories::FPA_CNABMAGNET)->retornaUltimoSequencialRegistro0($mesCompetencia, $nrOrg, $cdBanco, $cdoperinclusao)[0];

        if (end($ultimoSequencial)){
            $ultimoSequencial = end($ultimoSequencial);
            return $ultimoSequencial;
        }
        return false;
    }
    
    public function modificaSequenciaArquivoRemessa($mesCompetencia, $nrOrg, $cdBanco, $cdoperinclusao){

        $arquivosRemessa = $this->entityManager->getRepository(Repositories::FPA_CNABMAGNET)->retornaRegistros0Remessa($mesCompetencia, $nrOrg, $cdBanco, $cdoperinclusao);

        if (empty($arquivosRemessa) === false) {

            $contador = 1;
            foreach($arquivosRemessa as $arquivo) {

                $arquivo->setNrsequencialarquivo($contador);
                $this->entityManager->persist($arquivo);
                $this->entityManager->flush($arquivo);

                $contador++;
            }
        }
    }

    public function retornaRegistroPorVinculoSegmento($nrOrg, $mesCompetencia, $nrVinculom, $idSegmento, $cdoperinclusao){
        $resultado = $this->entityManager->getRepository(Repositories::FPA_CNABMAGNET)->retornaRegistroPorVinculoSegmento($nrOrg, $mesCompetencia, $nrVinculom, $idSegmento, $cdoperinclusao);
        return $resultado;
    }
}