<?php

namespace HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\itau;

use \HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\HoleriteUtil;

class HoleriteItauRepository extends HoleriteUtil {
    
    protected $requireds = array(
        'TIPOINCR' => 'Tipo de inscrição da empresa',
        'NROINSCR' => 'Incrição (CNPJ/CPF) da empresa',
        'NROAGENC' => 'Número da agência debitada',
        'NROCONTA' => 'Número da conta debitada',
        'NOMEMPRE' => 'Razão social da empresa',
        'ENDEMPRE' => 'Endereço da empresa',
        'NROEMPRE' => 'Número do endereço da empresa',
        'MUNEMPRE' => 'Múnicípio (do endereço) da empresa',
        'CEPEMPRE' => 'CEP da empresa',
        //'BANFAVOR' => 'Número do banco favorecido',
        'AGENCONT' => 'Agência do favorecido',
        'NOMFAVOR' => 'Nome do favorecido (vínculo)',
        'SEUNUMER' => 'Seu número (número de matrícula do vínculo)',
        /*reg3a*/
        'NUMINSCR' => 'Vínculo sem CPF',
        /*reg3d*/
        'CARGO'    => 'Funcionário sem cargo/função',
        'HORAS'    => 'Quantidade de horas de trabalho semanal',
        'VRBASE'   => 'Salário não cadastrado (Alteração salarial)'
        );


    /*Registro 0*/
    protected $layoutReg0 = array(
        /*campo            tipo  inicio  fim*/
        'CODBANCO' => array( '9',   1,   3 ),   /*CODBANCO 001 003 9(03) 341*/
        'CODLOTE'  => array( '9',   4,   7 ),   /*CODLOTE  004 007 9(04) 0000 NOTA 3*/
        'TIPREGIS' => array( '9',   8,   8 ),   /*TIPREGIS 008 008 9(01) 0*/
        'BRANCOS1' => array( 'X',   9,  14 ),   /*BRANCOS1 009 014 X(06) */
        'LAYOUTVE' => array( '9',  15,  17 ),   /*LAYOUTVE 015 017 9(03) 080*/
        'TIPOINCR' => array( '9',  18,  18 ),   /*TIPOINCR 018 018 9(01) 1 = CPF 2 = CNPJ*/
        'NROINSCR' => array( '9',  19,  32 ),   /*NROINSCR 019 032 9(14) NOTA 1 INSCRIÇÃO NÚMERO: CNPJ EMPRESA DEBITADA*/
        'BRANCOS2' => array( 'X',  33,  52 ),   /*BRANCOS2 033 052 X(20) BRANCOS*/
        'NROAGENC' => array( '9',  53,  57 ),   /*NROAGENC 053 057 9(05) NOTA 1 AGÊNCIA: NÚMERO AGÊNCIA DEBITADA*/
        'BRANCOS3' => array( 'X',  58,  58 ),   /*BRANCOS3 058 058 X(01) COMPLEMENTO DE REGISTRO*/
        'NROCONTA' => array( '9',  59,  70 ),   /*NROCONTA 059 070 9(12) NOTA 1 CONTA: NÚMERO DE C/C DEBITADA*/
        'BRANCOS4' => array( 'X',  71,  71 ),   /*BRANCOS4 071 071 X(01) COMPLEMENTO DE REGISTRO*/
        'NRODAC'   => array( '9',  72,  72 ),   /*NRODAC   072 072 9(01) NOTA 1 DAC DA AGÊNCIA/CONTA DEBITADA = 5*/
        'NOMEMPRE' => array( 'X',  73, 102 ),   /*NOMEMPRE 073 102 X(30) NOME DA EMPRESA*/
        'NOMBANCO' => array( 'X', 103, 132 ),   /*NOMBANCO 103 132 X(30) NOME DO BANCO = BANCO ITAU*/
        'BRANCOS5' => array( 'X', 133, 142 ),   /*BRANCOS5 133 142 X(10) COMPLEMENTO DE REGISTRO*/
        'ARQCODIG' => array( '9', 143, 143 ),   /*ARQCODIG 143 143 9(01) ARQUIVO-CÓDIGO CÓDIGO REMESSA/RETORNO 1=REMESSA 2=RETORNO = 1*/
        'DATGERAC' => array( '9', 144, 151 ),   /*DATGERAC 144 151 9(08) DDMMAAAA DATA DE GERAÇÃO DO ARQUIVO*/
        'HORGERAC' => array( '9', 152, 157 ),   /*HORGERAC 152 157 9(06) HHMMSS HORA DE GERAÇÃO DO ARQUIVO*/
        'ZEROS1'   => array( '9', 158, 166 ),   /*ZEROS1   158 166 9(09) COMPLEMENTO DE REGISTRO*/
        'DENSIDAD' => array( '9', 167, 171 ),   /*DENSIDAD 167 171 9(05) UNIDADE DE DENSIDADE DE GRAVAÇÃO DO ARQUIVO NOTA 2 = 00000*/
        'BRANCOS6' => array( 'X', 172, 240 )    /*BRANCOS6 172 240 X(69) COMPLEMENTO DE REGISTRO*/
        );
    protected $constReg0 = array(
        'CODBANCO' => '341',
        'CODLOTE'  => '0',
        'TIPREGIS' => '0',
        'BRANCOS1' => ' ',
        'LAYOUTVE' => '081',
        'BRANCOS2' => ' ',
        'BRANCOS3' => ' ',
        'BRANCOS4' => ' ',
        'NOMBANCO' => 'BANCO ITAU',
        'BRANCOS5' => ' ',
        'ARQCODIG' => '1',
        'ZEROS1'   => '0',
        'DENSIDAD' => '01600',
        'BRANCOS6' => ' '
        );
    protected $queryReg0 = "
SELECT
  DECODE(EH.CDCNPJESTRUT, NULL,
    DECODE(EH.CDCPFESTRUTURA, NULL, NULL, 1),
    2)                                        TIPOINCR, /*TIPOINCR 018 018 9(01) 1 = CPF 2 = CNPJ*/
  NVL(EH.CDCNPJESTRUT, EH.CDCNPJESTRUT)       NROINSCR, /*NROINSCR 019 032 9(14) NOTA 1 INSCRIÇÃO NÚMERO: CNPJ EMPRESA DEBITADA*/
  CC.CDAGENCIA                                NROAGENC, /*NROAGENC 053 057 9(05) NOTA 1 AGÊNCIA: NÚMERO AGÊNCIA DEBITADA*/
  SUBSTR(LPAD(CC.CDCONTCORR, 13, '0'), 1, 12) NROCONTA, /*NROCONTA 059 070 9(12) NOTA 1 CONTA: NÚMERO DE C/C DEBITADA*/
  SUBSTR(CC.CDCONTCORR, -1)                   NRODAC,   /*NRODAC   072 072 9(01) NOTA 1 DAC DA AGÊNCIA/CONTA DEBITADA = 5*/
  EH.NMRAZSOCESTRUT                           NOMEMPRE, /*NOMEMPRE 073 102 X(30) NOME DA EMPRESA*/
  TO_CHAR(SYSDATE, 'DDMMYYYY')                DATGERAC, /*DATGERAC 144 151 9(08) DDMMAAAA DATA DE GERAÇÃO DO ARQUIVO*/
  TO_CHAR(SYSDATE, 'HH24MISS')                HORGERAC  /*HORGERAC 152 157 9(06) HHMMSS HORA DE GERAÇÃO DO ARQUIVO*/
FROM ESTRUTURAM EM
JOIN ESTRUTURAH EH ON EH.NRORG        = EM.NRORG
                  AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                  AND EH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND NRORG        = EM.NRORG
                                                AND NRESTRUTURAM = EM.NRESTRUTURAM )
/*conta corrente e banco da estrutura legal*/
JOIN CONTCORRPARC CC ON CC.NRORG          = EM.NRORG
                    AND CC.NRPARCNEGOCIO  = EM.NRPARCNEGOCIO
                    AND CC.IDATIVO        = 'S'
                    AND CC.CDBANCO        = '341' /*TODO 
                    --AND CC.NRTPCONTCORR   = 2 /*1 Conta de Movimentação, 2 CNAB, 3 Conta Salário, 4 Pensão Judicial*/
						        AND CC.NRCONTCORRPARC = ( SELECT MAX(NRCONTCORRPARC) FROM CONTCORRPARC
                                              WHERE NRORG         = CC.NRORG
                                                AND IDATIVO       = CC.IDATIVO
                                                AND CDBANCO       = CC.CDBANCO
                                                AND NRPARCNEGOCIO = CC.NRPARCNEGOCIO
													                      AND NRTPCONTCORR  = CC.NRTPCONTCORR )
WHERE EM.NRORG        = :P_NRORG
  AND EM.NRESTRUTURAM = :P_NRESTRUTURAM
";
    
    /*Registro 1*/
    protected $layoutReg1 = array(
        /*campo            tipo  inicio  fim*/
        'CODBANCO' => array( '9',   1,   3 ),   /*CODBANCO 001 003 9(03) 341 CÓDIGO BANCO NA COMPENSAÇÃO*/
        'CODLOTE'  => array( '9',   4,   7 ),   /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE: LOTE IDENTIFICAÇÃO DE PAGTOS = 0001*/
        'TIPREGIS' => array( '9',   8,   8 ),   /*TIPREGIS 008 008 9(01) 1 TIPO DE REGISTRO REGISTRO HEADER DE LOTE*/
        'TIPOPERA' => array( 'X',   9,   9 ),   /*TIPOPERA 009 009 X(01) C=CRÉDITO TIPO DE OPERAÇÃO TIPO DA OPERAÇÃO*/
        'TIPPAGTO' => array( '9',  10,  11 ),   /*TIPPAGTO 010 011 9(02) NOTA 4 TIPO DE PAGAMENTO TIPO DE PAGTO*/
        'FORPAGTO' => array( '9',  12,  13 ),   /*FORPAGTO 012 013 9(02) NOTA 5 FORMA DE PAGAMENTO FORMA DE PAGAMENTO*/
        'LAYOUTVE' => array( '9',  14,  16 ),   /*LAYOUTVE 014 016 9(03) 040 LAYOUT DO LOTE Nº DA VERSÃO DO LAYOUT DO LOTE*/
        'BRANCOS1' => array( 'X',  17,  17 ),   /*BRANCOS1 017 017 X(01) COMPLEMENTO DE REGISTRO*/
        'TIPOINCR' => array( '9',  18,  18 ),   /*TIPOINCR 018 018 9(01) EMPRESA – INSCRIÇÃO TIPO INSCRIÇÃO EMPRESA DEBITADA 1 = CPF 2 = CNPJ*/
        'NROINSCR' => array( '9',  19,  32 ),   /*NROINSCR 019 032 9(14) NOTA 1 INSCRIÇÃO NÚMERO CNPJ EMPRESA DEBITADA*/
        'BRANCOS2' => array( 'X',  33,  52 ),   /*BRANCOS2 033 052 X(20) COMPLEMENTO DE REGISTRO*/
        'NROAGENC' => array( '9',  53,  57 ),   /*NROAGENC 053 057 9(05) NOTA 1 AGÊNCIA NÚMERO AGÊNCIA DEBITADA*/
        'BRANCOS3' => array( 'X',  58,  58 ),   /*BRANCOS3 058 058 X(01) COMPLEMENTO DE REGISTRO*/
        'NROCONTA' => array( '9',  59,  70 ),   /*NROCONTA 059 070 9(12) NOTA 1 CONTA NÚMERO DE C/C DEBITADA*/
        'BRANCOS4' => array( 'X',  71,  71 ),   /*BRANCOS4 071 071 X(01) COMPLEMENTO DE REGISTRO*/
        'NRODAC'   => array( '9',  72,  72 ),   /*NRODAC   072 072 9(01) NOTA 1 DAC DA AGÊNCIA/CONTA DEBITADA*/
        'NOMEMPRE' => array( 'X',  73, 102 ),   /*NOMEMPRE 073 102 X(30) NOME DA EMPRESA NOME DA EMPRESA DEBITADA*/
        'FINALOTE' => array( 'X', 103, 132 ),   /*FINALOTE 103 132 X(30) NOTA 6 FINALIDADE DO LOTE FINALIDADE DOS PAGTOS DO LOTE*/
        'HISTORCC' => array( 'X', 133, 142 ),   /*HISTORCC 133 142 X(10) NOTA 7 HISTÓRICO DE C/C COMPLEMENTO HISTÓRICO C/C DEBITADA*/
        'ENDEMPRE' => array( 'X', 143, 172 ),   /*ENDEMPRE 143 172 X(30) ENDEREÇO DA EMPRESA NOME DA RUA, AV, PÇA, ETC...*/
        'NROEMPRE' => array( '9', 173, 177 ),   /*NROEMPRE 173 177 9(05) NÚMERO NÚMERO DO LOCAL*/
        'COMEMPRE' => array( 'X', 178, 192 ),   /*COMEMPRE 178 192 X(15) COMPLEMENTO. CASA, APTO, SALA, ETC...*/
        'MUNEMPRE' => array( 'X', 193, 212 ),   /*MUNEMPRE 193 212 X(20) CIDADE NOME DA CIDADE*/
        'CEPEMPRE' => array( '9', 213, 220 ),   /*CEPEMPRE 213 220 9(08) CEP*/
        'ESTEMPRE' => array( 'X', 221, 222 ),   /*ESTEMPRE 221 222 X(02) SIGLA DO ESTADO*/
        'BRANCOS5' => array( 'X', 223, 230 ),   /*BRANCOS5 223 230 X(08) COMPLEMENTO DE REGISTRO*/
        'OCORRENC' => array( 'X', 231, 240 )    /*OCORRENC 231 240 X(10) NOTA 8 OCORRÊNCIAS: CÓDIGO OCORRÊNCIAS P/RETORNO*/
        );
    protected $constReg1 = array(
        'CODBANCO' => 341,
        'CODLOTE'  => 1,
        'TIPREGIS' => 1,
        'TIPOPERA' => 'C',
        'TIPPAGTO' => 30,
        'FORPAGTO' => 1,
        'LAYOUTVE' => 40,
        'BRANCOS1' => ' ',
        'BRANCOS2' => ' ',
        'BRANCOS3' => ' ',
        'BRANCOS4' => ' ',
        'HISTORCC' => ' ',
        'BRANCOS5' => ' ',
        'OCORRENC' => ' ',
        );
    protected $queryReg1 = "
SELECT
  DECODE(EHL.CDCNPJESTRUT, NULL,
    DECODE(EHL.CDCPFESTRUTURA, NULL, NULL, 1),
    2)                                         TIPOINCR, /*TIPOINCR 018 018 9(01) EMPRESA – INSCRIÇÃO TIPO INSCRIÇÃO EMPRESA DEBITADA 1 = CPF 2 = CNPJ*/
  NVL(EHL.CDCNPJESTRUT, EHL.CDCNPJESTRUT)      NROINSCR, /*NROINSCR 019 032 9(14) NOTA 1 INSCRIÇÃO NÚMERO CNPJ EMPRESA DEBITADA*/
  CC.CDAGENCIA                                 NROAGENC, /*NROAGENC 053 057 9(05) NOTA 1 AGÊNCIA NÚMERO AGÊNCIA DEBITADA*/
  SUBSTR(LPAD(CC.CDCONTCORR, 13, '0'), 1, 12)  NROCONTA, /*NROCONTA 059 070 9(12) NOTA 1 CONTA: NÚMERO DE C/C DEBITADA*/
  SUBSTR(CC.CDCONTCORR, -1)                    NRODAC,   /*NRODAC   072 072 9(01) NOTA 1 DAC DA AGÊNCIA/CONTA DEBITADA = 5*/
  EHL.NMRAZSOCESTRUT                           NOMEMPRE, /*NOMEMPRE 073 102 X(30) NOME DA EMPRESA NOME DA EMPRESA DEBITADA*/
  DECODE(:P_EVENTO, 3050, '08',
    DECODE(:P_TIPOMOVIMENT,  1, '01',
                            10, '01',
                             4, '04',
                            19, '05',
                             2, '07', '10'))   FINALOTE, /*FINALOTE 103 132 X(30) NOTA 6 FINALIDADE DO LOTE FINALIDADE DOS PAGTOS DO LOTE*/
  DECODE(EP.CDLOGRADOURO, NULL, EP.DSENDERECO,
    EP.CDLOGRADOURO || ' ' || EP.DSENDERECO)   ENDEMPRE, /*ENDEMPRE 143 172 X(30) ENDEREÇO DA EMPRESA NOME DA RUA, AV, PÇA, ETC...*/
  EP.NRIMOVELENDERECO                          NROEMPRE, /*NROEMPRE 173 177 9(05) NÚMERO NÚMERO DO LOCAL*/
  EP.DSCOMPLEENDERECO                          COMEMPRE, /*COMEMPRE 178 192 X(15) COMPLEMENTO. CASA, APTO, SALA, ETC...*/
  MU.NMMUNICIPIO                               MUNEMPRE, /*MUNEMPRE 193 212 X(20) CIDADE NOME DA CIDADE*/
  EP.NRCEPENDERECO                             CEPEMPRE, /*CEPEMPRE 213 220 9(08) CEP*/
  EP.SGESTADO                                  ESTEMPRE /*ESTEMPRE 221 222 X(02) SIGLA DO ESTADO*/
FROM ESTRUTURAM EML
JOIN ESTRUTURAH EHL ON EHL.NRORG        = EML.NRORG
                   AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                   AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND NRORG        = EML.NRORG
                                                AND NRESTRUTURAM = EML.NRESTRUTURAM )
/*conta corrente e banco da estrutura legal*/
JOIN CONTCORRPARC CC ON CC.NRORG          = EML.NRORG
                    AND CC.NRPARCNEGOCIO  = EML.NRPARCNEGOCIO
                    --AND CC.NRTPCONTCORR   = 2 /*1 Conta de Movimentação, 2 CNAB, 3 Conta Salário, 4 Pensão Judicial*/
                    AND CC.IDATIVO        = 'S'
                    AND CC.CDBANCO        = '341'
                    AND CC.NRCONTCORRPARC = ( SELECT MAX(NRCONTCORRPARC) FROM CONTCORRPARC
                                              WHERE NRORG         = CC.NRORG
                                                AND IDATIVO       = CC.IDATIVO
                                                AND CDBANCO       = CC.CDBANCO
                                                AND NRPARCNEGOCIO = CC.NRPARCNEGOCIO
                                                AND NRTPCONTCORR  = CC.NRTPCONTCORR )
/*Endereço da estrutura legal*/
LEFT JOIN ENDERECOPARC EP ON EP.NRORG          = EML.NRORG
                         AND EP.NRPARCNEGOCIO  = EML.NRPARCNEGOCIO /*alterar aqui*/
                         AND EP.CDTIPOENDERECO = 'PRINCIPAL'
LEFT JOIN MUNICIPIO MU ON ( MU.NRORG = EP.NRORG OR MU.NRORG = 0 OR MU.NRORG IS NULL )
                      AND MU.CDPAIS      = EP.CDPAIS
                      AND MU.SGESTADO    = EP.SGESTADO
                      AND MU.CDMUNICIPIO = EP.CDMUNICIPIO
/**/
WHERE EML.NRORG        = :P_NRORG
  AND EML.NRESTRUTURAM = :P_NRESTRUTURAM /*responsável*/
";
    
    /*Registro 3*/
    protected $layoutReg3A = array(
        /*campo            tipo  inicio  fim*/
        'CODBANCO' => array( '9',   1,   3 ),    /*CODBANCO 001 003 9(03) 341 CÓDIGO BANCO NA COMPENSAÇÃO*/
        'CODLOTE'  => array( '9',   4,   7 ),    /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE: LOTE IDENTIFICAÇÃO DE PAGTOS = 0001*/
        'TIPREGIS' => array( '9',   8,   8 ),    /*TIPREGIS 008 008 9(01) 3 TIPO DE REGISTRO REGISTRO HEADER DE LOTE*/
        'NROREGIS' => array( '9',   9,  13 ),    /*NROREGIS 009 013 9(05) NOTA 9 NÚMERO DO REGISTRO Nº SEQUENCIAL REGISTRO NO LOTE*/
        'CODSEG_A' => array( 'X',  14,  14 ),    /*CODSEG_A 014 014 X(01) A SEGMENTO CÓDIGO SEGMENTO REG. DETALHE*/
        'TIPMOVIM' => array( '9',  15,  17 ),    /*TIPMOVIM 015 017 9(03) NOTA 10 TIPO DE MOVIMENTO TIPO DE MOVIMENTO = 000*/
        'CAMARA'   => array( '9',  18,  20 ),    /*CAMARA   018 020 9(03) NOTA17*/
        'BANFAVOR' => array( '9',  21,  23 ),    /*BANFAVOR 021 023 9(03) BANCO FAVORECIDO CÓDIGO BANCO FAVORECIDO*/
        'AGENCONT' => array( 'X',  24,  43 ),    /*AGENCONT 024 043 X(20) NOTA 11 AGÊNCIA CONTA AGÊNCIA CONTA FAVORECIDO*/
        'NOMFAVOR' => array( 'X',  44,  73 ),    /*NOMFAVOR 044 073 X(30) NOME DO FAVORECIDO*/
        'SEUNUMER' => array( 'X',  74,  93 ),    /*SEUNUMER 074 093 X(20) SEU NÚMERO: Nº DOCTO ATRIBUÍDO PELA EMPRESA*/
        'DATPAGTO' => array( 'D',  94, 101 ),    /*DATPAGTO 094 101 9(08) DDMMAAAA DATA DE PAGTO: DATA PREVISTA PARA PAGTO*/
        'TIPMOEDA' => array( 'X', 102, 104 ),    /*TIPMOEDA 102 104 X(03) REA OU 009 TIPO TIPO DA MOEDA*/
        'CODISPB'  => array( '9', 105, 112 ),    /*CODISPB  105 112 9(08) NOTA 37*/
        'ZEROS1'   => array( '9', 113, 119 ),    /*ZEROS1   113 119 9(07) COMPLEMENTO DE REGISTRO*/
        'VALPAGTO' => array( '9', 120, 134 ),    /*VALPAGTO 120 134 9(13)V9(02) VALOR DO PAGTO VALOR PREVISTO DO PAGTO*/
        'NOSNUMER' => array( 'X', 135, 149 ),    /*NOSNUMER 135 149 X(15) NOTA 12 NOSSO NÚMERO Nº DOCTO ATRIBUÍDO PELO BANCO*/
        'BRANCOS1' => array( 'X', 150, 154 ),    /*BRANCOS1 150 154 X(05) 0 COMPLEMENTO DE REGISTRO*/
        'DATAEFET' => array( '9', 155, 162 ),    /*DATAEFET 155 162 9(08) 0 DDMMAAAA DATA EFETIVA: DATA REAL EFETIVAÇÃO DO PAGTO*/
        'VALOEFET' => array( '9', 163, 177 ),    /*VALOEFET 163 177 9(13)V9(02) VALOR REAL EFETIVAÇÃO DO PAGTO*/
        'FINALIDA' => array( 'X', 178, 195 ),    /*FINALIDA 178 195 X(18) NOTA 13 ' ' FINALIDADE DETALHE: INFORMAÇÃO COMPLEMENTAR P/ HIST. DE C/C*/
        'BRANCOS2' => array( 'X', 196, 197 ),    /*BRANCOS2 196 197 X(2)' ' COMPLEMENTO DE REGISTRO*/
        'NUMDOCUM' => array( '9', 198, 203 ),    /*NUMDOCUM 198 203 9(6) NOTA 14 ' ' Nº DO DOCUMENTO Nº DO DOC/TED/ OP/ CHEQUE NO RETORNO*/
        'NUMINSCR' => array( '9', 204, 217 ),    /*NUMINSCR 204 217 9(14) NOTA 15 0 Nº DE INSCRIÇÃO N DE INSCRIÇÃO DO FAVORECIDO (CPF/CNPJ)*/
        'FINALDOC' => array( 'X', 218, 219 ),    /*FINALDOC 218 219 X(02) NOTA 30 ' ' FINALIDADE DOC E STATUS: FINALIDADE DO DOC E STATUS DO FUNCIONÁRIO NA EMPRESA*/
        'FINALTED' => array( 'X', 220, 224 ),    /*FINALTED 220 224 X(05) NOTA 26 ' ' FINALIDADE TED FINALIDADE DA TED*/
        'BRANCOS3' => array( 'X', 225, 229 ),    /*BRANCOS3 225 229 X(05) COMPLEMENTO DE REGISTRO*/
        'AVISOFAV' => array( 'X', 230, 230 ),    /*AVISOFAV 230 230 X(01) NOTA 16 ' ' AVISO AO FAVORECIDO*/
        'OCORRENC' => array( 'X', 231, 240 )     /*OCORRENC 231 240 X(10) NOTA 8 ' ' OCORRÊNCIAS CÓDIGO OCORRÊNCIAS NO RETORNO*/
        );
    protected $layoutReg3D = array(
        /*campo            tipo  inicio  fim*/
        'CODBANCO' => array( '9',   1,   3 ),    /*CODBANCO 001 003 9(03) 341 CÓDIGO BANCO NA COMPENSAÇÃO*/
        'CODLOTE'  => array( '9',   4,   7 ),    /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE: LOTE IDENTIFICAÇÃO DE PAGTOS = 0001*/
        'TIPREGIS' => array( '9',   8,   8 ),    /*TIPREGIS 008 008 9(01) 3 TIPO DE REGISTRO REGISTRO HEADER DE LOTE*/
        'NROREGIS' => array( '9',   9,  13 ),    /*NROREGIS 009 013 9(05) NOTA 9 NÚMERO DO REGISTRO Nº SEQUENCIAL REGISTRO NO LOTE*/
        'CODSEG_D' => array( 'X',  14,  14 ),    /*CODSEG_D 014 014 X(01) A SEGMENTO CÓDIGO SEGMENTO REG. DETALHE*/
        'BRANCOS1' => array( 'X',  15,  17 ),    /*BRANCOS1 015 017 X(03) COMPLEMENTO DE REGISTRO*/
        'COMPETEN' => array( '9',  18,  23 ),    /*COMPETEN 018 023 9(06) MMAAAA MÊS/ANO DO PAGAMENTO*/
        'CENTRO'   => array( 'X',  24,  38 ),    /*CENTRO   024 038 X(15) CENTRO DE CUSTO, ÓRGÃO/CENTRO DE CUSTO*/
        'CODFUNC'  => array( 'X',  39,  53 ),    /*CODFUNC  039 053 X(15) CÓDIGO FUNCIONÁRIO*/
        'CARGO'    => array( 'X',  54,  83 ),    /*CARGO    054 083 X(30) CARGO DO FUNCIONÁRIO*/
        'FERIASDE' => array( '9',  84,  91 ),    /*FERIASDE 084 091 9(08) PERÍODO DE FÉRIAS “DE” DDMMAAAA*/
        'FERIASAT' => array( '9',  92,  99 ),    /*FERIASAT 092 099 9(08) PERÍODO DE FÉRIAS “ATÉ” DDMMAAAA*/
        'QDEDEPIR' => array( '9', 100, 101 ),    /*QDEDEPIR 100 101 9(02) DEPENDENTES I.R. QUANTIDADE DE DEPENDENTES IMP.DE RENDA*/
        'QDEDEPSF' => array( '9', 102, 103 ),    /*QDEDEPSF 102 103 9(02) DEPENDENTES S.F. QUANTIDADE DE DEPENDENTES SALÁRIO FAMÍLIA*/
        'HORAS'    => array( '9', 104, 105 ),    /*HORAS    104 105 9(02) HORAS SEMANAIS*/
        'SALCONTR' => array( '9', 106, 120 ),    /*SALCONTR 106 120 9(13)V9(02) VALOR DO SALÁRIO CONTRIBUIÇÃO*/
        'VRFGTS'   => array( '9', 121, 135 ),    /*VRFGTS   121 135 9(13)V9(02) VALOR DO F.G.T.S.*/
        'VRCRED'   => array( '9', 136, 150 ),    /*VRCRED   136 150 9(13)V9(02) VALOR TOTAL DOS CRÉDITOS*/
        'VRDEBI'   => array( '9', 151, 165 ),    /*VRDEBI   151 165 9(13)V9(02) VALOR TOTAL DOS DÉBITOS*/
        'VRLIQ'    => array( '9', 166, 180 ),    /*VRLIQ    166 180 9(13)V9(02) VALOR LIQUIDO DO PAGAMENTO*/
        'VRBASE'   => array( '9', 181, 195 ),    /*VRBASE   181 195 9(13)V9(02) VALOR DO SALÁRIO FIXO/BASE*/
        'BASEIRRF' => array( '9', 196, 210 ),    /*BASEIRRF 196 210 9(13)V9(02) VALOR DA BASE DO I.R.R.F.*/
        'BASEFGTS' => array( '9', 211, 225 ),    /*BASEFGTS 211 225 9(13)V9(02) VALOR DA BASE DO F.G.T.S.*/
        'DISPONIB' => array( '9', 226, 227 ),    /*DISPONIB 226 227 X(02) NOTA 27 PRAZO PARA DISPONIBILIZAÇÃO DO HOLERITE*/
        'BRANCOS2' => array( 'X', 228, 230 ),    /*BRANCOS2 228 230 X(03) COMPLEMENTO DE REGISTRO*/
        'OCORRENC' => array( 'X', 231, 240 )     /*OCORRENC 231 240 X(10) CÓDIGO OCORRÊNCIA PARA RETORNO*/
        );
    protected $layoutReg3E = array(
        /*campo            tipo  inicio  fim*/
        'CODBANCO' => array( '9',   1,   3 ),    /*CODBANCO 001 003 9(03) 341 CÓDIGO BANCO NA COMPENSAÇÃO*/
        'CODLOTE'  => array( '9',   4,   7 ),    /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE: LOTE IDENTIFICAÇÃO DE PAGTOS = 0001*/
        'TIPREGIS' => array( '9',   8,   8 ),    /*TIPREGIS 008 008 9(01) 3 TIPO DE REGISTRO REGISTRO HEADER DE LOTE*/
        'NROREGIS' => array( '9',   9,  13 ),    /*NROREGIS 009 013 9(05) NOTA 9 NÚMERO DO REGISTRO Nº SEQUENCIAL REGISTRO NO LOTE*/
        'CODSEG_E' => array( 'X',  14,  14 ),    /*CODSEG_E 014 014 X(01) A SEGMENTO CÓDIGO SEGMENTO REG. DETALHE*/
        'BRANCOS5' => array( 'X',  15,  17 ),    /*BRANCOS1 015 017 X(03) COMPLEMENTO DE REGISTRO*/
        'MOVIMENT' => array( 'X',  18,  18 ),    /*MOVIMENT 018 018 X(01) NOTA 24 TIPO DE MOVIMENTO*/
        
        //'INFOCOMP' => array( 'X',  19, 218 ),    /*INFOCOMP 019 218 X(200) INFORMAÇÕES COMPLEMENTARES PARA HOLERITE OU INFORME DE RENDIMENTOS ANEXO D*/
        'DESCRI1'  => array( 'X',  19,  48 ),    /*019 048 X(30)(1) DESCRIÇÃO DO CRÉDITO / DESCONTO (1)*/
        'BRANCOS1' => array( 'X',  49,  53 ),    /*049 053 X(05) COMPLEMENTO DE REGISTRO*/
        'VALOR1'   => array( '9',  54,  68 ),    /*054 068 9(13)V9(02) (1) VALOR DO CRÉDITO / DESCONTO (1)*/
        
        'DESCRI2'  => array( 'X',  69,  98 ),    /*069 098 X(30) (2) DESCRIÇÃO DO CRÉDITO / DESCONTO (2)*/
        'BRANCOS2' => array( 'X',  99, 103 ),    /*099 103 X(05) COMPLEMENTO DE REGISTRO*/
        'VALOR2'   => array( '9', 104, 118 ),    /*104 118 9(13)V9(02) (2) VALOR DO CRÉDITO / DESCONTO (2)*/
        
        'DESCRI3'  => array( 'X', 119, 148 ),    /*119 148 X(30) (3) DESCRIÇÃO DO CRÉDITO / DESCONTO (3)*/
        'BRANCOS3' => array( 'X', 149, 153 ),    /*149 153 X(05) COMPLEMENTO DE REGISTRO*/
        'VALOR3'   => array( '9', 154, 168 ),    /*154 168 9(13)V9(02) (3) VALOR DO CRÉDITO / DESCONTO (3)*/
        
        'DESCRI4'  => array( 'X', 169, 198 ),    /*169 198 X(30) (4) DESCRIÇÃO DO CRÉDITO / DESCONTO (4)*/
        'BRANCOS4' => array( 'X', 199, 203 ),    /*199 203 X(05) COMPLEMENTO DE REGISTRO*/
        'VALOR4'   => array( '9', 204, 218 ),    /*204 218 9(13)V9(02) (4) VALOR DO CRÉDITO / DESCONTO (4)*/
        
        'BRANCOS6' => array( 'X', 219, 230 ),    /*BRANCOS2 219 230 X(12) COMPLEMENTO DE REGISTRO*/
        'OCORRENC' => array( 'X', 231, 240 )     /*OCORRENC 231 240 X(10) CÓDIGO OCORRÊNCIA PARA RETORNO*/
        );
    protected $constReg3 = array(
        'CODBANCO' => 341,
        'CODLOTE'  => 1,
        'TIPREGIS' => 3,
        'CODSEG_A' => 'A',
        'CODSEG_D' => 'D',
        'CODSEG_E' => 'E',
        'TIPMOVIM' => '000',
        'CAMARA'   => '0',
        'ZEROS1'   => '0',
        'BRANCOS1' => ' ',
        'BRANCOS2' => ' ',
        'BRANCOS3' => ' ',
        'BRANCOS4' => ' ',
        'BRANCOS5' => ' ',
        'BRANCOS6' => ' ',
        'TIPMOEDA' => 'REA',
        'CODISPB'  => '888',
        'FINALIDA' => 'HP01', //Tipos de Pagamentos “30” (Salários) e “31” (Salários Poder Público) e Formas de Pagamentos “01” (Crédito em Conta Corrente no Itaú) e “60” (Cartão Salário)
        'NOSNUMER' => ' ',
        'DATAEFET' => '0',
        'VALOEFET' => '0',
        'NUMDOCUM' => '0',
        'FINALDOC' => '06',
        'FINALTED' => ' ',
        'AVISOFAV' => ' ',
        'OCORRENC' => ' ',
        'DISPONIB' => '0',
        'SEUNUMER' => ' ',
        
        'DATPAGTO' => '',
        'COMPETEN' => ''
        );
    protected $queryReg3AD = "
/*REGISTRO 3AD*/
SELECT
  PRI.*,
  PRI.VRLIQ AS VALPAGTO
FROM (
  SELECT
    /*A*/
    ROW_NUMBER() OVER(ORDER BY VM.NMVINCULOM)         NROREGIS,
    CC.CDBANCO                                        BANFAVOR,
    LPAD(SUBSTR(CC.CDAGENCIA, 1, 4), 5, '0')
      || ' ' || SUBSTR(LPAD(CC.CDCONTCORR, 13, '0'), 1, 12)
      || ' ' || SUBSTR(CC.CDCONTCORR, -1)             AGENCONT,
    VM.NMVINCULOM                                     NOMFAVOR,
    /*NVL(VM.CDMATRICULA, VM.NRVINCULOM)    SEUNUMER,*/
    PEH.NRCPFPESSOA                                   NUMINSCR,
    /*D*/
    EML.NMESTRUTURAM                                  CENTRO,
    NVL(VM.CDMATRICULA, VM.NRVINCULOM)                CODFUNC,
    OH.NMOCUPACAOH                                    CARGO,
    NVL((SELECT TO_CHAR(MAX(GF.DTINIGOZOFERIAS),'DDMMYYYY')
     FROM FPA_GOZOFERIAS GF
     JOIN FPA_FERIAS FE ON FE.NRORG      = GF.NRORG
                       AND FE.NRFERIAS   = GF.NRFERIAS
                       AND FE.NRVINCULOM = VM.NRVINCULOM
     WHERE GF.NRORG = VM.NRORG
       AND ( ( GF.DTINIGOZOFERIAS BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC) )
          OR ( GF.DTFIMGOZOFERIAS BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC) ) )
    ), '00000000')                                    FERIASDE,
    NVL((SELECT TO_CHAR(MAX(GF.DTFIMGOZOFERIAS),'DDMMYYYY')
      FROM FPA_GOZOFERIAS GF
      JOIN FPA_FERIAS FE ON FE.NRORG      = GF.NRORG
                        AND FE.NRFERIAS   = GF.NRFERIAS
                        AND FE.NRVINCULOM = VM.NRVINCULOM
      WHERE GF.NRORG = VM.NRORG
        AND ( ( GF.DTINIGOZOFERIAS BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC) )
           OR ( GF.DTFIMGOZOFERIAS BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC) ) )
    ), '00000000')                                    FERIASAT,
    NVL((SELECT SUM(ICF.VRREFFOLHA) FROM FPA_ITECALCFOLHA ICF
      WHERE ICF.NRORG          = CF.NRORG
        AND ICF.NRVINCULOM     = VM.NRVINCULOM
        AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
        AND ICF.NREVENTOM      = 1290), 0)            QDEDEPIR,
    NVL((SELECT SUM(ICF.VRREFFOLHA) FROM FPA_ITECALCFOLHA ICF
      WHERE ICF.NRORG          = CF.NRORG
        AND ICF.NRVINCULOM     = VM.NRVINCULOM
        AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
        AND ICF.NREVENTOM      = 300), 0)             QDEDEPSF,
    ROUND(ETH.QTHRSEMESCTRABH, 0)                     HORAS,
    ROUND(ATS.VRSALARIO, 2) * 100                     SALCONTR,
    NVL(ROUND((SELECT SUM(ICF.VREVENTOFOLHA) FROM FPA_ITECALCFOLHA ICF
      WHERE ICF.NRORG          = CF.NRORG
        AND ICF.NRVINCULOM     = VM.NRVINCULOM
        AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
        AND ICF.NREVENTOM IN (2010, 2050, 2040, 2020, 2060, 2080)
      ), 2), 0) * 100                                 VRFGTS,
    NVL(ROUND((SELECT SUM(ICF.VREVENTOFOLHA) FROM FPA_ITECALCFOLHA ICF
      WHERE ICF.NRORG          = CF.NRORG
        AND ICF.NRVINCULOM     = VM.NRVINCULOM
        AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
        AND ICF.NREVENTOM      = 3010
      ), 2), 0) * 100                                 VRCRED,
    NVL(ROUND((SELECT SUM(ICF.VREVENTOFOLHA) FROM FPA_ITECALCFOLHA ICF
      WHERE ICF.NRORG          = CF.NRORG
        AND ICF.NRVINCULOM     = VM.NRVINCULOM
        AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
        AND ICF.NREVENTOM      = 3020
      ), 2), 0) * 100                                 VRDEBI,
    NVL(ROUND((SELECT SUM(ICF.VREVENTOFOLHA) FROM FPA_ITECALCFOLHA ICF
      WHERE ICF.NRORG          = CF.NRORG
        AND ICF.NRVINCULOM     = VM.NRVINCULOM
        AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
        AND ICF.NREVENTOM      = :P_NREVENTOM
      ), 2), 0) * 100                                 VRLIQ,
    NVL(ROUND(ATS.VRSALARIO, 2) * 100, 0)             VRBASE,
    NVL(ROUND((SELECT SUM(ICF.VREVENTOFOLHA) FROM FPA_ITECALCFOLHA ICF
      WHERE ICF.NRORG          = CF.NRORG
        AND ICF.NRVINCULOM     = VM.NRVINCULOM
        AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
        AND ICF.NREVENTOM IN (3080, 3140)
      ), 2), 0) * 100                                 BASEIRRF,
    NVL(ROUND((SELECT SUM(ICF.VREVENTOFOLHA) FROM FPA_ITECALCFOLHA ICF
      WHERE ICF.NRORG          = CF.NRORG
        AND ICF.NRVINCULOM     = VM.NRVINCULOM
        AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
        AND ICF.NREVENTOM IN (3060, 3120)
      ), 2), 0) * 100                                 BASEFGTS,
    /*bknd*/
    VM.NRVINCULOM,
    CF.NROCORRECAL,
    CF.NRCALCULOFOLHA /*LISTAGG(CF.NRCALCULOFOLHA, ',') WITHIN GROUP (ORDER BY VM.NMVINCULOM) OVER () NRCALCULOFOLHA*/
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG          = VM.NRORG
                      AND VH.NRVINCULOM     = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRORG      = VH.NRORG
                                                  AND NRVINCULOM = VH.NRVINCULOM )
  /*Estrutura Legal*/
  JOIN ESTRUTURAM EML ON EML.NRORG        = VM.NRORG
                     AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
  /*Pessoa*/
  JOIN GPE_PESSOA PE ON PE.NRORG    = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN GPE_PESSOAH PEH ON PEH.NRORG          = VM.NRORG
                      AND PEH.NRPESSOA       = PE.NRPESSOA
                      AND PEH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                 WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                   AND NRORG    = PEH.NRORG
                                                   AND NRPESSOA = PEH.NRPESSOA )
  
  JOIN CONTCORRPARC CC ON CC.NRORG          = VM.NRORG
                      AND CC.NRPARCNEGOCIO  = PE.NRPARCNEGOCIO
                      AND CC.CDBANCO        = '341'
                      AND CC.IDATIVO        = 'S'
                      AND CC.NRCONTCORRPARC = ( SELECT MAX(NRCONTCORRPARC)
                                                FROM CONTCORRPARC
                                                WHERE NRORG         = CC.NRORG
                                                  AND NRPARCNEGOCIO = CC.NRPARCNEGOCIO
                                                  AND CDBANCO       = CC.CDBANCO
                                                  AND IDATIVO       = CC.IDATIVO
                                                  AND NRTPCONTCORR  = CC.NRTPCONTCORR )
  /*Ocupação*/
  LEFT JOIN GPE_OCUPACAOH OH ON OH.NRORG          = VM.NRORG
                            AND OH.NROCUPACAOM    = NVL( VH.NRCARGO, VH.NRFUNCAO )
                            AND OH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH
                                                      WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                        AND NRORG           = VM.NRORG
                                                        AND NROCUPACAOM     = NVL( VH.NRCARGO, VH.NRFUNCAO ) )
  /*Escala de trabalho*/
  LEFT JOIN GPE_ESCALATRABH ETH ON ETH.NRORG          = VM.NRORG
                                AND ETH.NRESCALATRABM  = VH.NRESCALATRABM
                                AND ETH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_ESCALATRABH
                                                           WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                             AND NRORG         = VM.NRORG
                                                             AND NRESCALATRABM = VH.NRESCALATRABM )
  /*alteração salarial*/
  LEFT JOIN GPE_ALTESALARIO ATS ON ATS.NRORG         = VM.NRORG
                               AND ATS.NRVINCULOM    = VM.NRVINCULOM
                               AND ATS.DTALTESALARIO = ( SELECT MAX(DTALTESALARIO) FROM GPE_ALTESALARIO
                                                          WHERE DTALTESALARIO <= LAST_DAY(:P_DTMESCOMPETENC)
                                                            AND NRORG      = VM.NRORG
                                                            AND NRVINCULOM = VM.NRVINCULOM )
                               AND ATS.NROCORRENCIA  = ( SELECT MAX(NROCORRENCIA) FROM GPE_ALTESALARIO
                                                         WHERE DTALTESALARIO = ATS.DTALTESALARIO
                                                           AND NRORG      = VM.NRORG
                                                           AND NRVINCULOM = VM.NRVINCULOM )
                               AND ATS.VRSALARIO     = ( SELECT MAX(VRSALARIO) FROM GPE_ALTESALARIO /*Quando o nr de ocorrência repetir ele considera o maior salário*/
                                                         WHERE DTALTESALARIO = ATS.DTALTESALARIO
                                                           AND NROCORRENCIA  = ATS.NROCORRENCIA
                                                           AND NRORG         = VM.NRORG
                                                           AND NRVINCULOM    = VM.NRVINCULOM )
  /*Cálculo de folha e item*/
  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG          = VM.NRORG
                          AND CF.NRTIPOMOVIMENT = :P_NRTIPOMOVIMENT
                          AND CF.NRTPMODALIDCAL = :P_NRTPMODALIDCAL
                          AND CF.DTOCORRENCIA   = :P_DTMESCOMPETENC
                          --AND CF.NROCORRECAL    = 1
  JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG          = CF.NRORG
                           AND ICF.NRVINCULOM     = VM.NRVINCULOM
                           AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                           AND ICF.NREVENTOM      = :P_NREVENTOM
                           AND ROUND(ICF.VREVENTOFOLHA, 2) > 0
  WHERE VM.NRORG =:P_NRORG
    AND ( VM.NMVINCULOM     BETWEEN :P_NMVINCULOM_INI     AND :P_NMVINCULOM_FIM     OR :P_NMVINCULOM_INI     IS NULL ) /*escopo alfabetico*/
    AND ( VM.CDMATRICULA    BETWEEN :P_CDMATRICULA_INI    AND :P_CDMATRICULA_FIM    OR :P_CDMATRICULA_INI    IS NULL ) /*escopo matrícula*/
    AND ( VM.DTADMISSAOVINC BETWEEN :P_DTADMISSAOVINC_INI AND :P_DTADMISSAOVINC_FIM OR :P_DTADMISSAOVINC_INI IS NULL ) /*escopo data admissão*/
    AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL ) /*lista de estrutura legal*/
    AND ( INSTR(','||:P_NRESTRUTGEREN||',', ','||VH.NRESTRUTGEREN||',' ) > 0 OR :P_NRESTRUTGEREN IS NULL ) /*lista de estrutura gerencial*/
    AND ( INSTR(','||:P_NRESTRUTSIND ||',', ','||VH.NRESTRUTSIND ||',' ) > 0 OR :P_NRESTRUTSIND  IS NULL ) /*lista de estrutura sindical*/
    AND ( INSTR(','||:P_NRVINCULOM   ||',', ','||VM.NRVINCULOM   ||',' ) > 0 OR :P_NRVINCULOM    IS NULL ) /*lista de vínculos*/
  ORDER BY VM.NMVINCULOM
) PRI
"; //todo e as ocorrências?
    protected $queryReg3E = "
/*REGISTRO 3E*/
SELECT
  DECODE(N, 0, 4, N) N,
  MOVIMENT, DESCRI, VALOR
  --NRTIPOMOVIMENT, NREVENTOM, NRVINCULOM
FROM (
  SELECT
    RES.*,
    MOD(ROW_NUMBER() OVER(PARTITION BY RES.MOVIMENT ORDER BY RES.MOVIMENT, RES.NREVENTOM), 4) N
  FROM (
    SELECT
      DECODE(EM.IDTIPOEVENTO, 'PROVENTO', 1, --crédito
                              'DESCONTO', 2, --débito
                              'ENCARGO',  2,
                              'TOTAL',    3 --acumulado
                              ) MOVIMENT,
      CONVERT(TRANSLATE(EH.NMEVENTOH, 'ªº', 'ao'), 'US7ASCII', 'WE8ISO8859P1') DESCRI,
      ROUND(ICF.VREVENTOFOLHA, 2) * 100 VALOR,
      /*bknd*/
      ICF.NREVENTOM
      /*CF.NRTIPOMOVIMENT,
      ICF.NREVENTOM,
      ICF.NRVINCULOM*/
    FROM FPA_CALCULOFOLHA CF
    JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG          = CF.NRORG
                             AND ICF.NRVINCULOM     = :P_NRVINCULOM
                             AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                             AND ROUND(ICF.VREVENTOFOLHA, 2) > 0
    /*eventos*/
    JOIN FPA_EVENTOM EM ON ( EM.NRORG = 0 OR EM.NRORG = CF.NRORG )
                       AND EM.NREVENTOM = ICF.NREVENTOM
    JOIN FPA_EVENTOH EH ON ( EH.NRORG = 0 OR EH.NRORG = CF.NRORG )
                       AND EH.NREVENTOM = EM.NREVENTOM
                       AND EH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM FPA_EVENTOH
                                                  WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                    AND NRORG     = EH.NRORG
                                                    AND NREVENTOM = EM.NREVENTOM )
    WHERE CF.NRORG = :P_NRORG
      AND ( INSTR(','||:P_NRCALCULOFOLHA||',', ','||CF.NRCALCULOFOLHA||',' ) > 0 )
  ) RES
)
";

    //MOVIMENT 1 Crédito 2 Débito 3 Acumulado
    
    /*Registro 5*/
    protected $layoutReg5 = array(
        /*campo            tipo  inicio  fim*/
        'CODBANCO' => array( '9',   1,   3 ), /*CODBANCO 001 003 9(03) 341 CÓDIGO DO BANCO CÓDIGO BANCO NA COMPENSAÇÃO*/
        'CODLOTE'  => array( '9',   4,   7 ), /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE LOTE DE SERVIÇO*/
        'TIPREGIS' => array( '9',   8,   8 ), /*TIPREGIS 008 008 9(01) 5 TIPO DE REGISTRO REGISTRO TRAILER DE LOTE*/
        'BRANCOS1' => array( 'X',   9,  17 ), /*BRANCOS1 009 017 X(09) ' ' COMPLEMENTO DE REGISTRO*/
        'TOTALREG' => array( '9',  18,  23 ), /*TOTALREG 018 023 9(06) NOTA 17 TOTAL QTDE REGISTROS QTDE REGISTROS DO LOTE*/
        'TOTALVAL' => array( '9',  24,  41 ), /*TOTALVAL 024 041 9(16)V9(2) NOTA 17 TOTAL VALOR PAGTOS SOMA VALOR DOS PGTOS DO LOTE*/
        'ZEROS1'   => array( '9',  42,  59 ), /*ZEROS1   042 059 9(18) ZEROS COMPLEMENTO DE REGISTRO*/
        'BRANCOS2' => array( 'X',  60, 230 ), /*BRANCOS2 060 230 X(171) ' ' COMPLEMENTO DE REGISTRO*/
        'OCORRENC' => array( 'X', 231, 240 )  /*OCORRENC 231 240 X(10) NOTA 8 ' ' OCORRÊNCIAS CÓDIGOS OCORRÊNCIAS P/ RETORNO*/
        );
    protected $constReg5 = array(
        'CODBANCO' => 341,
        'CODLOTE'  => 1,
        'TIPREGIS' => 5,
        'BRANCOS1' => ' ',
        //'TOTALREG' nrSequencial + 1
        //'TOTALVAL' vrTotal
        'ZEROS1'   => 0,
        'BRANCOS2' => ' ',
        'OCORRENC' => ' '
        );
    
    /*Registro 9*/
    protected $layoutReg9 = array(
        /*campo            tipo  inicio  fim*/
        'CODBANCO' => array( '9',  1,   3 ),   /*CODBANCO 001 003 9(03) 341 CÓDIGO DO BANCO CÓDIGO BANCO NA COMPENSAÇÃO*/
        'CODLOTE'  => array( '9',  4,   7 ),   /*CODLOTE  004 007 9(04) 9999 CÓDIGO DO LOTE LOTE DE SERVIÇO*/
        'TIPREGIS' => array( '9',  8,   8 ),   /*TIPREGIS 008 008 9(01) 9 TIPO DE REGISTRO REGISTRO TRAILER DE ARQUIVO*/
        'BRANCOS1' => array( 'X',  9,  17 ),   /*BRANCOS1 009 017 X(09)' ' COMPLEMENTO DE REGISTRO*/
        'QDELOTES' => array( '9', 18,  23 ),   /*QDELOTES 018 023 9(06) NOTA 17 1 TOTAL QTDE DE LOTES QTDE LOTES DO ARQUIVO*/
        'QDEREGIS' => array( '9', 24,  29 ),   /*QDEREGIS 024 029 9(06) NOTA 17 TOTAL QTDE REGISTROS Q T D E R E G ISTROS DO ARQUIVO*/
        'BRANCOS2' => array( 'X', 30, 240 )    /*BRANCOS2 030 240 X(211) COMPLEMENTO DE REGISTRO*/
        );
    protected $constReg9 = array(
        'CODBANCO' => 341,
        'CODLOTE'  => 9999,
        'TIPREGIS' => 9,
        'BRANCOS1' => ' ',
        'QDELOTES' => 1,
        //'QDEREGIS' => nrSequencial + 2,
        'BRANCOS2' => ' ',
        );
}