<?php
 
namespace HCM\Geral\Logic;

/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 05/12/2019
 * Time: 10:00
 */

class ImportacaoXSD {

    /* Tags XSD */
    const XS_ELEMENT = 'element';
    const XS_COMPLEXTYPE = 'complextype';
    const XS_SEQUENCE = 'sequence';
    const XS_ANNOTATION = 'annotation';
    const XS_DOCUMENTATION = 'documentation';
    const XS_ATTRIBUTE = 'attribute';
    const XS_ID = 'xs:ID';

    /** @var \Doctrine\ORM\EntityManager */
    protected $_entityManager;
    /** @var \HCM\Util\HCMParameters */
    protected $_hcmParameters;
    
    private $arrRegistros;
    private $nrevento;
    private $elementEsocial;
    private $complexTypes;

    public function __construct($entityManager, $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        
        $this->arrRegistros = array();
    }

    public function importarXSD($nrevento, $file) {
        try {
            $registros = $this->entityManager->getRepository(\HCM\Util\Repositories::ESO_REGISTRO)->findBy(array('nrevento' => $nrevento));
            if (!empty($registros)) {
                throw new \Exception("Evento já possui registros importados", 1);
            }
            
            $this->nrevento = $nrevento;
            
            $dom = new \DOMDocument("1.0", "UTF-8");
            $dom->preserveWhiteSpace = false;
            $dom->formatOutput = true;
            $dom->load($file->getPathname());
            
            $xpath = new \DOMXPath($dom);
            $xpath->registerNamespace('xs', 'http://www.w3.org/2001/XMLSchema');
            
            /*Ao invés de percorrer todos os elementos e complexTypes na função percorreXPath, selecionar aqui o elemento principal [esocial] e os seus complexTypes, 
            para que a função só percorra os elementos filhos usando os types quando necessário*/
            
            $complexTypes = $xpath->evaluate("/xs:schema/xs:complexType");
            if ($complexTypes->count() > 0) {
                $this->preparaComplexTypes($complexTypes);
            }
            
            $this->elementEsocial = $xpath->evaluate("/xs:schema/xs:element")->item(0);
            $this->percorreEsocialElement();
            
            $this->persistRegistros();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function preparaComplexTypes($complexTypes) {
        try {
            foreach($complexTypes as $node) {
                $this->complexTypes[$node->getAttribute('name')] = $node;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function percorreEsocialElement() {
        try {
            if ($this->elementEsocial->hasAttribute('name')) {
                $nivel = 1;
                $registro = $this->setNovoRegistro($this->elementEsocial, null);
                $this->percorreChildNodes($this->elementEsocial->childNodes, $nivel, $registro);
            } else {
                throw new \Exception('Elemento eSocial não encontrado', 1);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function percorreChildNodes($childNodes, $nivel, $registroPai=null) {
        try {
            foreach($childNodes as $node) {
                if ($node->localName == self::XS_ANNOTATION) {
                    $registroPai->setDsregistro($node->nodeValue);
                }
                if (($node->localName == self::XS_ELEMENT || $node->localName == self::XS_ATTRIBUTE) && $node->hasAttribute('name')) {
                    $registro = $this->setNovoRegistro($node, $registroPai);
                    //var_dump($nivel . ' - ' . $node->getAttribute('name') . ($node->hasAttribute('type') ? ' {'.$node->getAttribute('type').'}' : '') . ' | ' . $registroPai->getNmcampo());
                    if ($node->hasAttribute('type') && $node->getAttribute('type') != self::XS_ID) {
                        $this->percorreChildNodes($this->complexTypes[$node->getAttribute('type')]->childNodes, $nivel+1, $registro);
                    }
                }
                if ($node->hasChildNodes()) {
                    $this->percorreChildNodes($node->childNodes, ($node->localName == self::XS_ELEMENT ? $nivel+1 : $nivel), (isset($registro) ? $registro : $registroPai));
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function setNovoRegistro($node, $registroPai) {
        try {
            $esoRegistro = \HCM\Util\Repositories::ESO_REGISTRO;
            
            $registro = new $esoRegistro;
            $registro->setNrorg = $this->hcmParameters->getNrorgTrabalho();
            $registro->setNrregistro(1);
            $registro->setNrevento($this->nrevento);
            $registro->setNrcampo(count($this->arrRegistros)+1);
            $registro->setNmcampo($node->getAttribute('name'));
            $registro->setNrcampopai(is_null($registroPai) ? null : $registroPai->getNrcampo());
            $registro->setNmcampopai(is_null($registroPai) ? null : $registroPai->getNmcampo());
            $this->arrRegistros[] = $registro;
            
            return $registro;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function persistRegistros() {
        try {
            $this->entityManager->beginTransaction();
            foreach ($this->arrRegistros as $index => $registro) {
                
                $this->beforeInsertRegistro($registro);
                
                $registro->setNrregistro(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_REGISTRO', 1, 12, true));
                $this->entityManager->persist($registro);
                $this->entityManager->flush();
            }
            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    private function beforeInsertRegistro($registro) {
        try {
            // Verifica se em outras versões do evento possui nome de tabela e/ou campo preenchidos
            $evento = $this->entityManager->getRepository(\HCM\Util\Repositories::ESO_EVENTO)->findOneBy(array('nrevento' => $this->nrevento));
            $outrosEventos = $this->entityManager->getRepository(\HCM\Util\Repositories::ESO_EVENTO)->findBy(array('nmevento' => $evento->getNmevento()), array('dsversao' => 'DESC'));
            
            foreach ($outrosEventos as $outroEvento) {
                if ($outroEvento->getDsversao() == $evento->getDsversao()) {
                    continue;
                }
                
                $outroRegistro = $this->entityManager->getRepository(\HCM\Util\Repositories::ESO_REGISTRO)->findOneBy(
                    array('nrevento' => $outroEvento->getNrevento(), 'nmcampo' => $registro->getNmcampo(), 'nmcampopai' => $registro->getNmcampopai())
                );
                
                if (is_object($outroRegistro)) {
                    if (!is_null($outroRegistro->getDstabelabd()) || !is_null($outroRegistro->getDscampobd())) {
                        $registro->setDstabelabd($outroRegistro->getDstabelabd());
                        $registro->setDscampobd($outroRegistro->getDscampobd());
                        break;
                    }
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
}