<?php

namespace Vab\Services;

use Doctrine\ORM\EntityManager;
use Ems\Util\Utils;
use Folha\Util\Valor;
use Util\Environment;
use Util\Messages;
use VaB\Repository\CalculoBeneficioRepository;

use Folha\Entities\GpePessoa;
use Folha\Entities\FpaItecalcfolha;
use Folha\Entities\VabHistbenefvinc;
use Folha\Entities\GpeHorarioturno;
use Folha\Entities\GpeAltesitufunc;
use Folha\Entities\GpeEscalatrabh;
use Folha\Entities\GpeAlteescala;

use Folha\Repositories\GpeEscalatrabm;

use HCM\Geral\Entity\GpealtesitufuncEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpemovimentacaoEntity;

use HCM\HcmParameters;
use Ems\Services\{ EmailService};

use Util\{
    DateUtil,
    EmailMessages,
    //HCMParameters,
    Repositories,
    //Util
    NovoCodigo
};

use Vab\Logic\CalculoBeneficioUtil;

use \Zeedhi\Framework\ORM\DateTime;
use HCM\Util\MsgCalculo;

class CalculoBeneficioService {
    
    /** @var EntityManager */
    protected $entityManager;

    /** @var Environment */
    protected $environment;
    
    /** @var Repository */
    protected $calculoBeneficioRepository;

    protected $valorReferenciaService;
    
    /** @var const MOVIMENTO_BENEFICIO */
    public const MOVIMENTO_BENEFICIO = 21;
    
    //CONSTANTES
    const SABADO = 6;
    const DOMINGO = 7;
    const SABDOM = 67;
    const SEGUNDA = 1;
    const TERCA = 2;
    const QUARTA = 3;
    const QUINTA = 4;
    const SEXTA = 5;
    
    protected $histVinc;
    private $vinculoh;
    private $pessoah;
    private $nrcalculoconsumer;

    private $beneficios = array();

    private $nrtpbeneficio;
    private $nrestruturam;

    private $dtiniperiodobenef;
    private $dtfimperiodobenef;
    private $dtiniperiodocalculo;
    private $dtfimperiodocalculo;
    private $dtPagamentoBenef;
    private $idcalccomplementar;
    
    //Luan 23/06
    private $dtfimperiodobenefmod;
    private $dtiniperiodobenefmod;
    
    //luan 24/10
    private $feriasnaodescontadas;
    private $afastamentonaodescontados;
    private $valorAdescontar;
    
    //luan 31/10 implementação de proporcionalização no beneficio de Combustivel Mensal(Quando tem NRREFERENCIA)
    private $temFeriasNaCompetencia;
    private $diasFeriasNaCompetencia;
    private $temAfastamentoNaCompetencia;
    private $diasAfastamentoNaCompetencia;
    private $temAtestadoNaCompetencia;
    private $diasAtestadoNaCompetencia;
    private $diasNormais;

    /** @var GpeVinculom $vinculo */
    private $vinculo;

    private $dsdiadescansosemanal;

    private $nrafastamento;
    private $nrfalta;
    private $nrdiasaviso;
    private $nrdiasproxmes;
    private $nrdiasdsr;
    private $nrdiasferias;
    
    private $nrtpjornada;
    private $nrturno;
    private $nrHorarioTurno;
    
    private $geraBenefFeriado;

    private $diaInicialAupr;
    private $diaFinalAupr;

    private $diastotaisconsideracao;
    private $idMemoria;
    private $_movimentosVinculo;
    private $dtpagamento;
    private $nrReferencia;
    private $nrorgTrab;
    private $periodoInicialMensal;
    private $hcmParameters;
    private $dtCancelamentoBeneficiario;
    private $dtInscricaoBeneficiario;
    private $UltDiaCompetencia;
    private $competencia;
    private $row;
    private $diasTrabalhadosNoMes;
    private $cdoperador;
    private $deveCalcularFaltasPorSituacaoFuncional;
    private $referenciaFaltasPorSituacaoFuncional;
    private $deveAnalisarDescontoMinimizadores;
    private $referenciaAnalisaDescontoMinimizadores;
    private $objcompetenciaAdmissao;
    private $sequenciaCalculo;
    private $calcConsumer;
    private $fixaDias;

    protected $emailService;
    
    private $idcredindev;
    private $dtiniciocredindev;
    private $dtfinalcredindev;

    public function __construct(
        EntityManager      $entityManager,
        Environment        $environment,
        CalculoBeneficioRepository $calculoBeneficioRepository,
        EmailService $emailService
    ) {
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->calculoBeneficioRepository  = $calculoBeneficioRepository;
        $this->emailService            = $emailService;
    }
    
    public function calcularBeneficio($row)
    {   
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try{
            $foiCalculado = false;
            $dsordem = $row['DSORDEMCALCBENEF'];
            $dsinicio = $row['DSINICIOCALCBENEF'];
            $dsfim = $row['DSFIMCALCBENEF'];
            $dsinicio1 = $row['DSINICIOCALCBENEF1'];
            $dsfim1 = $row['DSFIMCALCBENEF1'];
            $dtinicio = isset($row['DTINICALCBENEF']) && !empty($row['DTINICALCBENEF']) ? substr($row['DTINICALCBENEF'], 0, 10) : null;
            $dtfinal = isset($row['DTFIMCALCBENEF']) && !empty($row['DTFIMCALCBENEF']) ? substr($row['DTFIMCALCBENEF'], 0, 10) : null;
            $dtpagamento = substr($row['DTPGTOCALCBENEF'], 0, 10);
            $nrtpbeneficio = $row['NRTPBENEFICIO'];
            $nrestruturam = $row['NRESTRUTURAM'];
            $tpperiodo = $row['TPPERIODOBENEF'];
            $competencia = substr($row['DTCOMPETENC'], 0, 10);
            $TipoEstruturaLegalOrg = $row['TIPOESTRUTURALEGALORG'];
            $this->UltDiaCompetencia = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO, true)->format('t/m/Y');
            $this->competencia = $competencia;
            $dtmescompetenc = DateUtil::getDataDeString($row['DTCOMPETENC'], DateUtil::FORMATO_BRASILEIRO, true);
            $nrorg = $row['NRORG'];
            $this->nrorgTrab = $nrorg;
            $this->cdoperador = $row['CDOPERADOR'];
            $nrcalculobenef = $nrcalculo = Utils::getEntityManager()->getRepository(Repositories::FPA_CALCULOFOLHA)->getNrCalculo(
                $nrorg, $dtmescompetenc, self::MOVIMENTO_BENEFICIO
            );
            $qtdiasbenef = $row['QTDIASBENEF'];
            $this->dtPagamentoBenef = $dtpagamento;
            $this->idcalccomplementar = $row['IDCALCCOMPLEMENTAR'];
            $this->idcredindev = $row['IDCALCCREDINDEV'];
            $this->dtiniciocredindev = $row['DTINICIOCREDINDEV'];
            $this->dtfinalcredindev = $row['DTFINALCREDINDEV'];
            $this->nrReferencia = null;
            $this->diasTrabalhadosNoMes = 0;
            $this->idMemoria = $row['IDMEMORIACALCULO'];
            $this->nrcalculoconsumer = $row['NRCALCULOCONSUMER'];
            $this->periodoInicialMensal = null;
            $this->row = $row;
            //!!!!!!!! IMPORTANTE, como estamos no HCMSERVICES precisamos de criar o hcmParameters
            $this->hcmParameters = new HcmParameters($this->nrorgTrab, $this->competencia, 0, $this->cdoperador, $TipoEstruturaLegalOrg);
            
            if($tpperiodo == 'MENSAL'){
                if(is_null($dtinicio)){
                    $this->periodoInicialMensal = $dtinicio = $this->hcmParameters->getDtCompetencia();
                }
                
                if(is_null($dtfinal)){
                    $dtfinal = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true)->format('t/m/Y');
                }
            }
            
            $this->dtiniperiodobenefmod = $dtinicio;
            $this->dtfimperiodobenefmod = $dtfinal;
            
            $this->referenciaAnalisaDescontoMinimizadores = 0;
            // Busca o valor de referência para analisar desconto minimizador
            $this->referenciaAnalisaDescontoMinimizadores = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)
                ->buscavalor(
                    $this->hcmParameters->getNrorgTrabalho(),
                    0,
                    $this->hcmParameters->getDtCompetencia(),
                    'ANALISA_DESCONTO_MINIMIZADORES'
                );
                
            // Verifica se deve analisar Desconto minimizadores
            $this->deveAnalisarDescontoMinimizadores = false;
            
            // Atualiza a flag para cálculo se a referência for válida e igual a 1
            if (!empty($this->referenciaAnalisaDescontoMinimizadores) && $this->referenciaAnalisaDescontoMinimizadores == 1) {
                $this->deveAnalisarDescontoMinimizadores = true;
            }
            
            $this->referenciaFaltasPorSituacaoFuncional = 0;
            // Busca o valor de referência para calcular faltas por situação funcional
            $this->referenciaFaltasPorSituacaoFuncional = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->hcmParameters->getNrorgTrabalho(), 0, $this->hcmParameters->getDtCompetencia(), 'BENEF_FALTASSITUACAOFUNCIONAL');
            
            // Verifica se deve calcular faltas com base na situação funcional
            $this->deveCalcularFaltasPorSituacaoFuncional = false;
            
            // Atualiza a flag para cálculo se a referência for válida e igual a 1
            if (!empty($this->referenciaFaltasPorSituacaoFuncional) && $this->referenciaFaltasPorSituacaoFuncional == 1) {
                $this->deveCalcularFaltasPorSituacaoFuncional = true;
            }
            
            $this->_movimentosVinculo = new GpemovimentacaoEntity($this->entityManager, $this->hcmParameters);
            $this->calcConsumer = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_CALCULOCONSUMER);
            $this->sequenciaCalculo = 'CALCULO_'.$row['NRCALCULOCONSUMER'];
            
            foreach($row['VINCULOS'] as $nrvinculo){
                $this->jornadaDiaria = null;
                $this->objcompetenciaAdmissao = null;
                
                $instancia = MsgCalculo::getInstance();
                $instancia->setDados($nrcalculobenef, intval($nrvinculo), $this->hcmParameters->getNrorgTrabalho(), "CALCBENEFICIO", null, $row['NRCALCULOCONSUMER']);
                
                $this->vinculoh = null;
                $this->pessoah = null;
                
                $vinculo = Utils::getEntityManager()->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array('nrorg' => $nrorg,
                'nrvinculom' => $nrvinculo));
                
                $this->vinculoh = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($this->nrorgTrab, $vinculo->getNrvinculom(), $this->competencia);
                
                if ($this->vinculoh->getNrescalatrabm() == null) {
                    MsgCalculo::getInstance()->adicionaMensagem("Histórico do vínculo não possui escala", MsgCalculo::ERRO);
                    $instancia->salvaMensagens();
                    $instancia->clear();
                    
                    //Em um continue (Exceção):
                    $this->atualizaCalculoConsumer();
                    
                    continue;
                }
                
                $eventos = $qtdBeneficiariosPerEvento = $eventoEparcelas = [];
                $beneficiarios = null;
                
                // Variável a ser comparada, caso não seja para considerar vínculos com rescisão no mês anterior
                $dtRescisaoVinc = substr($vinculo->getDtrescisaovinc(), 0, 10);
                $dtRescisaoVinc = $dtRescisaoVinc ? DateUtil::getDataDeString($dtRescisaoVinc, DateUtil::FORMATO_BRASILEIRO)->format('Y-m-d') : null;   
                $diaRescisao    = isset($dtRescisaoVinc) ? (int)substr($dtRescisaoVinc, 8, 2): null;
                
                // Variável a ser comparada, caso não seja para considerar vínculos com rescisão no mês anterior
                $competenciaPosterior  = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO);
                $competenciaPosterior  = $competenciaPosterior->add(new \DateInterval('P1M'));
                $competenciaPosterior  = $competenciaPosterior->format('Y-m-d');
                
                // Variável a ser utilizada juntamente com a $dtRescisaoVinc para calcular dias proporcionalizados                
                $dtAdmissaoVinc = substr($vinculo->getDtadmissaovinc(), 0, 10);
                $dtAdmissaoVinc = $dtAdmissaoVinc ? DateUtil::getDataDeString($dtAdmissaoVinc, DateUtil::FORMATO_BRASILEIRO)->format('Y-m-d') : null;   
                
                // Variável String com o valor competência que poderá ser comparada com outras variáveis String, todas no Ano/Mês/Dia
                $competenciaAtual  = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO);
                $competenciaAtual  = $competenciaAtual->format('Y-m-d');
                
                // Variavael booleana em que caso seja verdadeira, a empresa pagara o valor total do benefício para o beneficiário.
                $subsidio100PorAfastamentoOuDemissao = false;
                
                // Caso o vínculo seja de um sindicato é para considerar o campo 'percentual/valor associado' se e somente se este tiver valor
                #$idAssocSind = isset($histVinc['IDASSOCSINDICATO']) ? $histVinc['IDASSOCSINDICATO'] : 'N' ;
                $idAssocSind = $this->vinculoh->getIdassocsindicato() ? $this->vinculoh->getIdassocsindicato() : 'N' ; 

                $escalaTrabalho = Utils::getEntityManager()->getRepository(Repositories::GPE_ESCALATRABH)->_retornaUltHistórico(
                    $this->vinculoh->getNrescalatrabm(), 
                    $nrorg, 
                    $competencia
                );
                
                if (empty($escalaTrabalho) || $escalaTrabalho == null) {
                    MsgCalculo::getInstance()->adicionaMensagem("Não foi encontrado escala para o vínculo.", MsgCalculo::ERRO);
                    $instancia->salvaMensagens();
                    $instancia->clear();
                    
                    //Em um continue (Exceção):
                    $this->atualizaCalculoConsumer();
                    
                    continue;
                }
                
                $this->jornadaDiaria = $escalaTrabalho['QTHRESCALATRABH'];

                // Verificar se período de apuração é definido na escala de trabalho do vínculo
                if(isset($escalaTrabalho['IDUTAPURATESTBENEF']) && ($escalaTrabalho['IDUTAPURATESTBENEF'] == 'S' || $escalaTrabalho['IDUTAPURATESTBENEF'] == 'Y')){
                    $this->diaInicialAupr = $escalaTrabalho['NRDIAINIAPUR'];
                    $this->diaFinalAupr = $escalaTrabalho['NRDIAFINAPUR'];
                }
                
                switch ($tpperiodo) {
                    case 'MENSAL':
                        
                        $nrlotevinc = null;
                        
                        $beneficiarios = $this->calculoBeneficioRepository->getBeneficiarioPorVinculo2($nrorg, $vinculo->getNrvinculom(), $dtmescompetenc, $tpperiodo, $nrtpbeneficio, $nrestruturam);
                        
                        if(empty($beneficiarios)){
                            //!!! IMPORTANTE se tiver vazio os beneficiarios
                            //Em um continue (Exceção):
                            $this->atualizaCalculoConsumer();
                            
                            continue;
                        }
                        
                        #$this->obterMotivosParaNaoCalcular_verificarIsencao100Afastamento($nrorg, $competencia, $vinculo, $subsidio100PorAfastamentoOuDemissao, $motivos);
                        
                        $isLast = false;
                        
                        foreach($beneficiarios as $key => $beneficiario){
                            $this->pessoah = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaHistoricoAtualPorPessoa($this->nrorgTrab, $beneficiario['NRPESSOA'], $this->competencia);
                            $beneficio = $this->calculoBeneficioRepository->getBeneficioPorVinculo($competencia, $nrorg,$beneficiario['NRBENEFICIOM']);
                            
                            $this->dtInscricaoBeneficiario = DateUtil::getDataDeString(substr($beneficiario['DTINSCBENEFICIARIO'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
                            $this->dtCancelamentoBeneficiario = !is_null($beneficiario['DTCANCBENEFICIARIO']) ? DateUtil::getDataDeString(substr($beneficiario['DTCANCBENEFICIARIO'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true) : $beneficiario['DTCANCBENEFICIARIO'];
                            
                            //VALIDAÇÕES!!!
                            if(!empty($row['OCORRCALC']) && $row['OCORRCALC'] == 1){
                                $verificaLiquidoOcorr1 = $this->calculoBeneficioRepository->validaLiquidoBeneficioOcorrencia1(self::MOVIMENTO_BENEFICIO, $nrcalculobenef, $vinculo->getNrvinculom(), $competencia, $nrorg, $beneficio['NREVETITULARVINC'], $dtpagamento, 1);
                            
                                if(empty($verificaLiquidoOcorr1)){
                                    $verificaLiquidoOcorr1 = $this->calculoBeneficioRepository->validaLiquidoBeneficioOcorrencia1(self::MOVIMENTO_BENEFICIO, $nrcalculobenef, $vinculo->getNrvinculom(), $competencia, $nrorg, $beneficio['NREVENTOMENSA'], $dtpagamento, 1);
                                }
                                
                                if(!empty($verificaLiquidoOcorr1)){
                                    MsgCalculo::getInstance()->adicionaMensagem("Vinculo com liquído com mesma data de pagamento existente na ocorrência 1, em caso de cálculo complementar favor calcular na ocorrência 2 o benefício MENSAL do colaborador", MsgCalculo::ALERTA);
                                    $instancia->salvaMensagens();
                                    $instancia->clear();
                                    
                                    continue;
                                }
                            }
                            
                            //Variável a ser comparada caso o vinculo possua liquido
                            $verificaLiquido = $this->calculoBeneficioRepository->validaLiquidoBeneficio($nrcalculobenef, $vinculo->getNrvinculom(), $competencia, $nrorg, $beneficio['NREVETITULARVINC'], $dtpagamento);
                            
                            if(empty($verificaLiquido)){
                                $verificaLiquido = $this->calculoBeneficioRepository->validaLiquidoBeneficio($nrcalculobenef, $vinculo->getNrvinculom(), $competencia,$nrorg, $beneficio['NREVENTOMENSA'], $dtpagamento);
                            }
                            
                            // Verifica se algum Vinculo ja possui calculo de beneficio
                            if(isset($verificaLiquido) && $verificaLiquido){
                                MsgCalculo::getInstance()->adicionaMensagem("Vínculo possui o evento ". $beneficio['NREVETITULARVINC']." com a data de pagamento ".$dtpagamento, MsgCalculo::ALERTA);
                                $instancia->salvaMensagens();
                                $instancia->clear();
                                
                                continue;
                            }
                            
                            // No último beneficiario da iteração do mesmo benfício calcula o valor de acréscimo no desconto
                            if($key == count($beneficiarios)-1 || (isset($beneficiarios[$key+1]) && $beneficio['NRBENEFICIOM'] != $beneficiarios[$key+1]['NRBENEFICIOM'])){
                                $isLast = true;
                            }
                            
                            // Verifica conforme o parâmetro do cadastro do benefício se ele deve ser calculado no mês corrente
                            if(isset($beneficio['PERIODICIDADEMES']) && substr($beneficio['PERIODICIDADEMES'],DateUtil::getDataDeString($competencia,DateUtil::FORMATO_BRASILEIRO,true)->format('m')-1,1) == 'N'){
                                continue;
                            }
                            
                            // Caso o benefício esteja marcado para não gerar na admissão e a data de admissão do vínculo for na competência atual não gera o benefício
                            if((isset($beneficio['IDGERBENEFADMISSAO']) && $beneficio['IDGERBENEFADMISSAO'] == 'N') && (DateUtil::getDataDeString(substr($vinculo->getDtadmissaovinc(), 0, 10))->format('Ym') == DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO,true)->format('Ym'))){
                                $msgVinculoPossuiAdmissao = "Benefício ".$beneficio['NRBENEFICIOM']." não calculado para o vínculo. Motivo: Admitido no mês.";
                                MsgCalculo::getInstance()->adicionaMensagem($msgVinculoPossuiAdmissao, MsgCalculo::ALERTA);
                                $instancia->salvaMensagens();
                                $instancia->clear();

                                continue;
                            }
                            
                            // Caso o benefício esteja marcado para não gerar na rescisão e a data de rescisão do vínculo for na competência atual não gera o benefício
                            if((isset($beneficio['IDGERBENEFRESCISAO']) && $beneficio['IDGERBENEFRESCISAO'] == 'N') && $vinculo->getDtrescisaovinc() && (DateUtil::getDataDeString(substr($vinculo->getDtrescisaovinc(), 0, 10))->format('Ym') == DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO,true)->format('Ym'))){
                                $msgVinculoPossuiRescisao = "Benefício ".$beneficio['NRBENEFICIOM']." não calculado para o vínculo. Motivo: rescindido no mês.";
                                MsgCalculo::getInstance()->adicionaMensagem($msgVinculoPossuiRescisao, MsgCalculo::ALERTA);
                                $instancia->salvaMensagens();
                                $instancia->clear();

                                continue;
                            }
                            //FIM DAS VALIDAÇÕES
                            
                            // Caso vínculo tenha sindicato no histórico e tal sindicato tenha beneficio correspondente, os dados de desconto do vínculo serão a partir do benefício do sindicato.
                            // Caso vínculo tenha sindicato no histórico e tal sindicato não tenha benefício, os dados de desconto do vínculo serão a partir do benefício original.
                            if(!empty($this->vinculoh->getNrestrutsind()) && intval($this->vinculoh->getNrestrutsind())) {
                                $benefDescontoSind = $this->calculoBeneficioRepository->getBeneficioDescontoSind($competencia, $nrorg, $beneficio['NRTPBENEFICIO'], intval($this->vinculoh->getNrestrutsind()));
                                if(isset($benefDescontoSind) && !empty($benefDescontoSind)) $this->mergeBeneficioComBeneficioSind($beneficio, $benefDescontoSind);
                            }
                            
                            $tpBeneficio = Utils::getEntityManager()->getRepository(Repositories::TPBENEFICIO)->getTpbeneficio($beneficio['NRTPBENEFICIO'], $nrorg);
                            
                            $this->verificarIsencao100Demissao($subsidio100PorAfastamentoOuDemissao, $beneficio['CONSVINCRESMESANT'], $vinculo, $dtRescisaoVinc, $competenciaAtual);
                            
                            $ehParaCalcular_minimizadorBeneficioOriginal = $ehParaCalcular_minimizadorBeneficioCCT = true;
                            
                            $dtReferenciaRescisao = null; // Caso tenha valor, seu valor será a data da competência anterior referente ao dia informado no campo 'Rescindidos a partir do dia'
                            
                            if( isset($beneficio['NRDIARESCISAO']) && !empty($beneficio['NRDIARESCISAO']) ){
                                $objCompetenc = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO, true);
                                $dtReferenciaRescisao = clone $objCompetenc; 
                                $dtReferenciaRescisao->sub(new \DateInterval('P1M'));
                                $dtReferenciaRescisao->add(new \DateInterval('P'.( intval($beneficio['NRDIARESCISAO']) -1 ).'D'));
                                $dtReferenciaRescisao = $dtReferenciaRescisao->format('Y-m-d');   
                            }
                            
                            if(     
                                    ( 
                                        (isset($beneficio['CONSVINCRESMESANT']) && $beneficio['CONSVINCRESMESANT'] == 'S') // Calcula apenas se é para considerar vinculos com rescisão do mês anterior.
                                        && (
                                              !isset($dtReferenciaRescisao) // Se for considerar vínculos com rescisão do mês anterior, qualquer vínculo será calculado caso o campo 'Rescindidos até o dia' esteja vazio
                                              || $dtRescisaoVinc >= $dtReferenciaRescisao // Ou caso este esteja preenchido, considerará apenas os vínculos com data de rescisão a partir do dia informado no campo
                                           )
                                    )    
                                || !$dtRescisaoVinc                           // Se não for para considerar vinculos com rescisão do mês anterior, calculará o benefício se não houver data de rescisão do vínculo
                                || ($dtRescisaoVinc >= $competenciaAtual) // ou se a data de rescisão for maior que a competencia atual
                            ){
                                $MOTIVO = "Benefício ".$beneficio['NRBENEFICIOM']." não calculado para o vínculo '".$vinculo->getNmvinculom()."'. Ele(a) teve no período: ";
                                
                                $minimizadores = $this->obterMinimizadoresMensal($beneficio);
                                
                                if($this->deveAnalisarDescontoMinimizadores && $this->deveAnalisarDescontoMinimizadores == true){
                                    $minimizadores = $this->avaliarDescontoMinimizadores($vinculo->getNrvinculom(), $beneficio, $minimizadores, $beneficio['NREVETITULARVINC'], $beneficio['NREVENTOMENSA'], $tpperiodo);
                                }
                                
                                $motivos = [];
                                
                                //Analisa o motivo de afastamento
                                $this->obterMotivosParaNaoCalcular_verificarIsencao100Afastamento($nrorg, $competencia, $vinculo, $subsidio100PorAfastamentoOuDemissao, $motivos, null ,$minimizadores);
                                
                                //SO CONCEDE O SUBSIDIO DE AFASTAMENTO SE A OPCAO Gera desconto para afastado ESTIVER DESMARCADA NO BENEFICIO
                                if((isset($subsidio100PorAfastamentoOuDemissao) && $subsidio100PorAfastamentoOuDemissao) && (isset($beneficio['IDGERADESCAFAST']) && $beneficio['IDGERADESCAFAST'] == 'S')){
                                    $subsidio100PorAfastamentoOuDemissao = false;
                                }
                                
                                $this->validarMinimizadoresBeneficioCCT($nrorg, $competencia, $beneficio, $tpBeneficio, $vinculo, $beneficiario, $minimizadores, $ehParaCalcular_minimizadorBeneficioCCT, $subsidio100PorAfastamentoOuDemissao);
                                
                                //Analisa demais motivos
                                $this->obterMotivosParaNaoCalcular_verificarIsencao100Afastamento($nrorg, $competencia, $vinculo, $subsidio100PorAfastamentoOuDemissao, $motivos, "MENSAL",$minimizadores, false);
                                
                                $this->fixaDias = null;
                                
                                //PARA VALE COMBUSTIVEL (TPBENEFICIO 6) POR CAUSA DA AGILE, O BENEFICIO DEVE VERIFICAR OS DIAS NORMAIS COMO SE FOSSEM PERIODO DIARIO BUSCANDO OS DIAS UTEIS.
                                if(!empty($beneficio['NRTPBENEFICIO']) && $beneficio['NRTPBENEFICIO'] == 6){
                                    $this->diasNormais = $this->calculaDiasUteisBeneficioMensal($dtAdmissaoVinc, $dtRescisaoVinc, $escalaTrabalho, $tpBeneficio, $minimizadores, $beneficio, $vinculo);
                                } else {
                                    $this->diasNormais = $this->calculaDiasNormais($nrorg, $competencia, $dtAdmissaoVinc, $dtRescisaoVinc, $escalaTrabalho, $tpBeneficio);
                                }
                                
                                $proporcao = $this->proporcionalizarOuDuplicarDiasTrabalhadosNaCompetencia($dtAdmissaoVinc, $dtRescisaoVinc, $beneficio, $beneficiario, $competenciaAtual, $competenciaPosterior);
                                
                                foreach($motivos as $motivo){
                                    if((isset($motivo) && $motivo === 'FALTA') && (isset($minimizadores['IDGERABENEFFALTA']) && $minimizadores['IDGERABENEFFALTA'] === 'N')){
                                        $MOTIVO .= $motivo.', ';
                                        $ehParaCalcular_minimizadorBeneficioOriginal = false;
                                    }else if(isset($motivo) && $motivo === 'ATESTADO' && (isset($minimizadores['IDGERABENEFATEST']) && $minimizadores['IDGERABENEFATEST'] === 'N')){
                                        $MOTIVO .= $motivo.', ';
                                        $ehParaCalcular_minimizadorBeneficioOriginal = false;
                                    }else if(isset($motivo) && $motivo === 'AFASTAMENTO' && (isset($minimizadores['IDGERABENEFAFAST']) && $minimizadores['IDGERABENEFAFAST'] === 'N')){
                                        $MOTIVO .= $motivo.', ';
                                        $ehParaCalcular_minimizadorBeneficioOriginal = false; 
                                    }else if(isset($motivo) && $motivo === 'FERIAS' && (isset($minimizadores['IDGERABENEFFERIAS']) && $minimizadores['IDGERABENEFFERIAS'] === 'N')){
                                        $MOTIVO .= $motivo.', ';
                                        $ehParaCalcular_minimizadorBeneficioOriginal = false;
                                    }
                                }
                                
                                if((isset($ehParaCalcular_minimizadorBeneficioOriginal) && $ehParaCalcular_minimizadorBeneficioOriginal) && $ehParaCalcular_minimizadorBeneficioCCT){
                                    $this->calcularBeneficioMensalDiario($nrcalculobenef, $subsidio100PorAfastamentoOuDemissao, $beneficio, $eventos, $beneficiario, $vinculo, $vinculo->getNrvinculom(), $qtdBeneficiariosPerEvento, $proporcao, $tpperiodo, $isLast, null, $foiCalculado, null, $eventoEparcelas);
                                }else if((isset($beneficiario['IDTPBENEFICIARIO']) && $beneficiario['IDTPBENEFICIARIO'] == 'T') && !$ehParaCalcular_minimizadorBeneficioOriginal){
                                    $MOTIVO = substr($MOTIVO, 0, -2);
                                    $MOTIVO .= '.';
                                    MsgCalculo::getInstance()->adicionaMensagem($MOTIVO, MsgCalculo::ALERTA); 
                                    $instancia->salvaMensagens();
                                    $instancia->clear();
                                    
                                }
                            } else if(((isset($beneficiario) && !empty($beneficiario)) && $beneficiario['IDTPBENEFICIARIO'] == 'T') && isset($dtReferenciaRescisao) && $dtReferenciaRescisao > $dtRescisaoVinc){
                                MsgCalculo::getInstance()->adicionaMensagem("Benefício ".$beneficio['NRBENEFICIOM']." não calculado para o vínculo. Data de rescisão anterior ao dia ".intval($beneficio['NRDIARESCISAO'])." da competência anterior.", MsgCalculo::ALERTA);
                                $instancia->salvaMensagens();
                                $instancia->clear();
                            }
                            
                            $isLast = false;
                        }
                        
                        $this->persistirCalculo_Mensal($foiCalculado, $nrcalculobenef, $vinculo->getNrvinculom(), null, $eventos, $qtdBeneficiariosPerEvento, $dtpagamento, $nrlotevinc, $eventoEparcelas);  // Persiste o array retornado pela função da chamada na linha anterior
                        
                        break;
                        
                    case 'DIARIO':
                        
                        $motivos = [];
                        
                        $beneficiarios = $this->calculoBeneficioRepository->getBeneficiarioPorVinculo2($nrorg, $vinculo->getNrvinculom(), $dtmescompetenc, $tpperiodo, $nrtpbeneficio, $nrestruturam);
                        
                        if(empty($beneficiarios)){
                            //!!! IMPORTANTE se tiver vazio os beneficiarios pula o vinculo
                            //Em um continue (Exceção):
                            $this->atualizaCalculoConsumer();
                            
                            continue;
                        }

                        foreach($beneficiarios as $beneficiario){
                            $this->pessoah = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaHistoricoAtualPorPessoa($this->hcmParameters->getNrorgTrabalho(), $beneficiario['NRPESSOA'], $this->hcmParameters->getDtCompetencia());
                            $ehParaCalcular_minimizadorBeneficioCCT = true;
                            
                            $beneficio = $this->calculoBeneficioRepository->getBeneficioPorVinculo($this->hcmParameters->getDtCompetencia(), $this->hcmParameters->getNrorgTrabalho(), $beneficiario['NRBENEFICIOM']);
                            
                            $this->dtInscricaoBeneficiario = DateUtil::getDataDeString(substr($beneficiario['DTINSCBENEFICIARIO'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
                            $this->dtCancelamentoBeneficiario = !is_null($beneficiario['DTCANCBENEFICIARIO']) ? DateUtil::getDataDeString(substr($beneficiario['DTCANCBENEFICIARIO'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true) : $beneficiario['DTCANCBENEFICIARIO'];
                            
                            $minimizadores = $this->obterMinimizadoresDiario($beneficio);
                            
                            if($this->deveAnalisarDescontoMinimizadores && $this->deveAnalisarDescontoMinimizadores == true){
                                $minimizadores = $this->avaliarDescontoMinimizadores($vinculo->getNrvinculom(), $beneficio, $minimizadores, $beneficio['NREVETITULARVINC'], $beneficio['NREVENTOMENSA'], $tpperiodo);
                            }

                            $this->obterMotivosParaNaoCalcular_verificarIsencao100Afastamento($nrorg, $competencia, $vinculo, $subsidio100PorAfastamentoOuDemissao, $motivos, "DIARIO", $minimizadores);
                            
                            $this->nrReferencia = $beneficio['NRVALORREFERENCIAM'];
                            
                            $historicosBenef = $this->retornaHistoricoBenefEntrePeriodo($dtinicio, $dtfinal, $vinculo->getNrvinculom(), $beneficiario['NRBENEFICIOM'], $competencia, $nrorg);
                            //VALIDAÇÕES
                            if(isset($this->idcalccomplementar) && $this->idcalccomplementar == "N"){
                                // Verifica se algum Vinculo ja possui historicos de beneficio com intersecao entre o periodo inicial e final do formulario do benef
                                if(isset($historicosBenef) && $historicosBenef){
                                    MsgCalculo::getInstance()->adicionaMensagem("Vínculo possui o benefício ". $beneficiario['NRBENEFICIOM']." já calculado no período Inicial: ".$historicosBenef[0]->getDtiniapurbenef()->format('d/m/Y')." e período final: ".$historicosBenef[0]->getDtfinapurbenef()->format('d/m/Y')." que possui data de pagamento em ".$historicosBenef[0]->getDtpgtobenef()->format('d/m/Y'), MsgCalculo::ALERTA);
                                    $instancia->salvaMensagens();
                                    $instancia->clear();

                                    continue;
                                }
                            }
                            
                            $histBenefPagto = $this->retornaHistoricoBenefPagto($dtpagamento, $vinculo->getNrvinculom(), $beneficiario['NRBENEFICIOM'], $competencia, $nrorg);
                            //Verifica data de pagamento iguais
                            if(isset($histBenefPagto) && $histBenefPagto){
                                MsgCalculo::getInstance()->adicionaMensagem("Vínculo possui o benefício ". $beneficiario['NRBENEFICIOM']." já calculado com data de pagamento em ".$historicosBenef[0]->getDtpgtobenef()->format('d/m/Y'), MsgCalculo::ALERTA);
                                $instancia->salvaMensagens();
                                $instancia->clear();
        
                                continue;
                            }
                            
                            //Variável a ser comparada caso o vinculo possua liquido
                            $verificaLiquido = $this->calculoBeneficioRepository->validaLiquidoBeneficio($nrcalculobenef, $vinculo->getNrvinculom(), $competencia, $nrorg, $beneficio['NREVETITULARVINC'], $dtpagamento);
                            // Verifica se algum Vinculo ja possui calculo de beneficio
                            if(isset($verificaLiquido) && $verificaLiquido){
                                MsgCalculo::getInstance()->adicionaMensagem("Vinculo possui o evento ". $beneficio['NREVETITULARVINC']." com a data de pagamento ".$dtpagamento, MsgCalculo::ALERTA);
                                $instancia->salvaMensagens();
                                $instancia->clear();
                                
                                continue;
                            }
                            
                            // Verifica conforme o parâmetro do cadastro do benefício se ele deve ser calculado no mês corrente
                            if(isset($beneficio['PERIODICIDADEMES']) && substr($beneficio['PERIODICIDADEMES'],DateUtil::getDataDeString($competencia,DateUtil::FORMATO_BRASILEIRO,true)->format('m')-1,1) == 'N'){
                                continue;
                            }
                            
                            // Caso o benefício esteja marcado para não gerar na admissão e a data de admissão do vínculo for na competência atual não gera o benefício
                            if((isset($beneficio['IDGERBENEFADMISSAO']) && $beneficio['IDGERBENEFADMISSAO'] == 'N') && (DateUtil::getDataDeString(substr($vinculo->getDtadmissaovinc(), 0, 10))->format('Ym') == DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO,true)->format('Ym'))){
                                $msgVinculoPossuiAdmissao = "Benefício ".$beneficio['NRBENEFICIOM']." não calculado para o vínculo. Motivo: admitido no mês.";
                                MsgCalculo::getInstance()->adicionaMensagem($msgVinculoPossuiAdmissao, MsgCalculo::ALERTA);
                                $instancia->salvaMensagens();
                                $instancia->clear();
                                
                                continue;
                            }
                            
                            // Caso o benefício esteja marcado para não gerar na rescisão e a data de rescisão do vínculo for na competência atual não gera o benefício
                            if((isset($beneficio['IDGERBENEFRESCISAO']) && $beneficio['IDGERBENEFRESCISAO'] == 'N') && $vinculo->getDtrescisaovinc() && (DateUtil::getDataDeString($vinculo->getDtrescisaovinc())->format('Ym') == DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO,true)->format('Ym'))){
                                $msgVinculoPossuiRescisao = "Benefício ".$beneficio['NRBENEFICIOM']." não calculado para o vínculo. Motivo rescindido no mês.";
                                MsgCalculo::getInstance()->adicionaMensagem($msgVinculoPossuiRescisao, MsgCalculo::ALERTA);
                                $instancia->salvaMensagens();
                                $instancia->clear();
                                
                                continue;
                            }
                            
                            // Caso vínculo tenha sindicato no histórico e tal sindicato tenha beneficio correspondente, os dados de desconto do vínculo serão a partir do benefício do sindicato.
                            // Caso vínculo tenha sindicato no histórico e tal sindicato não tenha benefício, os dados de desconto do vínculo serão a partir do benefício original.
                            if(!empty($this->vinculoh->getNrestrutsind()) && $this->vinculoh->getNrestrutsind()) {
                                $benefDescontoSind = $this->calculoBeneficioRepository->getBeneficioDescontoSind($this->hcmParameters->getDtCompetencia(), $this->hcmParameters->getNrorgTrabalho(), $beneficio['NRTPBENEFICIO'], $this->vinculoh->getNrestrutsind());
                                if(isset($benefDescontoSind) && !empty($benefDescontoSind)) $this->mergeBeneficioComBeneficioSind($beneficio, $benefDescontoSind);
                            }
                            
                            $tpBeneficio = Utils::getEntityManager()->getRepository(Repositories::TPBENEFICIO)->getTpbeneficio($beneficio['NRTPBENEFICIO'], $nrorg);
                        
                            $this->verificarIsencao100Demissao($subsidio100PorAfastamentoOuDemissao, $beneficio['CONSVINCRESMESANT'], $vinculo, $dtRescisaoVinc, $competenciaAtual);
                            
                            $dtReferenciaRescisao = null; // Caso tenha valor, seu valor será a data da competência anterior referente ao dia informado no campo 'Rescindidos a partir do dia'
                            
                            if( isset($beneficio['NRDIARESCISAO']) && !empty($beneficio['NRDIARESCISAO']) ){
                                $objCompetenc = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO, true);
                                $dtReferenciaRescisao = clone $objCompetenc; 
                                $dtReferenciaRescisao->sub(new \DateInterval('P1M'));
                                $dtReferenciaRescisao->add(new \DateInterval('P'.( intval($beneficio['NRDIARESCISAO']) -1 ).'D'));
                                $dtReferenciaRescisao = $dtReferenciaRescisao->format('Y-m-d');   
                            }
                            
                            if(     
                                    ( 
                                        (isset($beneficio['CONSVINCRESMESANT']) && $beneficio['CONSVINCRESMESANT'] == 'S') // Calcula apenas se é para considerar vinculos com rescisão do mês anterior.
                                        && (
                                              !isset($dtReferenciaRescisao) // Se for considerar vínculos com rescisão do mês anterior, qualquer vínculo será calculado caso o campo 'Rescindidos até o dia' esteja vazio
                                              || ((isset($dtReferenciaRescisao) && isset($dtRescisaoVinc)) && ($dtRescisaoVinc >= $dtReferenciaRescisao)) // Ou caso este esteja preenchido, considerará apenas os vínculos com data de rescisão a partir do dia informado no campo
                                           )
                                    )    
                                || !$dtRescisaoVinc                           // Se não for para considerar vinculos com rescisão do mês anterior, calculará o benefício se não houver data de rescisão do vínculo
                                || ($dtRescisaoVinc >= $competenciaAtual) // ou se a data de rescisão for maior que a competencia atual
                            ){
                                
                                $arrayDiasCalculados = $this->calcularDias($vinculo, $beneficiario, $minimizadores, $dtinicio, $dtfinal, $dtpagamento, $qtdiasbenef); // Retorna array com o cálculo dos dias pertinentes para o Cálculo de Benefício Diário
                                
                                $this->validarMinimizadoresBeneficioCCT($this->nrorgTrab, $this->competencia, $beneficio, $tpBeneficio, $vinculo, $beneficiario, $minimizadores, $ehParaCalcular_minimizadorBeneficioCCT, $subsidio100PorAfastamentoOuDemissao, $arrayDiasCalculados);
                                
                                if((isset($arrayDiasCalculados['DIASCONSIDERACAO']) && $arrayDiasCalculados['DIASCONSIDERACAO'] > 0) && (isset($ehParaCalcular_minimizadorBeneficioCCT) && $ehParaCalcular_minimizadorBeneficioCCT))
                                    // Calcula o Benefício e o Persiste, apenas se a quantidades de dias de consideração for maior que 0.
                                    $this->calcularBeneficioMensalDiario($nrcalculobenef, $subsidio100PorAfastamentoOuDemissao, $beneficio, $eventos, $beneficiario, $vinculo, $vinculo->getNrvinculom(), $qtdBeneficiariosPerEvento, 1/*proporcao=1*/, $tpperiodo, false, $arrayDiasCalculados, $foiCalculado); // Dentro desta função é que se persiste o histórico diário
                                else if(isset($arrayDiasCalculados['DIASCONSIDERACAO']) && $arrayDiasCalculados['DIASCONSIDERACAO'] <= 0){
                                    MsgCalculo::getInstance()->adicionaMensagem('Nenhum Benefício calculado para o vínculo. Ele(a) não possui dias de consideração.', MsgCalculo::ALERTA);
                                    $instancia->salvaMensagens();
                                    $instancia->clear();
                                }
                                
                                // Para não acumular os valores dos eventos, o array de eventos é resetado para vazio. 
                                // Já que no histórico é persistido também por benefício. E não há dependentes do Titular.
                                $eventos = [];
                            }else if((isset($beneficiario['IDTPBENEFICIARIO']) && $beneficiario['IDTPBENEFICIARIO'] == 'T') && isset($dtReferenciaRescisao) && $dtReferenciaRescisao > $dtRescisaoVinc){
                                MsgCalculo::getInstance()->adicionaMensagem("Benefício ".$beneficio['NRBENEFICIOM']." não calculado para o vínculo. Data de rescisão anterior ao dia ".intval($beneficio['NRDIARESCISAO'])." da competência anterior.", MsgCalculo::ALERTA);
                                $instancia->salvaMensagens();
                                $instancia->clear();
                                // else if($dtRescisaoVinc < $competenciaAtual)
                                //     MsgCalculo::getInstance()->adicionaMensagem("Benefício ".$beneficio['NRBENEFICIOM']." não calculado para o vínculo '".$vinculo->getNmvinculom()."'. Data de rescisão anterior à competência vigente.", MsgCalculo::ALERTA);
                            } 
                            
                        }
                        
                        break;
                }
                #MemoriaBeneficio::getInstance()->concatenarMensagemVinc();
                $instancia->salvaMensagens();
                $instancia->clear();
                
                #$arrayMensagens['MSG'][] = $nrvinculo . '- Calculado';
                //modificar o totaldevinculsoprocessados para o count($row['VINCULOS'])
                
                //Ao final de cada vinculo:
                $this->atualizaCalculoConsumer();
            }
            #MemoriaBeneficio::getInstance()->salvarMsgArquivo();
            
            #if($this->idMemoria == 'S') 
            #$this->response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('memoriaCalculo', array(MemoriaBeneficio::getInstance()->getCaminhoArquivo())));

            #return $foiCalculado;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /** 
     * Função que modifica os minimizadores que descontam Faltas, Afastamentos, Atestados
     * - Se não houve pagamento na epoca de analise do minimizador não havera desconto para o minimizador.
     * - Aplicada tanto no Diario quando no Mensal.
     * @param $nrvinculom
     * @param $beneficio
     * @param $minimizadores
     * @param $eventoTitularVinc
     * @param $eventoMensal
     * @param $tpperiodo
     * @return array
     */
    private function avaliarDescontoMinimizadores($nrvinculom, $beneficio, $minimizadores, $eventoTitularVinc, $eventoMensal, $tpperiodo){
        $minimizadoresParaAvaliar = ['IDCONSFALTABENEF', 'IDCONSAFASTBENEF', 'IDCONSATESTBENEF'];
        $this->avaliaMinimizadores($minimizadores, $minimizadoresParaAvaliar, $tpperiodo, $nrvinculom, $eventoTitularVinc, $eventoMensal, $beneficio['NRBENEFICIOM']);
        return $minimizadores;
    }
    
    private function avaliaMinimizadores(&$minimizadores, $minimizadoresParaAvaliar, $tpperiodo, $nrvinculom, $eventoTitularVinc, $eventoMensal, $nrbeneficiom){
        if(empty($minimizadores)){
            return;
        }
        
        foreach($minimizadores as $nomeMinimizador => $minimizador){
            if (in_array($nomeMinimizador, $minimizadoresParaAvaliar) && $minimizador == 'S') {
                $nomeQtMinimizador = $this->verificaQtMinimizador($nomeMinimizador);
                $minimizador = $this->avaliaMinimizador($minimizador, $minimizadores[$nomeQtMinimizador], $tpperiodo, $nrvinculom, $eventoTitularVinc, $eventoMensal, $nrbeneficiom);
                $minimizadores[$nomeMinimizador] = $minimizador;
            }
        }
    }

    private function avaliaMinimizador($minimizador, $retrocedeQtsMeses, $tpperiodo, $nrvinculom, $eventoTitularVinc, $eventoMensal, $nrbeneficiom){
        $eMesAtual = $retrocedeQtsMeses == 0;
        if($eMesAtual){
            return $minimizador;
        }

        if($tpperiodo == 'MENSAL'){
            $minimizador = $this->modificaMinimizadorMensal($minimizador, $nrvinculom, $retrocedeQtsMeses, $eventoTitularVinc, $eventoMensal);
        } else if($tpperiodo == 'DIARIO'){
            $minimizador = $this->modificaMinimizadorDiario($minimizador, $nrvinculom, $retrocedeQtsMeses, $nrbeneficiom);
        }

        return $minimizador;
    }

    private function modificaMinimizadorMensal($minimizador, $nrvinculom, $retrocedeQtsMeses, $eventoTitularVinc, $eventoMensal){
        $competenciaRetrocedida = clone DateUtil::getDataDeString($this->periodoInicialMensal, DateUtil::FORMATO_BRASILEIRO, true)->sub(new \DateInterval('P'.$retrocedeQtsMeses.'M'));
        $nrcalculobenef = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->getNrCalculo(
            $this->hcmParameters->getNrorgTrabalho(), $competenciaRetrocedida->format('d/m/Y'), self::MOVIMENTO_BENEFICIO
        );

        $liquido = $this->verificaLiquidoMensal($nrcalculobenef, $nrvinculom, $competenciaRetrocedida, $eventoTitularVinc, $eventoMensal);
        
        //Se não houve pagamento na epoca de analise do minimizador não havera desconto para o minimizador.
        if($liquido == false){
            $minimizador = 'N';
        }

        return $minimizador;
    }

    private function verificaLiquidoMensal($nrcalculobenef, $nrvinculom, $competenciaRetrocedida, $eventoTitularVinc, $eventoMensal){
        $verificaLiquido = $this->calculoBeneficioRepository->validaLiquidoBeneficioSemDataPagamento($nrcalculobenef, $nrvinculom, $competenciaRetrocedida, $this->hcmParameters->getNrorgTrabalho(), $eventoTitularVinc);               
        if($verificaLiquido == false){
            $verificaLiquido = $this->calculoBeneficioRepository->validaLiquidoBeneficioSemDataPagamento($nrcalculobenef, $nrvinculom, $competenciaRetrocedida, $this->hcmParameters->getNrorgTrabalho(), $eventoMensal);
        }

        return $verificaLiquido;
    }

    private function modificaMinimizadorDiario($minimizador, $nrvinculom, $retrocedeQtsMeses, $nrbeneficiom){
        $dtinicio = clone DateUtil::getDataDeString($this->dtiniperiodobenefmod, DateUtil::FORMATO_BRASILEIRO, true)->sub(new \DateInterval('P'.$retrocedeQtsMeses.'M'));
        $dtfinal = clone DateUtil::getDataDeString($this->dtfimperiodobenefmod, DateUtil::FORMATO_BRASILEIRO, true)->sub(new \DateInterval('P'.$retrocedeQtsMeses.'M'));
        //Garantindo que a data final esteja no mesmo mes que a data inicio (ao retroceder algumas vezes se o mes tiver 31, e o mes de retrocesso tiver 30 dias: acaba acontecendo da data final cair no dia 01 do outro mes...)
        if($dtfinal->format('m/Y') != $dtinicio->format('m/Y')){
            $dtfinal = DateUtil::getUltimoDiaDoMes($dtinicio);
        }
        $dtmescompetenc = DateUtil::getDataDeString($dtinicio->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)->format('01/m/Y');

        $historicosBenef = $this->retornaHistoricoBenefEntrePeriodo($dtinicio, $dtfinal, $nrvinculom, $nrbeneficiom, $dtmescompetenc, $this->hcmParameters->getNrorgTrabalho());
        //Se não houve pagamento na epoca de analise do minimizador não havera desconto para o minimizador.
        if($historicosBenef == false || empty($historicosBenef)){
            $minimizador = 'N';
        }

        return $minimizador;
    }

    private function verificaQtMinimizador($nomeMinimizador){
        switch($nomeMinimizador){
            case 'IDCONSFALTABENEF':
                return 'QTMESESFALTABENEF';
                break;
            case 'IDCONSAFASTBENEF':
                return 'QTMESESAFASTBENEF';
                break;
            case 'IDCONSATESTBENEF':
                return 'QTMESESATESTBENEF';
                break;
        }
    }
    private function calcularDiasFerias($beneficio, $vinculo){
        $diasUteisFerias = 0;
        $feriados = 0;
        // ----- Verifica se o vínculo teve FÉRIAS no perído da competência
        $competencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        $ultimoDiaCompetencia =  DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true );
        
        $ferias = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->getPeriodoGozoFeriasVt(
            $vinculo->getNrvinculom(),
            DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true ), 
            DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true ),
            $this->hcmParameters->getNrorgTrabalho()
        );            
        
        if($ferias){
            $iniGozo = DateUtil::getDataDeString(substr($ferias['DTINIGOZOFERIAS'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true );
            $fimGozo = DateUtil::getDataDeString(substr($ferias['DTFIMGOZOFERIAS'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true );
            
            if($fimGozo > $ultimoDiaCompetencia){
                $fimGozo = $ultimoDiaCompetencia;
            }
            
            $dias = $fimGozo->diff($iniGozo, true);
            
            //retornaQtdeDiasTrabalhoPorVincNova Função nova para verificar os horarios que não deveriam gerar beneficio. Campo IDNGERABENEFDIARIO
            $diasUteisFerias = GpeEscalatrabm::retornaQtdeDiasTrabalhoPorVincNova( // dias totais do período menos dias de folga. Feriados são considerados como dias trabalhados
                $this->hcmParameters->getDtCompetencia(), 
                $iniGozo->format('d/m/Y'), 
                $dias->d, 
                $this->hcmParameters->getNrorgTrabalho(), 
                $vinculo->getNrvinculom(),
                1
            );
            
            $arFeriados = $this->calcularQtdeFeriados($iniGozo, $fimGozo, $vinculo);
            $feriados = count($arFeriados);
            
            $diasUteisFerias = ($diasUteisFerias - $feriados);
        }
        
        return $diasUteisFerias;
    }
    
    //Função que recebe a data de admissão, e rescisão, data inicio e fim e retorna a quantidade de dias trabalhados no mes
    public function calculaDiasUteisBeneficioMensal($dtAdmissao, $dtRescisao, $escalaTrab, $tpBeneficio, $minimizadores, $beneficio, $vinculo){
        
        $competencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        
        //01/10/2023
        $inicio = DateUtil::getDataDeString($competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
        //31/10/2023
        $fim = DateUtil::getDataDeString($competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
        
        // Converte as strings de data para objetos DateTime
        //22/08/2023
        $admissao = new DateTime($dtAdmissao);
        //NULA
        !is_null($dtRescisao) ? $rescisao = new DateTime($dtRescisao) : null;
    
        if($admissao >= $inicio && $admissao <= $fim){
            $inicio = $admissao;
        }
        
        if(!empty($rescisao) && ($rescisao <= $fim && $rescisao >= $inicio)){
            $fim = $rescisao;
        }
        
        // Calcula a diferença em dias
        $diasNormais = count(DateUtil::getDatasEntre($inicio, $fim));
        
        //Verificar se o valor de referencia especifico para pre fixar vale combustivel esta criado na org;
        $valorReferenciaPreFixaDias = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor(
            $this->hcmParameters->getNrorgTrabalho(),
            0,
            $this->hcmParameters->getDtCompetencia(),
            'PREFIXADIAS_VALECOMBUSTIVEL'
        );
        
        if(!empty($valorReferenciaPreFixaDias) && $valorReferenciaPreFixaDias == 1){
            //Se caso o tipo de jornada for 2 então fixar dias em 15, se caso 
            $this->fixaDias = $diasNormais = !empty($escalaTrab['NRTPJORNADA']) && $escalaTrab['NRTPJORNADA'] == 2 ? 15 : 21;
        }
        
        $feriados = 0;
        
        if(!empty($escalaTrab['NRTPJORNADA']) && $escalaTrab['NRTPJORNADA'] != 2){
            //todo: descontar feriados SOMENTE PARA NÃO ESCALA TPJORNADA 2
            $arFeriados = $this->calcularQtdeFeriados($inicio, $fim, $vinculo, 'N');
            $feriados = count($arFeriados);
        }
        
        $diasUteisFaltas = $this->calcularDiasFaltas($minimizadores, $vinculo);
        $diasUteisAtestados = $this->calcularDiasAtestados($minimizadores, $beneficio, $vinculo);
        $diasUteisAfastamentos = $this->calcularDiasAfastamentos($minimizadores, $beneficio, $vinculo);
        $diasUteisFerias = $this->calcularDiasFerias($beneficio, $vinculo);
        
        $diasNormais = $diasNormais - ($diasUteisFaltas + $diasUteisAtestados + $diasUteisAfastamentos + $diasUteisFerias + $feriados);
        
        $diasNormais = $diasNormais < 0 ? 0 : $diasNormais;
        
        return $diasNormais;
    }
    
    public function atualizaCalculoConsumer(){
        \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();
        // retorna registro do calculoconsumer da fila e seta a data hora de inicio
        $fpaCalculoConsumer = $this->calcConsumer->findOneBy(array('nrcalculoconsumer' => $this->nrcalculoconsumer, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()));

        $contador = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), $this->sequenciaCalculo, 1, 12, true);
        
        $fpaCalculoConsumer->setNrtotprocessvinc($contador);
        
        if($contador == $this->row['QNTTOTAL']){
            $fpaCalculoConsumer->setIdsituacao('CONCLUIDO');
            
            $query = $this->entityManager->getConnection()->prepare("
                DELETE 
                FROM NOVOCODIGO
                WHERE 
                    NRORG = :NRORG
                    AND CDCONTADOR = :CDCONTADOR"
            );
    
            $query->execute(
                [
                    "NRORG"      => $this->hcmParameters->getNrorgTrabalho(),
                    "CDCONTADOR" => $this->sequenciaCalculo,
                ]
            );
        }
        
        \Setforms2\Shared\Registry::getEntityManager()->persist($fpaCalculoConsumer);
        \Setforms2\Shared\Registry::getEntityManager()->flush();
        \Setforms2\Shared\Registry::getEntityManager()->commit();
    }
    
    public function obterMinimizadoresDiario($beneficio){
        $array = [];
        
        $array['IDCONSFALTABENEF']    = isset($beneficio['IDCONSFALTABENEF'])    && !empty($beneficio['IDCONSFALTABENEF'])    ? $beneficio['IDCONSFALTABENEF']    : 'N';
        $array['QTMESESFALTABENEF']   = isset($beneficio['QTMESESFALTABENEF'])   && !empty($beneficio['QTMESESFALTABENEF'])   ? $beneficio['QTMESESFALTABENEF']   : 0;
        $array['IDCONSATESTBENEF']    = isset($beneficio['IDCONSATESTBENEF'])    && !empty($beneficio['IDCONSATESTBENEF'])    ? $beneficio['IDCONSATESTBENEF']    : 'N';
        $array['QTMESESATESTBENEF']   = isset($beneficio['QTMESESATESTBENEF'])   && !empty($beneficio['QTMESESATESTBENEF'])   ? $beneficio['QTMESESATESTBENEF']   : 0;
        $array['IDCONSAFASTBENEF']    = isset($beneficio['IDCONSAFASTBENEF'])    && !empty($beneficio['IDCONSAFASTBENEF'])    ? $beneficio['IDCONSAFASTBENEF']    : 'N';
        $array['QTMESESAFASTBENEF']   = isset($beneficio['QTMESESAFASTBENEF'])   && !empty($beneficio['QTMESESAFASTBENEF'])   ? $beneficio['QTMESESAFASTBENEF']   : 0;
        $array['IDCONSFERIASBENEF']   = isset($beneficio['IDCONSFERIASBENEF'])   && !empty($beneficio['IDCONSFERIASBENEF'])   ? $beneficio['IDCONSFERIASBENEF']   : 'N';
        $array['QTMESESFERIASBENEF']  = isset($beneficio['QTMESESFERIASBENEF'])  && !empty($beneficio['QTMESESFERIASBENEF'])  ? $beneficio['QTMESESFERIASBENEF']  : 0;
        $array['IDCONSFERIADOBENEF']  = isset($beneficio['IDCONSFERIADOBENEF'])  && !empty($beneficio['IDCONSFERIADOBENEF'])  ? $beneficio['IDCONSFERIADOBENEF']  : 'N';
        $array['QTMESESFERIADOBENEF'] = isset($beneficio['QTMESESFERIADOBENEF']) && !empty($beneficio['QTMESESFERIADOBENEF']) ? $beneficio['QTMESESFERIADOBENEF'] : 0;
        $array['IDCONSDSRBENEF']      = isset($beneficio['IDCONSDSRBENEF'])      && !empty($beneficio['IDCONSDSRBENEF'])      ? $beneficio['IDCONSDSRBENEF']      : 'N';
        $array['QTMESESDSRBENEF']     = isset($beneficio['QTMESESDSRBENEF'])     && !empty($beneficio['QTMESESDSRBENEF'])     ? $beneficio['QTMESESDSRBENEF']     : 0;
        $array['IDPROPADMIS']         = isset($beneficio['IDPROPADMIS'])         && !empty($beneficio['IDPROPADMIS'])         ? $beneficio['IDPROPADMIS']         : 'N';
        $array['IDPROPRESC']          = isset($beneficio['IDPROPRESC'])          && !empty($beneficio['IDPROPRESC'])          ? $beneficio['IDPROPRESC']          : 'N';
        $array['IDPROPDTINSCR']       = isset($beneficio['IDPROPDTINSCR'])       && !empty($beneficio['IDPROPDTINSCR'])       ? $beneficio['IDPROPDTINSCR']       : 'N';
        $array['IDPROPDTCANC']        = isset($beneficio['IDPROPDTCANC'])        && !empty($beneficio['IDPROPDTCANC'])        ? $beneficio['IDPROPDTCANC']        : 'N';
        
        return $array;
    }
    
    private function definirPeriodoBeneficio($vinculo, $beneficiario, $dtinicio, $dtfinal, $dtpagamento, $minimizadores){

        $this->dtPagamentoBenef  = DateUtil::getDataDeString($dtpagamento, DateUtil::FORMATO_BRASILEIRO);

        $dtRescisaoVinc = substr($vinculo->getDtrescisaovinc(), 0, 10);
        $dtRescisaoVinc = $dtRescisaoVinc ? DateUtil::getDataDeString($dtRescisaoVinc, DateUtil::FORMATO_BRASILEIRO)->format('Y-m-d') : null;   

        $competenciaPosterior  = DateUtil::getDataDeString(substr($this->hcmParameters->getDtCompetencia(), 0, 10), DateUtil::FORMATO_BRASILEIRO);
        $competenciaPosterior  = $competenciaPosterior->add(new \DateInterval('P1M'));
        $competenciaPosterior  = $competenciaPosterior->format('Y-m-d');   

        $dtAdmissaoVinc = substr($vinculo->getDtadmissaovinc(), 0, 10);
        $dtAdmissaoVinc = $dtAdmissaoVinc ? DateUtil::getDataDeString($dtAdmissaoVinc, DateUtil::FORMATO_BRASILEIRO)->format('Y-m-d') : null;   

        $competenciaAtual  = DateUtil::getDataDeString(substr($this->hcmParameters->getDtCompetencia(), 0, 10), DateUtil::FORMATO_BRASILEIRO);
        $competenciaAtual  = $competenciaAtual->format('Y-m-d');   
        
        $dtinscrbeneficiario = substr($beneficiario->getDtinscbeneficiario(), 0, 10);
        $dtinscrbeneficiario = $dtinscrbeneficiario ? DateUtil::getDataDeString($dtinscrbeneficiario, DateUtil::FORMATO_BRASILEIRO)->format('Y-m-d') : null;   

        $dtcancbeneficiario  = null != $beneficiario->getDtcancbeneficiario() ? substr($beneficiario->getDtcancbeneficiario(), 0, 10) : null;
        $dtcancbeneficiario  = $dtcancbeneficiario ? DateUtil::getDataDeString($dtcancbeneficiario, DateUtil::FORMATO_BRASILEIRO)->format('Y-m-d') : null;   
        $diaCancBeneficiario = $dtcancbeneficiario ? (int)substr($dtcancbeneficiario, 8, 2) : null;

        /*
            - Caso as ambas opções 'Proporcionaliza na Admissão?' e 'Proporciona conforme data de inscrição ?' estiverem marcadas, 
                e data de admissão e inscrição estiverem na competênca corrente, será considerada a data mais avançada para proporcionalizar.
            - Caso apenas a opção 'Proporcionaliza na Admissão?' esteja marcada e estiver na competência vigente, 
                será considerada esta data para proporcionalizar.
            - Caso apenas a opção 'Proporciona conforme data de inscrição ?' esteja marcada e estiver na competência vigente,  
                será considerada esta data para proporcionalizar.
            - Caso nenhuma das opções estejam marcadas ou nem a data de inscrição nem a data de admissão estejam na competência,
                será considerada a data final informada no período de cálculo.
        */
        if((isset($minimizadores['IDPROPADMIS']) && $minimizadores['IDPROPADMIS'] == 'S') || (isset($minimizadores['IDPROPDTINSCR']) && $minimizadores['IDPROPDTINSCR'] == 'S')){
            $consideraAdmissao = $consideraInscri = null;
            if( $minimizadores['IDPROPADMIS'] == 'S' && $competenciaAtual < $dtAdmissaoVinc && $dtAdmissaoVinc < $competenciaPosterior )  $consideraAdmissao = true;
            if( $minimizadores['IDPROPDTINSCR'] == 'S' && $competenciaAtual < $dtinscrbeneficiario && $dtinscrbeneficiario < $competenciaPosterior ) $consideraInscri = true;
            
            if(isset($consideraAdmissao) && isset($consideraInscri)) {
                $maior = max($dtAdmissaoVinc, $dtinscrbeneficiario); 
                $this->dtiniperiodobenef = DateUtil::getDataDeString(substr($maior, 0, 10), 'Y-m-d', true);
            }
            else if(isset($consideraAdmissao)) $this->dtiniperiodobenef = DateUtil::getDataDeString(substr($vinculo->getDtadmissaovinc(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            else if(isset($consideraInscri)) $this->dtiniperiodobenef = DateUtil::getDataDeString(substr($beneficiario->getDtinscbeneficiario(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            else $this->dtiniperiodobenef = DateUtil::getDataDeString($dtinicio, DateUtil::FORMATO_BRASILEIRO, true);
        }else
            $this->dtiniperiodobenef = DateUtil::getDataDeString($dtinicio, DateUtil::FORMATO_BRASILEIRO, true);

        /*
            - Caso as ambas opções 'Proporcionaliza na Rescisão?' e 'Proporciona conforme data de cancelamento ?' estiverem marcadas, 
                e data de rescisão e cancelamento estiverem na competênca corrente, será considerada a data menos avançada para proporcionalizar.
            - Caso apenas a opção 'Proporcionaliza na Rescisão?' esteja marcada e estiver na competência vigente, 
                será considerada esta data para proporcionalizar.
            - Caso apenas a opção 'Proporciona conforme data de cancelamento ?' esteja marcada e estiver na competência vigente,  
                será considerada esta data para proporcionalizar.
            - Caso nenhuma das opções estejam marcadas ou nem a data de cancelamento nem a data de rescisão estejam na competência,
                será considerada a data inicial informada no período de cálculo.
        */

        if((isset($minimizadores['IDPROPRESC']) && $minimizadores['IDPROPRESC'] == 'S') || (isset($minimizadores['IDPROPDTCANC']) && $minimizadores['IDPROPDTCANC'] == 'S')){
            $consideraRescisao = $considerarCancel = null;
            if( isset($dtRescisaoVinc) && $minimizadores['IDPROPRESC'] == 'S' && $competenciaAtual < $dtRescisaoVinc && $dtRescisaoVinc < $competenciaPosterior )  $consideraRescisao = true;
            if( isset($dtcancbeneficiario) && (isset($minimizadores['IDPROPDTINSCR']) && $minimizadores['IDPROPDTINSCR'] == 'S') && $competenciaAtual < $dtcancbeneficiario && $dtcancbeneficiario < $competenciaPosterior ) $considerarCancel = true;
            
            if(isset($consideraRescisao) && isset($considerarCancel)){ 
                $menor = min($dtRescisaoVinc, $dtcancbeneficiario); 
                $this->dtfimperiodobenef = DateUtil::getDataDeString(substr($menor, 0, 10), 'Y-m-d', true);
            }    
            else if(isset($consideraRescisao)){ 
                $this->dtfimperiodobenef = DateUtil::getDataDeString(substr($vinculo->getDtrescisaovinc(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            } else if(isset($considerarCancel)){ 
                $this->dtfimperiodobenef = DateUtil::getDataDeString(substr($beneficiario->getDtcancbeneficiario(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            } else {
                $this->dtfimperiodobenef = DateUtil::getDataDeString($dtfinal, DateUtil::FORMATO_BRASILEIRO, true);
            }
        }else{
            $this->dtfimperiodobenef = DateUtil::getDataDeString($dtfinal, DateUtil::FORMATO_BRASILEIRO, true);
        }

    }
    
    private function getDsDescansoSemanal($nrvinculom){
        try {
            $this->nrtpjornada = null;
            $this->geraBenefFeriado = 'N';
            $dsDiasDescanso = self::SABDOM;
            
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $competencia = $this->hcmParameters->getDtCompetencia();
            $iniperiodo = clone $this->dtiniperiodobenef;
            $fimperiodo = clone $this->dtfimperiodobenef;

            $escalaTrabVinc = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->obterEscalaEntrePeriodo($iniperiodo, $fimperiodo, $this->hcmParameters->getDtCompetencia(), $nrvinculom, $this->hcmParameters->getNrorgTrabalho());
            
            // Caso haja turno e o turno tenha horários cadastrados, a contagem de dias deverá olhar apenas para o turno. Senão calculará da forma antiga.
            $this->nrturno = isset($escalaTrabVinc[0]) && isset($escalaTrabVinc[0]['NRTURNO']) && !empty($escalaTrabVinc[0]['NRTURNO']) ? $escalaTrabVinc[0]['NRTURNO'] : null; 
            if($this->nrturno) $this->nrHorarioTurno = GpeHorarioturno::getObjectByWhere("e.NRORG = :P_NRORG AND e.NRTURNO = :P_NRTURNO", ['P_NRORG' => $this->hcmParameters->getNrorgTrabalho(), "P_NRTURNO" => $this->nrturno]);
            
            if ((isset($escalaTrabVinc) && !empty($escalaTrabVinc)) && $escalaTrabVinc) {
                $escalaTrabalho = $this->entityManager->getRepository(Repositories::GPE_ESCALATRABH)->_retornaUltimoHistorico(
                    $escalaTrabVinc[0]['NRESCALATRABM'], 
                    $this->hcmParameters->getNrorgTrabalho(), 
                    $this->hcmParameters->getDtCompetencia()
                );

                // Verificar se período de apuração é definido na escala de trabalho do vínculo
                if($escalaTrabalho->getIdutapuratestbenef()  && ($escalaTrabalho->getIdutapuratestbenef() == 'S' || $escalaTrabalho->getIdutapuratestbenef() == 'Y')){ 
                    $this->diaInicialAupr = $escalaTrabalho->getNrdiainiapur();
                    $this->diaFinalAupr = $escalaTrabalho->getNrdiafinapur();
                }                 
                
                $this->nrtpjornada = isset($escalaTrabalho) && !empty($escalaTrabalho) ? $escalaTrabalho->getNrtpjornada() : null;
                
                $this->geraBenefFeriado = isset($escalaTrabalho) && $escalaTrabalho->getIdgerbenfer() ? $escalaTrabalho->getIdgerbenfer() : 'N'; // Parâmetro caso tenha valor 'S' não haverá subtração por feriado.
                
                $dsdescansosemanal = isset($escalaTrabalho) && !empty($escalaTrabalho) ? $escalaTrabalho->getDescansosemanal() : null;
                switch ($dsdescansosemanal) {
                    case('SABADO'): 
                        $dsDiasDescanso = self::SABADO;
                        break;
                    
                    case('SAB/DOM'): 
                        $dsDiasDescanso = self::SABDOM;
                        break;
                    
                    case('DOMINGO'): 
                        $dsDiasDescanso = self::DOMINGO;
                        break;
                    
                    case('SEGUNDA'): 
                        $dsDiasDescanso = self::SEGUNDA;
                        break;
                    
                    case('TERCA'): 
                        $dsDiasDescanso = self::TERCA;
                        break;
                    
                    case('QUARTA'): 
                        $dsDiasDescanso = self::QUARTA;
                        break;
                    
                    case('QUINTA'): 
                        $dsDiasDescanso = self::QUINTA;
                        break;
                    
                    case('SEXTA'): 
                        $dsDiasDescanso = self::SEXTA;
                        break;

                    case('6 X 1'): 
                        $dsDiasDescanso = self::DOMINGO;
                        break;

                    case('ESCALA'): 
                        $dsDiasDescanso = self::DOMINGO;
                        break;
                }
            }else {
                MsgCalculo::getInstance()->adicionaMensagem('Vínculo '.$nrvinculom.' não possui escala.', MsgCalculo::ALERTA);
                //throw new \Exception('Vínculo '.$vinculo->getNrvinculom().' - '.$vinculo->getNmvinculom().' não possui escala.');
            }
            
            return $dsDiasDescanso;
            
        }catch (\Exception $e){
            throw $e;
        }
    }
    
    private function calcularQtdeDiasNoPeriodo(&$diasDeTrabalhoNoPeriodo, &$diasDSR, &$diasFeriado, $vinculo, $dsDiasDescanso, $id_minimizadorFeriado, $qtdeMesesRetrocederFeriado, $id_minimizadorDSR, $qtdeMesesRetrocederDSR){
        $arFeriados = $arDsr = $arrayDiasFolgaVinc = [];
        $diasFeriado = 0;
        
        $dtinicio = $this->dtiniperiodobenef;
        $dtfinal = $this->dtfimperiodobenef;
        
        if(!($dtinicio instanceof \DateTime))
            $dtinicio = DateUtil::getDataDeString($this->dtiniperiodobenef, DateUtil::FORMATO_BRASILEIRO, true);
        
        if(!($dtfinal instanceof \DateTime))
            $dtfinal = DateUtil::getDataDeString($this->dtfimperiodobenef, DateUtil::FORMATO_BRASILEIRO, true);
        
        $dias = $dtfinal->diff($dtinicio, true);
        
        if(isset($id_minimizadorFeriado) && $id_minimizadorFeriado == 'S'){
            $dtPeriodoInicioCalc = clone $this->dtiniperiodobenef;
            $dtPeriodoInicioCalc->sub( new \DateInterval('P'.(string)$qtdeMesesRetrocederFeriado.'M') );

            $dtPeriodoFimCalc    = clone $this->dtfimperiodobenef;
            $dtPeriodoFimCalc->sub( new \DateInterval('P'.(string)$qtdeMesesRetrocederFeriado.'M') );
            
            if($dtPeriodoInicioCalc->format('m') == 2){
                while(($dtPeriodoFimCalc->format('m') > $dtPeriodoInicioCalc->format('m')) && $qtdeMesesRetrocederFeriado != 0){
                    $dtPeriodoFimCalc = $dtPeriodoFimCalc->sub(new \DateInterval('P1D'));
                }
            }
            
            $arFeriados = $this->calcularQtdeFeriados($dtPeriodoInicioCalc, $dtPeriodoFimCalc, $vinculo);
        }
        
        $diasFeriado = count($arFeriados);
        
        // if( isset($this->nrtpjornada) && ($this->nrtpjornada == 2 || $this->nrtpjornada == 3 || $this->nrtpjornada == 4 || $this->nrtpjornada == 7) ){
        
        //retornaQtdeDiasTrabalhoPorVincNova Função nova para verificar os horarios que não deveriam gerar beneficio. Campo IDNGERABENEFDIARIO
        $diasDeTrabalhoNoPeriodo = GpeEscalatrabm::retornaQtdeDiasTrabalhoPorVincNova( // dias totais do período menos dias de folga. Feriados são considerados como dias trabalhados
            $this->hcmParameters->getDtCompetencia(), 
            $this->dtiniperiodobenef->format('d/m/Y'), 
            $dias->d, 
            $this->hcmParameters->getNrorgTrabalho(), 
            $vinculo->getNrvinculom(),
            $this->nrReferencia
        );
        
        if( isset($this->nrturno) && $this->nrHorarioTurno && $diasDeTrabalhoNoPeriodo > 0){ // Deverá olhar para o turno apenas caso esteja na alteração de escala do vínculo e possua horários cadastrados
            $arrayDiasFolgaVinc = $this->calcularQtdeDiasFolgaVinc($this->dtiniperiodobenef, $dias, $vinculo->getNrvinculom()); 
            $diasFolga = count($arrayDiasFolgaVinc);
            $intersecaoFolgaFeriado = count(array_intersect($arrayDiasFolgaVinc, $arFeriados)); //quantidade de dias que coincidem Folga e Feriados
            $diasDeTrabalhoNoPeriodo = $diasDeTrabalhoNoPeriodo - ($diasFeriado - $intersecaoFolgaFeriado); // subtrair apenas feriados que não sejam em dias de folga
        }else{
            if(isset($id_minimizadorDSR) && $id_minimizadorDSR == 'S'){
                $dtPeriodoInicioCalc = clone $this->dtiniperiodobenef;
                $dtPeriodoInicioCalc->sub( new \DateInterval('P'.$qtdeMesesRetrocederDSR.'M') );
    
                $dtPeriodoFimCalc    = clone $this->dtfimperiodobenef;
                $dtPeriodoFimCalc->sub( new \DateInterval('P'.$qtdeMesesRetrocederDSR.'M') );
                
                if($dtPeriodoInicioCalc->format('m') == 2){
                    while(($dtPeriodoFimCalc->format('m') > $dtPeriodoInicioCalc->format('m')) && $qtdeMesesRetrocederDSR != 0){
                        $dtPeriodoFimCalc = $dtPeriodoFimCalc->sub(new \DateInterval('P1D'));
                    }
                }
                
                $arDsr = $this->calcularQtdeDiasDSR($dtPeriodoInicioCalc, $dtPeriodoFimCalc, $dsDiasDescanso);
            }
            
            $dias->d++;
            $intersecaoDsrFeriado  = count(array_intersect($arDsr, $arFeriados)); //quantidade de dias que coincidem DSR e Feriados
            $diasDSR     = count($arDsr);
            $diasDeTrabalhoNoPeriodo = $dias->d - $diasDSR - $diasFeriado + $intersecaoDsrFeriado;
        }
        
        //A RESPEITO DA ISSUE/TAREFA: 486965 - VALE COMBUSTÍVEL POR KM
        //AGILE - SE CASO O BENEFICIO FOR UM VALE COMBUSTIVEL, E TIVER COM O VALOR DE REFERENCIA CRIADO PARA DIAS PRÉ FIXADOS DE BENEFICIO, IRA FIXAR 15 DIAS PARA 12X36, 21 DIAS PARA DEMAIS ESCALAS.
        if(!empty($this->nrReferencia)){
            $diasPreFixados = $this->preFixaDiasCombustivelEscala();
            
            $diasDeTrabalhoNoPeriodo = !is_null($diasPreFixados) && !empty($diasPreFixados) ? $diasPreFixados : $diasDeTrabalhoNoPeriodo;
        }
    }
    
    //FUNÇÃO QUE IRA FIXAR 15 DIAS PARA 12X36, 21 DIAS PARA DEMAIS ESCALAS.
    private function preFixaDiasCombustivelEscala(){
        //Verificar se o valor de referencia especifico para pre fixar vale combustivel esta criado na org;
        $valorReferenciaPreFixaDias = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor(
            $this->hcmParameters->getNrorgTrabalho(),
            0,
            $this->hcmParameters->getDtCompetencia(),
            'PREFIXADIAS_VALECOMBUSTIVEL'
        );
        
        if(!empty($valorReferenciaPreFixaDias) && $valorReferenciaPreFixaDias == 1){
            //PRE FIXAR OS DIAS DE ACORDO COM A ESCALA.
            if(!empty($this->nrtpjornada) && $this->nrtpjornada == 2){
                return 15;
            } else {
                return 21;
            }
        }
        
        return null;
    }
    
    private function calcularQtdeDiasEmAviso(&$diasAviso, $vinculo, $dsDiasDescanso, $id_minimizadorDSR){
        
        $dtrescisao = null;

        if ($vinculo->getDtrescisaovinc()) 
            $dtrescisao = DateUtil::getDataDeString(substr($vinculo->getDtrescisaovinc(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);

        //if ($this->vinculo->getDtavisoprevio() and $dtrescisao->isBetween($iniperiodo, $fimperiodo)) {
        if( (isset($dtrescisao) && $dtrescisao)
            && $vinculo->getDtavisoprevio()
            && DateUtil::comparaDatas($this->dtiniperiodobenef, $dtrescisao) 
            && DateUtil::comparaDatas($dtrescisao, $this->dtfimperiodobenef)
        ){
            $arrayDsr = [];
            
            $diasAviso = DateUtil::getQtdeDiasEntre($this->dtiniperiodobenef, $dtrescisao) + 1;
            
            if(isset($id_minimizadorDSR) && $id_minimizadorDSR == 'S') $arrayDsr = $this->calcularQtdeDiasDSR($this->dtiniperiodobenef, $dtrescisao, $dsDiasDescanso);
            
            $diasAviso -=  count($arrayDsr);

        }

    }
    
    private function calcularQtdeDiasFerias(&$diasFerias, $vinculo, $qtdeMesesRetroceder, $id_minimizadorDSR, $id_minimizadorFeriado, $dsDiasDescanso){
  
        $dtPeriodoInicioCalc = clone $this->dtiniperiodobenef;
        $dtPeriodoInicioCalc->sub( new \DateInterval('P'.(string)$qtdeMesesRetroceder.'M') );
        
        $dtPeriodoFimCalc    = clone $this->dtfimperiodobenef;
        $dtPeriodoFimCalc->sub( new \DateInterval('P'.(string)$qtdeMesesRetroceder.'M') );
        
        if($dtPeriodoInicioCalc->format('m') == 2){
            while(($dtPeriodoFimCalc->format('m') > $dtPeriodoInicioCalc->format('m')) && $qtdeMesesRetroceder != 0){
                $dtPeriodoFimCalc = $dtPeriodoFimCalc->sub(new \DateInterval('P1D'));
            }
        }
        
        $ferias = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->getPeriodoGozoFeriasVt(
            $vinculo->getNrvinculom(), 
            $dtPeriodoInicioCalc,
            $dtPeriodoFimCalc, 
            $this->hcmParameters->getNrorgTrabalho()
        );

        if (isset($ferias) && $ferias) {
            $arDsr = $arFeriados = [];
            
            $inicio = DateUtil::retornaMaiorData( substr($ferias['DTINIGOZOFERIAS'], 0, 10), $dtPeriodoInicioCalc->format('d/m/Y') );
            $final = DateUtil::retornaMenorData( substr($ferias['DTFIMGOZOFERIAS'], 0, 10), $dtPeriodoFimCalc->format('d/m/Y') );

            $totalFerias = DateUtil::truncateData(DateUtil::getDataDeString($inicio))->diff(DateUtil::truncateData(DateUtil::getDataDeString($final)));

            if(isset($id_minimizadorFeriado) && $id_minimizadorFeriado == 'S') $arFeriados = $this->calcularQtdeFeriados(DateUtil::getDataDeString($inicio), DateUtil::getDataDeString($final), $vinculo);

            $feriados = count($arFeriados);

            if( isset($this->nrturno) && $this->nrHorarioTurno ){ // Deverá olhar para o turno apenas caso esteja na alteração de escala do vínculo e possua horários cadastrados
                $arrayDiasFolgaVinc = $this->calcularQtdeDiasFolgaVinc(DateUtil::getDataDeString($inicio), $totalFerias, $vinculo->getNrvinculom(), DateUtil::truncateData(DateUtil::getDataDeString($inicio))->format('01/m/Y')); 
                $diasFolga = count($arrayDiasFolgaVinc);
                $intersecaoFolgaFeriado = count(array_intersect($arrayDiasFolgaVinc, $arFeriados)); //quantidade de dias que coincidem Folga e Feriados
                $diasFerias = ($totalFerias->d+1) - $feriados - $diasFolga + $intersecaoFolgaFeriado;
            }else{
                if($id_minimizadorDSR == 'S') $arDsr = $this->calcularQtdeDiasDSR( DateUtil::getDataDeString($inicio), DateUtil::getDataDeString($final), $dsDiasDescanso);
                $intersecaoDsrFeriado = count(array_intersect($arDsr, $arFeriados)); //quantidade de dias que coincidem DSR e Feriados
                $diasFerias = ($totalFerias->d+1) - $feriados - count($arDsr) + $intersecaoDsrFeriado;
            }

        }
        
    } 
    
    private function calcularQtdeDiasFeriasRetroativa($beneficiario, &$diasFerias, $vinculo, $qtdeMesesRetroceder, $id_minimizadorDSR, $id_minimizadorFeriado, $dsDiasDescanso, $dtinicioRetroativa, $dtfinalRetroativa){
        $dtPeriodoInicioCalc = clone $dtinicioRetroativa;
        $dtPeriodoInicioCalc = $dtPeriodoInicioCalc->sub(new \DateInterval('P'.($qtdeMesesRetroceder).'M'));
        
        $dtPeriodoFimCalc    = clone $dtfinalRetroativa;
        $dtPeriodoFimCalc = $dtPeriodoFimCalc->sub(new \DateInterval('P'.($qtdeMesesRetroceder).'M'));
        
        if($dtPeriodoInicioCalc->format('m') == 2){
            while(($dtPeriodoFimCalc->format('m') > $dtPeriodoInicioCalc->format('m')) && $qtdeMesesRetroceder != 0){
                $dtPeriodoFimCalc = $dtPeriodoFimCalc->sub(new \DateInterval('P1D'));
            }
        }
        
        $ferias = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->getPeriodoGozoFeriasVt(
            $vinculo->getNrvinculom(), 
            $dtPeriodoInicioCalc, 
            $dtPeriodoFimCalc, 
            $this->hcmParameters->getNrorgTrabalho()
        );

        if (!empty($ferias) && $ferias) {
            $arDsr = $arFeriados = [];
            //$inicio = substr($ferias['DTINIGOZOFERIAS'], 0, 10);
            $inicio = DateUtil::retornaMaiorData( substr($ferias['DTINIGOZOFERIAS'], 0, 10), $dtPeriodoInicioCalc->format('d/m/Y') );
            //$final = substr($ferias['DTFIMGOZOFERIAS'], 0, 10);
            $final = DateUtil::retornaMenorData( substr($ferias['DTFIMGOZOFERIAS'], 0, 10), $dtPeriodoFimCalc->format('d/m/Y') );

            $totalFerias = DateUtil::getDataDeString($inicio)->diff(DateUtil::getDataDeString($final));

            if(!empty($id_minimizadorFeriado) && $id_minimizadorFeriado == 'S') $arFeriados = $this->calcularQtdeFeriados(DateUtil::truncateData(DateUtil::getDataDeString($inicio)), DateUtil::truncateData(DateUtil::getDataDeString($final)), $vinculo);
            $feriados = count($arFeriados);

            if( isset($this->nrturno) && $this->nrHorarioTurno ){ // Deverá olhar para o turno apenas caso esteja na alteração de escala do vínculo e possua horários cadastrados
                $arrayDiasFolgaVinc = $this->calcularQtdeDiasFolgaVinc(DateUtil::truncateData(DateUtil::getDataDeString($inicio)), $totalFerias, $vinculo->getNrvinculom()); 
                $diasFolga = count($arrayDiasFolgaVinc);
                $intersecaoFolgaFeriado = count(array_intersect($arrayDiasFolgaVinc, $arFeriados)); //quantidade de dias que coincidem Folga e Feriados
                $diasFerias = ($totalFerias->d+1) - $feriados - $diasFolga + $intersecaoFolgaFeriado;
            }else{
                if($id_minimizadorDSR == 'S') $arDsr = $this->calcularQtdeDiasDSR( DateUtil::truncateData(DateUtil::getDataDeString($inicio)), DateUtil::truncateData(DateUtil::getDataDeString($final)), $dsDiasDescanso);
                $intersecaoDsrFeriado = count(array_intersect($arDsr, $arFeriados)); //quantidade de dias que coincidem DSR e Feriados
                $diasFerias = ($totalFerias->d+1) - $feriados - count($arDsr) + $intersecaoDsrFeriado;
            }

        }
        
    }
    
    public function testeTeste(){
        return 1;
    }
    
    private function calcularQtdeDias_AtesOuAfastRetroativa($beneficiario, &$dias_AtesOuAfast, $id_tpCalc, $qtdeMesesRetroceder, $vinculo, $dsDiasDescanso, $id_minimizadorFeriado, $id_minimizadorDSR, $dtinicioRetroativa, $dtfinalRetroativa){
        $dtPeriodoInicioCalc = clone $dtinicioRetroativa;
        $dtPeriodoInicioCalc = $dtPeriodoInicioCalc->sub(new \DateInterval('P'.($qtdeMesesRetroceder).'M'));
        
        $dtPeriodoFimCalc    = clone $dtfinalRetroativa;
        $dtPeriodoFimCalc = $dtPeriodoFimCalc->sub(new \DateInterval('P'.($qtdeMesesRetroceder).'M'));
        
        if($dtPeriodoInicioCalc->format('m') == 2){
            while(($dtPeriodoFimCalc->format('m') > $dtPeriodoInicioCalc->format('m')) && $qtdeMesesRetroceder != 0){
                $dtPeriodoFimCalc = $dtPeriodoFimCalc->sub(new \DateInterval('P1D'));
            }
        }
        
        $dtmescompetenciaRetrocedida = DateUtil::truncateData(DateUtil::getPrimeiroDiaDoMes($dtPeriodoInicioCalc))->format('d/m/Y');
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        
        $listNrsitufuncm = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCH)->getAfastamentoIncideVale($dtmescompetenciaRetrocedida);
        $listNrsitufuncm = implode(', ', $listNrsitufuncm);
        
        $altesitfuncs = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncional_AtesOuAfast(
            $this->hcmParameters->getNrorgTrabalho(), 
            $vinculo->getNrvinculom(), 
            $listNrsitufuncm, 
            $dtPeriodoInicioCalc, 
            $dtPeriodoFimCalc,
            $id_tpCalc
        );
        
        if(!empty($altesitfuncs) && $altesitfuncs){
            foreach($altesitfuncs as $altesitfunc){
            
                $dtInic = $dtFim = null;
                
                if ( $altesitfunc->getDtinisitufunc() < $dtPeriodoInicioCalc ) {
                    $dtInic = $dtPeriodoInicioCalc;
                } else {
                    $dtInic = $altesitfunc->getDtinisitufunc();
                }
    
                //Caso a situação funcional não possual data final, considera o dia anterior da data inicial da próxima situação que gera vale
                if (!$altesitfunc->getDtfimsitufunc()) {
                    
                    $dtinisitufunc = $altesitfunc->getDtinisitufunc();
                    
                    $prox_altesitufunc = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaProximaSituFuncional(
                        $this->hcmParameters->getNrorgTrabalho(), 
                        $vinculo->getNrvinculom(),
                        $dtinisitufunc,
                        $dtPeriodoFimCalc,
                        $listNrsitufuncm
                    );
                    
                    if (!empty($prox_altesitufunc) && $prox_altesitufunc) {
                        //Dia anterior da próxima situação geradora de vale
                        $dtFim = end($prox_altesitufunc)->getDtinisitufunc()->sub(new \DateInterval('P1D'));
                    } else {
                        //Caso não tenha próxima situação que gera vale considera a data final do periodo de apuração
                        $dtFim = clone $dtPeriodoFimCalc;
                    }
                }else {
                    
                    if ($altesitfunc->getDtfimsitufunc() > $dtPeriodoFimCalc) {
                        $dtFim = $dtPeriodoFimCalc;
                    } else {
                        $dtFim = $altesitfunc->getDtfimsitufunc();
                    }
                }
        
                
                if ($dtFim >= $dtInic) {
                    
                    $arFeriados = $arDsr = $arrayDiasFolgaVinc = $arrayDiasAtestado = [];
    
                    $qnt = $dtFim->diff($dtInic, true);
                    
                    $diasTotaisAtestOuAfast = ($qnt->d + $qnt->m*30 + $qnt->y*365 + 1);
                    
                    if(!empty($id_minimizadorFeriado) && $id_minimizadorFeriado == 'S') $arFeriados = $this->calcularQtdeFeriados($dtInic, $dtFim, $vinculo);
                    $feriados = count($arFeriados);
    
                    if( isset($this->nrturno)  && $this->nrHorarioTurno){ // Deverá olhar para o turno apenas caso esteja na alteração de escala do vínculo e possua horários cadastrados
                        $arrayDiasFolgaVinc = $this->calcularQtdeDiasFolgaVinc($dtInic, $qnt, $vinculo->getNrvinculom()); 
                        $diasFolga = count($arrayDiasFolgaVinc);
                        $intersecaoFolgaFeriado = count(array_intersect($arrayDiasFolgaVinc, $arFeriados)); //quantidade de dias que coincidem Atestado/Afastamento e Feriados
                        $dias_AtesOuAfast += $diasTotaisAtestOuAfast - $feriados - $diasFolga + $intersecaoFolgaFeriado;
                    }else{
                        $intersecaoDsrFeriado = count(array_intersect($arDsr, $arFeriados)); // quantidade de dias que coincidem DSR e Feriados
                    
                        if($id_minimizadorDSR == 'S') $arDsr = $this->calcularQtdeDiasDSR($dtInic, $dtFim, $dsDiasDescanso);
                        $dsr = count($arDsr);
                        
                        $dias_AtesOuAfast += $diasTotaisAtestOuAfast - $feriados - $dsr + $intersecaoDsrFeriado;
                    }
                }
                
            }
            
        }
        
    }
    
    /*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
    -=                                       -=
    -= ABAIXO, REFERENTE AO CÁLCULO DIÁRIO   -=
    -=                                       -=
    -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

    public function calcularDias($vinculo, $beneficiario, $minimizadores, $dtinicio, $dtfinal, $dtpagamento, $qtdiasbenef){
        
        //Transformando o array de Beneficiario em um objeto.
        $beneficiario = Utils::getEntityManager()->getRepository(Repositories::VAB_BENEFICIARIO)->findOneBy(array('nrorg' => $this->nrorgTrab, 'nrbeneficiario' => $beneficiario['NRBENEFICIARIO']));
        
        $diasConsideracao = $diasAviso = $diasDeTrabalhoNoPeriodo = $diasFaltas = $diasAtestado = $diasAfastamento = $diasFerias = $diasDSR = $diasFeriado = 0;
        
        $this->definirPeriodoBeneficio($vinculo, $beneficiario, $dtinicio, $dtfinal, $dtpagamento, $minimizadores);
        
        $dsDiasDescanso = $this->getDsDescansoSemanal( $vinculo->getNrvinculom() );
        
        $this->calcularQtdeDiasNoPeriodo($diasDeTrabalhoNoPeriodo, $diasDSR, $diasFeriado, $vinculo, $dsDiasDescanso, $minimizadores['IDCONSFERIADOBENEF'], $minimizadores['QTMESESFERIADOBENEF'], $minimizadores['IDCONSDSRBENEF'], $minimizadores['QTMESESDSRBENEF'] );
        
        $this->calcularQtdeDiasEmAviso($diasAviso, $vinculo, $dsDiasDescanso, $minimizadores['IDCONSDSRBENEF']);
        
        $dtPeriodoInicioCalc = clone $this->dtiniperiodobenef;
        // Se considerar falta, calcula quantidade de faltas
        if((isset($minimizadores['IDCONSFALTABENEF']) && $minimizadores['IDCONSFALTABENEF'] === 'S') && ($beneficiario->getDtinscbeneficiario() <= $dtPeriodoInicioCalc->sub(new \DateInterval('P'.(string)$minimizadores['QTMESESFALTABENEF'].'M')) ) ) $diasFaltas = $this->calcularDiasFaltas( $minimizadores, $vinculo );

        $dtPeriodoInicioCalc = clone $this->dtiniperiodobenef;
        // Se considerar atestado, calcula quantidade de dias de atestado
        if((isset($minimizadores['IDCONSATESTBENEF']) && $minimizadores['IDCONSATESTBENEF'] === 'S') && ($beneficiario->getDtinscbeneficiario() <= $dtPeriodoInicioCalc->sub(new \DateInterval('P'.(string)$minimizadores['QTMESESATESTBENEF'].'M')) ) ) $this->calcularQtdeDias_AtesOuAfast( $diasAtestado, 'ATESTADO', $minimizadores['QTMESESATESTBENEF'], $vinculo, $dsDiasDescanso, $minimizadores['IDCONSFERIADOBENEF'], $minimizadores['IDCONSDSRBENEF']);
        
        $dtPeriodoInicioCalc = clone $this->dtiniperiodobenef;
        // Se considerar afastamento, calcula quantidade de dias afastado
        if((isset($minimizadores['IDCONSAFASTBENEF']) && $minimizadores['IDCONSAFASTBENEF'] === 'S') && ($beneficiario->getDtinscbeneficiario() <= $dtPeriodoInicioCalc->sub(new \DateInterval('P'.(string)$minimizadores['QTMESESAFASTBENEF'].'M')) ) ) $this->calcularQtdeDias_AtesOuAfast( $diasAfastamento, 'AFASTAMENTO', $minimizadores['QTMESESAFASTBENEF'], $vinculo, $dsDiasDescanso, $minimizadores['IDCONSFERIADOBENEF'], $minimizadores['IDCONSDSRBENEF']);

        $dtPeriodoInicioCalc = clone $this->dtiniperiodobenef;
        // Se considerar ferias, calcula quantidade de dias de férias
        if((isset($minimizadores['IDCONSFERIASBENEF']) && $minimizadores['IDCONSFERIASBENEF'] === 'S') && ($beneficiario->getDtinscbeneficiario() <= $dtPeriodoInicioCalc->sub(new \DateInterval('P'.(string)$minimizadores['QTMESESFERIASBENEF'].'M')) ) ) $this->calcularQtdeDiasFerias($diasFerias, $vinculo, $minimizadores['QTMESESFERIASBENEF'], $minimizadores['IDCONSDSRBENEF'], $minimizadores['IDCONSFERIADOBENEF'], $dsDiasDescanso);
        
        $this->feriasnaodescontadas = 0;
        $this->afastamentonaodescontados = 0;
        $this->valorAdescontar = 0;
        $creditoIndevidoFerias = 0;
        $creditoIndevidoAfasta = 0;
        
        if(!empty($this->idcredindev) && $this->idcredindev == 'S'){
            //Procura o registro na histbenefvinc correspondente aquela competencia retrocedida e inicio e fim de apuração do mês.
            //Se houver ele vai calcular retroativamente as ferias e os afastamentos
            $dtinicioRetroativa = DateUtil::truncateData(DateUtil::getDataDeString($this->dtiniciocredindev, DateUtil::FORMATO_BRASILEIRO));
            $dtfinalRetroativa = DateUtil::truncateData(DateUtil::getDataDeString($this->dtfinalcredindev, DateUtil::FORMATO_BRASILEIRO));
            
            $dtmescompetencRetroativa = DateUtil::truncateData(DateUtil::getPrimeiroDiaDoMes($dtinicioRetroativa));
    
            $beneficio = $this->calculoBeneficioRepository->getBeneficioPorVinculo($dtmescompetencRetroativa->format('d/m/Y'), $this->hcmParameters->getNrorgTrabalho(), $beneficiario->getNrbeneficiom());
            
            $nreventomensal = $beneficio['NREVENTOMENSA'];
            
            $histbenefvincRetroativos = $this->retornaHistoricoBenefEntrePeriodoSemCalcComplementar($dtinicioRetroativa, $dtfinalRetroativa, $vinculo->getNrvinculom(), $beneficiario->getNrbeneficiom(), $dtmescompetencRetroativa, $this->hcmParameters->getNrorgTrabalho());
            
            if(!empty($histbenefvincRetroativos) && $histbenefvincRetroativos){
                foreach($histbenefvincRetroativos AS $histbenefvincRetroativo){
                    if($histbenefvincRetroativo->getNreventom() == $nreventomensal){
                        //CALCULA QUANTIDADE DE FERIAS RETROATIVA - A PEDIDO DA AGILE ORG1410 SERÁ RECALCULADO AS FERIAS E VERIFICADO SE HOUVE PAGAMENTO INDEVIDO
                        //RETROCEDE 2 MESES PARA VERIFICAR AS FERIAS 
                        $diasFeriasRetrocedida = 0;
                        if(isset($minimizadores['IDCONSFERIASBENEF']) && $minimizadores['IDCONSFERIASBENEF'] === 'S') $this->calcularQtdeDiasFeriasRetroativa($beneficiario, $diasFeriasRetrocedida, $vinculo, $minimizadores['QTMESESFERIASBENEF'], $minimizadores['IDCONSDSRBENEF'], $minimizadores['IDCONSFERIADOBENEF'], $dsDiasDescanso, $dtinicioRetroativa, $dtfinalRetroativa);
                        
                        //CALCULA QUANTIDADE DE AFASTAMENTOS RETROATIVA - A PEDIDO DA AGILE ORG1410 SERÁ RECALCULADO OS AFASTAMENTOS E VERIFICADO SE HOUVE PAGAMENTO INDEVIDO
                        //RETROCEDE 2 MESES PARA VERIFICAR OS AFASTAMENTOS 
                        $diasAfastRetrocedida = 0;
                        if(isset($minimizadores['IDCONSAFASTBENEF']) && $minimizadores['IDCONSAFASTBENEF'] === 'S') $this->calcularQtdeDias_AtesOuAfastRetroativa($beneficiario, $diasAfastRetrocedida, 'AFASTAMENTO', $minimizadores['QTMESESAFASTBENEF'], $vinculo, $dsDiasDescanso, $minimizadores['IDCONSFERIADOBENEF'], $minimizadores['IDCONSDSRBENEF'], $dtinicioRetroativa, $dtfinalRetroativa);
        
                        if($histbenefvincRetroativo->getQtdiasferias() < $diasFeriasRetrocedida){
                            $this->feriasnaodescontadas = $diasFeriasRetrocedida - $histbenefvincRetroativo->getQtdiasferias();
                            $creditoIndevidoFerias += $histbenefvincRetroativo->getVrbeneficio() * $this->feriasnaodescontadas;
                            MsgCalculo::getInstance()->adicionaMensagem('Foi descontado para o vínculo: '.$vinculo->getNmvinculom().'. Férias que foram gozadas na competência: '.$dtmescompetencRetroativa->format('d/m/Y').'. Dias descontados: '.$this->feriasnaodescontadas.'. Total Desconto: R$'.$creditoIndevidoFerias, MsgCalculo::ALERTA);
                        }
                        
                        if($histbenefvincRetroativo->getQtdiasafasta() < $diasAfastRetrocedida){
                            $this->afastamentonaodescontados = $diasAfastRetrocedida - $histbenefvincRetroativo->getQtdiasafasta();
                            $creditoIndevidoAfasta += $histbenefvincRetroativo->getVrbeneficio() * $this->afastamentonaodescontados;
                            MsgCalculo::getInstance()->adicionaMensagem('Foi descontado para o vínculo: '.$vinculo->getNmvinculom().'. Afastamento na competência: '.$dtmescompetencRetroativa->format('d/m/Y').'. Dias descontados: '.$this->afastamentonaodescontados.'. Total Desconto: R$'.$creditoIndevidoAfasta, MsgCalculo::ALERTA);
                        }
                    }
                }
                $this->valorAdescontar = $creditoIndevidoFerias + $creditoIndevidoAfasta;
            }
        }
        
        if(!empty($qtdiasbenef) && $qtdiasbenef > 0){
            $diasDeTrabalhoNoPeriodo = $qtdiasbenef;
        }
        
        if(!empty($this->idcalccomplementar) && $this->idcalccomplementar == 'S'){
            $diasConsideracao = $qtdiasbenef;
        }elseif( $diasAviso > 0 ){
            $diasConsideracao = $diasAviso - $diasFaltas - $diasAtestado - $diasAfastamento - $diasFerias;
        } else{
            $diasConsideracao = $diasDeTrabalhoNoPeriodo - $diasFaltas - $diasAtestado - $diasAfastamento - $diasFerias;
        }
        
        return $this->montarArrayDiasCalculados($diasConsideracao, $diasAviso, $diasDeTrabalhoNoPeriodo, $diasFaltas, $diasAtestado, $diasAfastamento, $diasFerias, $diasDSR, $diasFeriado, empty($diasFeriasRetrocedida) ? 0 : $diasFeriasRetrocedida, empty($diasAfastRetrocedida) ? 0 : $diasAfastRetrocedida);
    }
    
    private function montarArrayDiasCalculados($diasConsideracao, $diasAviso, $diasDeTrabalhoNoPeriodo, $diasFaltas, $diasAtestado, $diasAfastamento, $diasFerias, $diasDSR, $diasFeriado, $diasFeriasRetrocedida, $diasAfastamentoRetrocedida){

            $diasCalculados = [];

            $diasCalculados['DIASCONSIDERACAO']        = $diasConsideracao;
            $diasCalculados['DIASAVISO']               = $diasAviso;
            $diasCalculados['DIASDETRABALHONOPERIODO'] = $diasDeTrabalhoNoPeriodo;
            $diasCalculados['DIASFALTAS']              = $diasFaltas;
            $diasCalculados['DIASATESTADO']            = $diasAtestado;
            $diasCalculados['DIASAFASTAMENTO']         = $diasAfastamento;
            $diasCalculados['DIASFERIAS']              = $diasFerias;
            $diasCalculados['DIASDSR']                 = $diasDSR;
            $diasCalculados['DIASFERIADO']             = $diasFeriado;
            $diasCalculados['DIASFERIASNAODESCONTADO'] = $diasFeriasRetrocedida;
            $diasCalculados['DIASAFASTNAODESCONTADO']  = $diasAfastamentoRetrocedida;

            return $diasCalculados;
    }
    
    private function retornaHistoricoBenefEntrePeriodoSemCalcComplementar($dtinicio, $dtfinal, $vinculo, $nrbeneficiom, $dtmescompetenc, $nrorg){
        try{
            $param = array(
                'P_NRORG'          => $nrorg,
                'P_NRBENEFICOM'     => $nrbeneficiom,
                'P_DTMESCOMPETENC'  => $dtmescompetenc,
                'P_DTINIAPURBENEFFORM' => $dtinicio,
                'P_DTFIMAPURBENEFFORM' => $dtfinal,
                'P_NRVINCULOM'      => $vinculo
            );
            
            $historicos = VabHistbenefvinc::getObjectsByWhere(
                "NRORG = :P_NRORG AND
                DTMESCOMPETENC = :P_DTMESCOMPETENC AND 
                DTINIAPURBENEF <= :P_DTFIMAPURBENEFFORM AND 
                DTFINAPURBENEF >= :P_DTINIAPURBENEFFORM AND
                (IDCALCCOMPLEMENTAR = 'N' OR IDCALCCOMPLEMENTAR IS NULL) AND
                (NRBENEFICIOM = :P_NRBENEFICOM) AND
                NRVINCULOM = :P_NRVINCULOM", $param
            );
            
            if($historicos){
                return $historicos;
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function retornaHistoricoBenefEntrePeriodo($dtinicio, $dtfinal, $vinculo, $nrbeneficiom, $dtmescompetenc, $nrorg){
        try{
            $param = array(
                'P_NRORG'          => $nrorg,
                'P_NRBENEFICOM'     => $nrbeneficiom,
                'P_DTMESCOMPETENC'  => $dtmescompetenc,
                'P_DTINIAPURBENEFFORM' => $dtinicio,
                'P_DTFIMAPURBENEFFORM' => $dtfinal,
                'P_NRVINCULOM'      => $vinculo
            );
            
            $historicos = VabHistbenefvinc::getObjectsByWhere(
                "NRORG = :P_NRORG AND
                DTMESCOMPETENC = :P_DTMESCOMPETENC AND 
                DTINIAPURBENEF <= :P_DTFIMAPURBENEFFORM AND 
                DTFINAPURBENEF >= :P_DTINIAPURBENEFFORM AND
                (NRBENEFICIOM = :P_NRBENEFICOM) AND
                NRVINCULOM = :P_NRVINCULOM", $param
            );
            
            if($historicos){
                return $historicos;
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function retornaHistoricoBenefPagto($dtpgtobenef, $vinculo, $nrbeneficiom, $dtmescompetenc, $nrorg){
        try{
            $param = array(
                'P_NRORG'          => $nrorg,
                'P_NRBENEFICOM'     => $nrbeneficiom,
                'P_DTMESCOMPETENC'  => $dtmescompetenc,
                'P_DTPGTOBENEF'     => "$dtpgtobenef",
                'P_NRVINCULOM'      => $vinculo
            );
            
            $historicos = VabHistbenefvinc::getObjectsByWhere(
                "NRORG = :P_NRORG AND
                DTMESCOMPETENC = :P_DTMESCOMPETENC AND 
                DTPGTOBENEF = :P_DTPGTOBENEF AND
                (NRBENEFICIOM = :P_NRBENEFICOM) AND
                NRVINCULOM = :P_NRVINCULOM", $param
            );

            if($historicos){
                return $historicos;
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function persistirCalculo_Mensal(&$foiCalculado, $nrcalculobenef, $nrvinculom, $proporcao = null, $eventos, $qtdBeneficiariosPerEvento, $dtpagamento = null, $nrlotevinc = null, $eventoEparcelas = []){
        $dtpagamento = $dtpagamento ? $dtpagamento : $this->hcmParameters->getDtCompetencia();
        foreach($eventos as $nrEvento => $valorEvento){
            if($valorEvento >= 0){
                $qtdParc = null;
                $row = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy( 
                    [ 
                     'nrcalculofolha' => $nrcalculobenef,
                     'nrvinculom'     => $nrvinculom,
                     'nreventom'      => $nrEvento,
                     'nrorg'          => $this->hcmParameters->getNrorgTrabalho(),
                     'dtocorrencia'   => Dateutil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true),
                    ]
                );

                if($row && $row->getNrlotelancevento()){
                    MsgCalculo::getInstance()->adicionaMensagem('Evento '.$nrEvento.' para o vínculo '.$nrvinculom.' já exportado para folha. ', MsgCalculo::ALERTA);

                }else{
                    $foiCalculado = true;

                    if($row){
                        $this->entityManager->remove($row);
                        $this->entityManager->flush();
                    }
                    
                    $eventoh = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
                        $nrEvento,
                        $this->hcmParameters->getNrorgTrabalho(), 
                        $this->hcmParameters->getNrorgPadrao(), 
                        $this->hcmParameters->getDtCompetencia() 
                    );

                    //Checa se o evento tem valor de parcela. Se tiver faz a divisão do valor pela parcela e seta a qtde de parcela.
                    if($eventoEparcelas[$nrEvento]) {
                        $valorEvento = $valorEvento / $eventoEparcelas[$nrEvento]; 
                        $qtdParc = $eventoEparcelas[$nrEvento];
                    }
                    
                    $valorEvento = Valor::arredonda(
                        $valorEvento, 
                        $eventoh->historico->getIdtipoarredonda(), 
                        $eventoh->historico->getNrdigarredonda(),
                        $eventoh->historico->getVrlimitearred()
                    );

                    try {
                        $referencia = $qtdBeneficiariosPerEvento[$nrEvento]['qtde'];
                        
                        if(!is_null($qtdBeneficiariosPerEvento[$nrEvento]['quantidadeBenef'])){
                            $referencia = $qtdBeneficiariosPerEvento[$nrEvento]['qtde'] * $qtdBeneficiariosPerEvento[$nrEvento]['quantidadeBenef'];
                        }

                        $event = new FpaItecalcfolha();
                        $event->setNritemcalcfolha(NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCFOLHA', 1, 22, true));
                        $event->setNrcalculofolha($nrcalculobenef);
                        $event->setNrvinculom($nrvinculom);
                        $event->setNreventom($nrEvento);
                        $event->setVreventofolha($valorEvento);
                        $event->setVrreffolha($referencia);
                        $event->setNrlotevinc($nrlotevinc);
                        $event->setDtocorrencia(Dateutil::getDataDeString($dtpagamento, DateUtil::FORMATO_BRASILEIRO, true));
                        $event->setNrorg($this->hcmParameters->getNrorgTrabalho());
                        $event->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                        $event->setDtinclusao(DateUtil::getDataAtual());
                        $event->setCdoperinclusao($this->hcmParameters->getCdoperador());
                        $event->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $event->setDtultatu(DateUtil::getDataAtual());
                        $event->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $event->setQtocorproxperi($qtdParc);
                        
                        $this->entityManager->persist($event);
                        $this->entityManager->flush();
                    } catch (\Exception $e) {
                        throw $e;    
                    }

                }
            }
            
        }
    }  
    
    private function validaJornadaReduzida($beneficio, $nrvinculom, &$vlrlinearbeneficio){
        try {
            $qtdeHorasJrndReduz = isset($beneficio['QTMAXHSREDUZIDA']) ? $beneficio['QTMAXHSREDUZIDA'] : null;
            $vlrJornadaReduz = isset($beneficio['VRHSREDUZIDA']) ? $beneficio['VRHSREDUZIDA'] : null;
            $hrsTrabalhadasMes = null;
            
            $objDtCompetencia = DateUtil::getDataDeString( substr($this->competencia, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            $objUltDiaCompetencia = DateUtil::getDataDeString( substr($this->UltDiaCompetencia, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            $escalaTrabVinc = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->obterEscalaEntrePeriodo($objDtCompetencia, $objUltDiaCompetencia, $this->competencia, $nrvinculom, $this->nrorgTrab);
            
            if (isset($escalaTrabVinc) && !empty($escalaTrabVinc)) {
                $escalaTrabalho = $this->entityManager->getRepository(Repositories::GPE_ESCALATRABH)->_retornaUltimoHistorico(
                    $escalaTrabVinc[0]['NRESCALATRABM'], 
                    $this->nrorgTrab, 
                    $this->competencia
                );
            }
            
            if(isset($escalaTrabalho) && $escalaTrabalho->getQthrescalatrabh() > 0) $hrsTrabalhadasMes = $escalaTrabalho->getQthrescalatrabh() * 30;
    
            if(isset($hrsTrabalhadasMes) && isset($qtdeHorasJrndReduz) && isset($vlrJornadaReduz) && $hrsTrabalhadasMes <= $qtdeHorasJrndReduz) 
                $vlrlinearbeneficio = $vlrJornadaReduz;
        } 
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function _quantidadeMesAfastados($dt_inicio, $dt_fim, $vinculo){
        try {
            $alteracao_situacao_funcional = new GpealtesitufuncEntity($this->entityManager, $this->hcmParameters);
            $situacaoFuncional = new GpesitufuncmEntity($this->entityManager, $this->hcmParameters);
            $alteracao_situacao_funcional->retornaAlteracaoSituacaoFuncional(
               $vinculo->getNrorg(), $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc()
            );
            
            $mes_ini = $dt_inicio->format('m');
            $mes_fim = $dt_fim->format('m');
            $dt_aux_inicio = $dt_inicio;
            $mes_afastado = 0;
            
            for ($i = $dt_inicio; $i <= $dt_fim; $i = DateUtil::adicionaIntervalo($i, 1, DateUtil::MES)) {
                
                if ($i == $dt_inicio) {
                    $dt_aux_inicio = $dt_inicio;
                    $dt_aux_fim = DateUtil::adicionaIntervalo($dt_inicio, 1, DateUtil::MES);
                    if($dt_aux_fim > $dt_fim){                          
                        $dt_aux_fim = $dt_fim;      
                    }
                } elseif ($i == $dt_fim) {
                    $dt_aux_inicio = DateUtil::setDay($dt_fim, 1);
                    $dt_aux_fim = $dt_fim;
                }else {
                    $dt_aux_inicio = $i;
                    $dt_aux_fim = DateUtil::adicionaIntervalo($dt_aux_inicio, 1, DateUtil::MES);
                }
                
                $afastamento = $alteracao_situacao_funcional->retornaDiasAfastamento(
                    $situacaoFuncional, $dt_aux_inicio, $dt_aux_fim, false
                );
                $dias_afastados = 0;
                for ($x=0; $x < count($afastamento); $x++) {
                    $sit_func = $situacaoFuncional->retornaSituacaoFuncional(
                        0, $this->nrorgTrab, $afastamento[$x]
                    );
                    if ($sit_func == true) {
                        if (($sit_func->getNrsitufuncm() === 12) or 
                            ($sit_func->getNrsitufuncm() === 15) or 
                            ($sit_func->getNrsitufuncm() === 16) or 
                            ($sit_func->getNrsitufuncm() === 100042)) {
                            $dias_afastados++;
                        }
                    }
                }
            }
            return $dias_afastados;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getMultiplicadorMensalidade($beneficio, $vinculo, $histVinc, $nreventomensa){
        try {
            
            $multiplicar = true;
            $vlrMultiplicador = isset($beneficio['VRMULTIPLICADOR']) && !empty($beneficio['VRMULTIPLICADOR']) ? floatval($beneficio['VRMULTIPLICADOR']) : 1;
    
            // ===== ID: "Vínculos Empregatícios que não considerarão o multiplicador:"
            // Verifica se o tipo de vínculo do vínculo é o tipo de vínculo do benefício CCT.
            // Se for, o multiplicador será 1.
            
            if(isset($beneficio['DSVINCULOEMPREG']) && !empty($beneficio['DSVINCULOEMPREG'])){
                $tpvinculos = explode(", ", $beneficio['DSVINCULOEMPREG']);
                foreach($tpvinculos as $nrtipovinc){
                    if(intval($nrtipovinc) == $vinculo->getNrvinculoempreg()) $multiplicar = false;
                }
            }
            
            // ===== ID: "Não considerarão o multiplicador por possuírem os seguintes eventos:" e "Retroceder quantos meses para apurar os eventos (frente ao período)?"
            // Verifica se na folha da competência informada há os eventos informados pelo ID acima
            // Caso haja, o multiplicador será 1.
            if(isset($beneficio['DSINCIDEVENTO']) && !empty($beneficio['DSINCIDEVENTO'])){
                $dadosRetornoValidEventos = $this->retornaEventosFolhaMensal($vinculo, $beneficio['DSINCIDEVENTO'], $beneficio['QTMESRETROCMULT'], true);
            }
            
            if(isset($dadosRetornoValidEventos) && $dadosRetornoValidEventos['consideraMultiplicador'] == false){
               $multiplicar = false;
            }else{
                $escalaTrabalho = $this->entityManager->getRepository(Repositories::GPE_ESCALATRABH)->_retornaUltimoHistorico(
                    $this->histVinc->getNrescalatrabm(), 
                    $this->nrorgTrab, 
                    $this->competencia
                );
                
                $dataInicio = DateUtil::getDataDeString($this->competencia);
                $dataFim = DateUtil::getUltimoDiaDoMes($dataInicio);
                
                // Verificar se período de apuração é definido na escala de trabalho do vínculo
                if((isset($escalaTrabalho) )&& $escalaTrabalho->getIdutapuratestbenef()  && ($escalaTrabalho->getIdutapuratestbenef() == 'S' || $escalaTrabalho->getIdutapuratestbenef() == 'Y')){ 
                    $diaInicialAupr = $escalaTrabalho->getNrdiainiapur();
                    $diaFinalAupr = $escalaTrabalho->getNrdiafinapur();
                    
                    $mesAnterior = DateUtil::subtraiIntervalo($dataInicio, 1, DateUtil::MES);
                    
                    $dataInicio = DateUtil::getDataDeString($diaInicialAupr.'/'.$mesAnterior->format('m/Y'));
                    $dataFim = DateUtil::getDataDeString($diaFinalAupr.'/'.$dataFim->format('m/Y'));
                
                }  
                
                $dias = $this->_quantidadeMesAfastados($dataInicio, $dataFim, $vinculo);
                
                $multiplicar = $dias > 0 ? false : $multiplicar;
            }

            // ===== ID: "Não considerar multiplicador para vínculos em período de experiência:"
            // Caso o id esteja marcado,  será verificado se data atual é menor ou igual à data de fim de experiência
            if(isset($beneficio['IDNAOGERABENPEREXP']) && !empty($beneficio['IDNAOGERABENPEREXP']) && $beneficio['IDNAOGERABENPEREXP'] == 'S'){
                $dtAtual = $dtFimContratExpe = null;
                $diasProrr = 0;
    
                $dtAtual = DateUtil::getDataDeString( substr(DateUtil::getDataAtual(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
                
                if($histVinc->getDtfimcontrdetermin() && !empty($histVinc->getDtfimcontrdetermin())) $dtFimContratExpe = DateUtil::getDataDeString( substr($histVinc->getDtfimcontrdetermin(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
                
                if( $dtFimContratExpe && $dtAtual->format('Y/m/d') <= $dtFimContratExpe->format('Y/m/d')) $multiplicar = false;
    
            }
            
            // ===== ID: "Não considerar multiplicador para vínculos rescindidos:"
            // Caso o id esteja marcado e haja rescisão do vínculo, o multiplicador será 1
            if(isset($beneficio['IDNAOGERABENRESC']) && !empty($beneficio['IDNAOGERABENRESC']) && $beneficio['IDNAOGERABENRESC'] == 'S'){
                if($vinculo->getDtrescisaovinc() && $vinculo->getDtrescisaovinc()->format('d/m/Y') <= $this->UltDiaCompetencia){ 
                    $multiplicar = false;
                }
            }
            
            // Caso não seja para multiplicar devido às dcondições acima o multiplicador deve ter valor default (1).
            if($multiplicar === false) $vlrMultiplicador = 1;
            if($multiplicar === 0) $vlrMultiplicador = 0;
            
            return $vlrMultiplicador;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function calcularBeneficioMensalDiario($nrcalculobenef, $subsidio100PorAfastamentoOuDemissao, $beneficio, &$eventos, $beneficiario, $vinculo, $nrvinculom, &$qtdBeneficiariosPerEvento, $proporcao, $tpperiodo, $isLast, $arrayDiasCalculados = null, &$foiCalculado = false, $qtdiasbenef = null, &$eventoEparcelas = []) {
        try {
            // Transforma as datas para serem comparáveis entre si
            $dtcompetencia       = substr($this->competencia, 0, 10);
            $ultimoDiaComp       = substr($this->competencia, 0, 10);
            $dtinivigencia       = isset($beneficio['DTINICIOVIGENCIA'])        ? substr($beneficio['DTINICIOVIGENCIA'], 0, 10) : null;
            $dtfimvigencia       = isset($beneficio['DTFINALVIGENCIA'])         ? substr($beneficio['DTFINALVIGENCIA'], 0, 10)  : null;
            $dtcancbeneficiario  = isset($beneficiario['DTCANCBENEFICIARIO'])   ? substr($beneficiario['DTCANCBENEFICIARIO'], 0, 10) : null;
            $dtinscrbeneficiario = isset($beneficiario['DTINSCBENEFICIARIO'])   ? substr($beneficiario['DTINSCBENEFICIARIO'], 0, 10) : null;
            $dtcompetencia       = !empty($dtcompetencia)                       ? \DateTime::createFromFormat(DateUtil::FORMATO_BRASILEIRO, $dtcompetencia)->format('Y-m-d')       : null;   
            $ultimoDiaComp       = !empty($ultimoDiaComp)                       ? \DateTime::createFromFormat(DateUtil::FORMATO_BRASILEIRO, $ultimoDiaComp)->format('Y-m-t')       : null;   
            $dtinivigencia       = !empty($dtinivigencia)                       ? \DateTime::createFromFormat(DateUtil::FORMATO_BRASILEIRO, $dtinivigencia)->format('Y-m-d')       : null;   
            $dtfimvigencia       = !empty($dtfimvigencia)                       ? \DateTime::createFromFormat(DateUtil::FORMATO_BRASILEIRO, $dtfimvigencia)->format('Y-m-d')       : null;  
            $dtcancbeneficiario  = !empty($dtcancbeneficiario)                  ? \DateTime::createFromFormat(DateUtil::FORMATO_BRASILEIRO, $dtcancbeneficiario)->format('Y-m-d')  : null;  
            $dtinscrbeneficiario = !empty($dtinscrbeneficiario)                 ? \DateTime::createFromFormat(DateUtil::FORMATO_BRASILEIRO, $dtinscrbeneficiario)->format('Y-m-d') : null;
            
            $beneficiario = Utils::getEntityManager()->getRepository(Repositories::VAB_BENEFICIARIO)->findOneBy(array('nrorg' => $this->nrorgTrab, 'nrbeneficiario' => $beneficiario['NRBENEFICIARIO']));
            
            if( (isset($beneficio) && $beneficio)
                && strtotime($dtinivigencia) <= strtotime($dtcompetencia)
                && ( !isset($dtfimvigencia) || strtotime($dtcompetencia) <= strtotime($dtfimvigencia) )
                && ( !isset($dtinscrbeneficiario) || strtotime($ultimoDiaComp) >= strtotime($dtinscrbeneficiario) ) 
                && ( !isset($dtcancbeneficiario) || strtotime($dtcompetencia) < strtotime($dtcancbeneficiario) )     
              ){
                $vincBenefiarioEvento         = []; // Array para gravar em VAB_APURABENEFICIO a relação Vínculo/Beneficiário/TipoEvento/Evento
                $vincBenefiarioEvento_mens    = []; // Array para gravar em VAB_APURABENEFICIO a relação Vínculo/Beneficiário/TipoEvento/Evento
                $vincBenefiarioEvento_vinc    = []; // Array para gravar em VAB_APURABENEFICIO a relação Vínculo/Beneficiário/TipoEvento/Evento
                $vincBenefiarioEvento_empresa = []; // Array para gravar em VAB_APURABENEFICIO a relação Vínculo/Beneficiário/TipoEvento/Evento
    
                $tpBeneficiario     = $beneficiario->getIdtpbeneficiario() ? $beneficiario->getIdtpbeneficiario() : null;
                $eventoBeneficiario = intval($beneficiario->getNreventom());
                
                $vlrEventoMensal    = null; // Variaveis a serem usadas para cálculos de 
                $vlrEventoVinc      = 0;    // Evento Vínculo e Evento Empresa, respectivamente
                
                $vlrEventoEmpr      = 0;    // Esta em específico é usada apenas para o cálculo de subsidiado diferente de 100%
                
                $efetuarCalculo     = true; // Variável booleana, para verificar se é para calcular, com base nas verificações por nascimento e sexo
                
                $nreventomensa = $vlrlinearbeneficio = $eventVinc =       $tpParticipacao =   $eventEmpresa =
                $nrtabelafolha = $nrFaixaSalarial =    $idadeBeneficiario = $idadeMaxSubsidio = $tpRelac =
                $vlrPiso =       $vlrTeto =        $isDeficiente =      $sexoBeneficiario = $sexoSubsidio =
                $nreventotaxa =  $vreventotaxa = null;
                
                
                if($tpperiodo === 'MENSAL'){
                    $nreventotaxa = intval($beneficio['NREVENTOTAXA']);
                    $vreventotaxa = $beneficio['VREVENTOTAXA'];
                }else{
                    $nreventotaxa = null;
                    $vreventotaxa = null;
                }
                
                $idSubsidiario100 = $this->entityManager->getRepository(Repositories::VAB_BENEFICIARIO)->getIdSubisido100($beneficiario->getNrorg(), $beneficiario->getNrbeneficiario());
                $percentSubsidio = $this->entityManager->getRepository(Repositories::VAB_BENEFICIARIO)->getPercentSubsidio($beneficiario->getNrorg(), $beneficiario->getNrbeneficiario());
                
                // Histórico do vínculo é carregado a fim de se ter acesso ao valor IDASSOCSINDICATO 
                $this->histVinc = $this->vinculoh;
                $histVinc = $this->vinculoh;
                
                // Caso o vínculo seja de um sindicato é para considerar o campo 'percentual/valor associado' se e somente se este tiver valor
                $idAssocSind = $this->histVinc->getIdassocsindicato() ? $this->histVinc->getIdassocsindicato() : 'N' ; 
    
                $vlrPisoMens = isset($beneficio['VRPISOTITULARMENS']) && !empty($beneficio['VRPISOTITULARMENS']) ? $beneficio['VRPISOTITULARMENS'] : null;
                $vlrTetoMens = isset($beneficio['VRTETOTITULARMENS']) && !empty($beneficio['VRTETOTITULARMENS']) ? $beneficio['VRTETOTITULARMENS'] : null;
                
                if( isset($tpBeneficiario) && $tpBeneficiario === 'T' ){
                    $nreventomensa             = intval($beneficio['NREVENTOMENSA']);
                    $vlrlinearbeneficio        = isset($beneficio['VRLINEARBENEFICIO']) && !empty($beneficio['VRLINEARBENEFICIO']) ? $beneficio['VRLINEARBENEFICIO'] : null;
                    $eventVinc                 = intval($beneficio['NREVETITULARVINC']);
                    $tpParticipacao            = $beneficio['NRTIPOPARTICIPATITULAR'];
                    $percentOuVlrParticipacao  = $idAssocSind == 'S' && isset($beneficio['VRTITULARSIND']) ? $beneficio['VRTITULARSIND'] : $beneficio['VRPARTICIPATITULAR'];
                    $eventEmpresa              = intval($beneficio['NREVETITULAREMP']);
                    $nrtabelafolha             = isset($beneficio['NRTABELAFOLHAPRI']) && !empty($beneficio['NRTABELAFOLHAPRI']) ? explode(", ", $beneficio['NRTABELAFOLHAPRI']) : null;
                    $nrFaixaSalarial           = isset($beneficio['NRTABFAIXASALTIT']) && !empty($beneficio['NRTABFAIXASALTIT']) ? explode(", ", $beneficio['NRTABFAIXASALTIT']) : null;
                    $idadeMaxSubsidio          = null;
                    $sexoSubsidio              = null;
                    $vlrPiso                   = $idAssocSind == 'S' && isset($beneficio['VRPISOTITULARSIND']) ? $beneficio['VRPISOTITULARSIND'] : $beneficio['VRPISOTITULAR']; 
                    $vlrTeto                   = $idAssocSind == 'S' && isset($beneficio['VRTETOTITULARSIND']) ? $beneficio['VRTETOTITULARSIND'] : $beneficio['VRTETOTITULAR'];
    
    
                    $this->validaJornadaReduzida($beneficio, $nrvinculom, $vlrlinearbeneficio);
    
                }else if( isset($tpBeneficiario) && $tpBeneficiario === 'D' ){
                    $tpRelacAndDtNascAndNmpessoa = $this->calculoBeneficioRepository->getTipoRelacionamentoPorPessoa(
                        $beneficiario->getNrvinculom(), 
                        $beneficiario->getNrpessoa(), 
                        $this->nrorgTrab,
                        $this->competencia
                    );
    
                    $tpRelac               = $tpRelacAndDtNascAndNmpessoa['NRTIPORELACIONA'];
                    $idadeBeneficiario     = isset($tpRelacAndDtNascAndNmpessoa['DTNASCPESSOA']) ? substr($tpRelacAndDtNascAndNmpessoa['DTNASCPESSOA'], 0, -9) : null;
                    $sexoBeneficiario      = isset($tpRelacAndDtNascAndNmpessoa['IDSEXOPESSOA']) ? $tpRelacAndDtNascAndNmpessoa['IDSEXOPESSOA'] : null;
                    
                    if ($this->pessoah->getNrcondfispes() && ($this->pessoah->getNrcondfispes() !== 1)) {
                        $isDeficiente = true;
                    } else {
                        $isDeficiente = false;
                    }
                    
                    if(isset($idadeBeneficiario) && $idadeBeneficiario){
                        $idadeBeneficiario = $idadeBeneficiario ? DateUtil::getDataDeString($idadeBeneficiario) : null;
                        $idadeBeneficiario = $idadeBeneficiario->diff(DateUtil::getDataAtual())->format('%y');                
                    }
                    
                    $nreventomensa             = intval($beneficio['NREVENTOMENSADEP']);
                    $vlrlinearbeneficio        = isset($beneficio['VRLINEARBENEFICIODEP']) && !empty($beneficio['VRLINEARBENEFICIODEP']) ? $beneficio['VRLINEARBENEFICIODEP'] : null;
                    $eventVinc                 = intval($beneficio['NREVEDEPEVINC']);
                    $tpParticipacao            = $beneficio['NRTIPOPARTICIPADEP'];
                    $percentOuVlrParticipacao  = $idAssocSind == 'S' && isset($beneficio['VRDEPSIND']) ? $beneficio['VRDEPSIND'] : $beneficio['VRPARTICIPADEPE'];
                    $eventEmpresa              = intval($beneficio['NREVEDEPEMPRESA']);
                    $nrtabelafolha             = isset($beneficio['NRTABELAFOLHASEC']) && !empty($beneficio['NRTABELAFOLHASEC']) ? explode(", ", $beneficio['NRTABELAFOLHASEC']) : null;
                    $nrFaixaSalarial           = isset($beneficio['NRTABFAIXASALDEP']) && !empty($beneficio['NRTABFAIXASALDEP']) ? explode(", ", $beneficio['NRTABFAIXASALDEP']) : null;
                    $idadeMaxSubsidio          = $tpRelac == 3 ? $beneficio['NRIDADEMAXSUBFILHOS'] : $beneficio['NRIDADEMAXSUBDEP'];
                    $sexoSubsidio              = $tpRelac == 3 ? $beneficio['SEXOSUBSIDIOFILHO'] : $beneficio['SEXOSUBSIDIODEP'];
                    $vlrPiso                   = $idAssocSind == 'S' && isset($beneficio['VRPISODEPSIND']) ? $beneficio['VRPISODEPSIND'] : $beneficio['VRPISODEP'];
                    $vlrTeto                   = $idAssocSind == 'S' && isset($beneficio['VRTETODEPSIND']) ? $beneficio['VRTETODEPSIND'] : $beneficio['VRTETODEP'];
                }
                
                // Se o beneficiário for dependente, tiver idade de subsídio no benefício, e não tiver dtnascimento no BD, não será calculada a Mensalidade.
                // Se o beneficiário for dependente, a mensalidade for por faixa etária e não tiver dtnascimento no BD, não será calculada a Mensalidade.
                if( isset($tpBeneficiario) && $tpBeneficiario === 'D' 
                    && ( 
                            ( isset($idadeMaxSubsidio)    && !isset($idadeBeneficiario) ) 
                         || ( !isset($vlrlinearbeneficio) && !isset($idadeBeneficiario) )
                       ) 
                ){
                    MsgCalculo::getInstance()->adicionaMensagem('Dependente '.$tpRelacAndDtNascAndNmpessoa['NMPESSOA'].' não possui data de nascimento.', MsgCalculo::ALERTA);
                    $efetuarCalculo = false;
                }
                
                // Se o beneficiário for dependente, tiver sexo Subsídio no benefício, e não tiver sexo cadastrado no BD, não será calculada a Mensalidade.
                if( (isset($tpBeneficiario) && $tpBeneficiario === 'D') && isset($sexoSubsidio) && !isset($sexoBeneficiario) ){
                    MsgCalculo::getInstance()->adicionaMensagem('Dependente '.$tpRelacAndDtNascAndNmpessoa['NMPESSOA'].' não possui sexo cadastrado.', MsgCalculo::ALERTA);
                    $efetuarCalculo = false;
                }
                
                if($efetuarCalculo){
                    // Calcula o valor para o evento setado no field "Evento", da aba "Beneficiário Por Evento"   
                    if( isset($eventoBeneficiario) && !empty($eventoBeneficiario) ){
                        if( array_key_exists($eventoBeneficiario, $eventos) ){
                            $vlrAnterior                  = $eventos[$eventoBeneficiario];
                            $eventos[$eventoBeneficiario] = $this->calcularEventoBeneficiario($vincBenefiarioEvento, $eventoBeneficiario, $beneficio, $beneficiario, $nrtabelafolha, $proporcao, $tpperiodo, $eventos[$eventoBeneficiario]);
                            $vlrPosterior                 = $eventos[$eventoBeneficiario];
                            
                            $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventoBeneficiario, $vlrAnterior, $vlrPosterior);
                        
                        }else{
                            $eventos[$eventoBeneficiario] = $this->calcularEventoBeneficiario($vincBenefiarioEvento, $eventoBeneficiario, $beneficio, $beneficiario, $nrtabelafolha, $proporcao, $tpperiodo);
    
                            $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventoBeneficiario);
                        }
                        $eventoEparcelas[$eventoBeneficiario] = intval($beneficio['QTDPARCELAS']);
                    }
                    
                    // Calcula o evento setado ou no field "Evento Mensalidade" ou no field "Evento Mensalidade (Dependente)"
                    if( isset($nreventomensa) && !empty($nreventomensa) && (empty($eventoBeneficiario))){

                        
                        // Função retorna valor a ser multiplicado a partir de outros parâmetros avaliados. Tais valores e parâmetros estão no benefício do sindicato.
                        $vlrMultiplicador = $this->getMultiplicadorMensalidade($beneficio, $vinculo, $histVinc, $nreventomensa);
                        
                        if( array_key_exists($nreventomensa, $eventos)){
                            $vlrAnterior             = $eventos[$nreventomensa];
                            $eventos[$nreventomensa] = $this->calcularEventoMensalidade($vincBenefiarioEvento_mens, $nreventomensa, $vlrMultiplicador, $beneficio, $beneficiario, $nrtabelafolha, $vlrEventoMensal, $vlrlinearbeneficio, $tpperiodo, $proporcao, $vlrPisoMens, $vlrTetoMens, $eventos[$nreventomensa]);
                            $vlrPosterior            = $eventos[$nreventomensa];
    
                            $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $nreventomensa, $vlrAnterior, $vlrPosterior, $beneficiario->getQtbeneficio());
    
                        }else{
                            $eventos[$nreventomensa] = $this->calcularEventoMensalidade($vincBenefiarioEvento_mens, $nreventomensa, $vlrMultiplicador, $beneficio, $beneficiario, $nrtabelafolha, $vlrEventoMensal, $vlrlinearbeneficio, $tpperiodo, $proporcao, $vlrPisoMens, $vlrTetoMens);
                            
                            $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $nreventomensa, null, null, $beneficiario->getQtbeneficio());
                        }
                        
                        $eventoEparcelas[$nreventomensa] = intval($beneficio['QTDPARCELAS']);
                        
                        // Caso seja o último beneficiário do vínculo para que já possua o valor total da mensalidade
                        if(isset($isLast) && !empty($isLast)){
                            // Se não usufruiu do multiplicador por possui um dos eventos da lista
                            if(isset($vlrMultiplicador) && $vlrMultiplicador == 1 && !empty($beneficio['DSINCIDEVENTO']) && !empty($beneficio['NREVETITULARVINC'])){
                                // Caso tenha o valor fixo ou percentual sobre o valor da mensalidade é realizado o acréscimo no desconto do vínculo
                                if(!is_null($beneficio['VRFIXACRESDESC'])){
                                     $vlrAcresDesconto = $beneficio['VRFIXACRESDESC'];
                                }elseif(!is_null($beneficio['VRPERACRESCDESC'])){
                                    $vlrAcresDesconto = $eventos[$nreventomensa] * $beneficio['VRPERACRESCDESC'] / 100;
                                }else{
                                    $vlrAcresDesconto = 0;
                                }
                                $eventos[$beneficio['NREVETITULARVINC']] += $vlrAcresDesconto;
                            }
                        }
                    }
                    
                    // - Se a opção 'subsidado 100%' estiver como sim, a empresa pagará o valor total do benefício
                    // - Se o funcionário estiver afastado com excessão de licensa maternidade e de demissão, com atributo 'Gera Vale-Transporte Benefícios' 
                    //    da situação funcional como não, com data inicial de afastamento anterior à competencia, 
                    //    e com data final maior que o último dia da competencia, a empresa pagara tudo.
                    // - Se a opção para 'considerar vínculos rescindidos do mês anterior' estiver marcada e o vínculo estiver rescindido no mês anterior, 
                    //    a empresa pagará tudo.
                    if((isset($idSubsidiario100) && $idSubsidiario100 == 'S') || (isset($subsidio100PorAfastamentoOuDemissao) && $subsidio100PorAfastamentoOuDemissao)){
    
                        // Calcula o evento setado ou no field "Evento Titular (Empresa)" ou no field "Evento Dependente (Empresa)", apenas se houver mensalidade
                        if( (isset($vlrEventoMensal) && $vlrEventoMensal) && isset($eventEmpresa) && !empty($eventEmpresa) && ($vlrEventoMensal > $vlrEventoVinc) ){
                            if(array_key_exists($eventEmpresa, $eventos)){
                                $vlrAnterior             = $eventos[$eventEmpresa];
                                $eventos[$eventEmpresa] += $vincBenefiarioEvento_empresa[$eventEmpresa] = $vlrEventoMensal; // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal já está proporcionalizado
                                $vlrPosterior            = $eventos[$eventEmpresa];
        
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventEmpresa, $vlrAnterior, $vlrPosterior);
        
                            }else{
                                $eventos[$eventEmpresa] = $vincBenefiarioEvento_empresa[$eventEmpresa] = $vlrEventoMensal; // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal já está proporcionalizado
        
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventEmpresa);
                            }
                            $eventoEparcelas[$eventEmpresa] = intval($beneficio['QTDPARCELAS']);
                        }
    
                        // Calcula o evento setado ou no field "Evento Titular (Vínculo)" ou no field "Evento Dependente (Vínculo)"
                        if( isset($eventVinc) && !empty($eventVinc)){
                            if(array_key_exists($eventVinc, $eventos)){
                                
                                $vlrAnterior         = $eventos[$eventVinc];
                                $eventos[$eventVinc] += $vincBenefiarioEvento_vinc[$eventVinc] = 0;
                                $vlrPosterior        = $eventos[$eventVinc];
                                
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventVinc, $vlrAnterior, $vlrPosterior);
        
                            }else{
                                $eventos[$eventVinc] = $vincBenefiarioEvento_vinc[$eventVinc] = 0;
        
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventVinc);
                            }
                            $eventoEparcelas[$eventVinc] = intval($beneficio['QTDPARCELAS']);
                        }
                        
                        if($tpperiodo == 'DIARIO'){
                            $eventos[$eventEmpresa] = 0;
                        }
                    }
                    else
                    // Se o campo '% Subsídio' estiver com valor, a Empresa pagará o percentual da Mensalidade calculado a partir deste valor. 
                    if(isset($percentSubsidio) && $percentSubsidio >= 0){
    
                        // Calcula o evento setado ou no field "Evento Titular (Empresa)" ou no field "Evento Dependente (Empresa)", apenas se houver mensalidade
                        if( $vlrEventoMensal && isset($eventEmpresa) && !empty($eventEmpresa)){
    
                            $vlrEventoEmpr = $vlrEventoMensal * ($percentSubsidio/100); // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal já está proporcionalizado
    
                            if(array_key_exists($eventEmpresa, $eventos)){
                                $vlrAnterior             = $eventos[$eventEmpresa];
                                $eventos[$eventEmpresa] += $vincBenefiarioEvento_empresa[$eventEmpresa] = $vlrEventoEmpr;
                                $vlrPosterior            = $eventos[$eventEmpresa];
        
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventEmpresa, $vlrAnterior, $vlrPosterior);
        
                            }else{
                                $eventos[$eventEmpresa] = $vincBenefiarioEvento_empresa[$eventEmpresa] = $vlrEventoEmpr;
        
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventEmpresa);
                            }
                            $eventoEparcelas[$eventEmpresa] = intval($beneficio['QTDPARCELAS']);
                        }
    
                        // Calcula o evento setado ou no field "Evento Titular (Vínculo)" ou no field "Evento Dependente (Vínculo)"
                        if( isset($eventVinc) && !empty($eventVinc)){
                            if(array_key_exists($eventVinc, $eventos)){
                                
                                $vlrAnterior         = $eventos[$eventVinc];
                                $eventos[$eventVinc] += $vincBenefiarioEvento_vinc[$eventVinc] = ($vlrEventoMensal - $vlrEventoEmpr); // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal e $vlrEventoEmpr já estão proporcionalizados
                                $vlrPosterior        = $eventos[$eventVinc];
                                
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventVinc, $vlrAnterior, $vlrPosterior);
        
                            }else{
                                $eventos[$eventVinc] = $vincBenefiarioEvento_vinc[$eventVinc] = ($vlrEventoMensal - $vlrEventoEmpr); // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal e $vlrEventoEmpr já estão proporcionalizados
        
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventVinc);
                            }
                            $eventoEparcelas[$eventVinc] = intval($beneficio['QTDPARCELAS']);
                        } 
                    }
                    else
                    // Se beneficiario é titular, ou deficiente, ou dependente com idade menor que a de subsidio, ou com o sexo Subsídio igual ao sexo Cadastrado, 
                    // há desconto do benefício. Senão, pagará o valor integral.
                    if( 
                           ( $tpBeneficiario == 'T' )    
                        || ( $isDeficiente ) 
                        || (
                                ( !isset($idadeMaxSubsidio) || ($idadeBeneficiario <= $idadeMaxSubsidio) )
                             && ( !isset($sexoSubsidio) || ($sexoBeneficiario === $sexoSubsidio) ) 
                           )
                            
                     ){
                        // Calcula o evento setado ou no field "Evento Titular (Vínculo)" ou no field "Evento Dependente (Vínculo)"
                        if( isset($eventVinc) && !empty($eventVinc)){
                            if(array_key_exists($eventVinc, $eventos)){
                                $vlrAnterior         = $eventos[$eventVinc];
                                $eventos[$eventVinc] = $this->calcularEventoVinculo($vincBenefiarioEvento_vinc, $tpperiodo, $eventVinc, $vlrPiso, $vlrTeto, $beneficio, $beneficiario, $vlrEventoVinc, $vlrEventoMensal, $tpParticipacao, $percentOuVlrParticipacao, $nrFaixaSalarial, $proporcao, $eventos[$eventVinc]);
                                $vlrPosterior        = $eventos[$eventVinc];
                                
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventVinc, $vlrAnterior, $vlrPosterior, $beneficiario->getQtbeneficio());
        
                            }else{
                                $eventos[$eventVinc] = $this->calcularEventoVinculo($vincBenefiarioEvento_vinc, $tpperiodo, $eventVinc, $vlrPiso, $vlrTeto, $beneficio, $beneficiario, $vlrEventoVinc, $vlrEventoMensal, $tpParticipacao, $percentOuVlrParticipacao, $nrFaixaSalarial, $proporcao);
                                
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventVinc, null, null, $beneficiario->getQtbeneficio());
                            }
                            
                            $eventoEparcelas[$eventVinc] = intval($beneficio['QTDPARCELAS']);
                        }           
                        
                        // Calcula o evento setado ou no field "Evento Titular (Empresa)" ou no field "Evento Dependente (Empresa)", apenas se houver mensalidade
                        if( (isset($vlrEventoMensal) && $vlrEventoMensal) && isset($eventEmpresa) && !empty($eventEmpresa) && ($vlrEventoMensal > $vlrEventoVinc)){
                            if(array_key_exists($eventEmpresa, $eventos)){
                                $vlrAnterior             = $eventos[$eventEmpresa];
                                $eventos[$eventEmpresa] += $vincBenefiarioEvento_empresa[$eventEmpresa] = ($vlrEventoMensal - $vlrEventoVinc); // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal e $vlrEventoVinc já estão proporcionalizados
                                $vlrPosterior            = $eventos[$eventEmpresa];
        
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventEmpresa, $vlrAnterior, $vlrPosterior, $beneficiario->getQtbeneficio());
        
                            }else{
                                $eventos[$eventEmpresa] = $vincBenefiarioEvento_empresa[$eventEmpresa] = ($vlrEventoMensal - $vlrEventoVinc); // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal e $vlrEventoVinc já estão proporcionalizados
        
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventEmpresa, null, null, $beneficiario->getQtbeneficio());
                            }
                            $eventoEparcelas[$eventEmpresa] = intval($beneficio['QTDPARCELAS']);
                        }
                        
                    }else{
                        // Calcula o evento setado ou no field "Evento Titular (Vínculo)" ou no field "Evento Dependente (Vínculo)"
                        if( isset($eventVinc) && !empty($eventVinc)){
                            if(array_key_exists($eventVinc, $eventos)){
                                $vlrAnterior          = $eventos[$eventVinc];
                                $eventos[$eventVinc] += $vincBenefiarioEvento_vinc[$eventVinc] = $vlrEventoMensal; // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal já está proporcionalizados
                                $vlrPosterior         = $eventos[$eventVinc];
                                
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventVinc, $vlrAnterior, $vlrPosterior);
        
                            }else{
                                $eventos[$eventVinc] = $vincBenefiarioEvento_vinc[$eventVinc] = $vlrEventoMensal; // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal já está proporcionalizados
        
                                $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $eventVinc);
                            }
                            $eventoEparcelas[$eventVinc] = intval($beneficio['QTDPARCELAS']);
                        }
                    }
                    
                    // Caso o benefício seja do tipo Mensal, é Persistido os valores na tabela de histórico Mensal. 
                    // Se for Diário, será persistido na tabela de histórico Diário
                    if($tpperiodo === 'MENSAL'){
                        
                        // Caso haja evento setado no field 'Eventos Taxa de Inclusão/Implantação' e a data de inscrição do beneficiário estiver na competência vigente,
                        // deverá calcular tal evento para o beneficiário
                        if($dtcompetencia <= $dtinscrbeneficiario && $dtinscrbeneficiario <= $ultimoDiaComp){
                            if( isset($nreventotaxa) && !empty($nreventotaxa)){
                                if(array_key_exists($nreventotaxa, $eventos)){
                                    $vlrAnterior            = $eventos[$nreventotaxa];
                                    $eventos[$nreventotaxa] += $vreventotaxa; 
                                    $vlrPosterior           = $eventos[$nreventotaxa];
                                    $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $nreventotaxa, $vlrAnterior, $vlrPosterior);
            
                                }else{
                                    $eventos[$nreventotaxa] = $vreventotaxa; 
    
                                    $this->validaQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento, $eventos, $nreventotaxa);
                                }
                                $eventoEparcelas[$nreventotaxa] = intval($beneficio['QTDPARCELAS']);
                            }
                        }
                        
                        $this->persistirHistoricoCalculoDIRF_Mensal(
                            $nrcalculobenef,
                            $beneficio['NRBENEFICIOM'],
                            $beneficiario->getNrvinculom(), 
                            $beneficiario->getNrpessoa(),
                            $proporcao,
                            $vincBenefiarioEvento, 
                            $vincBenefiarioEvento_mens, 
                            $vincBenefiarioEvento_vinc, 
                            $vincBenefiarioEvento_empresa
                        );
        
                    }else if($tpperiodo === 'DIARIO'){
    
                        if(!empty($eventVinc) && $eventVinc){
                            $eventoh = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
                                $eventVinc, 
                                $this->nrorgTrab, 
                                0, 
                                $this->competencia
                            );                    
        
                            if( !$subsidio100PorAfastamentoOuDemissao && isset($vlrPiso) ){
                                $vlrPiso = Valor::arredonda(
                                    $vlrPiso, 
                                    $eventoh->historico->getIdtipoarredonda(), 
                                    $eventoh->historico->getNrdigarredonda(),
                                    $eventoh->historico->getVrlimitearred()
                                );           
                            }else
                                $vlrPiso = null;
        
                            if( !$subsidio100PorAfastamentoOuDemissao && isset($vlrTeto) ){
                                $vlrTeto = Valor::arredonda(
                                    $vlrTeto, 
                                    $eventoh->historico->getIdtipoarredonda(), 
                                    $eventoh->historico->getNrdigarredonda(),
                                    $eventoh->historico->getVrlimitearred()
                                );          
                            }else
                                $vlrTeto = null;
                        }
                        
                        $eventEmpresa = array('EVENTO' => $eventEmpresa, 'VALOR' => (isset($eventos[$eventEmpresa]) && !empty($eventos[$eventEmpresa])) ? $eventos[$eventEmpresa] : 0);
                        $eventVinc = array('EVENTO' => $eventVinc, 'VALOR' => (isset($eventos[$eventVinc]) && !empty($eventos[$eventVinc])) ? $eventos[$eventVinc] : 0);
                        $eventomensal = array('EVENTO' => $nreventomensa, 'VALOR' => (isset($eventos[$nreventomensa]) && !empty($eventos[$nreventomensa])) ? $eventos[$nreventomensa] : 0);
                        $eventoBeneficiario = array('EVENTO' => $eventoBeneficiario, 'VALOR' => (isset($eventos[$eventoBeneficiario]) && !empty($eventos[$eventoBeneficiario])) ? $eventos[$eventoBeneficiario] : 0);
                        
                        $this->calculaHistorico_Diario(
                            $foiCalculado,
                            $nrcalculobenef,
                            $beneficio['NRBENEFICIOM'],
                            $beneficiario->getNrvinculom(), 
                            $beneficiario->getQtbeneficio(),
                            $arrayDiasCalculados,
                            $vlrPisoMens,
                            $vlrTetoMens,
                            $vlrPiso,
                            $vlrTeto,
                            $eventEmpresa,
                            $eventVinc,
                            $eventomensal,
                            $eventoBeneficiario,
                            $beneficio['NRTIPOPARTICIPATITULAR']
                        );
                    }
                    
                    $this->resetarFlagArrayQtdeBeneficiariosPerEvento($qtdBeneficiariosPerEvento);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function calculaHistorico_Diario(&$foiCalculado, $nrcalculobenef, $nrbeneficiom, $nrvinculom, $qtdeBeneficio, $arrayDiasCalculados, $vlrPisoMens, $vlrTetoMens, $vlrPiso, $vlrTeto, $eventEmpresa, $eventVinc, $eventMensal, $eventoBeneficiario, $tipoParticipacaoTitular = null){
        $vlrEventMensal = null;
        $vlrEventVinc = null;
        $vlrEventEmpresa = null;
        $vlrEventBeneficiario = null;
        
        $vlrEventMensal = ($qtdeBeneficio * $eventMensal['VALOR'] * $arrayDiasCalculados['DIASCONSIDERACAO']) - $this->valorAdescontar;
        
        $vlrEventVinc = $qtdeBeneficio * $eventVinc['VALOR'] * $arrayDiasCalculados['DIASCONSIDERACAO'];
        
        // Arredondar o valor do evento mensalidade depois de multiplicar pela quantidade de dias.
        $eventoMensal = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
            $eventMensal['EVENTO'], 
            $this->hcmParameters->getNrorgTrabalho(), 
            $this->hcmParameters->getNrorgPadrao(), 
            $this->hcmParameters->getDtCompetencia() 
        );
        
        if($eventoMensal){
            $vlrEventMensal = Valor::arredonda(
                $vlrEventMensal, 
                $eventoMensal->historico->getIdtipoarredonda(), 
                $eventoMensal->historico->getNrdigarredonda(),
                $eventoMensal->historico->getVrlimitearred()
            );
        }
        
        if( isset($vlrPisoMens) && $vlrEventMensal < $vlrPisoMens ) $vlrEventMensal = $vlrPisoMens;
        
        if( isset($vlrTetoMens) && $vlrEventMensal > $vlrTetoMens ) $vlrEventMensal = $vlrTetoMens;

        // Arredondar o valor do evento de desconto depois de multiplicar pela quantidade de dias.
        $eventoVinc = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
            $eventVinc['EVENTO'], 
            $this->hcmParameters->getNrorgTrabalho(), 
            $this->hcmParameters->getNrorgPadrao(), 
            $this->hcmParameters->getDtCompetencia() 
        );
    
        //Se for % de salario ele vai aplicar uma taxa unica
        if(isset($tipoParticipacaoTitular) && $tipoParticipacaoTitular == 'PERC_SAL'){
            if(isset($eventoVinc) && $eventoVinc->getIdtipoevento() == 'DESCONTO'){
                $vlrEventVinc = $eventVinc['VALOR'];
            }
        }
        
        if(!empty($eventoVinc) && $eventoVinc){
            $vlrEventVinc = Valor::arredonda(
                $vlrEventVinc, 
                $eventoVinc->historico->getIdtipoarredonda(), 
                $eventoVinc->historico->getNrdigarredonda(),
                $eventoVinc->historico->getVrlimitearred()
            );
        }

        if( isset($vlrPiso) && $vlrEventVinc < $vlrPiso ) $vlrEventVinc = $vlrPiso;
        
        if( isset($vlrTeto) && $vlrEventVinc > $vlrTeto ) $vlrEventVinc = $vlrTeto;


        
        if(isset($eventEmpresa['EVENTO']) && $eventEmpresa['EVENTO']){
            $vlrEventEmpresa = $vlrEventMensal - $vlrEventVinc;
        }
        
        if(isset($eventoBeneficiario['VALOR']) && $eventoBeneficiario['VALOR']){
            $vlrEventBeneficiario = $qtdeBeneficio * $eventoBeneficiario['VALOR'] * $arrayDiasCalculados['DIASCONSIDERACAO'];

            // Arredondar o valor do evento do beneficiário depois de multiplicar pela quantidade de dias.
            $objEventBenef = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
                $eventoBeneficiario['EVENTO'], 
                $this->hcmParameters->getNrorgTrabalho(), 
                $this->hcmParameters->getNrorgPadrao(), 
                $this->hcmParameters->getDtCompetencia() 
            );
            if($objEventBenef){
                $vlrEventBeneficiario = Valor::arredonda(
                    $vlrEventBeneficiario, 
                    $objEventBenef->historico->getIdtipoarredonda(), 
                    $objEventBenef->historico->getNrdigarredonda(),
                    $objEventBenef->historico->getVrlimitearred()
                );
            }
        }

        //Mensal
        if(isset($vlrEventMensal) && $vlrEventMensal > 0){
            $this->persistirHistorico_Diario($foiCalculado, $nrcalculobenef, $nrbeneficiom, $nrvinculom, $qtdeBeneficio, $arrayDiasCalculados, $eventMensal['EVENTO'], $eventMensal['VALOR'], $vlrEventMensal);
        }
        //Vinc
        if(isset($vlrEventVinc) && $vlrEventVinc > 0){
            $this->persistirHistorico_Diario($foiCalculado, $nrcalculobenef, $nrbeneficiom, $nrvinculom, $qtdeBeneficio, $arrayDiasCalculados, $eventVinc['EVENTO'], $eventVinc['VALOR'], $vlrEventVinc);
        }
        
        //Empresa  
        if(isset($vlrEventEmpresa) && $vlrEventEmpresa > 0){
            $this->persistirHistorico_Diario($foiCalculado, $nrcalculobenef, $nrbeneficiom, $nrvinculom, $qtdeBeneficio, $arrayDiasCalculados, $eventEmpresa['EVENTO'], $eventEmpresa['VALOR'], $vlrEventEmpresa);
        }
        
        //beneficiario
        if(isset($vlrEventBeneficiario) && $vlrEventBeneficiario > 0){
            $this->persistirHistorico_Diario($foiCalculado, $nrcalculobenef, $nrbeneficiom, $nrvinculom, $qtdeBeneficio, $arrayDiasCalculados, $eventoBeneficiario['EVENTO'], $eventoBeneficiario['VALOR'], $vlrEventBeneficiario);
        }
    } 
    
    public function persistirHistorico_Diario(&$foiCalculado, $nrcalculobenef, $nrbeneficiom, $nrvinculom, $qtdeBeneficio, $arrayDiasCalculados, $nrEvento, $valorEvento, $vlrTotalBenef){
        
        if(isset($valorEvento) && $valorEvento >= 0){
            
            $foiCalculado = true;

            $row = $this->entityManager->getRepository(Repositories::VAB_HISTBENEFVINC)->findOneBy( 
                [ 
                 'nrcalculofolha' => $nrcalculobenef,
                 'dtmescompetenc' => Dateutil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true),
                 'nrbeneficiom'   => $nrbeneficiom,
                 'nrvinculom'     => $nrvinculom,
                 'nreventom'      => $nrEvento,
                 'nrorg'          => $this->hcmParameters->getNrorgTrabalho(),
                 'dtpgtobenef'    => DateUtil::truncateData($this->dtPagamentoBenef)
                ]
            );
            
            if(isset($row) && $row){
                $this->entityManager->remove($row);
                $this->entityManager->flush();
            }

            try {
                $this->calculoBeneficioRepository->persistirHistorico_Diario(
                    $this->nrorgTrab, 
                    $this->competencia, 
                    $this->cdoperador,
                    $nrcalculobenef, 
                    $nrbeneficiom, 
                    $nrvinculom,
                    $nrEvento, 
                    $valorEvento, 
                    $arrayDiasCalculados, 
                    $this->dtiniperiodobenefmod, 
                    $this->dtfimperiodobenefmod, 
                    $this->dtPagamentoBenef->format('d/m/Y h:m:s'),
                    $vlrTotalBenef,
                    $qtdeBeneficio,
                    $this->idcalccomplementar,
                    $this->valorAdescontar
                );

            } catch (\Exception $e) {
                throw $e;    
            }
        }
    }
    
    public function calcularEventoVinculo(&$vincBenefiarioEvento_vinc, $tpperiodo, $eventVinc, $vlrPiso, $vlrTeto, $beneficio, $beneficiario, &$vlrEventoVinc, $vlrEventoMensal, $tpParticipacao, $percentOuVlrParticipacao, $nrFaixaSalarial, $proporcao, $vlrBeneficio = 0){
        
        $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
            $eventVinc, 
            $this->hcmParameters->getNrorgTrabalho(), 
            $this->hcmParameters->getNrorgPadrao(), 
            $this->hcmParameters->getDtCompetencia() 
        );
    
        switch($tpParticipacao){
            case 'FAIXA_SAL':

                $vlrAntesDeArredondar = $vlrArredondado = $vlrNaoArredondado = null;
                
                if($tpperiodo == 'MENSAL'){
                    
                    $valor = $this->calcularValorTipoTabela($beneficiario, $nrFaixaSalarial, 'faixaSalarial');
                    
                    $vlrArredondado = Valor::arredonda(
                        $vlrAntesDeArredondar = ($valor * $proporcao), // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                        $evento->historico->getIdtipoarredonda(), 
                        $evento->historico->getNrdigarredonda(),
                        $evento->historico->getVrlimitearred()
                    );

                    if( isset($vlrPiso) && $vlrAntesDeArredondar < $vlrPiso ){ // Teto e Piso para Diário considera depois de multiplicar pela quantidade de dias
    
                        $vlrArredondado = Valor::arredonda(
                            $vlrPiso, 
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
    
                    }
                    
                    if( isset($vlrTeto) && $vlrAntesDeArredondar > $vlrTeto ){ // Teto e Piso para Diário considera depois de multiplicar pela quantidade de dias
    
                        $vlrArredondado = Valor::arredonda(
                            $vlrTeto, 
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
    
                    }    
                    
                    $vlrBeneficio += $vincBenefiarioEvento_vinc[$eventVinc] = $vlrEventoVinc = $vlrArredondado;
                    
                    $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Valor) '.$valor.' = '.$vlrArredondado;
                    
                    #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, $vlrPiso, $vlrTeto, null, $tpParticipacao, null);
                
                }elseif($tpperiodo == 'DIARIO'){ // Como no cálculo diario há a multiplicação pelo número de dias, o arredondamento deve ocorrer depois dessa multiplicação.
                    $valor = $this->calcularValorTipoTabela($beneficiario, $nrFaixaSalarial, 'faixaSalarial');
                    
                    $vlrNaoArredondado = ($this->calcularValorTipoTabela($beneficiario, $nrFaixaSalarial, 'faixaSalarial') * $proporcao); // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                    $vlrBeneficio += $vincBenefiarioEvento_vinc[$eventVinc] = $vlrEventoVinc = $vlrNaoArredondado;
                    
                    $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Valor) '.$valor.' = '.$vlrNaoArredondado;
                    #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, $vlrPiso, $vlrTeto, null, $tpParticipacao, null);
                }    

                break;
            

            case 'VALOR':

                $vlrAntesDeArredondar = $vlrArredondado = $vlrNaoArredondado = null;
                
                if($tpperiodo == 'MENSAL'){
                    $referenciaProporciona = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor(
                        $this->hcmParameters->getNrorgTrabalho(),
                        0,
                        $this->hcmParameters->getDtCompetencia(),
                        'PROPORC_EVENT_VINC_FIXO'
                    );

                    $vlrArredondado = Valor::arredonda(
                        $vlrAntesDeArredondar = ( !empty($referenciaProporciona) && $referenciaProporciona == 1 ? $percentOuVlrParticipacao * $beneficiario->getQtbeneficio() : (($percentOuVlrParticipacao * $beneficiario->getQtbeneficio()) * $proporcao)), // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                        $evento->historico->getIdtipoarredonda(), 
                        $evento->historico->getNrdigarredonda(),
                        $evento->historico->getVrlimitearred()
                    );
                    
                    if( isset($vlrPiso) && $vlrAntesDeArredondar < $vlrPiso ){ // Teto e Piso para Diário considera depois de multiplicar pela quantidade de dias
    
                        $vlrArredondado = Valor::arredonda(
                            $vlrPiso, 
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
    
                    }    
                    
                    if( isset($vlrTeto) && $vlrAntesDeArredondar > $vlrTeto ){ // Teto e Piso para Diário considera depois de multiplicar pela quantidade de dias
    
                        $vlrArredondado = Valor::arredonda(
                            $vlrTeto, 
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
    
                    }    
    
                    $vlrBeneficio += $vincBenefiarioEvento_vinc[$eventVinc] = $vlrEventoVinc = $vlrArredondado;
                    
                    $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Valor) '.$percentOuVlrParticipacao.' = '.$vlrArredondado;

                    #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, $vlrPiso, $vlrTeto, null, $tpParticipacao, null);
                
                }elseif($tpperiodo == 'DIARIO'){ // Como no cálculo diario há a multiplicação pelo número de dias, o arredondamento deve ocorrer depois dessa multiplicação.
                    $vlrNaoArredondado = ($percentOuVlrParticipacao * $proporcao); // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                    $vlrBeneficio += $vincBenefiarioEvento_vinc[$eventVinc] = $vlrEventoVinc = $vlrNaoArredondado;
                    
                    $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Valor) '.$percentOuVlrParticipacao.' = '.$vlrNaoArredondado;
                    
                    #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, $vlrPiso, $vlrTeto, null, $tpParticipacao, null);
                }    

                break;
            
            case 'PERC_SAL':
                
                $vlrAntesDeArredondar = $vlrArredondado = $vlrNaoArredondado = null;
                
                $alteSalario = $this->calculoBeneficioRepository->getSalarioPorCompetencia(
                    $this->hcmParameters->getNrorgTrabalho(), 
                    $beneficiario->getNrvinculom(), 
                    $this->hcmParameters->getDtCompetencia()
                );

                if($tpperiodo == 'MENSAL'){
                    $valor = $alteSalario['VRSALARIO'] * ($percentOuVlrParticipacao/100);
                    $vlrArredondado = Valor::arredonda(
                        $vlrAntesDeArredondar = ($valor * $proporcao), // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                        $evento->historico->getIdtipoarredonda(), 
                        $evento->historico->getNrdigarredonda(),
                        $evento->historico->getVrlimitearred()
                    );
                    
                    if( isset($vlrPiso) && $vlrAntesDeArredondar < $vlrPiso ){ // Teto e Piso para Diário considera depois de multiplicar pela quantidade de dias
    
                        $vlrArredondado = Valor::arredonda(
                            $vlrPiso, 
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
    
                    }    
                    
                    if( isset($vlrTeto) && $vlrAntesDeArredondar > $vlrTeto ){ // Teto e Piso para Diário considera depois de multiplicar pela quantidade de dias
    
                        $vlrArredondado = Valor::arredonda(
                            $vlrTeto, 
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
    
                    }    
    
                    $vlrBeneficio += $vincBenefiarioEvento_vinc[$eventVinc] = $vlrEventoVinc = $vlrArredondado;
                    
                    $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Valor) '.$valor.' = '.$vlrArredondado;

                    #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, $vlrPiso, $vlrTeto, null, $tpParticipacao, null);
                
                }elseif($tpperiodo == 'DIARIO'){ // Como no cálculo diario há a multiplicação pelo número de dias, o arredondamento deve ocorrer depois dessa multiplicação.
                    $valor = $alteSalario['VRSALARIO'] * ($percentOuVlrParticipacao/100);
                    
                    $vlrNaoArredondado = ($alteSalario['VRSALARIO'] * ($percentOuVlrParticipacao/100) * $proporcao); // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                    $vlrBeneficio += $vincBenefiarioEvento_vinc[$eventVinc] = $vlrEventoVinc = $vlrNaoArredondado;
                    
                    $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Valor) '.$valor.' = '.$vlrNaoArredondado;
                    #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, $vlrPiso, $vlrTeto, null, $tpParticipacao, null);
                }    
                
                break;
            
            case 'PERC_MENS':
                if($vlrEventoMensal){
                    
                    $vlrAntesDeArredondar = $vlrArredondado = $vlrNaoArredondado = null;
                    
                    if($tpperiodo == 'MENSAL'){

                        $vlrArredondado = Valor::arredonda(
                            $vlrAntesDeArredondar = $vlrEventoMensal * ($percentOuVlrParticipacao/100), // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal já está proporcionalizado
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
    
                        if( isset($vlrPiso) && $vlrAntesDeArredondar < $vlrPiso ){ // Teto e Piso para Diário é considerado depois de multiplicar pela quantidade de dias
    
                            $vlrArredondado = Valor::arredonda(
                                $vlrPiso, 
                                $evento->historico->getIdtipoarredonda(), 
                                $evento->historico->getNrdigarredonda(),
                                $evento->historico->getVrlimitearred()
                            );
    
    
                            $vlrAntesDeArredondar = $vlrPiso;
                        }    
                        
                        if( isset($vlrTeto) && $vlrAntesDeArredondar > $vlrTeto ){ // Teto e Piso para Diário é considerado depois de multiplicar pela quantidade de dias
    
                            $vlrArredondado = Valor::arredonda(
                                $vlrTeto, 
                                $evento->historico->getIdtipoarredonda(), 
                                $evento->historico->getNrdigarredonda(),
                                $evento->historico->getVrlimitearred()
                            );
    
                        }    
    
                        $vlrBeneficio += $vincBenefiarioEvento_vinc[$eventVinc] = $vlrEventoVinc = $vlrArredondado;
                        
                        $calculoBeneficio = '((Valor mensal) '.$vlrEventoMensal.' * (Percentual) '.$percentOuVlrParticipacao.' / 100) = '.$vlrArredondado;

                        #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $percentOuVlrParticipacao, 0, $vlrPiso, $vlrTeto, null, $tpParticipacao, null);
                
                    }elseif($tpperiodo == 'DIARIO'){ // Como no cálculo diario há a multiplicação pelo número de dias, o arredondamento deve ocorrer depois dessa multiplicação.
                        $vlrNaoArredondado = $vlrEventoMensal * ($percentOuVlrParticipacao/100); // Se deve haver proporcionalização por dias trabalhados, $vlrEventoMensal já está proporcionalizado
                        $vlrBeneficio += $vincBenefiarioEvento_vinc[$eventVinc] = $vlrEventoVinc = $vlrNaoArredondado;
                        
                        $calculoBeneficio = '((Valor mensal) '.$vlrEventoMensal.' * (Percentual) '.$percentOuVlrParticipacao.' / 100) = '.$vlrNaoArredondado;
                        #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $percentOuVlrParticipacao, 0, $vlrPiso, $vlrTeto, null, $tpParticipacao, null);
                    }    
                    
                }    
                break;
        }
        
        return $vlrBeneficio;
    }
    
    public function calcularEventoMensalidade(&$vincBenefiarioEvento_mens, $nreventomensa, $vlrMultiplicador, $beneficio, $beneficiario, $nrtabelafolha, &$vlrEventoMensal, $vlrlinearbeneficio, $tpperiodo, $proporcao, $vlrPisoMens, $vlrTetoMens, $vlrBeneficio = 0){
        try {
            $vlrAntesDeArredondar = $vlrArredondado = $vlrNaoArredondado = null;
            
            $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
                $nreventomensa, 
                $this->nrorgTrab, 
                0, 
                $this->competencia 
            );
            
            //O beneficio e um vale combustivel quando possui um valor de referencia vinculado ao beneficioe e e calculado quando o possui.
            if(isset($beneficio['NRVALORREFERENCIAM'])){ 
                /*$valorReferencia = FpaValorreferenciam::findOneByNrvalorreferenciam($beneficio['NRVALORREFERENCIAM']);
                if (is_object($valorReferencia)) {
                    $valorReferencia = $valorReferencia->retornaUltimoHistoricoCompetencia($this->competencia )->getVrreferencia();
                }*/
                
                $valorReferencia = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAH)->retornaUltimoHistoricoCompetenciaNrValorReferencia(
                    $this->nrorgTrab,
                    $beneficio['NRVALORREFERENCIAM'],
                    $this->competencia 
                )->getVrreferencia();
    
                $alteSalario = $this->calculoBeneficioRepository->getSalarioPorCompetencia(
                    $this->nrorgTrab,
                    $beneficiario->getNrvinculom(), 
                    $this->competencia
                );
    
                $valorSal = floatval($alteSalario['VRSALARIO']);
                $faixasTabela = $this->entityManager->getRepository(Repositories::FPA_FAIXATABELA)->retornaFaixasPorTabela($beneficio['NRTABELAFOLHATERC'], $this->competencia);
                
                $calculoBeneficio = '(Valor de Referencia): '.$valorReferencia.' * '.$beneficiario->getQtbeneficio();
                
                //Beneficio vale combustivel mensal.
                if($tpperiodo == 'MENSAL'){
                    $calculoBeneficio = '(Valor de Referencia): '.$valorReferencia.' * '.$beneficiario->getQtbeneficio(). ' /'.(!empty($this->fixaDias) ? $this->fixaDias : 30).' * ' .$this->diasNormais. '(Composição do mês: ' .$this->diasNormais.'))' ;
    
                    $vlrBeneficio = (($valorReferencia * $beneficiario->getQtbeneficio())/(!empty($this->fixaDias) ? $this->fixaDias : 30))*$this->diasNormais;
                    
                    //Comparar o valor mensal com a tabela.
                    $this->calculaFaixasTabelaComb($faixasTabela, $valorSal, $vlrBeneficio, $calculoBeneficio);
                }
                //Beneficio vale combustivel diário.
                else if($tpperiodo == 'DIARIO') {
                    //Quando for diário deve-se calcular a referência vezes a qtde de km/dia do beneficiário.
                    
                    $vlrBeneficio = $valorReferencia * $beneficiario->getQtbeneficio();
                    
                    //Comparar o valor diário com a tabela.
                    $this->calculaFaixasTabelaComb($faixasTabela, $valorSal, $vlrBeneficio, $calculoBeneficio);
    
                    //Dividir o resultado pela qtde de beneficio para obter o valor real do km.
                    $vlrBeneficio = $vlrBeneficio / $beneficiario->getQtbeneficio();
                }
                $vincBenefiarioEvento_mens[$nreventomensa] = $vlrEventoMensal = $vlrArredondado = $vlrBeneficio;
                #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, $vlrMultiplicador, $vlrPisoMens, $vlrTetoMens);
    
            } else if($vlrlinearbeneficio){
                if( $tpperiodo == 'MENSAL' ){
                    $vlrArredondado = Valor::arredonda(
                        $vlrAntesDeArredondar = (($vlrlinearbeneficio * $beneficiario->getQtbeneficio())* $proporcao * $vlrMultiplicador), // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados e multiplicador, este parâmetro real já é passado proporcionalizado e multiplicado
                        $evento->historico->getIdtipoarredonda(), 
                        $evento->historico->getNrdigarredonda(),
                        $evento->historico->getVrlimitearred()
                    );
    
                    if( isset($vlrPisoMens) && $vlrAntesDeArredondar < $vlrPisoMens ){ // Teto e Piso para Diário considera depois de multiplicar pela quantidade de dias
                        $vlrArredondado = Valor::arredonda(
                            $vlrPisoMens, 
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
                    }
                    if( isset($vlrTetoMens) && $vlrAntesDeArredondar > $vlrTetoMens ){ // Teto e Piso para Diário considera depois de multiplicar pela quantidade de dias
                        $vlrArredondado = Valor::arredonda(
                            $vlrTetoMens, 
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
                    }    
    
                    $vlrBeneficio += $vincBenefiarioEvento_mens[$nreventomensa] = $vlrEventoMensal = $vlrArredondado;
                    $calculoBeneficio = '(Valor) '.$vlrlinearbeneficio.' * (Proporcao) '.$proporcao.' * (Multiplicador) '.$vlrMultiplicador.' = '.$vlrArredondado;
                    
                    #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, $vlrMultiplicador, $vlrPisoMens, $vlrTetoMens);
                    
                }elseif( $tpperiodo == 'DIARIO' ){ // Como no cálculo diario há a multiplicação pelo número de dias, o arredondamento deve ocorrer depois dessa multiplicação.
                    $vlrNaoArredondado = ($vlrlinearbeneficio * $proporcao);
                    $vlrBeneficio += $vincBenefiarioEvento_mens[$nreventomensa] = $vlrEventoMensal = $vlrNaoArredondado;
                    
                    $calculoBeneficio = '(Valor) '.$vlrlinearbeneficio.' * (Proporcao) '.$proporcao.' * (Multiplicador) '.$vlrMultiplicador.' = '.$vlrNaoArredondado;
                    #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, $vlrMultiplicador, $vlrPisoMens, $vlrTetoMens);
                }
                
            }else{
                
                if( $tpperiodo == 'MENSAL' ){
                    $valor = $this->calcularValorTipoTabela($beneficiario, $nrtabelafolha);
                    $vlrArredondado = Valor::arredonda(
                        $vlrAntesDeArredondar = (($valor * $beneficiario->getQtbeneficio()) * $proporcao * $vlrMultiplicador), // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados e multiplicador, este parâmetro real já é passado proporcionalizado e multiplicado
                        $evento->historico->getIdtipoarredonda(),
                        $evento->historico->getNrdigarredonda(),
                        $evento->historico->getVrlimitearred()
                    );
    
                    if( isset($vlrPisoMens) && $vlrAntesDeArredondar < $vlrPisoMens ){ // Teto e Piso para Diário considera depois de multiplicar pela quantidade de dias
                        $vlrArredondado = Valor::arredonda(
                            $vlrPisoMens, 
                            $evento->historico->getIdtipoarredonda(),
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
                    }
                    if( isset($vlrTetoMens) && $vlrAntesDeArredondar > $vlrTetoMens ){ // Teto e Piso para Diário considera depois de multiplicar pela quantidade de dias
                        $vlrArredondado = Valor::arredonda(
                            $vlrTetoMens, 
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
                    }    
    
                    $vlrBeneficio += $vincBenefiarioEvento_mens[$nreventomensa] = $vlrEventoMensal = $vlrArredondado;
                    
                    $calculoBeneficio = '(Valor) '.$valor.' * (Proporcao) '.$proporcao.' * (Multiplicador) '.$vlrMultiplicador.' = '.$vlrArredondado;
    
                    #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, $vlrMultiplicador, $vlrPisoMens, $vlrTetoMens, $nrtabelafolha);
                    
                }elseif( $tpperiodo == 'DIARIO' ){ // Como no cálculo diario há a multiplicação pelo número de dias, o arredondamento deve ocorrer depois dessa multiplicação.
                    $valor = $this->calcularValorTipoTabela($beneficiario, $nrtabelafolha);
                    $vlrNaoArredondado = ($valor * $proporcao); // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                    $vlrBeneficio += $vincBenefiarioEvento_mens[$nreventomensa] = $vlrEventoMensal = $vlrNaoArredondado;
                    
                    $calculoBeneficio = '(Valor) '.$valor.' * (Proporcao) '.$proporcao.' * (Multiplicador) '.$vlrMultiplicador.' = '.$vlrNaoArredondado;
                    #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, $vlrMultiplicador, $vlrPisoMens, $vlrTetoMens, $nrtabelafolha);
                }
            }
            
            return $vlrBeneficio;
        } 
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function calculaFaixasTabelaComb($faixasTabela, $valorSal, &$vlrBeneficio, &$calculoBeneficio) {
        foreach($faixasTabela as $faixa) {
            if(($faixa->getVrmaxresultcalc() >= $valorSal) && ($faixa->getVrlimitsupfaixa() <= $valorSal)) {
                
                if($vlrBeneficio >= $faixa->getVraplicaresult()) {
                    $vlrBeneficio = $faixa->getVraplicaresult();
                    $calculoBeneficio = 'De acordo com a tabela salarial, o valor do vale combustivel foi superior ao limite e portanto o Vale Combustivel terá o valor de: '.$vlrBeneficio;
                    continue;
                }
            }
        }
    }
    
    public function resetarFlagArrayQtdeBeneficiariosPerEvento(&$qtdBeneficiariosPerEvento){
        try {
            foreach($qtdBeneficiariosPerEvento as &$value){
                $value['flag'] = 0;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function persistirHistoricoCalculoDIRF_Mensal($nrcalculobenef, $nrbeneficiom, $nrvinculom, $nrPessoaBeneficiario, $proporcao, $vincBenefiarioEvento, $vincBenefiarioEvento_mens, $vincBenefiarioEvento_vinc, $vincBenefiarioEvento_empresa){
        // Persiste o cálculo para a relação Benefício/Vínculo/Beneficiario/EventoDoBeneficiario
        $this->calculoBeneficioRepository->persistirCalculoDIRF_Mensal($nrcalculobenef, $nrbeneficiom, $nrvinculom, $nrPessoaBeneficiario, $proporcao, $vincBenefiarioEvento, 'B', $this->nrorgTrab, $this->competencia, $this->cdoperador);
        
        // Persiste o cálculo para a relação Benefício/Vínculo/Beneficiario/EventoMensalidade
        $this->calculoBeneficioRepository->persistirCalculoDIRF_Mensal($nrcalculobenef, $nrbeneficiom, $nrvinculom, $nrPessoaBeneficiario, $proporcao, $vincBenefiarioEvento_mens, 'M', $this->nrorgTrab, $this->competencia, $this->cdoperador);
        
        // Persiste o cálculo para a relação Benefício/Vínculo/Beneficiario/EventoVínculo
        $this->calculoBeneficioRepository->persistirCalculoDIRF_Mensal($nrcalculobenef, $nrbeneficiom, $nrvinculom, $nrPessoaBeneficiario, $proporcao, $vincBenefiarioEvento_vinc, 'V', $this->nrorgTrab, $this->competencia, $this->cdoperador);
        
        // Persiste o cálculo para a relação Benefício/Vínculo/Beneficiario/EventoEmpresa
        $this->calculoBeneficioRepository->persistirCalculoDIRF_Mensal($nrcalculobenef, $nrbeneficiom, $nrvinculom, $nrPessoaBeneficiario, $proporcao, $vincBenefiarioEvento_empresa, 'E', $this->nrorgTrab, $this->competencia, $this->cdoperador);
    }
    
    public function validaQtdeBeneficiariosPerEvento(&$qtdBeneficiariosPerEvento, $eventos, $evento, $vlrAnterior = null, $vlrPosterior = null, $qtBeneficio = null){
        try {
            if( is_null($vlrAnterior) && is_null($vlrPosterior) ){
    
                if( $eventos[$evento] > 0 ) // Se flag for 1, e se o evento for calculado novamente p/ o msm beneficiario, a 'qtde' não aumentará
                    $qtdBeneficiariosPerEvento[$evento] = ['qtde'=>1, 'flag'=>1, 'quantidadeBenef' => $qtBeneficio];
                else
                    $qtdBeneficiariosPerEvento[$evento] = ['qtde'=>0, 'flag'=>0];
                    
            }else if( $qtdBeneficiariosPerEvento[$evento]['flag'] == 0 && $vlrAnterior != $vlrPosterior ){
                
                $qtdBeneficiariosPerEvento[$evento]['qtde']++;
                $qtdBeneficiariosPerEvento[$evento]['flag']++;
    
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function calcularEventoBeneficiario(&$vincBenefiarioEvento, $eventoBeneficiario, $beneficio, $beneficiario, $nrtabelafolha, $proporcao, $tpperiodo, $vlrBeneficio = 0){
        try {
            $vlrArredondado = $vlrNaoArredondado = null;
            
            $qtdeBeneficio = $beneficiario->getQtbeneficio();
            $qtdeBeneficio = $qtdeBeneficio ? $qtdeBeneficio : 1; 
            
            $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
                $eventoBeneficiario, 
                $this->nrorgTrab, 
                0, 
                $this->competencia 
            );
            
            switch($beneficiario->getIdvalordifebenef()){
                case 'REFERENCIA':
                    
                    $referencia = $this->valorReferenciaService->getReferenciaPorNrValorReferencia(
                        $this->nrorgTrab, 
                        $this->competencia, 
                        $beneficiario->getNrvalorreferenciam()
                    );
                    
                    if($tpperiodo == 'MENSAL'){
                        $vlrArredondado = Valor::arredonda(
                            ($referencia['VRREFERENCIA'] * $proporcao), // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
                        $vlrBeneficio += $benefiarioEventos[$eventoBeneficiario] = ($vlrArredondado * $qtdeBeneficio);
                        
                        $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Referencia) '.$referencia['VRREFERENCIA'].' = ('.$vlrArredondado.' * (Quantidade) '.$qtdeBeneficio.') = '.($vlrArredondado * $qtdeBeneficio);
    
                        #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, 0, 0, null, $beneficiario->getIdvalordifebenef(), $referencia['VRREFERENCIA']);
                    
                    }elseif($tpperiodo == 'DIARIO'){ // Como no cálculo diario há a multiplicação pelo número de dias, o arredondamento deve ocorrer depois dessa multiplicação.
                        $vlrNaoArredondado = ($referencia['VRREFERENCIA'] * $proporcao); // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                        $vlrBeneficio += $benefiarioEventos[$eventoBeneficiario] = ($vlrNaoArredondado * $qtdeBeneficio);
                        
                        $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Referencia) '.$referencia['VRREFERENCIA'].' = ('.$vlrArredondado.' * (Quantidade) '.$qtdeBeneficio.') = '.($vlrNaoArredondado * $qtdeBeneficio);
                        #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, 0, 0, null, $beneficiario->getIdvalordifebenef(), $referencia['VRREFERENCIA']);
                    }
                    
                    break;
                case 'TABELA':
                    $vlrBeneficioTabela = $this->calcularValorTipoTabela($beneficiario, $nrtabelafolha); 
    
                    if($tpperiodo == 'MENSAL'){
                        $vlrArredondado = Valor::arredonda(
                            ($vlrBeneficioTabela * $proporcao), // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
                        $vlrBeneficio += $benefiarioEventos[$eventoBeneficiario] = ($vlrArredondado * $qtdeBeneficio); 
                        
                        $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Vr Tabela) '.$vlrBeneficioTabela.' = ('.$vlrArredondado.' * (Quantidade) '.$qtdeBeneficio.') = '.($vlrArredondado * $qtdeBeneficio);
    
                        #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, 0, 0, $vlrBeneficioTabela, $beneficiario->getIdvalordifebenef(), null);
                    
                    }elseif($tpperiodo == 'DIARIO'){ // Como no cálculo diario há a multiplicação pelo número de dias, o arredondamento deve ocorrer depois dessa multiplicação.
                        $vlrNaoArredondado = ($vlrBeneficioTabela * $proporcao); // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                        $vlrBeneficio += $benefiarioEventos[$eventoBeneficiario] = ($vlrNaoArredondado * $qtdeBeneficio);
                        
                        $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Vr Tabela) '.$vlrBeneficioTabela.' = ('.$vlrArredondado.' * (Quantidade) '.$qtdeBeneficio.') = '.($vlrNaoArredondado * $qtdeBeneficio);
                        #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, 0, 0, $vlrBeneficioTabela, $beneficiario->getIdvalordifebenef(), null);
                    }
                    
                    break;
                case 'VALOR':
    
                    if($tpperiodo == 'MENSAL'){
                        $vlrArredondado = Valor::arredonda(
                            ($beneficiario->getVrbeneficio() * $proporcao), // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                            $evento->historico->getIdtipoarredonda(), 
                            $evento->historico->getNrdigarredonda(),
                            $evento->historico->getVrlimitearred()
                        );
                        $vlrBeneficio += $benefiarioEventos[$eventoBeneficiario] = (($vlrArredondado *  $qtdeBeneficio) / (!empty($this->fixaDias) ? $this->fixaDias : 30 )) * $this->diasNormais;
                        
                        $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Valor) '.$beneficiario->getVrbeneficio().' = ('.$vlrArredondado.' * (Quantidade) '.$qtdeBeneficio.') / '.(!empty($this->fixaDias) ? $this->fixaDias : 30 ).' ) *  '.$this->diasNormais.'(Dias Normais) ) = '.$vlrBeneficio;
    
                        #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, 0, 0, null, $beneficiario->getIdvalordifebenef(), null);
                    
                    }elseif($tpperiodo == 'DIARIO'){ // Como no cálculo diario há a multiplicação pelo número de dias, o arredondamento deve ocorrer depois dessa multiplicação.
                        $vlrNaoArredondado = ($beneficiario->getVrbeneficio() * $proporcao); // Caso seja benefício mensal e deva haver proporcionalização por dias trabalhados, este parâmetro real já é passado proporcionalizado
                        $vlrBeneficio += $benefiarioEventos[$eventoBeneficiario] = ($vlrNaoArredondado *  $qtdeBeneficio);
                        
                        $calculoBeneficio = '(Proporcao) '.$proporcao.' * (Valor) '.$beneficiario->getVrbeneficio().' = ('.$vlrArredondado.' * (Quantidade) '.$qtdeBeneficio.') = '.($vlrNaoArredondado * $qtdeBeneficio);
                        #MemoriaBeneficio::getInstance()->setCalcBenefVinc($evento->getNreventom(), $vlrBeneficio, $calculoBeneficio, $proporcao, 0, 0, 0, null, $beneficiario->getIdvalordifebenef(), null);
                    }  
                    
                    break;
            }
            
            return $vlrBeneficio;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function calcularValorTipoTabela($beneficiario, $nrtabelafolha, $tpCalc = 'faixaEtaria'){
        try {
            $dtcompetencia = DateUtil::getDataDeString($this->competencia);
            // $faixasTabela = FpaFaixatabela::findByNrtabelafolha($nrtabelafolha);
            $faixasTabela = $this->entityManager->getRepository(Repositories::FPA_FAIXATABELA)->retornaFaixasPorTabela($nrtabelafolha, $dtcompetencia);
            $pessoa = GpePessoa::findOneByNrorgAndNrpessoa($beneficiario->getNrorg(), $beneficiario->getNrpessoa());
            $vlrBeneficioTabela = 0;
            $idadeOuSalario = null;
            
            if (is_object($pessoa)) {
                
                if($tpCalc === 'faixaEtaria'){
                    
                    $dtNascPessoa = $pessoa->getHistorico($dtcompetencia)->getDtnascpessoa();
                    
                    $objDtCompetencia = DateUtil::getDataDeString( substr($dtcompetencia->format('d/m/Y'), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
                    
    
                    if (($dtNascPessoa instanceof DateTime) === false) 
                        $dtNascPessoa = DateUtil::getDataDeString( substr($dtNascPessoa, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
                    else 
                        $dtNascPessoa = DateUtil::getDataDeString( $dtNascPessoa->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
                    
    
                    if(    $dtNascPessoa // Se houver a data de nascimento cadastrada
                        && $objDtCompetencia->format('m/d') <= $dtNascPessoa->format('m/d') // E se o aniversário estiver dentro da competencia,
                        && $dtNascPessoa->format('m/d') <= $objDtCompetencia->format('m/t') // este será equivalente ao primeiro dia da competência
                    ){  
                        $dtNascPessoa = DateUtil::getDataDeString( $dtNascPessoa->format('01/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
                    }    
    
                    $idadeOuSalario = $dtNascPessoa ? $dtNascPessoa->diff($objDtCompetencia)->format('%y') : null;
    
                }else if($tpCalc === 'faixaSalarial'){
                    $alteSalario = $this->calculoBeneficioRepository->getSalarioPorCompetencia(
                        $this->nrorgTrab, 
                        $beneficiario->getNrvinculom(), 
                        $this->competencia
                    );
                    $idadeOuSalario = $alteSalario['VRSALARIO'];
                }
    
                foreach ($faixasTabela as $faixa) {
                    
                    if ( isset($idadeOuSalario) 
                         && $idadeOuSalario >= $faixa->getVrlimitsupfaixa() 
                         && $idadeOuSalario <= $faixa->getVrmaxresultcalc() 
                      ){
                        $percentCalc = $faixa->getVrpecalculoresu();
                        $vraplicaresult = $faixa->getVraplicaresult();
                        if(isset($percentCalc) && !empty($percentCalc)){
                            $vlrBeneficioTabela += $vraplicaresult*($percentCalc/100);    
                        }else{
                            $vlrBeneficioTabela += $vraplicaresult;    
                        }                                
                    }
                    
                }
    
            }
    
            return $vlrBeneficioTabela;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function proporcionalizarOuDuplicarDiasTrabalhadosNaCompetencia($dtAdmissaoVinc, $dtRescisaoVinc, $beneficio, $beneficiario, $competenciaAtual, $competenciaPosterior){
        try {
            /*
                Parte-se da premissa que um vínculo admitido numa competência, não será rescindido ou terá benefício cancelado ainda nesta.
                Parte-se da premissa que um vínculo rescindido numa competência, não terá sido admitido ou ter benefício inscrito ainda nesta.
            */
            $qtdDiasTrabalhados = null;
            $proporcao = 1;
            
            $ultimoDiaMes = (int)substr($this->UltDiaCompetencia, 0, 2);
            $diaAdmissao  = (int)substr($dtAdmissaoVinc, 8, 2);
            $diaRescisao  = isset($dtRescisaoVinc) ? (int)substr($dtRescisaoVinc, 8, 2): null;
    
            $dtinscrbeneficiario = substr($beneficiario['DTINSCBENEFICIARIO'], 0, 10);
            $dtinscrbeneficiario  = $dtinscrbeneficiario ? DateUtil::getDataDeString($dtinscrbeneficiario, DateUtil::FORMATO_BRASILEIRO)->format('Y-m-d') : null;   
            $diaInscrBeneficiario = (int)substr($dtinscrbeneficiario, 8, 2);
    
            $dtcancbeneficiario = null != $beneficiario['DTCANCBENEFICIARIO'] ? substr($beneficiario['DTCANCBENEFICIARIO'], 0, 10) : null;
            $dtcancbeneficiario  = $dtcancbeneficiario ? DateUtil::getDataDeString($dtcancbeneficiario, DateUtil::FORMATO_BRASILEIRO)->format('Y-m-d') : null;   
            $diaCancBeneficiario = $dtcancbeneficiario ? (int)substr($dtcancbeneficiario, 8, 2) : null;
            
            $compEmObjetoDateTime = clone DateUtil::getDataDeString($this->competencia, DateUtil::FORMATO_BRASILEIRO, true);
            $compAnterior = $compEmObjetoDateTime->sub(new \DateInterval('P1M'))->format('d/m/Y');
            
            $competAnterior = DateUtil::getDataDeString($compAnterior)->format('Y-m-d');
            
            // Caso seja para duplicar a mensalidade e também para proporcionalizar ou na admissão ou na inscrição, na competência corrente a mensalidade será proporcionalizada e apenas na seguinte será duplicada.
            // Em outras palavras, apenas quando a data de inscrição do beneficiário estiver na competencia anterior, é que a mensalidade será duplicada.
            if((isset($beneficio['IDDUPLICMENS']) && $beneficio['IDDUPLICMENS'] == 'S') && ((isset($beneficio['IDPROPADMIS']) && $beneficio['IDPROPADMIS'] == 'S') || (isset($beneficio['IDPROPDTINSCR']) && $beneficio['IDPROPDTINSCR'] == 'S')) && ($competAnterior <= $dtinscrbeneficiario && $dtinscrbeneficiario < $competenciaAtual)){
                $proporcao = 2;
            // Caso seja para duplicar a mensalidade mas não para proporcionalizar nem na admissão nem na inscriçao, apenas duplicará se a data de inscrição estiver na competência corrent
            }else if((isset($beneficio['IDDUPLICMENS']) && $beneficio['IDDUPLICMENS'] == 'S') && (isset($beneficio['IDPROPADMIS']) && $beneficio['IDPROPADMIS'] == 'N') && (isset($beneficio['IDPROPDTINSCR']) && $beneficio['IDPROPDTINSCR'] == 'N') && ($competenciaAtual <= $dtinscrbeneficiario && $dtinscrbeneficiario < $competenciaPosterior)){
                $proporcao = 2;
            // Caso não seja para duplicar a mensalidade, mas seja para para proporcionalizar, proporcionalizará normalmente na competência corrente
            }else{
    
                /*
                    - Caso as ambas opções 'Proporcionaliza na Admissão?' e 'Proporciona conforme data de inscrição ?' estiverem marcadas, 
                        e data de admissão e inscrição estiverem na competênca corrente, será considerada a data mais avançada para proporcionalizar.
                    - Caso apenas a opção 'Proporcionaliza na Admissão?' esteja marcada e estiver na competência vigente, 
                        será considerada esta data para proporcionalizar.
                    - Caso apenas a opção 'Proporciona conforme data de inscrição ?' esteja marcada e estiver na competência vigente,  
                        será considerada esta data para proporcionalizar.
                */
                if((isset($beneficio['IDPROPADMIS']) && $beneficio['IDPROPADMIS'] == 'S') || (isset($beneficio['IDPROPDTINSCR']) && $beneficio['IDPROPDTINSCR'] == 'S')){
                    $diffAdmissao = $diffDtIniVig = null;
                    if( $beneficio['IDPROPADMIS'] == 'S' && $competenciaAtual < $dtAdmissaoVinc && $dtAdmissaoVinc < $competenciaPosterior )  $diffAdmissao = $ultimoDiaMes - $diaAdmissao + 1;
                    if( $beneficio['IDPROPDTINSCR'] == 'S' && $competenciaAtual < $dtinscrbeneficiario && $dtinscrbeneficiario < $competenciaPosterior ) $diffDtIniVig = $ultimoDiaMes - $diaInscrBeneficiario + 1;
                    
                    if(isset($diffAdmissao) && isset($diffDtIniVig)) $qtdDiasTrabalhados = min($diffAdmissao, $diffDtIniVig); // dias trabalhados será a menor diferença entre (admissão e última dia) e (inscrição e último dia)
                    else if(isset($diffAdmissao)) $qtdDiasTrabalhados = $diffAdmissao;
                    else if(isset($diffDtIniVig)) $qtdDiasTrabalhados = $diffDtIniVig;
                }
                /*
                    - Caso as ambas opções 'Proporcionaliza na Rescisão?' e 'Proporciona conforme data de cancelamento ?' estiverem marcadas, 
                        e data de rescisão e cancelamento estiverem na competênca corrente, será considerada a data menos avançada para proporcionalizar.
                    - Caso apenas a opção 'Proporcionaliza na Rescisão?' esteja marcada e estiver na competência vigente, 
                        será considerada esta data para proporcionalizar.
                    - Caso apenas a opção 'Proporciona conforme data de cancelamento ?' esteja marcada e estiver na competência vigente,  
                        será considerada esta data para proporcionalizar.
                */
                
                if((isset($beneficio['IDPROPRESC']) && $beneficio['IDPROPRESC'] == 'S') || (isset($beneficio['IDPROPDTCANC']) && $beneficio['IDPROPDTCANC'] == 'S')){
                    $diffRescisao = $diffCancBeneficiario = null;
                    if( isset($dtRescisaoVinc) && $beneficio['IDPROPRESC'] == 'S' && $competenciaAtual < $dtRescisaoVinc && $dtRescisaoVinc < $competenciaPosterior )  $diffRescisao = $diaRescisao;
                    if( isset($dtcancbeneficiario) && $beneficio['IDPROPDTINSCR'] == 'S' && $competenciaAtual < $dtcancbeneficiario && $dtcancbeneficiario < $competenciaPosterior ) $diffCancBeneficiario = $diaCancBeneficiario;
                    
                    if(isset($diffRescisao) && isset($diffCancBeneficiario)) $qtdDiasTrabalhados = min($diffRescisao, $diffCancBeneficiario); // dias trabalhados será a menor data entre rescisão e cancelamento
                    else if(isset($diffRescisao)) $qtdDiasTrabalhados = $diffRescisao;
                    else if(isset($diffCancBeneficiario)) $qtdDiasTrabalhados = $diffCancBeneficiario;
                }
                if(isset($qtdDiasTrabalhados)) $proporcao = $qtdDiasTrabalhados/30;
    
            }
            
            return $proporcao;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function calcularDiasTrabalhadosMesAdmissaoRescisao($nrorg, $competencia, $beneficio, $vinculo, &$tpCompetencia = null){
        try {
            $dtAdmissaoVinc = DateUtil::getDataDeString( substr($vinculo->getDtadmissaovinc(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            $dtRescisaoVinc = null;
            if($vinculo->getDtrescisaovinc()){
                $dtRescisaoVinc = DateUtil::getDataDeString( substr($vinculo->getDtrescisaovinc(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            }
            $objCompetenc = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO, true);
            
            // ===== CAMPO(S): "Gera Benefício somente para vínculos com mais de x dias trabalhados no mês:" e "Retroceder quantos meses para apurar os dias trabalhados (frente ao período)?"    
            // - Função responsável por calcular a quantidade de dias trabalhados na competência informada pelo minimizador diário para validar minimizador do Benefício CCT
            $objRetrocedido = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO, true);
            if((isset($beneficio['QTDIASMINMES']) && !empty($beneficio['QTDIASMINMES']) && (!empty($beneficio['QTMESRETROCDIASTRAB']) && $beneficio['QTMESRETROCDIASTRAB'] > 0) )){
                $objRetrocedido = $objRetrocedido->sub(new \DateInterval('P'.$beneficio['QTMESRETROCDIASTRAB'].'M'));
                $objCompetenc = $objCompetenc->sub(new \DateInterval('P'.$beneficio['QTMESRETROCDIASTRAB'].'M'));
            }
            
            $this->objcompetenciaAdmissao = $objCompetenc;
            
            $diasTrabalhados = null;
            
            if( $dtAdmissaoVinc->format('Y/m') == $objCompetenc->format('Y/m') || (isset($dtRescisaoVinc) && $dtRescisaoVinc->format('Y/m') == $objCompetenc->format('Y/m')) ) {
                // Calculará a quatidade de dias trabalhados nos seguintes casos:
                if( $dtAdmissaoVinc->format('Y/m') == $objCompetenc->format('Y/m') && ( isset($dtRescisaoVinc) && $dtRescisaoVinc->format('Y/m') == $objCompetenc->format('Y/m')) ){ // Caso admissão e rescisão estejam na competência
                    $diasTrabalhados = intval( $dtRescisaoVinc->format('d') ) - intval($dtAdmissaoVinc->format('d')) + 1;                                         
                    $tpCompetencia = "'Admissão/Rescisão'";
                }elseif( $dtAdmissaoVinc->format('Y/m') == $objCompetenc->format('Y/m') ){ // Caso apenas admissão esteja na competência
                    // Caso o mês seja fevereiro considera 30 dias da mesma forma que a base da folha de pagamento
                    if( $objCompetenc->format('m') == '02' ){
                        $ultimoDiaComp = 30;
                    }else{
                        $ultimoDiaComp = intval( $objCompetenc->format('t') );
                    }
                    $diasTrabalhados = $ultimoDiaComp - intval($dtAdmissaoVinc->format('d')) + 1; 
                    $tpCompetencia = "'Admissão'";
                }elseif ( isset($dtRescisaoVinc) && $dtRescisaoVinc->format('Y/m') == $objCompetenc->format('Y/m') ){ // Caso apenas rescisão esteja na competência
                    $diasTrabalhados = intval( $dtRescisaoVinc->format('d') ) - intval($objCompetenc->format('d')) + 1;
                    $tpCompetencia = "'Rescisão'";
                }    
            }else{
                $diasTrabalhados = $objRetrocedido->format('m') == '02' ? 30 : intval( $objRetrocedido->format('t') ); // Caso o mês seja fevereiro considera 30 dias da mesma forma que a base da folha de pagamento
            }
            
            return $diasTrabalhados;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function calcularDiasFaltas(array $minimizadores, $vinculo){
        try {
            // Inicializa o contador de dias de falta
            $diasFaltas = 0;
            // Inicializa a quantidade de meses retrocedidos com padrão "0"
            $mesesRetroceder = 0;
            
            $this->dtiniperiodobenef = DateUtil::getDataDeString(
                $this->hcmParameters->getDtcompetencia(), 
                DateUtil::FORMATO_BRASILEIRO, 
                true
            );
            
            // Verifica se é necessário minimizar faltas e retroceder meses
            if (isset($minimizadores['IDCONSFALTABENEF']) && $minimizadores['IDCONSFALTABENEF'] === 'S') {
                // Se sim, obtém a quantidade de meses para retroceder
                $mesesRetroceder = isset($minimizadores['QTMESESFALTABENEF']) ? (int)$minimizadores['QTMESESFALTABENEF'] : 0;
            }
            
            if($this->deveCalcularFaltasPorSituacaoFuncional == true){
                // Calcula a quantidade de faltas com base nos parâmetros fornecidos (Situação funcional)
                $diasFaltas = $this->calcularFaltasSitufunc($diasFaltas, $mesesRetroceder, $vinculo->getNrvinculom(), $vinculo);
            } else {
                $diasFaltas = $this->calcularQtdeFaltas($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getDtcompetencia(), $diasFaltas, $mesesRetroceder, $vinculo->getNrvinculom(), $vinculo);
            }
                
            return $diasFaltas;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    //Responsavel por buscar situações funcionais de faltas
    public function retornaArraySituFuncFaltas($nrvinculom, $nrsitufuncm, $dtInicio, $dtFim){
        $arraySituFuncFaltas = [];

        $where = "NRORG = :P_NRORG AND
                  NRVINCULOM = :P_NRVINCULOM AND
                  DTINISITUFUNC <= '$dtFim' AND (DTFIMSITUFUNC >= '$dtInicio' OR DTFIMSITUFUNC IS NULL) AND NRSITUFUNCM = :P_NRSITUFUNCM";
                       
        $param = array(
            'P_NRORG' => $this->hcmParameters->getNrorgTrabalho(),
            'P_NRVINCULOM' => $nrvinculom,
            'P_NRSITUFUNCM' => $nrsitufuncm);
    
        $altesitfuncs = GpeAltesitufunc::getObjectsByWhere($where, $param);
        
        foreach($altesitfuncs as $altesitfunc){
            array_push($arraySituFuncFaltas, $altesitfunc);
        }
        
        return $arraySituFuncFaltas;
    }
    
    private function getHistoricoEscalaRetroc($nrvinculom, $competencia, $iniperiodoAnaliseRetroc, $fimperiodoAnaliseRetroc){
        $competencia = $competencia;
        $iniperiodo = $iniperiodoAnaliseRetroc;
        $fimperiodo = $fimperiodoAnaliseRetroc;
        
        try {
             /**
             * @var GpeEscalatrabh $escalaTrab
             * @var GpeAlteescala $escalaTrabVinc
             */
            $params = array(    
                            'P_NRORG' => $this->hcmParameters->getNrorgTrabalho(),
                            'P_NRVINCULOM' => $nrvinculom,
                            'P_INIPERIODO' => $iniperiodoAnaliseRetroc,
                            'P_FIMPERIODO' => $fimperiodoAnaliseRetroc,
                            'P_COMPETENCIA' => $competencia,
                           );
                           
            $where = "e.NRVINCULOM = :P_NRVINCULOM 
                        AND e.NRORG = :P_NRORG
                        AND e.DTINIESCALA = (SELECT MAX(DTINIESCALA) DTINIESCALA 
                                                FROM GPE_ALTEESCALA MAX_ALTEESCALA 
                                                WHERE MAX_ALTEESCALA.DTINIESCALA <= :P_COMPETENCIA 
                                                AND MAX_ALTEESCALA.NRORG = e.NRORG 
                                                AND MAX_ALTEESCALA.NRVINCULOM = e.NRVINCULOM
                                           )";
            $escalaTrabVinc = GpeAlteescala::getObjectByWhere($where, $params);
            
            $nrturno = isset($escalaTrabVinc) && !empty($escalaTrabVinc) ? $escalaTrabVinc->getNrturno() : null; // Caso haja turno a contagem de dias deverá olhar apenas para o turno
            
            if (!empty($escalaTrabVinc)) {
                $params = array(
                    'P_NRORG' => $this->hcmParameters->getNrorgTrabalho(),
                    'P_NRESCALATRABM' => $escalaTrabVinc->getNrescalatrabm(), 
                    'P_DTMESCOMPETENC' => $competencia, 
                );
                $where = "NRORG = :P_NRORG 
                            AND e.NRESCALATRABM = :P_NRESCALATRABM 
                            AND e.DTMESCOMPETENC = (SELECT MAX(MAX_ESCALATRABH.DTMESCOMPETENC) 
                                                    FROM GPE_ESCALATRABH MAX_ESCALATRABH 
                                                    WHERE MAX_ESCALATRABH.DTMESCOMPETENC <= :P_DTMESCOMPETENC 
                                                    AND MAX_ESCALATRABH.NRESCALATRABM = e.NRESCALATRABM 
                                                    AND MAX_ESCALATRABH.NRORG = e.NRORG
                                                  )";
                $escalatrabalho = GpeEscalatrabh::getObjectByWhere($where, $params);
                
                // histórico é carregado novamente apenas para acessar os valores 'IDGERBENFER' e 'IDUTAPURATESTBENEF' 
                $historico = GpeEscalatrabm::_retornaHistorico(
                    $escalatrabalho->getNrescalatrabh(), 
                    $this->hcmParameters->getNrorgTrabalho()
                );
                
                return $historico;
            }
        }catch (\Exception $e){
            throw $e;
        }
    }
    
    private function calculaFaltasSituFunc($iniSitufunc, $fimSitufunc){
        $faltasSituFunc = 0;
        
        //Verificar se a data fim e maior do q a data inicio
        if($fimSitufunc > $iniSitufunc){
            //Calcula a diferença de dias entre essas datas.
            $qntDiasFaltasSitu = $iniSitufunc->diff($fimSitufunc, true)->d+1;
            $faltasSituFunc = $faltasSituFunc + $qntDiasFaltasSitu;
        } else {
            //Se caso a data fim for igual a data inicio adiciona somente uma falta.
            $faltasSituFunc++;
        }
        
        return $faltasSituFunc;
    }
    
    private function calcularFaltasSitufunc(&$diasFaltas, $mesesRetroceder, $nrvinculom, $vinculo){
        $dataFormularioIsUltimoDiaMes = false;
        $dtPeriodoInicioCalc = clone DateUtil::truncateData(DateUtil::getDataDeString($this->dtiniperiodobenefmod, DateUtil::FORMATO_BRASILEIRO, true));
        $dtPeriodoInicioCalc = $dtPeriodoInicioCalc->sub(new \DateInterval('P'.(string)$mesesRetroceder.'M'));
        
        $dtPeriodoFimCalc = clone DateUtil::getDataDeString($this->dtfimperiodobenefmod, DateUtil::FORMATO_BRASILEIRO, true);
        
        if($dtPeriodoFimCalc->format('d/m/Y') == $dtPeriodoFimCalc->format('t/m/Y')){
            $dataFormularioIsUltimoDiaMes = true;
        }
        
        $dtPeriodoFimCalc = $dtPeriodoFimCalc->sub(new \DateInterval('P'.(string)$mesesRetroceder.'M'));
        
        //Antes de buscar as datas das faltas do vinculo, precisamos verificar se a dataRetrocedidaFim e maior q o ultimo dia do mes da competencia retrocedida...
        $UltimoDiaDataRetrocedida = DateUtil::truncateData(DateUtil::getUltimoDiaDoMes($dtPeriodoInicioCalc));
        if($dtPeriodoFimCalc > $UltimoDiaDataRetrocedida){
            $dtPeriodoFimCalc = $UltimoDiaDataRetrocedida;
        }
        
        if(($dtPeriodoFimCalc->format('d/m/Y') != $dtPeriodoFimCalc->format('t/m/Y')) && $dataFormularioIsUltimoDiaMes == true){
            $dtPeriodoFimCalc = DateUtil::getDataDeString($dtPeriodoFimCalc->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
        }
        
        //EARLY RETURN SE CASO A DATA INSCRIÇÃO DO BENEFICIARIO FOR MAIOR QUE A DATA INICIO DE PERIODO DE CALCULO.
        //$dtPeriodoInicioCalc normalmente será retrocedida (Parametro do beneficio para buscar faltas retrocedendo x meses)
        //EXEMPLO: INSCRIÇÃO 01/06/2024 / BUSCA FALTAS NO MES 05 01/05/2024 (1 MES RETROCEDIDO) O vinculo ainda não possuia o beneficio então não ira descontar a falta.
        if($this->dtInscricaoBeneficiario > $dtPeriodoInicioCalc){
            $diasFaltas = 0;
            return $diasFaltas;
        }
        
        //Calculo de VT utilizando origem de faltas pela situação funcional falta(DIA) no periodo retrocedido.
        $arraySituFuncFaltas = $this->retornaArraySituFuncFaltas($nrvinculom, 100072, $dtPeriodoInicioCalc, $dtPeriodoFimCalc);
        
        $escalaTrabVinc = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->obterEscalaEntrePeriodo($dtPeriodoInicioCalc, $dtPeriodoFimCalc, $this->hcmParameters->getDtCompetencia(), $nrvinculom, $this->hcmParameters->getNrorgTrabalho());
        // Caso haja turno e o turno tenha horários cadastrados, a contagem de dias deverá olhar apenas para o turno. Senão calculará da forma antiga.
        $nrturno = isset($escalaTrabVinc[0]) && isset($escalaTrabVinc[0]['NRTURNO']) && !empty($escalaTrabVinc[0]['NRTURNO']) ? $escalaTrabVinc[0]['NRTURNO'] : null; 
        if($nrturno) $nrHorarioTurno = GpeHorarioturno::getObjectByWhere("e.NRORG = :P_NRORG AND e.NRTURNO = :P_NRTURNO", ['P_NRORG' => $this->hcmParameters->getNrorgTrabalho(), "P_NRTURNO" => $nrturno]);
        
        $historico = $this->getHistoricoEscalaRetroc($nrvinculom, $UltimoDiaDataRetrocedida, $dtPeriodoInicioCalc, $dtPeriodoFimCalc);
        
        $descansoRemuneradoEFeriado = array();
        $descansos = array();
        $feriados = array();
        $qtdeFaltas = 0;
        
        if(!empty($arraySituFuncFaltas)){
            foreach($arraySituFuncFaltas as $arraySituFuncFalta){
                $dataInicioSitufunc = $arraySituFuncFalta->getDtinisitufunc();
                $dataFimSitufunc = $arraySituFuncFalta->getDtfimsitufunc();
                
                // Se a data fim for nula, define como a data fim do periodo RETROCEDIDO do formulario do beneficio
                //Se a data fim for maior define a a data fim como a data fim do periodo RETROCEDIDO do formulario do beneficio.
                if (is_null($dataFimSitufunc) || $dataFimSitufunc > $dtPeriodoFimCalc) {
                    $dataFimSitufunc = $dtPeriodoFimCalc;
                }
                
                //Se a data inicio da situação funcional for menor do que a data inicio do periodo RETROCEDIDO do formulario do beneficio
                //Define a data inicio da situação funcional como a data inicio do periodo RETROCEDIDO do formulario do beneficio
                if($dataInicioSitufunc < $dtPeriodoInicioCalc){
                    $dataInicioSitufunc = $dtPeriodoInicioCalc;
                }
                
                if(!empty($nrturno) && isset($nrturno)){
                    $qnt = $dataInicioSitufunc->diff((!is_null($dataFimSitufunc) && $dataFimSitufunc <=  $dtPeriodoFimCalc) ? $dataFimSitufunc : $dtPeriodoFimCalc, true);
                    
                    $descansos = $this->calcularQtdeDiasFolgaVinc($dataInicioSitufunc, $qnt, $nrvinculom, $dtPeriodoInicioCalc->format('01/m/Y'));
                } else {
                    $dsDiasDescanso = $this->getDsDescansoSemanal( $vinculo->getNrvinculom() );
                    $descansos = $this->calcularQtdeDiasDSR($dtPeriodoInicioCalc, $dtPeriodoFimCalc, $dsDiasDescanso);
                }
                
                if((!empty($historico) && isset($historico)) && $historico['NRTPJORNADA'] == 2){
                    $feriados = array();
                } else {
                    $feriados = $this->calcularQtdeFeriados($dataInicioSitufunc, (!is_null($dataFimSitufunc) && $dataFimSitufunc <=  $dtPeriodoFimCalc) ? $dataFimSitufunc : $dtPeriodoFimCalc, $vinculo);
                }
                
                if(!empty($descansos)){
                    foreach($descansos as $descanso){
                        if(!in_array($descanso, $descansoRemuneradoEFeriado)){
                            $descansoRemuneradoEFeriado[] = $descanso;
                        }
                    }
                }
                
                if(!empty($feriados)){
                    foreach($feriados as $feriado){
                        if(!in_array($feriado, $descansoRemuneradoEFeriado)){
                            $descansoRemuneradoEFeriado[] = $feriado;
                        }
                    }
                }
                
                $qtdeFaltas += $this->calculaFaltasSituFunc($dataInicioSitufunc, $dataFimSitufunc);
            }
        }
        
        //Pega a quantidade de DSRs do vinculo, no periodo da situação funcional das faltas e diminui da quantidade de faltas.
        if($qtdeFaltas >= count($descansoRemuneradoEFeriado)){
            $qtdeFaltas = $qtdeFaltas - count($descansoRemuneradoEFeriado);
        } else {
            //Se o vinculo tiver DSR's + Feriados maior do que faltas a quantidade de faltas devem ser 0.
            $qtdeFaltas = 0;
        }
        
        return $qtdeFaltas ;
    }
    
    private function calcularQtdeFaltas($nrorg, $competencia, &$diasFaltas, $qtdeMesesRetroceder, $nrvinculom, $vinculo){
        try {
            $dtPeriodoInicioCalc = clone $this->dtiniperiodobenef;
            $nrtpmovimento = '1';
            $nreventom = "490, 495, 1040, 1043";
            
            //Evento que possui referencia em horas
            $nreventomHoras = "1041, 1042, 100105994";
            
            $dtmescomp_falta = DateUtil::getPrimeiroDiaDoMes( $dtPeriodoInicioCalc->sub(new \DateInterval('P'.(string)$qtdeMesesRetroceder.'M')) );
    
            $faltas = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->getSomaItemCalculo(
                $nrvinculom, 
                $nrorg, 
                $dtmescomp_falta, 
                $dtmescomp_falta, 
                $nreventom, 
                $nrtpmovimento, 
                $dtmescomp_falta
            );
            
            if(isset($faltas["VALOR"])){
                $diasFaltas = (int)$faltas['VALOR'];
            }
            
            //Modificação a pedido do artur, 19/09/2024 as 18:00 considerando as faltas que tem referencia em horas.
            if(!empty($this->jornadaDiaria)){
                $faltasHoras = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->getSomaItemCalculo(
                    $nrvinculom, 
                    $this->hcmParameters->getNrorgTrabalho(), 
                    $dtmescomp_falta, 
                    $dtmescomp_falta, 
                    $nreventomHoras, 
                    $nrtpmovimento, 
                    $dtmescomp_falta
                );
                
                if(isset($faltasHoras["VALOR"])){
                    $diasFaltas += $faltasHoras['VALOR'] / $this->jornadaDiaria;
                }
            }
            
            return $diasFaltas;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function setPeriodoApuracaoPelaEscala(&$dtiniperiodoAnt, &$dtfimperiodoAnt, $qtdeMesesRetroceder){
        try {
            $competenciaReferencia = DateUtil::getDataDeString($this->competencia, DateUtil::FORMATO_BRASILEIRO, true);
    
            // Subtrai (x-1) meses da competência para operar de acordo com a data final evidente na tela de escala
            if($qtdeMesesRetroceder>0){
                $competenciaReferencia = $competenciaReferencia->sub(new \DateInterval('P'.($qtdeMesesRetroceder-1).'M'));
            }
            $dtfimperiodoAnt = clone $competenciaReferencia;
            $dtfimperiodoAnt = $dtfimperiodoAnt->add(new \DateInterval('P'. ($this->diaFinalAupr-1).'D'));
            
            // Subtrai mais um mes para operar de acordo com a data inicial evidente na tela de escala
            $competenciaReferencia = $competenciaReferencia->sub(new \DateInterval('P1M')); 
            $dtiniperiodoAnt = clone $competenciaReferencia;
            $dtiniperiodoAnt = $dtiniperiodoAnt->add(new \DateInterval('P'. ($this->diaInicialAupr-1).'D'));
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function calcularQtdeDias_AtesOuAfast(&$dias_AtesOuAfast, $id_tpCalc, $qtdeMesesRetroceder, $vinculo, $dsDiasDescanso, $id_minimizadorFeriado, $id_minimizadorDSR){
        try {
            $dataFormularioIsUltimoDiaMes = false;
            $dtPeriodoInicioCalc = clone $this->dtiniperiodobenef;
            $dtPeriodoInicioCalc->sub( new \DateInterval('P'.(string)$qtdeMesesRetroceder.'M') );
            
            $dtPeriodoFimCalc    = clone $this->dtfimperiodobenef;
            
            if($dtPeriodoFimCalc->format('d/m/Y') == $dtPeriodoFimCalc->format('t/m/Y')){
                $dataFormularioIsUltimoDiaMes = true;
            }
        
            $dtPeriodoFimCalc->sub( new \DateInterval('P'.(string)$qtdeMesesRetroceder.'M') );
    
            if($dtPeriodoInicioCalc->format('m') == 2){
                while(($dtPeriodoFimCalc->format('m') > $dtPeriodoInicioCalc->format('m')) && $qtdeMesesRetroceder != 0){
                    $dtPeriodoFimCalc = $dtPeriodoFimCalc->sub(new \DateInterval('P1D'));
                }
            }
            
            if(($dtPeriodoFimCalc->format('d/m/Y') != $dtPeriodoFimCalc->format('t/m/Y')) && $dataFormularioIsUltimoDiaMes == true){
                $dtPeriodoFimCalc = DateUtil::getDataDeString($dtPeriodoFimCalc->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
            }
    
            // Caso seja para contar dias de Atestado e na escala do vínculo tenha período pré-definido e id 'Utiliza para Atestados' preenchido,
            // função abaixo adequará perído de consideração de atestado segundo o informado no histórico da Escala do vínculo
            if($id_tpCalc == 'ATESTADO' && $this->diaInicialAupr && $this->diaFinalAupr) $this->setPeriodoApuracaoPelaEscala($dtPeriodoInicioCalc, $dtPeriodoFimCalc, $qtdeMesesRetroceder);
            
            $dtmescompetencia = $this->competencia;
            $nrorg = $this->nrorgTrab;
    
            $listNrsitufuncm = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCH)->getAfastamentoIncideVale($dtmescompetencia);
            $listNrsitufuncm = implode(', ', $listNrsitufuncm);
            
            $altesitfuncs = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncional_AtesOuAfast(
                $this->nrorgTrab, 
                $vinculo->getNrvinculom(), 
                $listNrsitufuncm, 
                $dtPeriodoInicioCalc, 
                $dtPeriodoFimCalc,
                $id_tpCalc
            );
        
            foreach($altesitfuncs as $altesitfunc){
                $dtInic = $dtFim = null;
                
                if ( $altesitfunc->getDtinisitufunc() < $dtPeriodoInicioCalc ){
                    $dtInic = $dtPeriodoInicioCalc;
                } else {
                    $dtInic = $altesitfunc->getDtinisitufunc();
                }
                
                //Caso a situação funcional não possual data final, considera o dia anterior da data inicial da próxima situação que gera vale
                if (!$altesitfunc->getDtfimsitufunc()) {
                    
                    $dtinisitufunc = $altesitfunc->getDtinisitufunc();
                    
                    $prox_altesitufunc = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaProximaSituFuncional(
                        $this->nrorgTrab, 
                        $vinculo->getNrvinculom(),
                        $dtinisitufunc,
                        $dtPeriodoFimCalc,
                        $listNrsitufuncm
                    );
                    
    
                    if ($prox_altesitufunc) 
                        //Dia anterior da próxima situação geradora de vale
                        $dtFim = end($prox_altesitufunc)->getDtinisitufunc()->sub(new \DateInterval('P1D'));
                    else 
                        //Caso não tenha próxima situação que gera vale considera a data final do periodo de apuração
                        $dtFim = clone $dtPeriodoFimCalc;
    
                }else {
                    if ($altesitfunc->getDtfimsitufunc() > $dtPeriodoFimCalc){
                        $dtFim = $dtPeriodoFimCalc;
                    } else {
                        $dtFim = $altesitfunc->getDtfimsitufunc();
                    }
                }
                
                if ($dtFim >= $dtInic) {
                    $arFeriados = $arDsr = $arrayDiasFolgaVinc = $arrayDiasAtestado = [];
    
                    $qnt = $dtFim->diff($dtInic, true);
                    
                    $diasTotaisAtestOuAfast = ($qnt->d + $qnt->m*30 + $qnt->y*365 + 1);
                    
                    if($id_minimizadorFeriado == 'S') $arFeriados = $this->calcularQtdeFeriados($dtInic, $dtFim, $vinculo);
                    $feriados = count($arFeriados);
                    
                    if( isset($this->nrturno)  && $this->nrHorarioTurno){ // Deverá olhar para o turno apenas caso esteja na alteração de escala do vínculo e possua horários cadastrados
                        //Dias/Horario de Afastamentos ou atestados que estão em dias normais mas não geram beneficio diario para minimizar os dias totais de atestado / afastamento.
                        $minimizaDiasAtestOuAfastNgeraBenefDiario = GpeEscalatrabm::retornaQtdeDiasNgeraBenefDiario(
                            $this->hcmParameters->getDtCompetencia(), 
                            $dtInic->format('d/m/Y'), 
                            $qnt->d, 
                            $this->hcmParameters->getNrorgTrabalho(), 
                            $this->nrturno
                        );
                        
                        $arrayDiasFolgaVinc = $this->calcularQtdeDiasFolgaVinc($dtInic, $qnt, $vinculo->getNrvinculom()); 
                        $diasFolga = count($arrayDiasFolgaVinc);
                        $intersecaoFolgaFeriado = count(array_intersect($arrayDiasFolgaVinc, $arFeriados)); //quantidade de dias que coincidem Atestado/Afastamento e Feriados
                        $dias_AtesOuAfast += ($diasTotaisAtestOuAfast - $minimizaDiasAtestOuAfastNgeraBenefDiario) - $feriados - $diasFolga + $intersecaoFolgaFeriado;
                    }else{
                        $intersecaoDsrFeriado = count(array_intersect($arDsr, $arFeriados)); // quantidade de dias que coincidem DSR e Feriados
                    
                        if($id_minimizadorDSR == 'S') $arDsr = $this->calcularQtdeDiasDSR($dtInic, $dtFim, null);
                        $dsr = count($arDsr);
                        
                        $dias_AtesOuAfast += $diasTotaisAtestOuAfast - $feriados - $dsr + $intersecaoDsrFeriado;
                    }
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function calcularQtdeDiasFolgaVinc($dtInic, $dias, $nrVinculo, $competenciaForcada = null){
        try {
            $array = [];
            
            $diasDeFolgaNoPeriodo = GpeEscalatrabm::retornaDiasFolgaVinc(
                !empty($competenciaForcada) ? $competenciaForcada : $this->competencia, 
                $dtInic->format('d/m/Y'), 
                $dias->d, 
                $this->nrorgTrab, 
                $nrVinculo
            );
    
            foreach($diasDeFolgaNoPeriodo as $value) $array[] = $value['DTDATA'];
    
            return $array;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function calcularQtdeDiasDSR($dtIni, $dtFim, $dsDiasDescanso){
        try {
            $dtiniperiodoapur = clone $dtIni;
            $dtfimperiodoapur = clone $dtFim;
    
            $diasDsr = array();
    
            switch($dsDiasDescanso){
                case (self::SABADO):
                    while($dtiniperiodoapur->format('N') != 6){
                        $dtiniperiodoapur->add(new \DateInterval('P1D'));
                    }
                    while($dtiniperiodoapur<=$dtfimperiodoapur){
                        $diasDsr[] = $dtiniperiodoapur->format('d/m/Y');
                        $dtiniperiodoapur->add(new \DateInterval('P7D'));
                    }
                    break;
                
                
                case (self::DOMINGO):
                    while($dtiniperiodoapur->format('N') != 7){
                        $dtiniperiodoapur->add(new \DateInterval('P1D'));
                    }
                    while($dtiniperiodoapur<=$dtfimperiodoapur){
                        $diasDsr[] = $dtiniperiodoapur->format('d/m/Y');
                        $dtiniperiodoapur->add(new \DateInterval('P7D'));
                    }
                    break;
                
                
                case (self::SABDOM):
                    while($dtiniperiodoapur->format('N') != 7 and $dtiniperiodoapur->format('N') != 6){
                        $dtiniperiodoapur->add(new \DateInterval('P1D'));
                    }
                    if($dtiniperiodoapur->format('N') == 7){
                        $diasDsr[] = $dtiniperiodoapur->format('d/m/Y');
                        $dtiniperiodoapur->add(new \DateInterval('P6D'));
                    }
                    while($dtiniperiodoapur<$dtfimperiodoapur){
                        $diasDsr[] = $dtiniperiodoapur->format('d/m/Y');
                        $dtiniperiodoapur->add(new \DateInterval('P1D'));
                        $diasDsr[] = $dtiniperiodoapur->format('d/m/Y');
                        $dtiniperiodoapur->add(new \DateInterval('P6D'));
                    }
                    if(($dtfimperiodoapur->format('N') == 6 or $dtfimperiodoapur->format('N') == 7) and ($dtiniperiodoapur<=$dtfimperiodoapur)){
                        $diasDsr[] = $dtfimperiodoapur->format('d/m/Y');
                    }
                    break;
                
                
                case (self::SEGUNDA):
                    while($dtiniperiodoapur->format('N') != 1){
                        $dtiniperiodoapur->add(new \DateInterval('P1D'));
                    }
                    while($dtiniperiodoapur<=$dtfimperiodoapur){
                        $diasDsr[] = $dtiniperiodoapur->format('d/m/Y');
                        $dtiniperiodoapur->add(new \DateInterval('P7D'));
                    }
                    break;
                
                
                case (self::TERCA):
                    while($dtiniperiodoapur->format('N') != 1){
                        $dtiniperiodoapur->add(new \DateInterval('P1D'));
                    }
                    while($dtiniperiodoapur<=$dtfimperiodoapur){
                        $diasDsr[] = $dtiniperiodoapur->format('d/m/Y');
                        $dtiniperiodoapur->add(new \DateInterval('P7D'));
                    }
                    break;
                
                
                case (self::QUARTA):
                    while($dtiniperiodoapur->format('N') != 3){
                        $dtiniperiodoapur->add(new \DateInterval('P1D'));
                    }
                    while($dtiniperiodoapur<=$dtfimperiodoapur){
                        $diasDsr[] = $dtiniperiodoapur->format('d/m/Y');
                        $dtiniperiodoapur->add(new \DateInterval('P7D'));
                    }
                    break;
                
                
                case (self::QUINTA):
                    while($dtiniperiodoapur->format('N') != 4){
                        $dtiniperiodoapur->add(new \DateInterval('P1D'));
                    }
                    while($dtiniperiodoapur<=$dtfimperiodoapur){
                        $diasDsr[] = $dtiniperiodoapur->format('d/m/Y');
                        $dtiniperiodoapur->add(new \DateInterval('P7D'));
                    }
                    break;
                
                
                case (self::SEXTA):
                    while($dtiniperiodoapur->format('N') != 5){
                        $dtiniperiodoapur->add(new \DateInterval('P1D'));
                    }
                    while($dtiniperiodoapur<=$dtfimperiodoapur){
                        $diasDsr[] = $dtiniperiodoapur->format('d/m/Y');
                        $dtiniperiodoapur->add(new \DateInterval('P7D'));
                    }
                    break;
                
            }
            return $diasDsr;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function calcularQtdeFeriados($dtiniperiodoapur, $dtfimperiodoapur, $vinculo){
        try {
            $result = array();
            $vinculoh = $this->vinculoh;
            
            if(is_object($vinculoh)){
                $nrestrutlegal = $vinculoh->getNrestrutlegal();
            }
            
            $parametrosOrg = $this->entityManager->getRepository(Repositories::GPE_PARAMORG)->findOneBy(array('nrorg'=>$this->nrorgTrab));
            
            //Busca calendario da estrutra de movimentacao.
            if($parametrosOrg->getNrtpestrutcalend()){
                
                $movimentacoes = $this->_movimentosVinculo->carregaMovimentacoesPorTipoEstrutura($vinculo->getNrorg(), $vinculo->getNrvinculom(), $this->UltDiaCompetencia, $parametrosOrg->getNrtpestrutcalend());
                
                if($movimentacoes){
                    
                    foreach ($movimentacoes as $movimentacao){
                        $nrestrutlegal = $movimentacao->getNrestruturam();
                    }
                }
            }
            
            if($this->geraBenefFeriado == 'N'){
                $feriados = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->getFeriados(
                    $dtiniperiodoapur->format('d/m/Y'), 
                    $dtfimperiodoapur->format('d/m/Y'), 
                    $nrestrutlegal, 
                    $this->competencia,
                    $this->nrorgTrab
                );
                foreach($feriados as $feriado){
                    $result[] = substr($feriado['DTCALENDARIO'], 0, 10);
                }
            }
            
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function calcularQtdeAtestados(&$diasAtestado, $qtdeMesesRetroceder, $nrvinculom){
       
        $dtPeriodoInicioCalc = clone $this->dtiniperiodobenef;
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $nrtpmovimento = '1';
        $nreventom = "";
        //Evento que possui referencia em horas
        $nreventomHoras = "100106245";
        $eventos = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaListaEventosRefAtestado($nrorg, $nrorgpadrao, $dtmescompetenc);
        foreach($eventos as $evento){
            if($evento->getNreventom() != '100106245'){
                $nreventom = ($nreventom == "" ? $evento->getNreventom() : $nreventom . ", " . $evento->getNreventom());
            }
        }
        
        $dtmescomp_atestado = DateUtil::getPrimeiroDiaDoMes( $dtPeriodoInicioCalc->sub(new \DateInterval('P'.(string)$qtdeMesesRetroceder.'M')) );

        $atestados = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->getSomaItemCalculo(
            $nrvinculom, 
            $this->hcmParameters->getNrorgTrabalho(), 
            $dtmescomp_atestado, 
            $dtmescomp_atestado, 
            $nreventom, 
            $nrtpmovimento, 
            $dtmescomp_atestado
        );
        
        if(isset($atestados["VALOR"])){
            $diasAtestado = (int)$atestados['VALOR'];
        }
        
        //Modificação a pedido do artur, 19/09/2024 as 18:00 considerando os atestados que tem referencia em horas.
        if(!empty($this->jornadaDiaria)){
            $atestadosHoras = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->getSomaItemCalculo(
                $nrvinculom, 
                $this->hcmParameters->getNrorgTrabalho(), 
                $dtmescomp_atestado, 
                $dtmescomp_atestado, 
                $nreventomHoras, 
                $nrtpmovimento, 
                $dtmescomp_atestado
            );
            
            if(isset($atestadosHoras["VALOR"])){
                $diasAtestado += $atestadosHoras['VALOR'] / $this->jornadaDiaria;
            }
        }
    }
    
    private function calcularDiasAtestados($minimizadores, $beneficio, $vinculo){
        try {
            $diasAtestado= 0;
            $this->dtiniperiodobenef = DateUtil::getDataDeString($this->competencia, DateUtil::FORMATO_BRASILEIRO, true);
            $this->dtfimperiodobenef = DateUtil::getDataDeString(DateUtil::getDataDeString($this->competencia, DateUtil::FORMATO_BRASILEIRO, true)->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
            
            if((isset($minimizadores['IDCONSATESTBENEF']) && $minimizadores['IDCONSATESTBENEF'] == 'S') && isset($minimizadores['QTMESESATESTBENEF'])){
                $this->calcularQtdeDias_AtesOuAfast($diasAtestado, 'ATESTADO', $minimizadores['QTMESESATESTBENEF'], $vinculo, null, $minimizadores['IDCONSFERIADOBENEF'], $minimizadores['IDCONSDSRBENEF']);
            } else {
                $this->calcularQtdeDias_AtesOuAfast($diasAtestado, 'ATESTADO', 0, $vinculo, null, $minimizadores['IDCONSFERIADOBENEF'], $minimizadores['IDCONSDSRBENEF']);
            }
            
            // Caso não encontre atestados nas situações funcionais do vínculo verifica a referência dos eventos de atestado nos lançamentos de folha 
            if($diasAtestado == 0){
                $this->dtiniperiodobenef = DateUtil::getDataDeString($this->competencia, DateUtil::FORMATO_BRASILEIRO, true);
                $this->calcularQtdeAtestados($diasAtestado, $minimizadores['QTMESESATESTBENEF'], $vinculo->getNrvinculom());
            }
                
            return $diasAtestado;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function obterQtdeMesesPosAfastamento($alteSituacaoFunc){
        $qtdeMesesPosUltAfast = null;
        
        $objCompetenc = DateUtil::getDataDeString(substr($this->UltDiaCompetencia, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $objDtIniAfast = DateUtil::getDataDeString( substr($alteSituacaoFunc->getDtinisitufunc(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        //$mesUltAfast = $objDtIniAfast->format('d/m/Y'); // Variável cujo únivo fim é agregar valor à mensagem de feedBack ao usuário do sistema.
        
        if($objDtIniAfast && $objDtIniAfast->format('Y/m/d') <= $objCompetenc->format('Y/m/d')){
            $objDtIniAfast = DateUtil::getDataDeString(DateUtil::getPrimeiroDiaDoMes($objDtIniAfast)->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
            $diff = $objCompetenc->diff($objDtIniAfast, true);   
            $qtdeMesesPosUltAfast = ($diff->m + $diff->y*12);
            $qtdeMesesPosUltAfast ++; //Considerar o próprio mês que é tirado na subtração
        }

        return $qtdeMesesPosUltAfast;
    }
    
    private function obterDadosAfastValidarCalc($nrorg, $competencia, $listSituFunc, $refQtdeMes, $geraDesconto, $beneficio, $vinculo){
        try {
            //$ultAfastamento = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaUltimoAfastamento($this->hcmParameters->getNrorgTrabalho(), $this->hcmParameters->getNrorgPadrao(), $vinculo->getNrvinculom(), $this->hcmParameters->getDtCompetencia());
            $listSituFunc = explode(", ", $listSituFunc);
            $dadosRetorno = array();
    
            $ultimaSituFunc = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaUltimaSituacaoPorDatainicio( $nrorg, $vinculo->getNrvinculom(), substr($this->UltDiaCompetencia, 0, 10));
    
            foreach($listSituFunc as $nrSituFunc){
                
                if($ultimaSituFunc->getNrsitufuncm() == $nrSituFunc){
                    $qtdeMesPosAfast = $this->obterQtdeMesesPosAfastamento($ultimaSituFunc);
                    $situFuncH = Utils::getEntityManager()->getRepository(Repositories::GPE_SITUFUNCM)->retornaUltimoHistoricoCompetencia(
                        $nrorg, 
                        0, 
                        $ultimaSituFunc->getNrsitufuncm(), 
                        $competencia
                    );
        
                    // Quando a quantidade de meses do afastamento for menor ou igual a quantidade de meses a se considerar                
                    if(!is_null($qtdeMesPosAfast) && $qtdeMesPosAfast <= $refQtdeMes && $refQtdeMes != 0){ 
                        
                        $dadosRetorno[] = array(
                                                "qtdeMesesPosAfast" => $qtdeMesPosAfast,
                                                "qtdeMesesRef" => $refQtdeMes,
                                                "nomeSituFunc" => $situFuncH->getNmsitufunch(),
                                                "geraDesconto" => $geraDesconto,
                                                "geraBeneficio" => 'S',
                                                "dtIniSituFunc" => $ultimaSituFunc->getDtinisitufunc()->format('d/m/Y'),
                                          );
                    }else{
                        $dadosRetorno[] = array(
                                                "qtdeMesesPosAfast" => $qtdeMesPosAfast,
                                                "qtdeMesesRef" => $refQtdeMes,
                                                "nomeSituFunc" => $situFuncH->getNmsitufunch(),
                                                "geraDesconto" => $geraDesconto,
                                                "geraBeneficio" => 'N',
                                                "dtIniSituFunc" => $ultimaSituFunc->getDtinisitufunc()->format('d/m/Y'),
                                          );
                    }
                }
            }
            
            return $dadosRetorno;
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function validarMinimizadorCCTAfastamento($nrorg, $competencia, $dadosRetornoAfast, $beneficio, $tpBeneficio, $vinculo, $beneficiario, &$ehParaCalcular_minimizadorBeneficioCCT, &$subsidio100PorAfastamentoOuDemissao){
        try {
            if(count($dadosRetornoAfast) > 0){
                $geraBeneficio = true;
    
                //Verifica se é ou não é para gerar beneficio
                foreach($dadosRetornoAfast as $dado){
                    if($dado['geraBeneficio'] == 'N'){
                        $msg = "Benefício DO SINDICADO do tipo '".$tpBeneficio['DSTPBENEFICIO']."' NÃO GERADO para o vínculo '".$vinculo->getNmvinculom()."'. Passou-se ".$dado['qtdeMesesPosAfast']." mes(es) após o início do(a) '".$dado["nomeSituFunc"]."' iniciado(a) em '".$dado['dtIniSituFunc']."', quando não poderia ter passado mais de ".$dado['qtdeMesesRef']." mes(es)." ;
                        if(!empty($beneficiario) && $beneficiario['IDTPBENEFICIARIO'] == 'T') MsgCalculo::getInstance()->adicionaMensagem($msg, MsgCalculo::ALERTA);
                        $ehParaCalcular_minimizadorBeneficioCCT = $geraBeneficio  = false;
                    }
                }
                
                //Verifica se é ou não é para descontar ou não o beneficio
                if(!empty($geraBeneficio)){
                    foreach($dadosRetornoAfast as $dado){
                        if($dado['geraBeneficio'] == 'S' && $dado['geraDesconto'] == 'N'){
                            $msg = "Benefício DO SINDICATO do tipo '".$tpBeneficio['DSTPBENEFICIO']."' NÃO DESCONTADO do vínculo '".$vinculo->getNmvinculom()."'. Passou-se ".$dado['qtdeMesesPosAfast']." mes(es) após o início do(a) '".$dado["nomeSituFunc"]."', quando o limite para isso é ".$dado['qtdeMesesRef']." mes(es)." ;
                            if(!empty($beneficiario) && $beneficiario['IDTPBENEFICIARIO'] == 'T') MsgCalculo::getInstance()->adicionaMensagem($msg, MsgCalculo::ALERTA);
                            // caso não seja para descontar, a variável abaixo é atualizado com o valor verdadeiro
                            $subsidio100PorAfastamentoOuDemissao = true;
                        }
                    }                  
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function calcularDiasAfastamentos($minimizadores, $beneficio, $vinculo){
        try {
            $diasAfastamento= 0;
            $this->dtiniperiodobenef = DateUtil::getDataDeString($this->competencia, DateUtil::FORMATO_BRASILEIRO, true);
            $this->dtfimperiodobenef = DateUtil::getDataDeString($this->UltDiaCompetencia, DateUtil::FORMATO_BRASILEIRO, true);
            
            //Passa o $id_tpCalc = null para considerar todas situações funcionais que não geram vale/benefício (calcularQtdeDias_AtesOuAfast)
            if($minimizadores['IDCONSAFASTBENEF'] == 'S' && isset($minimizadores['QTMESESAFASTBENEF'])){
                $this->calcularQtdeDias_AtesOuAfast( $diasAfastamento, 'AFASTAMENTO', $minimizadores['QTMESESAFASTBENEF'], $vinculo, null, $minimizadores['IDCONSFERIADOBENEF'], $minimizadores['IDCONSDSRBENEF']);
            }else{
                $this->calcularQtdeDias_AtesOuAfast( $diasAfastamento, 'AFASTAMENTO', 0, $vinculo, null, $minimizadores['IDCONSFERIADOBENEF'], $minimizadores['IDCONSDSRBENEF']);
            }
            return $diasAfastamento;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function validarMinimizadoresBeneficioCCT($nrorg, $competencia, $beneficio, $tpBeneficio, $vinculo, $beneficiario, $minimizadores, &$ehParaCalcular_minimizadorBeneficioCCT, &$subsidio100PorAfastamentoOuDemissao, &$arrayDiasCalculados = null){
        try {
            // ===== CAMPO(S): "Gera Benefício somente para vínculos com mais de x dias trabalhados no mês"    
            // - Função responsável por calcular a quantidade de dias trabalhados no mês para validar minimizador do Benefício CCT. 
            // - '$diasTrabalhadosAdmissaoRescisao' terá valor apenas se a competência corrente é da admissão, rescisão ou ambos.
            $tpCompetencia = null;
            $diasTrabalhadosAdmissaoRescisao = $this->calcularDiasTrabalhadosMesAdmissaoRescisao($nrorg, $competencia, $beneficio, $vinculo, $tpCompetencia);
            $diasFaltas = $this->calcularDiasFaltas($minimizadores, $vinculo);
            $diasAtestados = $this->calcularDiasAtestados($minimizadores, $beneficio, $vinculo);
            $diasAfastamentosMemoria = $this->calcularDiasAfastamentos($minimizadores, $beneficio, $vinculo);
            $diasAfastamentos = 0;
            $diasTrabalhados = ( $diasTrabalhadosAdmissaoRescisao - $diasFaltas - $diasAfastamentos - $diasAfastamentosMemoria);
            
            if((isset($beneficio['QTDIASATEST']) && $beneficio['QTDIASATEST']) && $diasAtestados > $beneficio['QTDIASATEST']){
                $diasAtestados += $beneficio['QTDIASDESCATEST'];
                $arrayDiasCalculados['DIASATESTADO'] += $beneficio['QTDIASDESCATEST'];
                $arrayDiasCalculados['DIASCONSIDERACAO'] -= $beneficio['QTDIASDESCATEST'];
            }
            
            if((isset($beneficio['QTDIASFALTAS']) && $beneficio['QTDIASFALTAS']) && $diasFaltas > $beneficio['QTDIASFALTAS']){
                $diasFaltas += $beneficio['QTDIASDESCFALTAS'];
                $arrayDiasCalculados['DIASFALTAS'] += $beneficio['QTDIASDESCFALTAS'];
                $arrayDiasCalculados['DIASCONSIDERACAO'] -= $beneficio['QTDIASDESCFALTAS'];
            }
            
            $beneficio['DIASTRABALHADOSTOTAL'] = $diasTrabalhadosAdmissaoRescisao;
            $beneficio['DIASAFASTAMENTOS'] = $diasAfastamentosMemoria;
            $beneficio['DIASATESTADO'] = $diasAtestados;
            $beneficio['DIASFALTAS'] = $diasFaltas;
            $beneficio['DIASTRABALHADOS'] = $diasTrabalhados;
            
            #MemoriaBeneficio::getInstance()->setInfoBenefVinc($beneficio);
            
            //TODAS AS ORGS VERIFICAM DIAS TRABALHADOS NO MÊS.
            $verificaXdiasTrabalhadosNoMes = true;
            
            //ORG 1410 NÃO VERIFICA DIAS TRABALHADOS NO MÊS
            if($this->hcmParameters->getNrorgTrabalho() == 1410){
                $verificaXdiasTrabalhadosNoMes = false;
            }
            
            /*ORG 1410(AGILE VERIFICA DIAS TRABALHADOS NO MES SOMENTE SE FOR NO MESMO MES DA ADMISSÃO.)*/
            /* ===== CAMPO(S): "Gera Benefício somente para vínculos com mais de x dias trabalhados no mês:" e "Retroceder quantos meses para apurar os dias trabalhados (frente ao período)?" */
            if($vinculo->getDtadmissaovinc()->format('m/Y') == $this->objcompetenciaAdmissao->format('m/Y') && $this->hcmParameters->getNrorgTrabalho() == 1410){
                $verificaXdiasTrabalhadosNoMes = true;
            }
            
            // ===== CAMPO(S): "Gera Benefício somente para vínculos com mais de x dias trabalhados no mês:" e "Retroceder quantos meses para apurar os dias trabalhados (frente ao período)?"    
            // - Responsavel por verificar a quantidade de dias trabalhados se esta maior do que a quantidade minima para o beneficio
            if( (isset($beneficio['QTDIASMINMES']) && intval($beneficio['QTDIASMINMES'])>0 && isset($diasTrabalhados) && $diasTrabalhados <= intval($beneficio['QTDIASMINMES']) ) && $verificaXdiasTrabalhadosNoMes == true){
                $ehParaCalcular_minimizadorBeneficioCCT = false;
                if(isset($beneficiario['IDTPBENEFICIARIO']) && $beneficiario['IDTPBENEFICIARIO'] == 'T'){
                    $msg = "Benefício DO SINDICATO do tipo '".$tpBeneficio['DSTPBENEFICIO']."' NÃO GERADO para o vínculo '".$vinculo->getNmvinculom()."'. Na atual competencia que é de sua $tpCompetencia ele trabalhou ".$diasTrabalhados." dias quando precisaria trabalhar mais que ".$beneficio['QTDIASMINMES']." dias." ;
                    MsgCalculo::getInstance()->adicionaMensagem($msg, MsgCalculo::ALERTA);
                }    
            }
            
            // ===== CAMPO(S): "Não gerar benefícios para vínculos com mais de x faltas:" e "Retroceder quantos meses para apurar as faltas (frente ao período)?"    
            // - Função responsável por calcular a quantidade de faltas na competência informada pelo minimizador diário para validar minimizador do Benefício CCT
            //$diasFaltas = $this->calcularDiasFaltas($minimizadores, $beneficio, $vinculo);
            
            if( isset($beneficio['QTDIASMAXFALTAS']) && intval($beneficio['QTDIASMAXFALTAS'])>0 && $diasFaltas > intval($beneficio['QTDIASMAXFALTAS']) ){
                $ehParaCalcular_minimizadorBeneficioCCT = false;
                if(isset($beneficiario['IDTPBENEFICIARIO']) && $beneficiario['IDTPBENEFICIARIO'] == 'T'){
                    $objCompetenc = DateUtil::getDataDeString($this->hcmParameters->getDtcompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
                    $dtmescomp_falta = DateUtil::getPrimeiroDiaDoMes( $objCompetenc->sub(new \DateInterval('P'.$minimizadores['QTMESESFALTABENEF'].'M')) )->format('d/m/Y');
                    $msg = "Benefício DO SINDICATO do tipo '".$tpBeneficio['DSTPBENEFICIO']."' NÃO GERADO para o vínculo '".$vinculo->getNmvinculom()."'. Ele faltou ".$diasFaltas." dias na competência ".$dtmescomp_falta.", quando não poderia faltar mais que ".$beneficio['QTDIASMAXFALTAS']." dias." ;
                    MsgCalculo::getInstance()->adicionaMensagem($msg, MsgCalculo::ALERTA);
                }    
            }
            
            // ===== CAMPO(S): "Não gerar benefícios para vínculos com mais de x atestados:" e "Retroceder quantos meses para apurar os atestados (frente ao período)?"
            // - Função responsável por calcular a quantidade de faltas na competência informada pelo minimizador diário para validar minimizador do Benefício CCT
            //$diasAtestados = $this->calcularDiasAtestados($minimizadores, $beneficio, $vinculo);
            
            if( isset($beneficio['QTDIASMAXATEST']) && intval($beneficio['QTDIASMAXATEST'])>0 && $diasAtestados > intval($beneficio['QTDIASMAXATEST']) ){
                $ehParaCalcular_minimizadorBeneficioCCT = false;
                if(isset($beneficiario['IDTPBENEFICIARIO']) && $beneficiario['IDTPBENEFICIARIO'] == 'T'){
                    $objCompetenc = DateUtil::getDataDeString($competencIa, DateUtil::FORMATO_BRASILEIRO, true);
                    $dtmescomp_atest = DateUtil::getPrimeiroDiaDoMes( $objCompetenc->sub(new \DateInterval('P'.(string)$minimizadores['QTMESESATESTBENEF'].'M')) )->format('d/m/Y');
                    $msg = "Benefício DO SINDICATO do tipo '".$tpBeneficio['DSTPBENEFICIO']."' NÃO GERADO para o vínculo '".$vinculo->getNmvinculom()."'. Ele teve
                    ".$diasAtestados." atestados na competência ".$dtmescomp_atest.", quando não poderia ter mais que ".$beneficio['QTDIASMAXATEST']." atestados." ;
                    MsgCalculo::getInstance()->adicionaMensagem($msg, MsgCalculo::ALERTA);
                }
            }
            
            // ===== CAMPO(S): "Situação Funcional Afastamento 1", "Até Quantos Meses de Afastamento 1" e "Gera Desconto 1"    
            // - Escopo responsável por validar os minimizadores relativos a afastamentos 1 do beneficio CCT 
            $dadosRetornoAfast = array();
            if(isset($beneficio['DSAFASTAMENTO1']) && !empty($beneficio['DSAFASTAMENTO1'])) $dadosRetornoAfast = $this->obterDadosAfastValidarCalc($nrorg, $competencia, $beneficio['DSAFASTAMENTO1'], $beneficio['QTMESMAXAFAST'], $beneficio['IDGERADESCAFAST1'], $beneficio, $vinculo); // Função responsável por obter dados de afastamento do vínculo a fim de validar minimizador do Benefício CCT
            $this->validarMinimizadorCCTAfastamento($nrorg, $competencia, $dadosRetornoAfast, $beneficio, $tpBeneficio, $vinculo,$beneficiario,  $ehParaCalcular_minimizadorBeneficioCCT, $subsidio100PorAfastamentoOuDemissao);
            
            // ===== CAMPO(S): "Situação Funcional Afastamento 2", "Até Quantos Meses de Afastamento 2" e "Gera Desconto 2"    
            // - Escopo responsável por validar os minimizadores relativos a afastamentos 2 do beneficio CCT 
            $dadosRetornoAfast = array();
            if(isset($beneficio['DSAFASTAMENTO2']) && !empty($beneficio['DSAFASTAMENTO2'])) $dadosRetornoAfast = $this->obterDadosAfastValidarCalc($nrorg, $competencia, $beneficio['DSAFASTAMENTO2'], $beneficio['QTMESMAXAFAST2'], $beneficio['IDGERADESCAFAST2'], $beneficio, $vinculo); // Função responsável por obter dados de afastamento do vínculo a fim de validar minimizador do Benefício CCT
            $this->validarMinimizadorCCTAfastamento($nrorg, $competencia, $dadosRetornoAfast, $beneficio, $tpBeneficio, $vinculo,$beneficiario,  $ehParaCalcular_minimizadorBeneficioCCT, $subsidio100PorAfastamentoOuDemissao);
            
            // ===== CAMPO(S): "Situação Funcional Não Gera Benefício"    
            // - Escopo responsável por validar os minimizadores relativos às situações que não geram benefício
            $ultimaSituFunc = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncionalDentroPeriodo( $this->hcmParameters->getNrorgTrabalho(), $vinculo->getNrvinculom(), substr($this->hcmParameters->getDtCompetencia(), 0, 10),substr($this->UltDiaCompetencia, 0, 10) );
            
            $listSituFunc = isset($beneficio['DSAFASTAMENTO3']) && !empty($beneficio['DSAFASTAMENTO3']) ? explode(", ", $beneficio['DSAFASTAMENTO3']) : null;            
            if(!empty($listSituFunc)){
                if(is_array($listSituFunc)){
                    foreach($ultimaSituFunc as $situFunc){
                        if( isset($listSituFunc) && in_array($situFunc->getNrsitufuncm(), $listSituFunc)){
                            $ehParaCalcular_minimizadorBeneficioCCT = false;
                            if(!empty($beneficiario['IDTPBENEFICIARIO']) && $beneficiario['IDTPBENEFICIARIO'] == 'T'){
                                $situFuncH = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCM)->retornaUltimoHistoricoCompetencia(
                                    $this->hcmParameters->getNrorgTrabalho(), 
                                    0, 
                                    $situFunc->getNrsitufuncm(), 
                                    $this->hcmParameters->getDtCompetencia()
                                );
                                $msg = "[PARAMETRO 'Situação Funcional Não Gera Benefício'] Benefício DO SINDICATO do tipo '".$tpBeneficio['DSTPBENEFICIO']."'.  NÃO GERADO para o vínculo '".$vinculo->getNmvinculom()."'. Ela possui situação funcional '".$situFuncH->getNmsitufunch()."', dentro do periodo, quando não poderia ter." ;
                                MsgCalculo::getInstance()->adicionaMensagem($msg, MsgCalculo::ALERTA);
                            }
                            continue;
                        }
                    }
                } else {
                    if( isset($listSituFunc) && in_array($ultimaSituFunc->getNrsitufuncm(), $listSituFunc)){
                        $ehParaCalcular_minimizadorBeneficioCCT = false;
                        if(!empty($beneficiario['IDTPBENEFICIARIO']) && $beneficiario['IDTPBENEFICIARIO'] == 'T'){
                            $situFuncH = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCM)->retornaUltimoHistoricoCompetencia(
                                $this->hcmParameters->getNrorgTrabalho(), 
                                0, 
                                $ultimaSituFunc->getNrsitufuncm(), 
                                $this->hcmParameters->getDtCompetencia()
                            );
                            $msg = "[PARAMETRO 'Situação Funcional Não Gera Benefício'] Benefício DO SINDICATO do tipo '".$tpBeneficio['DSTPBENEFICIO']."' NÃO GERADO para o vínculo '".$vinculo->getNmvinculom()."'. Sua atual situação funcional é '".$situFuncH->getNmsitufunch()."', dentro do periodo, quando não poderia ter." ;
                            MsgCalculo::getInstance()->adicionaMensagem($msg, MsgCalculo::ALERTA);
                        }
                    }
                }
            }
            
            // ===== CAMPO(S): "Perderão o benefício por possuírem os seguintes eventos:" e "Retroceder quantos meses para apurar os eventos (frente ao período)?"    
            // - Escopo responsável por validar os minimizadores do Beneficio CCT relativos a perda de benefícios por existências de determinados eventos
            $dadosRetornoValidEventos = array();
            if(isset($beneficio['DSINCIDEVENPERDER']) && !empty($beneficio['DSINCIDEVENPERDER'])){
                $dadosRetornoValidEventos = $this->retornaEventosFolhaMensal($vinculo, $beneficio['DSINCIDEVENPERDER'], $beneficio['QTMESRETROCSEMBEN']);
            }
    
            if(isset($dadosRetornoValidEventos) && !empty($dadosRetornoValidEventos) && $dadosRetornoValidEventos['geraBeneficio'] == false){
                $listaEventos = implode(", ", $dadosRetornoValidEventos['listaEventos']);
                $msg = "Benefício DO SINDICATO do tipo '".$tpBeneficio['DSTPBENEFICIO']."' NÃO GERADO para o vínculo '".$vinculo->getNmvinculom()."'. Ele teve na folha da competência '".$dadosRetornoValidEventos['competenciaRef']."' o(s) seguinte(s) evento(s) '".$listaEventos."'." ;   
                if(!empty($beneficiario) && $beneficiario['IDTPBENEFICIARIO'] == 'T') MsgCalculo::getInstance()->adicionaMensagem($msg, MsgCalculo::ALERTA);
                $ehParaCalcular_minimizadorBeneficioCCT = false;
            }
            
            // ===== CAMPO(S): "Não gera benefício para vínculos em período de experiência"
            // - Escopo responsável por validar os minimizadores do Beneficio CCT relativos a perda de benefícios por existências de determinados eventos
            if(isset($beneficio['IDNAOGERBENVINCEXP']) && $beneficio['IDNAOGERBENVINCEXP'] == 'S'){
                #$histVinc = Utils::getEntityManager()->getRepository(Repositories::GPE_VINCULOM)->_retornaHistorico($vinculo->getNrvinculom(), $nrorg, $this->competencia);
                
                $dtAtual = $dtFimContratExpe = null;
    
                $dtAtual = DateUtil::getDataDeString( substr(DateUtil::getDataAtual(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
                
                if($this->vinculoh->getDtfimcontrdetermin() && !empty($this->vinculoh->getDtfimcontrdetermin()))
                    $dtFimContratExpe = $this->vinculoh->getDtfimcontrdetermin();
                
                if((isset($dtFimContratExpe) && $dtFimContratExpe) && $dtAtual->format('Y/m/d') <= $dtFimContratExpe->format('Y/m/d')){
                    $msg = "Benefício '".$tpBeneficio['DSTPBENEFICIO']."' não calculado para o vínculo '".$vinculo->getNmvinculom()."'. Ele(a) está no período de experiência.";
                    MsgCalculo::getInstance()->adicionaMensagem($msg, MsgCalculo::ALERTA);
                    $ehParaCalcular_minimizadorBeneficioCCT = false;
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function retornaEventosFolhaMensal($vinculo, $listEventos, $qtdeMesRetroc, $incluiTodosMeses=false){
        try {
            if(!isset($qtdeMesRetroc) || empty($qtdeMesRetroc)) $qtdeMesRetroc = 0;
            
            if($incluiTodosMeses){
                $dadosRetorno = array(
                                    "listaEventos" => [],
                                    "geraBeneficio" => true,
                                    "consideraMultiplicador" => true,
                                    "competenciaRef" => [],
                                );
                
                for($i = 0; $i <= $qtdeMesRetroc; $i++){
                    $objCompetenc = DateUtil::getDataDeString($this->competencia, DateUtil::FORMATO_BRASILEIRO, true);
                    $objCompReferencia = DateUtil::getPrimeiroDiaDoMes( $objCompetenc->sub(new \DateInterval('P'.(string)$i.'M')) );
                    
                    $nrCalculoFolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->getNrCalculo($this->nrorgTrab, $objCompReferencia->format('d/m/Y'), 1);
                        
                    if(isset($nrCalculoFolha)) $itensCalc = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->retornaItemPeriodo($this->nrorgTrab, $vinculo->getNrvinculom(), $nrCalculoFolha, $listEventos);
                    
                    if($itensCalc){
                        foreach($itensCalc as $item){
                            if($item->getVreventofolha() && $item->getVreventofolha() > 0){
                                $dadosRetorno['geraBeneficio'] = false;
                                $dadosRetorno['consideraMultiplicador'] = false;
                                if(!in_array($item->getNreventom(),$dadosRetorno['listaEventos'])){
                                    $dadosRetorno['listaEventos'][] = $item->getNreventom();
                                }
                                $dadosRetorno['competenciaRef'][] = $objCompReferencia->format('d/m/Y');
                            }
                        }
                    }
                }
            }else{
                $objCompetenc = DateUtil::getDataDeString($this->competencia, DateUtil::FORMATO_BRASILEIRO, true);
                $objCompReferencia = DateUtil::getPrimeiroDiaDoMes( $objCompetenc->sub(new \DateInterval('P'.(string)$qtdeMesRetroc.'M')) );
        
                $dadosRetorno = array(
                                    "listaEventos" => [],
                                    "geraBeneficio" => true,
                                    "consideraMultiplicador" => true,
                                    "competenciaRef" => $objCompReferencia->format('d/m/Y'),
                                );
                
                $nrCalculoFolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->getNrCalculo($this->nrorgTrab, $objCompReferencia->format('d/m/Y'), 1);
                    
                if(isset($nrCalculoFolha)) $itensCalc = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->retornaItemPeriodo($this->nrorgTrab, $vinculo->getNrvinculom(), $nrCalculoFolha, $listEventos);
                
                if($itensCalc){
                    foreach($itensCalc as $item){
                        if($item->getVreventofolha() && $item->getVreventofolha() > 0){
                            $dadosRetorno['geraBeneficio'] = false;
                            $dadosRetorno['consideraMultiplicador'] = false;
                            $dadosRetorno['listaEventos'][] = $item->getNreventom();
                        }
                    }
                }
            }
            
            return $dadosRetorno;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function obterMinimizadoresMensal($beneficio){
        $array = [];
        
        $array['IDGERABENEFFALTA']  = isset($beneficio['IDGERABENEFFALTA'])  && !empty($beneficio['IDGERABENEFFALTA'])   ? $beneficio['IDGERABENEFFALTA']  : 'N';
        $array['IDGERABENEFATEST']  = isset($beneficio['IDGERABENEFATEST'])  && !empty($beneficio['IDGERABENEFATEST'])   ? $beneficio['IDGERABENEFATEST']  : 'N';
        $array['IDGERABENEFAFAST']  = isset($beneficio['IDGERABENEFAFAST'])  && !empty($beneficio['IDGERABENEFAFAST'])   ? $beneficio['IDGERABENEFAFAST']  : 'N';
        $array['IDGERABENEFFERIAS'] = isset($beneficio['IDGERABENEFFERIAS']) && !empty($beneficio['IDGERABENEFFERIAS'])  ? $beneficio['IDGERABENEFFERIAS'] : 'N';
        $array['IDGERABENEFRETOAFAST']  = isset($beneficio['IDGERABENEFRETOAFAST'])  && !empty($beneficio['IDGERABENEFRETOAFAST'])   ? $beneficio['IDGERABENEFRETOAFAST']  : 'N';
        
        // Abaixo para minimizadores informados no Benefício do Sindicato
        $array['IDCONSFALTABENEF']    = isset($beneficio['IDCONSFALTABENEF'])    && !empty($beneficio['IDCONSFALTABENEF'])    ? $beneficio['IDCONSFALTABENEF']    : 'N';
        $array['QTMESESFALTABENEF']   = isset($beneficio['QTMESESFALTABENEF'])   && !empty($beneficio['QTMESESFALTABENEF'])   ? $beneficio['QTMESESFALTABENEF']   : 0;
        $array['IDCONSAFASTBENEF']    = isset($beneficio['IDCONSAFASTBENEF'])    && !empty($beneficio['IDCONSAFASTBENEF'])    ? $beneficio['IDCONSAFASTBENEF']    : 'N';
        $array['QTMESESAFASTBENEF']   = isset($beneficio['QTMESESAFASTBENEF'])   && !empty($beneficio['QTMESESAFASTBENEF'])   ? $beneficio['QTMESESAFASTBENEF']   : 0;
        $array['IDCONSATESTBENEF']    = isset($beneficio['IDCONSATESTBENEF'])    && !empty($beneficio['IDCONSATESTBENEF'])    ? $beneficio['IDCONSATESTBENEF']    : 'N';
        $array['QTMESESATESTBENEF']   = isset($beneficio['QTMESESATESTBENEF'])   && !empty($beneficio['QTMESESATESTBENEF'])   ? $beneficio['QTMESESATESTBENEF']   : 0;
        $array['QTMESESFERIASBENEF']  = isset($beneficio['QTMESESFERIASBENEF'])  && !empty($beneficio['QTMESESFERIASBENEF'])  ? $beneficio['QTMESESFERIASBENEF']  : 0;
        $array['IDCONSFERIADOBENEF']    = isset($beneficio['IDCONSFERIADOBENEF'])    && !empty($beneficio['IDCONSFERIADOBENEF'])    ? $beneficio['IDCONSFERIADOBENEF']    : 'N';
        $array['QTMESESFERIADOBENEF']   = isset($beneficio['QTMESESFERIADOBENEF'])   && !empty($beneficio['QTMESESFERIADOBENEF'])   ? $beneficio['QTMESESFERIADOBENEF']   : 0;
        $array['IDCONSDSRBENEF']    = isset($beneficio['IDCONSDSRBENEF'])    && !empty($beneficio['IDCONSDSRBENEF'])    ? $beneficio['IDCONSDSRBENEF']    : 'N';
        $array['QTMESESDSRBENEF']   = isset($beneficio['QTMESESDSRBENEF'])   && !empty($beneficio['QTMESESDSRBENEF'])   ? $beneficio['QTMESESDSRBENEF']   : 0;
        
        return $array;
    }
    
    private function verificarIsencao100Demissao(&$subsidio100PorAfastamentoOuDemissao, $consvincresmesant, $vinculo, $dtRescisaoVinc, $competenciaAtual){
        if($consvincresmesant == 'S' && isset($dtRescisaoVinc) && $dtRescisaoVinc < $competenciaAtual){
            $subsidio100PorAfastamentoOuDemissao = true;
        }
    }
    
    private function mergeBeneficioComBeneficioSind(&$beneficio, &$benefDescontoSind){
        // Campos específicos existentes apenas no Benefício do CCT
        $beneficio['QTDIASMINMES']          = isset($benefDescontoSind['QTDIASMINMES'])       && !empty($benefDescontoSind['QTDIASMINMES'])       ? $benefDescontoSind['QTDIASMINMES']        : null;     
        $beneficio['QTDIASMAXFALTAS']       = isset($benefDescontoSind['QTDIASMAXFALTAS'])    && !empty($benefDescontoSind['QTDIASMAXFALTAS'])    ? $benefDescontoSind['QTDIASMAXFALTAS']     : null;
        $beneficio['QTDIASMAXATEST']        = isset($benefDescontoSind['QTDIASMAXATEST'])     && !empty($benefDescontoSind['QTDIASMAXATEST'])     ? $benefDescontoSind['QTDIASMAXATEST']      : null;     
        $beneficio['QTMESMAXAFAST']         = isset($benefDescontoSind['QTMESMAXAFAST'])      && !empty($benefDescontoSind['QTMESMAXAFAST'])      ? $benefDescontoSind['QTMESMAXAFAST']       : null;     
        $beneficio['QTDIASATEST']           = isset($benefDescontoSind['QTDIASATEST'])        && !empty($benefDescontoSind['QTDIASATEST'])        ? $benefDescontoSind['QTDIASATEST']         : null; 
        $beneficio['QTDIASDESCATEST']       = isset($benefDescontoSind['QTDIASDESCATEST'])    && !empty($benefDescontoSind['QTDIASDESCATEST'])    ? $benefDescontoSind['QTDIASDESCATEST']     : null; 
        $beneficio['QTDIASFALTAS']          = isset($benefDescontoSind['QTDIASFALTAS'])       && !empty($benefDescontoSind['QTDIASFALTAS'])       ? $benefDescontoSind['QTDIASFALTAS']        : null;
        $beneficio['QTDIASDESCFALTAS']      = isset($benefDescontoSind['QTDIASDESCFALTAS'])   && !empty($benefDescontoSind['QTDIASDESCFALTAS'])   ? $benefDescontoSind['QTDIASDESCFALTAS']    : null; 
        $beneficio['VRMULTIPLICADOR']       = isset($benefDescontoSind['VRMULTIPLICADOR'])    && !empty($benefDescontoSind['VRMULTIPLICADOR'])    ? $benefDescontoSind['VRMULTIPLICADOR']     : null;     
        $beneficio['DSVINCULOEMPREG']       = isset($benefDescontoSind['DSVINCULOEMPREG'])    && !empty($benefDescontoSind['DSVINCULOEMPREG'])    ? $benefDescontoSind['DSVINCULOEMPREG']     : null;     
        $beneficio['DSINCIDEVENTO']         = isset($benefDescontoSind['DSINCIDEVENTO'])      && !empty($benefDescontoSind['DSINCIDEVENTO'])      ? $benefDescontoSind['DSINCIDEVENTO']       : null;
        $beneficio['VRFIXACRESDESC']        = isset($benefDescontoSind['VRFIXACRESDESC'])     && !empty($benefDescontoSind['VRFIXACRESDESC'])     ? $benefDescontoSind['VRFIXACRESDESC']      : null;     
        $beneficio['VRPERACRESCDESC']       = isset($benefDescontoSind['VRPERACRESCDESC'])    && !empty($benefDescontoSind['VRPERACRESCDESC'])    ? $benefDescontoSind['VRPERACRESCDESC']     : null;     
        $beneficio['IDNAOGERBENVINCEXP']    = isset($benefDescontoSind['IDNAOGERBENVINCEXP']) && !empty($benefDescontoSind['IDNAOGERBENVINCEXP']) ? $benefDescontoSind['IDNAOGERBENVINCEXP']  : 'N';     
        $beneficio['IDNAOGERABENPEREXP']    = isset($benefDescontoSind['IDNAOGERABENPEREXP']) && !empty($benefDescontoSind['IDNAOGERABENPEREXP']) ? $benefDescontoSind['IDNAOGERABENPEREXP']  : null;     
        $beneficio['IDNAOGERABENRESC']      = isset($benefDescontoSind['IDNAOGERABENRESC'])   && !empty($benefDescontoSind['IDNAOGERABENRESC'])   ? $benefDescontoSind['IDNAOGERABENRESC']    : null;     
        $beneficio['NRBENEFICIOCCT']        = isset($benefDescontoSind['NRBENEFICIOCCT'])     && !empty($benefDescontoSind['NRBENEFICIOCCT'])     ? $benefDescontoSind['NRBENEFICIOCCT']      : null;     

        $beneficio['QTMAXHSREDUZIDA']       = isset($benefDescontoSind['QTMAXHSREDUZIDA'])    && !empty($benefDescontoSind['QTMAXHSREDUZIDA'])    ? intval($benefDescontoSind['QTMAXHSREDUZIDA'])    : null;
        $beneficio['VRHSREDUZIDA']          = isset($benefDescontoSind['VRHSREDUZIDA'])       && !empty($benefDescontoSind['VRHSREDUZIDA'])       ? floatval($benefDescontoSind['VRHSREDUZIDA'])     : null;
        $beneficio['DSINCIDEVENPERDER']     = isset($benefDescontoSind['DSINCIDEVENPERDER'])  && !empty($benefDescontoSind['DSINCIDEVENPERDER'])  ? $benefDescontoSind['DSINCIDEVENPERDER']          : null;
        $beneficio['QTMESRETROCMULT']       = isset($benefDescontoSind['QTMESRETROCMULT'])    && !empty($benefDescontoSind['QTMESRETROCMULT'])    ? intval($benefDescontoSind['QTMESRETROCMULT'])    : null;
        $beneficio['QTMESRETROCSEMBEN']     = isset($benefDescontoSind['QTMESRETROCSEMBEN'])  && !empty($benefDescontoSind['QTMESRETROCSEMBEN'])  ? intval($benefDescontoSind['QTMESRETROCSEMBEN'])  : null;
        $beneficio['QTMESRETROCDIASTRAB']   = isset($benefDescontoSind['QTMESRETROCDIASTRAB'])  && !empty($benefDescontoSind['QTMESRETROCDIASTRAB'])  ? intval($benefDescontoSind['QTMESRETROCDIASTRAB'])  : null;
        $beneficio['DSAFASTAMENTO1']        = isset($benefDescontoSind['DSAFASTAMENTO1'])     && !empty($benefDescontoSind['DSAFASTAMENTO1'])     ? $benefDescontoSind['DSAFASTAMENTO1']             : null;
        $beneficio['IDGERADESCAFAST1']      = isset($benefDescontoSind['IDGERADESCAFAST1'])   && !empty($benefDescontoSind['IDGERADESCAFAST1'])   ? $benefDescontoSind['IDGERADESCAFAST1']           : null;
        $beneficio['DSAFASTAMENTO2']        = isset($benefDescontoSind['DSAFASTAMENTO2'])     && !empty($benefDescontoSind['DSAFASTAMENTO2'])     ? $benefDescontoSind['DSAFASTAMENTO2']             : null;
        $beneficio['QTMESMAXAFAST2']        = isset($benefDescontoSind['QTMESMAXAFAST2'])     && !empty($benefDescontoSind['QTMESMAXAFAST2'])     ? intval($benefDescontoSind['QTMESMAXAFAST2'])     : null;
        $beneficio['IDGERADESCAFAST2']      = isset($benefDescontoSind['IDGERADESCAFAST2'])   && !empty($benefDescontoSind['IDGERADESCAFAST2'])   ? $benefDescontoSind['IDGERADESCAFAST2']           : null;
        $beneficio['DSAFASTAMENTO3']        = isset($benefDescontoSind['DSAFASTAMENTO3'])     && !empty($benefDescontoSind['DSAFASTAMENTO3'])     ? $benefDescontoSind['DSAFASTAMENTO3']             : null;

        // Campos integrais comuns para as duas tabelas
        $beneficio['NREVENTOMENSA']        = isset($benefDescontoSind['NREVENTOMENSA'])        && !empty($benefDescontoSind['NREVENTOMENSA'])        ? $benefDescontoSind['NREVENTOMENSA']        : $beneficio['NREVENTOMENSA'];     
        $beneficio['NREVENTOMENSADEP']     = isset($benefDescontoSind['NREVENTOMENSADEP'])     && !empty($benefDescontoSind['NREVENTOMENSADEP'])     ? $benefDescontoSind['NREVENTOMENSADEP']     : $beneficio['NREVENTOMENSADEP'];     
        $beneficio['VRTETOTITULARMENS']    = isset($benefDescontoSind['VRTETOTITULARMENS'])    && !empty($benefDescontoSind['VRTETOTITULARMENS'])    ? $benefDescontoSind['VRTETOTITULARMENS']    : $beneficio['VRTETOTITULARMENS'];     
        $beneficio['VRPISOTITULARMENS']    = isset($benefDescontoSind['VRPISOTITULARMENS'])    && !empty($benefDescontoSind['VRPISOTITULARMENS'])    ? $benefDescontoSind['VRPISOTITULARMENS']    : $beneficio['VRPISOTITULARMENS'];     
        $beneficio['VRLINEARBENEFICIO']    = isset($benefDescontoSind['VRLINEARBENEFICIO'])    && !empty($benefDescontoSind['VRLINEARBENEFICIO'])    ? $benefDescontoSind['VRLINEARBENEFICIO']    : $beneficio['VRLINEARBENEFICIO'];     
        $beneficio['VRLINEARBENEFICIODEP'] = isset($benefDescontoSind['VRLINEARBENEFICIODEP']) && !empty($benefDescontoSind['VRLINEARBENEFICIODEP']) ? $benefDescontoSind['VRLINEARBENEFICIODEP'] : $beneficio['VRLINEARBENEFICIODEP'];     

        $beneficio['VRPARTICIPATITULAR']   = isset($benefDescontoSind['VRPARTICIPATITULAR'])   && !empty($benefDescontoSind['VRPARTICIPATITULAR'])   ? floatval($benefDescontoSind['VRPARTICIPATITULAR'])  : $beneficio['VRPARTICIPATITULAR'];
        $beneficio['VRPISOTITULAR']        = isset($benefDescontoSind['VRPISOTITULAR'])        && !empty($benefDescontoSind['VRPISOTITULAR'])        ? floatval($benefDescontoSind['VRPISOTITULAR'])       : $beneficio['VRPISOTITULAR'];
        $beneficio['VRTETOTITULAR']        = isset($benefDescontoSind['VRTETOTITULAR'])        && !empty($benefDescontoSind['VRTETOTITULAR'])        ? floatval($benefDescontoSind['VRTETOTITULAR'])       : $beneficio['VRTETOTITULAR'];
        $beneficio['VRPARTICIPADEPE']      = isset($benefDescontoSind['VRPARTICIPADEPE'])      && !empty($benefDescontoSind['VRPARTICIPADEPE'])      ? floatval($benefDescontoSind['VRPARTICIPADEPE'])     : $beneficio['VRPARTICIPADEPE'];
        $beneficio['VRPISODEP']            = isset($benefDescontoSind['VRPISODEP'])            && !empty($benefDescontoSind['VRPISODEP'])            ? floatval($benefDescontoSind['VRPISODEP'])           : $beneficio['VRPISODEP'];
        $beneficio['VRTETODEP']            = isset($benefDescontoSind['VRTETODEP'])            && !empty($benefDescontoSind['VRTETODEP'])            ? floatval($benefDescontoSind['VRTETODEP'])           : $beneficio['VRTETODEP'];

        $beneficio['IDPROPADMIS']          = isset($benefDescontoSind['IDPROPADMIS'])          && !empty($benefDescontoSind['IDPROPADMIS'])          ? $benefDescontoSind['IDPROPADMIS']          : $beneficio['IDPROPADMIS'];     
        $beneficio['IDPROPRESC']           = isset($benefDescontoSind['IDPROPRESC'])           && !empty($benefDescontoSind['IDPROPRESC'])           ? $benefDescontoSind['IDPROPRESC']           : $beneficio['IDPROPRESC'];     

        // Campos para desconto de benefício comuns para as duas tabelas
        $beneficio['NRIDADEMAXSUBFILHOS']    = isset($benefDescontoSind['NRIDADEMAXSUBFILHOS'])    && !empty($benefDescontoSind['NRIDADEMAXSUBFILHOS'])    ? $benefDescontoSind['NRIDADEMAXSUBFILHOS']    : $beneficio['NRIDADEMAXSUBFILHOS'];     
        $beneficio['NRIDADEMAXSUBDEP']       = isset($benefDescontoSind['NRIDADEMAXSUBDEP'])       && !empty($benefDescontoSind['NRIDADEMAXSUBDEP'])       ? $benefDescontoSind['NRIDADEMAXSUBDEP']       : $beneficio['NRIDADEMAXSUBDEP'];     
        $beneficio['SEXOSUBSIDIOFILHO']      = isset($benefDescontoSind['SEXOSUBSIDIOFILHO'])      && !empty($benefDescontoSind['SEXOSUBSIDIOFILHO'])      ? $benefDescontoSind['SEXOSUBSIDIOFILHO']      : $beneficio['SEXOSUBSIDIOFILHO'];       
        $beneficio['SEXOSUBSIDIODEP']        = isset($benefDescontoSind['SEXOSUBSIDIODEP'])        && !empty($benefDescontoSind['SEXOSUBSIDIODEP'])        ? $benefDescontoSind['SEXOSUBSIDIODEP']        : $beneficio['SEXOSUBSIDIODEP'];         
        $beneficio['NREVETITULARVINC']       = isset($benefDescontoSind['NREVETITULARVINC'])       && !empty($benefDescontoSind['NREVETITULARVINC'])       ? $benefDescontoSind['NREVETITULARVINC']       : $beneficio['NREVETITULARVINC'];     
        $beneficio['NRTIPOPARTICIPATITULAR'] = isset($benefDescontoSind['NRTIPOPARTICIPATITULAR']) && !empty($benefDescontoSind['NRTIPOPARTICIPATITULAR']) ? $benefDescontoSind['NRTIPOPARTICIPATITULAR'] : $beneficio['NRTIPOPARTICIPATITULAR'];  
        $beneficio['NRTABFAIXASALTIT']       = isset($benefDescontoSind['NRTABFAIXASALTIT'])       && !empty($benefDescontoSind['NRTABFAIXASALTIT'])       ? $benefDescontoSind['NRTABFAIXASALTIT']       : $beneficio['NRTABFAIXASALTIT'];     
        $beneficio['VRTITULARSIND']          = isset($benefDescontoSind['VRTITULARSIND'])          && !empty($benefDescontoSind['VRTITULARSIND'])          ? $benefDescontoSind['VRTITULARSIND']          : $beneficio['VRTITULARSIND'];           
        $beneficio['VRPISOTITULARSIND']      = isset($benefDescontoSind['VRPISOTITULARSIND'])      && !empty($benefDescontoSind['VRPISOTITULARSIND'])      ? $benefDescontoSind['VRPISOTITULARSIND']      : $beneficio['VRPISOTITULARSIND'];       
        $beneficio['VRTETOTITULARSIND']      = isset($benefDescontoSind['VRTETOTITULARSIND'])      && !empty($benefDescontoSind['VRTETOTITULARSIND'])      ? $benefDescontoSind['VRTETOTITULARSIND']      : $beneficio['VRTETOTITULARSIND'];       
        $beneficio['NREVEDEPEVINC']          = isset($benefDescontoSind['NREVEDEPEVINC'])          && !empty($benefDescontoSind['NREVEDEPEVINC'])          ? $benefDescontoSind['NREVEDEPEVINC']          : $beneficio['NREVEDEPEVINC'];           
        $beneficio['NRTIPOPARTICIPADEP']     = isset($benefDescontoSind['NRTIPOPARTICIPADEP'])     && !empty($benefDescontoSind['NRTIPOPARTICIPADEP'])     ? $benefDescontoSind['NRTIPOPARTICIPADEP']     : $beneficio['NRTIPOPARTICIPADEP'];      
        $beneficio['NRTABFAIXASALDEP']       = isset($benefDescontoSind['NRTABFAIXASALDEP'])       && !empty($benefDescontoSind['NRTABFAIXASALDEP'])       ? $benefDescontoSind['NRTABFAIXASALDEP']       : $beneficio['NRTABFAIXASALDEP'];     
        $beneficio['VRDEPSIND']              = isset($benefDescontoSind['VRDEPSIND'])              && !empty($benefDescontoSind['VRDEPSIND'])              ? $benefDescontoSind['VRDEPSIND']              : $beneficio['VRDEPSIND'];          
        $beneficio['VRPISODEPSIND']          = isset($benefDescontoSind['VRPISODEPSIND'])          && !empty($benefDescontoSind['VRPISODEPSIND'])          ? $benefDescontoSind['VRPISODEPSIND']          : $beneficio['VRPISODEPSIND'];           
        $beneficio['VRTETODEPSIND']          = isset($benefDescontoSind['VRTETODEPSIND'])          && !empty($benefDescontoSind['VRTETODEPSIND'])          ? $benefDescontoSind['VRTETODEPSIND']          : $beneficio['VRTETODEPSIND'];
    }
    
    private function obterMotivosParaNaoCalcular_verificarIsencao100Afastamento($nrorg, $dtmescompetenc,$vinculo, &$subsidio100PorAfastamentoOuDemissao, &$motivos = null, $tpperiodo = null, $minimizadores = null, $verificaAfastamento = true){
        try {
            $this->temFeriasNaCompetencia = false;
            $this->temAfastamentoNaCompetencia = false;
            $this->temAtestadoNaCompetencia = false;
            
            if(!$motivos) $motivos = [];
            
            if(!empty($tpperiodo) && $tpperiodo == 'MENSAL'){
                $this->diasFeriasNaCompetencia = 0;
                $this->diasAtestadoNaCompetencia = 0;
                // ----- Verifica se o vínculo FALTOU no período da competência
                $nrtpmovimento = '1';
                $nreventom = "490, 495, 1040, 1043";
                
                $competencia = clone DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true);
                $competencia = !empty($minimizadores['QTMESESFALTABENEF']) && $minimizadores['QTMESESFALTABENEF'] > 0 ? $competencia->sub( new \DateInterval('P'.(string)$minimizadores['QTMESESFALTABENEF'].'M') ) : $competencia;
                
                if($this->deveCalcularFaltasPorSituacaoFuncional == true){
                    $faltas = $this->calcularDiasFaltas($minimizadores, $vinculo);
                } else { 
                    $faltas = Utils::getEntityManager()->getRepository(Repositories::FPA_CALCULOFOLHA)->getSomaItemCalculo(
                        $vinculo->getNrvinculom(), 
                        $nrorg, 
                        DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true),
                        DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true),
                        $nreventom, 
                        $nrtpmovimento, 
                        DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)
                    );
                    
                    if( $faltas && (int)$faltas['VALOR'] > 0){
                        $faltas = (int)$faltas['VALOR'];
                    }  
                }
                
                if($faltas > 0){
                    $motivos[] = "FALTA";
                    $temFalta = true;
                }

                // ----- Verifica se o vínculo ficou de ATESTADO no período da competência
                $competencia = clone DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true);
                $competencia = !empty($minimizadores['QTMESESATESTBENEF']) && $minimizadores['QTMESESATESTBENEF'] > 0 ? $competencia->sub( new \DateInterval('P'.(string)$minimizadores['QTMESESATESTBENEF'].'M') ) : $competencia;
                
                $listNrsitufuncm = Utils::getEntityManager()->getRepository(Repositories::GPE_SITUFUNCH)->getAfastamentoIncideVale(substr($dtmescompetenc, 0, 10));
                $listNrsitufuncm = implode(', ', $listNrsitufuncm);
                
                $altesitfuncs = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncional_AtesOuAfast(
                    $nrorg, 
                    $vinculo->getNrvinculom(), 
                    $listNrsitufuncm, 
                    DateUtil::getDataDeString($competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true),
                    DateUtil::getDataDeString($competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true), 
                    "ATESTADO"
                );
                
                if($altesitfuncs){
                    $motivos[] = "ATESTADO";
                    $temAtestado = true;
                    $this->temAtestadoNaCompetencia = true;
                    
                    //Utilizar estas propriedades somente em Calculo Mensal (Pois verifica o fim da competencia(Mes inteiro))
                    foreach($altesitfuncs as $altesitfunc){
                        //Se a data fim da situação funcional for nula, ira analisar do inicio da situação ate o final da competencia.
                        if(is_null($altesitfunc->getDtfimsitufunc())){
                            $this->diasAtestadoNaCompetencia += count(DateUtil::getDatasEntre($altesitfunc->getDtinisitufunc(), DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)));
                        }//Se a data fim for menor ou igual do que o ultimo dia da competencia ira-se utilizar a data inicio da situação funcional ate a data fim da situação funcional 
                        else if($altesitfunc->getDtfimsitufunc() <= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)){
                            $this->diasAtestadoNaCompetencia += count(DateUtil::getDatasEntre($altesitfunc->getDtinisitufunc(), $altesitfunc->getDtfimsitufunc()));
                        }//Se a data fim da situação func for maior ou igual que o ultimo da da competencia deve-se utilizar a data inicio da situação ate a data fim da competencia. 
                        else if($altesitfunc->getDtfimsitufunc() >= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)){
                            $this->diasAtestadoNaCompetencia += count(DateUtil::getDatasEntre($altesitfunc->getDtinisitufunc(), DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)));
                        }
                    }
                }
                
                // ----- Verifica se o vínculo teve FÉRIAS no perído da competência
                $competencia = clone DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true);
                $competencia = !empty($minimizadores['QTMESESFERIASBENEF']) && $minimizadores['QTMESESFERIASBENEF'] > 0 ? $competencia->sub( new \DateInterval('P'.(string)$minimizadores['QTMESESFERIASBENEF'].'M') ) : $competencia;
                
                $ferias = Utils::getEntityManager()->getRepository(Repositories::FPA_FERIAS)->getPeriodoGozoFeriasVt(
                    $vinculo->getNrvinculom(),
                    DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true ), 
                    DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true ),
                    $nrorg
                );
                
                if($ferias){
                    $motivos[] = "FERIAS";
                    $temFerias = true;
                    $this->temFeriasNaCompetencia = true;
                    
                    $iniGozo = DateUtil::getDataDeString(substr($ferias['DTINIGOZOFERIAS'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true );
                    $fimGozo = DateUtil::getDataDeString(substr($ferias['DTFIMGOZOFERIAS'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true );
                    
                    //Utilizar estas propriedades somente em Calculo Mensal (Pois verifica o fim da competencia(Mes inteiro))
                    if(($fimGozo <= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)) && ($iniGozo >= DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true))){
                        $this->diasFeriasNaCompetencia += count(DateUtil::getDatasEntre($iniGozo, $fimGozo));
                    } else if(($iniGozo >= DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)) && ($fimGozo > DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true))) {
                        $this->diasFeriasNaCompetencia += count(DateUtil::getDatasEntre($iniGozo, DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)));
                    } else if($fimGozo <= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true) && ($iniGozo < DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true))){
                        $this->diasFeriasNaCompetencia += count(DateUtil::getDatasEntre(DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true), $fimGozo));
                    }
                    #var_dump($this->diasFeriasNaCompetencia);die;
                    
                }
                
            } else if(!empty($tpperiodo) && $tpperiodo == 'DIARIO'){
                $this->diasFeriasNaCompetencia = 0;
                $this->diasAtestadoNaCompetencia = 0;
                // ----- Verifica se o vínculo FALTOU no período da competência
                $nrtpmovimento = '1';
                $nreventom = "490, 495, 1040, 1043";
            
                $competencia = clone DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true);
                $competencia = !empty($minimizadores['QTMESESFALTABENEF']) && $minimizadores['QTMESESFALTABENEF'] > 0 ? $competencia->sub( new \DateInterval('P'.(string)$minimizadores['QTMESESFALTABENEF'].'M') ) : $competencia;

                if($this->deveCalcularFaltasPorSituacaoFuncional == true){
                    $faltas = $this->calcularDiasFaltas($minimizadores, $vinculo);
                } else {
                    $faltas = Utils::getEntityManager()->getRepository(Repositories::FPA_CALCULOFOLHA)->getSomaItemCalculo(
                        $vinculo->getNrvinculom(), 
                        $nrorg,
                        DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true),
                        DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true),
                        $nreventom, 
                        $nrtpmovimento, 
                        DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)
                    );
                    
                    if( $faltas && (int)$faltas['VALOR'] > 0){
                        $faltas = (int)$faltas['VALOR'];
                    }
                }
                
                if($faltas > 0){
                    $motivos[] = "FALTA";
                    $temFalta = true;
                }
        
                // ----- Verifica se o vínculo ficou de ATESTADO no período da competência
                $competencia = DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true);
                
                $listNrsitufuncm = Utils::getEntityManager()->getRepository(Repositories::GPE_SITUFUNCH)->getAfastamentoIncideVale(substr($dtmescompetenc, 0, 10));
                $listNrsitufuncm = implode(', ', $listNrsitufuncm);
        
                $altesitfuncs = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncional_AtesOuAfast(
                    $nrorg, 
                    $vinculo->getNrvinculom(), 
                    $listNrsitufuncm, 
                    DateUtil::getDataDeString($competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true),
                    DateUtil::getDataDeString($competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true), 
                    "ATESTADO"
                );
                
                if($altesitfuncs){
                    $motivos[] = "ATESTADO";
                    $temAtestado = true;
                    $this->temAtestadoNaCompetencia = true;
                    
                    //Utilizar estas propriedades somente em Calculo Mensal (Pois verifica o fim da competencia(Mes inteiro))
                    foreach($altesitfuncs as $altesitfunc){
                        //Se a data fim da situação funcional for nula, ira analisar do inicio da situação ate o final da competencia.
                        if(is_null($altesitfunc->getDtfimsitufunc())){
                            $this->diasAtestadoNaCompetencia += count(DateUtil::getDatasEntre($altesitfunc->getDtinisitufunc(), DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)));
                        }//Se a data fim for menor ou igual do que o ultimo dia da competencia ira-se utilizar a data inicio da situação funcional ate a data fim da situação funcional 
                        else if($altesitfunc->getDtfimsitufunc() <= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)){
                            $this->diasAtestadoNaCompetencia += count(DateUtil::getDatasEntre($altesitfunc->getDtinisitufunc(), $altesitfunc->getDtfimsitufunc()));
                        }//Se a data fim da situação func for maior ou igual que o ultimo da da competencia deve-se utilizar a data inicio da situação ate a data fim da competencia. 
                        else if($altesitfunc->getDtfimsitufunc() >= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)){
                            $this->diasAtestadoNaCompetencia += count(DateUtil::getDatasEntre($altesitfunc->getDtinisitufunc(), DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)));
                        }
                    }
                }
                
                // ----- Verifica se o vínculo teve FÉRIAS no perído da competência
                $competencia = DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true);
                
                $ferias = Utils::getEntityManager()->getRepository(Repositories::FPA_FERIAS)->getPeriodoGozoFeriasVt(
                    $vinculo->getNrvinculom(),
                    DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true ), 
                    DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true ),
                    $nrorg
                );            
                
                if($ferias){
                    $motivos[] = "FERIAS";
                    $temFerias = true;
                    $this->temFeriasNaCompetencia = true;
        
                    $iniGozo = DateUtil::getDataDeString(substr($ferias['DTINIGOZOFERIAS'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true );
                    $fimGozo = DateUtil::getDataDeString(substr($ferias['DTFIMGOZOFERIAS'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true );
                    
                    /*
                    //Utilizar estas propriedades somente em Calculo Mensal (Pois verifica o fim da competencia(Mes inteiro))
                    if($fimGozo <= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)){
                        $this->diasFeriasNaCompetencia += count(DateUtil::getDatasEntre($iniGozo, $fimGozo));
                    } else if($fimGozo >= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)){
                        $this->diasFeriasNaCompetencia += count(DateUtil::getDatasEntre($iniGozo, DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)));
                    }*/
                    
                    //Utilizar estas propriedades somente em Calculo Mensal (Pois verifica o fim da competencia(Mes inteiro))
                    if(($fimGozo <= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)) &&  ($iniGozo >= DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true))){
                        $this->diasFeriasNaCompetencia += count(DateUtil::getDatasEntre($iniGozo, $fimGozo));
                    } else if(($iniGozo >= DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)) && ($fimGozo > DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true))) {
                        $this->diasFeriasNaCompetencia += count(DateUtil::getDatasEntre($iniGozo, DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)));
                    } else if($fimGozo <= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true) && ($iniGozo < DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true))){
                        $this->diasFeriasNaCompetencia += count(DateUtil::getDatasEntre(DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true), $fimGozo));
                    }
                    
                }
            }
            
            if($verificaAfastamento && $verificaAfastamento == true){
                $this->diasAfastamentoNaCompetencia = 0;
                
                // ----- Verifica se o vínculo ficou de AFASTAMENTO no perído da competência
                $competencia = DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true);
                
                if(isset($minimizadores) && !empty($minimizadores)){
                    //Verifica se o parametro de gerar beneficio no afastamento esta marcado como Não, e faz uma logica em conjunto com o campo
                    //Gera beneficio no retorno do afastamento, para que se caso o vinculo tiver retornando do afastamento na competencia deve-se gerar.
                    if((!empty($minimizadores['IDGERABENEFAFAST']) && $minimizadores['IDGERABENEFAFAST'] == 'N') && (!empty($minimizadores['IDGERABENEFRETOAFAST']) && $minimizadores['IDGERABENEFRETOAFAST'] == 'S')){
                        $competencia = ((!empty($minimizadores['IDCONSAFASTBENEF']) && $minimizadores['IDCONSAFASTBENEF'] == 'S') && isset($minimizadores['QTMESESAFASTBENEF'])) && $minimizadores['QTMESESAFASTBENEF'] > 0 ? $competencia->sub( new \DateInterval('P'.(string)$minimizadores['QTMESESAFASTBENEF'].'M') ) : $competencia;
                    }
                }
                
                $listNrsitufuncm = Utils::getEntityManager()->getRepository(Repositories::GPE_SITUFUNCH)->getAfastamentoIncideVale($dtmescompetenc);
                $listNrsitufuncm = implode(', ', $listNrsitufuncm);
                
                $altesitfuncs = Utils::getEntityManager()->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncional_AtesOuAfast(
                    $nrorg, 
                    $vinculo->getNrvinculom(), 
                    $listNrsitufuncm, 
                    DateUtil::getDataDeString( $competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true),
                    DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true), 
                    "AFASTAMENTO"
                );
                
                if($altesitfuncs){
                    $limpaAfastamento = false;
                    
                    $competenciaBase = substr($competencia->format('d/m/Y'), 0, 10);
                    $primeiroDiaComp = $competenciaBase ? DateTime::createFromFormat(DateUtil::FORMATO_BRASILEIRO, $competenciaBase)->format('Y-m-d') : null;   
                    $ultimoDiaComp   = $competenciaBase ? DateTime::createFromFormat(DateUtil::FORMATO_BRASILEIRO, $competenciaBase)->format('Y-m-t') : null;   
                    
                    foreach($altesitfuncs as $altesitfunc ){
                        
                        //Utilizar estas propriedades somente em Calculo Mensal (Pois verifica o fim da competencia(Mes inteiro))
                        //Se a data fim da situação funcional for nula, ira analisar do inicio da situação ate o final da competencia.
                        if(is_null($altesitfunc->getDtfimsitufunc())){
                            $this->diasAfastamentoNaCompetencia += count(DateUtil::getDatasEntre($altesitfunc->getDtinisitufunc(), DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)));
                            
                            if((!empty($minimizadores['IDGERABENEFAFAST']) && $minimizadores['IDGERABENEFAFAST'] == 'N') && (!empty($minimizadores['IDGERABENEFRETOAFAST']) && $minimizadores['IDGERABENEFRETOAFAST'] == 'S')){
                                if($limpaAfastamento == true){
                                    $limpaAfastamento = false;  
                                }
                            }
                        }//Se a data fim for menor ou igual do que o ultimo dia da competencia ira-se utilizar a data inicio da situação funcional ate a data fim da situação funcional 
                        else if($altesitfunc->getDtfimsitufunc() < DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)){
                            $this->diasAfastamentoNaCompetencia += count(DateUtil::getDatasEntre($altesitfunc->getDtinisitufunc(), $altesitfunc->getDtfimsitufunc()));
                            
                            if((!empty($minimizadores['IDGERABENEFAFAST']) && $minimizadores['IDGERABENEFAFAST'] == 'N') && (!empty($minimizadores['IDGERABENEFRETOAFAST']) && $minimizadores['IDGERABENEFRETOAFAST'] == 'S')){
                                if($limpaAfastamento == false){
                                    $limpaAfastamento = true;    
                                }
                            }
                        }//Se a data fim da situação func for maior que o ultimo dia da competencia deve-se utilizar a data inicio da situação ate a data fim da competencia. 
                        else if($altesitfunc->getDtfimsitufunc() >= DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)){
                            $this->diasAfastamentoNaCompetencia += count(DateUtil::getDatasEntre($altesitfunc->getDtinisitufunc(), DateUtil::getDataDeString( $competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true)));
                        
                            if((!empty($minimizadores['IDGERABENEFAFAST']) && $minimizadores['IDGERABENEFAFAST'] == 'N') && (!empty($minimizadores['IDGERABENEFRETOAFAST']) && $minimizadores['IDGERABENEFRETOAFAST'] == 'S')){
                                if($limpaAfastamento == true){
                                    $limpaAfastamento = false;  
                                }
                            }
                        }
                                        
                        $altesitfunc->getDtinisitufunc();
                        $altesitfunc->getDtinisitufunc();
                        
                        $dtIniSituFunc = $altesitfunc->getDtinisitufunc() ? DateTime::createFromFormat( DateUtil::FORMATO_BRASILEIRO, $altesitfunc->getDtinisitufunc()->format('d/m/Y') )->format('Y-m-d') : null;   
                        $dtFimSituFunc = $altesitfunc->getDtfimsitufunc() ? DateTime::createFromFormat( DateUtil::FORMATO_BRASILEIRO, $altesitfunc->getDtfimsitufunc()->format('d/m/Y') )->format('Y-m-d') : null;   
            
                        $situFuncH = Utils::getEntityManager()->getRepository(Repositories::GPE_SITUFUNCM)->retornaUltimoHistoricoCompetencia(
                            $nrorg, 
                            0, 
                            $altesitfunc->getNrsitufuncm(), 
                            $dtmescompetenc
                        );
            
                        // Se vínculo estiver afastado, que não seja licensa gestante nem demissão, possivelmente a empresa pagará o benefício total
                        if( 
                               $altesitfunc->getNrsitufuncm() != 5     // 5 = licensa gestante
                            && $altesitfunc->getNrsitufuncm() != 11    // 11 = prolongação licensa gestante
                            && $altesitfunc->getNrsitufuncm() != 13    // 13 = demissão
                            && $situFuncH->getIdgervaletransp() == 'N' // “Gera Vale-Transporte Benefícios“ deve estar como Não
                            && $dtIniSituFunc < $primeiroDiaComp       // Afastamento deve ter se iniciando antes do início da competência
                            
                            && (    !$dtFimSituFunc 
                                 || (
                                        !($primeiroDiaComp <= $dtFimSituFunc && $dtFimSituFunc <= $ultimoDiaComp) // Não pode ter retorno durante competencia
                                        && $dtFimSituFunc > $ultimoDiaComp // A data final de afastamento deve ser maior que o último dia da competência
                                    )
                               )    
                        )
                            $subsidio100PorAfastamentoOuDemissao = true;
            
                    }
                    
                    //Indica que de fato o vinculo n possui direito ao BENEFICIO pois possiu afastamento 
                    if($limpaAfastamento == false){
                        $motivos[] = "AFASTAMENTO";
                        $temAfastamento = true;
                        $this->temAfastamentoNaCompetencia = true;
                    }
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    //Função que recebe a data de admissão, e rescisão, data inicio e fim e retorna a quantidade de dias trabalhados no mes
    public function calculaDiasNormais($nrorg, $competencia, $dtAdmissao, $dtRescisao, $escalaTrab = null, $tpBeneficio = null){
        try {
            $competencia = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO, true);
            
            //01/10/2023
            $inicio = DateUtil::getDataDeString($competencia->format('d/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
            //31/10/2023
            $fim = DateUtil::getDataDeString($competencia->format('t/m/Y'), DateUtil::FORMATO_BRASILEIRO, true);
            
            // Converte as strings de data para objetos DateTime
            //22/08/2023
            $admissao = new DateTime($dtAdmissao);
            
            //NULA
            !is_null($dtRescisao) ? $rescisao = new DateTime($dtRescisao) : $rescisao = null;
            
            if($admissao >= $inicio && $admissao <= $fim){
                $inicio = $admissao;
            }
            
            if($rescisao && ($rescisao <= $fim && $rescisao >= $inicio)){
                $fim = $rescisao;
            }
            
            $diasNormais = count(DateUtil::getDatasEntreAjustada($inicio, $fim));
            
            //A RESPEITO DA ISSUE/TAREFA: 486965 - VALE COMBUSTÍVEL POR KM
            //TPBENEFICIO == 6 (COMBUSTIVEL)
            //Se caso o valor de referencia estiver ativo... usa os dados da escala para analisar o tipo de jornada.
            if(!empty($escalaTrab) && (!empty($tpBeneficio['NRTPBENEFICIO']) && $tpBeneficio['NRTPBENEFICIO'] == 6) ){
                //Verificar se o valor de referencia especifico para pre fixar vale combustivel esta criado na org;
                $valorReferenciaPreFixaDias = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor(
                    $this->hcmParameters->getNrorgTrabalho(),
                    0,
                    $this->hcmParameters->getDtCompetencia(),
                    'PREFIXADIAS_VALECOMBUSTIVEL'
                );
                
                if(!empty($valorReferenciaPreFixaDias) && $valorReferenciaPreFixaDias == 1){
                    //Se caso o tipo de jornada for 2 então fixar dias em 15, se caso 
                    $diasNormais = !empty($escalaTrab['NRTPJORNADA']) && $escalaTrab['NRTPJORNADA'] == 2 ? 15 : 21;
                }
            }
            
            $diasNormais = $diasNormais - ($this->diasAfastamentoNaCompetencia + $this->diasFeriasNaCompetencia);
            
            return $diasNormais;
        } catch (\Exception $e) {
            throw $e;
        }
    }
}